//------------------------------------------------------------------------------
// Multi Board Synchronous Scan With Phase Comparison
//------------------------------------------------------------------------------

#ifdef __linux__
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ncurses.h>
#include <sys/select.h>
#include <sys/statvfs.h>
#include <pthread.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/time.h>
#include <errno.h>
#else
#include <conio.h>
#include <stdio.h>
#include <wtypes.h>
#include <windows.h>
#include "Tools.h"
#endif

#include "GscApi.h"

#if !defined(ULONGLONG)
typedef unsigned long long         ULONGLONG;
#endif



int ValueRead, indata, data_in;
int ulNumBds, ulBdNum, ulAuxBdNum, ulErr = 0;
int ulValue;
U16 saved_x, saved_y, CurX=2, CurY=2;
char mmchar, kbchar, datebuf[16], timebuf[16];
int uData[65535], Data[262144], inphase[1024];

GSC_DEVICES_STRUCT      gscDevices[5];
BOARD_INFO              boardInfo[5];
#ifndef __linux__
HANDLE                  hBufferThresholdEvent;
HANDLE                  hBufferThresholdEventAux;
HANDLE                  hChannelsReadyEvent;
HANDLE                  hChannelsReadyEventAux;
HANDLE                  hMultiIniBoardBuffLowHigh;
HANDLE                  hMultiAuxBoardBuffLowHigh;
#endif

// forward declarations
void MultiBoardSync3(void);

// ncurses implementation of screen functions
// linux system call implementation of keyboard functions
//
#ifdef __linux__

WINDOW *term;

void kbflush();

void PutCursor( U32 y, U32 x )
{
  move(x, y);
}

void CursorVisible(BOOL CVState)
{
  if (CVState == TRUE)
    curs_set(1);
  else
    curs_set(0);
}

void ClrScr(void)
{
  clear();
}

int cprintf( const char * format, ...)
{
  char sLine[120];
  va_list args;
  int chars_printed = 0;

  va_start( args, format);

  chars_printed = vsprintf( sLine, format, args );
  addstr( sLine );
  refresh();
  return chars_printed;
}

char anykey(void)
{
  char ch;

  printf("\n  Press AnyKey to Continue...."); // Prompt for AnyKey
  refresh();
  ch = getch();
  kbflush();                                    // Empty KB Buffer
  return(toupper( ch ));    // Return the toupper of the key
}

void ShowAPIError(U32 APIReturnCode)
{
  char errorMsg[100];

  clear();
  printf("\n  An Error has Occured in the API for the Device Driver.");
  printf("\n  Below is all the information that was available about ");
  printf("\n  the encountered Error.                             \n ");

  GscGetErrorString( APIReturnCode, errorMsg );

  printf( "\n Return Code %d = %s", (int)APIReturnCode, errorMsg );
  refresh();
  anykey();
}

int kbhit(void)
{
  struct timeval tv;
  fd_set read_fd;

  /* Do not wait at all, not even a microsecond */
  tv.tv_sec=0;
  tv.tv_usec=0;

  /* Must be done first to initialize read_fd */
  FD_ZERO(&read_fd);

  /* Makes select() ask if input is ready:
   * 0 is the file descriptor for stdin    */
  FD_SET(0,&read_fd);

  /* The first parameter is the number of the
   * largest file descriptor to check + 1. */
  if(select(1, &read_fd,
            NULL, /*No writes*/
            NULL, /*No exceptions*/
            &tv)
     == -1)
    return 0;  /* An error occured */

  /* read_fd now holds a bit map of files that are
   * readable. We test the entry for the standard
   * input (file 0). */
  if(FD_ISSET(0,&read_fd))
    /* Character pending on stdin */
    return 1;

  /* no characters were pending */
  return 0;
}

void kbflush()
{
  char ch;

  while (kbhit())
  {
    read(0, &ch, 1);
  }
}

char prompt_for_key(char *message)
{
  char sLine[120];

  sprintf(sLine, "%s", message); // Display the Message
  addstr(sLine);
  kbflush();                     // Empty KB Buffer
  refresh();
  do{}while( !kbhit() );         // Wait for a Key Press
  return(toupper( getch() ));    // Return the toupper of the key
}

void Busy_Signal(int speed)
{
  int BS_x, BS_y;
  U32 BSLoop;

  // Save Cursor Position
  getyx( term, BS_y, BS_x );

  for(BSLoop = speed;BSLoop>0x0000;BSLoop--)
  {
    move(70, 2);
    attrset(COLOR_PAIR( COLOR_CYAN ));
    addstr("*\b");
    refresh();
    usleep(50000);

    move(70, 2);
    attrset(COLOR_PAIR( COLOR_RED ));
    addstr("|");
    refresh();
    usleep(50000);

    move(70, 2);
    attrset(COLOR_PAIR( COLOR_MAGENTA ));
    addstr("/");
    refresh();
    usleep(50000);

    move(70, 2);
    attrset(COLOR_PAIR( COLOR_WHITE ));
    addstr("-");
    usleep(50000);

    move(70, 2);
    attrset(COLOR_PAIR( COLOR_BLUE ));
    attron( A_STANDOUT );
    addstr("\\");
    refresh();
    usleep(50000);

    move(70, 2);
    attrset(COLOR_PAIR( COLOR_GREEN ));
    addstr("|");
    refresh();
    usleep(100000);

    move(70, 2);
    attrset(COLOR_PAIR( COLOR_CYAN ));
    addstr("/");
    refresh();
    usleep(100000);

    move(70, 2);
    attrset(COLOR_PAIR( COLOR_RED ));
    addstr("-");
    refresh();
    usleep(100000);

    move(70, 2);
    attrset(COLOR_PAIR( COLOR_MAGENTA ));
    addstr("\\");
    refresh();
    usleep(100000);

  }

  move(70, 2);
  addstr(" ");
  use_default_colors();
  attrset(COLOR_PAIR(0));
  move(BS_x, BS_y);
  refresh();

}

void Sleep(int ms)
{
  usleep( ms * 1000 );
}

// structures and constants for use with linux ipc functions
typedef struct _THREAD_MESSAGE
{
  long int type;
  int command;
} THREAD_MESSAGE, *PTHREAD_MESSAGE;

#define MSG_GSCANA_AUTOCALCOMPLETE      1
#define MSG_GSCANA_THRESHOLDLOWTOHIGH   2
#define MSG_GSCANA_CHANNELSREADY        4

// pthreads variables
int callback_qid = 0;

#else
//==============================================================================
void PutCursor(U32 FixedX, U32 FixedY)
{
  PositionCursor((U16) FixedX,(U16) FixedY);
}
#endif



//-----------------------------------------------------------------------
int main( int argc, char *argv[])
{
  int status = 0;
  int i = 0;
#ifdef __linux__
  char sLine[120];
#endif

#ifdef __linux__
  // create message queue for interrupt callback communication
  callback_qid = msgget(IPC_PRIVATE, 0666 | IPC_CREAT| IPC_EXCL);
  if(callback_qid == -1)
  {
    printf("Problem creating message queue, err = %s.\n", strerror(errno));
    return 1;
  }

  // initialize ncurses
  term = initscr();
  nonl();
  cbreak();
  noecho();

#endif

  CursorVisible(FALSE);

  if ((status = GscFindBoards( &ulNumBds, gscDevices )) != 0)
  {
    ShowAPIError(status);
#ifdef __linux__
    // Delete the message queue
    if (msgctl(callback_qid, IPC_RMID, (struct msqid_ds *)0) == -1)
    {
      sprintf(sLine, "Problem deleting message queue, error = %s, queue id = %d",
              strerror(errno), callback_qid);
      addstr(sLine);
    }
    endwin();
#endif
    do{Sleep(100);}while( !kbhit() );         // Wait for a Key Press
    exit(0);
  }
  if(ulNumBds < 1)
  {
    ClrScr();
    PutCursor(0,2);
    cprintf("  ERROR: No Boards found in the system\n");
  }
  else
  {
    ulBdNum = 0;
    ClrScr();
#ifdef __linux__
    addstr("\n\n");
    addstr("  ====================================================\n");
    addstr("   Select Board Number to Test                          \n");
    addstr("  ====================================================\n");

    addstr("        Bus/   Vendor/                DLL     Driver     FPGA    \n");
    addstr(" Board  Slot   Device    Serial #   Version   Version   Version  \n");
    addstr(" -----  ----  ---------  ---------  --------  --------  -------- \n");
#else
    printf("\n\n");
    printf("  ====================================================\n");
    printf("   Select Board Number to Test                          \n");
    printf("  ====================================================\n");

    printf("        Bus/   Vendor/                DLL     Driver     FPGA    \n");
    printf(" Board  Slot   Device    Serial #   Version   Version   Version  \n");
    printf(" -----  ----  ---------  ---------  --------  --------  -------- \n");
#endif
    for(i=0; i < ulNumBds; i++)
    {
      // Open the board
      if ((status = GscOpen( i+1, GSC_API_VERSION)) != 0)
      {
        ShowAPIError(status);
        exit(0);
      }

      // Get the board info
      if ((status = GscGetInfo( i+1, &boardInfo[ i])) == 0)
      {
#ifdef __linux__
        sprintf(sLine, "   %d    %d/%d  %04X/%04X   %s ",
          (int)i+1,
          gscDevices[i].busNumber,
          gscDevices[i].slotNumber,
          gscDevices[i].vendorId,
          gscDevices[i].deviceId,
          gscDevices[i].serialNumber);
        addstr( sLine );

        // Display the versions
        sprintf(sLine, "       %s    %s  %s\n",
          boardInfo[i].apiVersion, boardInfo[i].driverVersion,
          boardInfo[i].fpgaVersion);
        addstr( sLine );
#else
        printf("   %d    %d/%d  %04X/%04X  %s ",
          i+1,
          gscDevices[i].busNumber,
          gscDevices[i].slotNumber,
          gscDevices[i].vendorId,
          gscDevices[i].deviceId,
          gscDevices[i].serialNumber);

        // Display the versions
        printf(" %s  %s  %s\n", boardInfo[i].apiVersion, boardInfo[i].driverVersion,
          boardInfo[i].fpgaVersion);
#endif
      }
      else
        printf("Problem getting board info, err = %d\n", status);

      // Close the board
      GscClose( i+1);
    }

    //printf("\n");
    do
    {
      kbchar = prompt_for_key("\n  Please Make selection: ");
      switch(kbchar)
      {
      case '1':
        {
          ulBdNum = 1;
          break;
        }
      case '2':
        {
          ulBdNum = 2;
          break;
        }
      case '3':
        {
          ulBdNum = 3;
          break;
        }
      case '4':
        {
          ulBdNum = 4;
          break;
        }
      case '5':
        {
          ulBdNum = 5;
          break;
        }
      }
      if((ulBdNum == 0) || (ulBdNum > ulNumBds))
        printf("  Error - This is not a valid entry");
    } while((ulBdNum == 0) || (ulBdNum > ulNumBds));

    ulAuxBdNum = 0;
    if(ulNumBds >1) {
      ulAuxBdNum = ulBdNum;
      ClrScr();
      cprintf("\n\n");
      cprintf("  ====================================================\n");
      cprintf("   Select Auxillary Board Number to Use               \n");
      cprintf("   For Examples Requiring Two Boards (0 = Disabled)   \n");
      cprintf("  ====================================================\n");
      for(i=0; i < ulNumBds; i++)
      {
#ifdef __linux__
        sprintf(sLine, "   %d    %d/%d  %04X/%04X   %s ",
          (int)i+1,
          gscDevices[i].busNumber,
          gscDevices[i].slotNumber,
          gscDevices[i].vendorId,
          gscDevices[i].deviceId,
          gscDevices[i].serialNumber);
        addstr( sLine );

        // Display the versions
        sprintf(sLine, "       %s    %s  %s\n",
          boardInfo[i].apiVersion, boardInfo[i].driverVersion,
          boardInfo[i].fpgaVersion);
        addstr( sLine );
#else
        printf("   %d    %d/%d  %04X/%04X  %s ",
          i+1,
          gscDevices[i].busNumber,
          gscDevices[i].slotNumber,
          gscDevices[i].vendorId,
          gscDevices[i].deviceId,
          gscDevices[i].serialNumber);

        // Display the versions
        printf(" %s  %s  %s\n", boardInfo[i].apiVersion, boardInfo[i].driverVersion,
          boardInfo[i].fpgaVersion);
#endif

      }
      //printf("\n\n");
      do
      {
        kbchar = prompt_for_key("\n  Please Make selection: \n");
        switch(kbchar)
        {
        case '0':
          {
            ulAuxBdNum = 0;
            break;
          }
        case '1':
          {
            ulAuxBdNum = 1;
            break;
          }
        case '2':
          {
            ulAuxBdNum = 2;
            break;
          }
        case '3':
          {
            ulAuxBdNum = 3;
            break;
          }
        case '4':
          {
            ulAuxBdNum = 4;
            break;
          }
        case '5':
          {
            ulAuxBdNum = 5;
            break;
          }
        }
        if((ulAuxBdNum == ulBdNum) || (ulAuxBdNum > ulNumBds))
          printf("  Error - This is not a valid entry");
      } while((ulAuxBdNum == ulBdNum) || (ulBdNum > ulNumBds));
    }
  }

  if (ulNumBds > 0)
    MultiBoardSync3();

  prompt_for_key("Hit any key to exit.");

#ifdef __linux__
  // Delete the message queue
  if (msgctl(callback_qid, IPC_RMID, (struct msqid_ds *)0) == -1)
  {
    sprintf(sLine, "Problem deleting message queue, error = %s, queue id = %d",
            strerror(errno), callback_qid);
    addstr(sLine);
    refresh();
  }
#endif

#ifdef __linux__
  endwin();
#endif

  // close the boards
  GscClose( ulBdNum );
  GscClose( ulAuxBdNum );
  return 0;
}


//==============================================================================
// Callback for channels ready interrupt
void CALLBACK isrChannelsReadyHandler( int board, int channel, int type)
{
#ifdef __linux__
  THREAD_MESSAGE  threadMessage;
  int status = 0;
  char sLine[80];
#endif

#ifdef __linux__
  memset( &threadMessage, 0, sizeof(threadMessage));
  threadMessage.type = MSG_GSCANA_CHANNELSREADY;
  status = msgsnd(callback_qid, (void *)&threadMessage, sizeof(int), 0);
  if (status)
  {
    sprintf(sLine, "msgsnd status = %s, queue id = %d",
            strerror(errno), callback_qid);
    addstr( sLine );
  }
#else
  if (board == ulBdNum)
    SetEvent( hChannelsReadyEvent );
  else
    SetEvent( hChannelsReadyEventAux );
#endif
}


//==============================================================================
// Callback for buffer threshold interrupt
void CALLBACK isrBufferThresholdHandler( int board, int channel, int type)
{
  //U32 ulBuffSize = 0;
  //U32 indata = 0;
  int status = 0;
#ifdef __linux__
  char sLine[80];
  THREAD_MESSAGE  threadMessage;
#endif
  //QueryPerformanceCounter( &presentTime );
  //GscAnaRegisterInterrupt( board, GSC_INTR_ANA_THRESHOLDLOWTOHIGH, NULL); // Lo->Hi transition
  //GscLocalRegisterRead( ulBdNum, GSC_ANALOG_RAG, &indata); // Disable Clock
  //GscLocalRegisterWrite( ulBdNum, GSC_ANALOG_RAG, indata | 0x10000); // Disable Clock
  //GscAnaSetOption( board, GSC_ANA_CLOCK_SOURCE, GSC_ANA_CLOCK_MANUAL); // turn sampling off
  //GscLocalRegisterRead( board, GSC_ANALOG_BS, &ulBuffSize );
  //GscLocalRegisterRead( ulBdNum, GSC_ANALOG_IBC,&ulIbc ); // Threshold
  //printf("Got buffer threshold low==>high transition interrupt, buffer size = %d.\n", ulBuffSize);
  //printf("Got buffer threshold low==>high transition interrupt, ibc reg = 0x%x.\n", ulIbc);
  //PutCursor( 5, 10);
  //elapsedDMATime = ((double) (presentTime.QuadPart - startTime.QuadPart)) / (double) frequency.QuadPart;
  //printf("intr: scan time = %lf secs \n", elapsedDMATime);
  //Sleep(100);
#ifdef __linux__
  memset( &threadMessage, 0, sizeof(threadMessage));
  threadMessage.type = MSG_GSCANA_THRESHOLDLOWTOHIGH;
  status = msgsnd(callback_qid, (void *)&threadMessage, sizeof(int), 0);
  if (status)
  {
    sprintf(sLine, "msgsnd status = %s, queue id = %d",
            strerror(errno), callback_qid);
    addstr( sLine );
  }
#else
  if (board == ulBdNum)
    SetEvent( hBufferThresholdEvent );
  else
    SetEvent( hBufferThresholdEventAux );
#endif
}


//------------------------------------------------------------------------------
// Multi Board Synchronous Scan With Phase Comparison
//------------------------------------------------------------------------------
void MultiBoardSync3(void)
{
#ifdef __linux__
  //pthread_t       writerThread;
  //pthread_attr_t  fsThreadAttribute;
  THREAD_MESSAGE  threadMessage;
  //struct statvfs  vfsbuf;
  U32             EventStatus = 0;
  int             msecs = 0;
#else
  HANDLE          theHandles[MAXIMUM_WAIT_OBJECTS];
  HANDLE          fbHandle = NULL;
  //HANDLE sbHandle;
  DWORD           EventStatus = 0;
#endif
  int             status = 0;
  int             ok;
  int             i;
  char            sMsg[100];

  ClrScr();
  CurX=CurY=2;
  cprintf("  Multiboard Sync test:     \n\n");
  //fprintf(TestData,"  Multiboard Sync test:     \n\n");
  cprintf("  Initializing ...   Please wait ~ 7 seconds  \n\n\n");
  cprintf("  Make the following LVDS connections: \n\n");
  cprintf("    Board2                    Board1 \n");
  cprintf(" EXT_CLK_OUT_LO   ===>     EXT_CLK_IN_LO \n");
  cprintf(" EXT_CLK_OUT_HI   ===>     EXT_CLK_IN_HI \n");
  cprintf(" EXT_SYNC_OUT_LO  ===>     EXT_SYNC_IN_LO \n");
  cprintf(" EXT_SYNC_OUT_HI  ===>     EXT_SYNC_IN_HI \n");
  cprintf("\n\n Connect a 5khz @ 9vpp sine wave to all inputs.    \n\n");

  if ((status = GscOpen( ulBdNum, GSC_API_VERSION)) != 0)
  {
    GscGetErrorString(status, sMsg);
    cprintf(sMsg);
    return;
  }
  GscAnaInitializeBoard( ulBdNum );
  if ((status = GscOpen( ulAuxBdNum, GSC_API_VERSION)) != 0)
  {
    GscGetErrorString(status, sMsg);
    cprintf(sMsg);
    return;
  }
  GscAnaInitializeBoard( ulAuxBdNum );

  anykey();
  ClrScr();
  CurX=CurY=2;
  cprintf("  Multiboard Sync test:     \n\n");

  GscAnaSetInputBufferThreshold( ulBdNum, 0x80 );
  if(ulAuxBdNum)
    GscAnaSetInputBufferThreshold( ulAuxBdNum, 0x80 );

  // ulAuxBdNum will be the Initiator board//
  GscAnaSetOption( ulAuxBdNum, GSC_ANA_CLOCK_SOURCE, GSC_ANA_CLOCK_SECONDARY );
  GscAnaSetOption( ulAuxBdNum, GSC_ANA_SAMPLE_RATE, 16000 );

  // ulBdNum will be the Target board
  GscAnaSetOption( ulBdNum, GSC_ANA_SAMPLE_RATE, 16000 );
  Sleep(200);

  //Set ulBdNum as Target board
  GscAnaSetOption( ulBdNum, GSC_ANA_CLOCK_SOURCE, GSC_ANA_CLOCK_EXTERNAL );

#ifndef __linux__
  hChannelsReadyEvent = CreateEvent( NULL, TRUE, FALSE, (LPCWSTR)"ChannelsReadyEvent" );
  hChannelsReadyEventAux = CreateEvent( NULL, TRUE, FALSE, (LPCWSTR)"AuxChannelsReadyEvent" );
  theHandles[0] = hChannelsReadyEvent;
  theHandles[1] = hChannelsReadyEventAux;
#endif

  if ((status = GscAnaRegisterInterrupt( ulBdNum, GSC_INTR_ANA_CHANNELSREADY,
					 isrChannelsReadyHandler )) != 0)
    printf("Error registering for channels ready interrupt, errno = %d\n", status );

  if ((status = GscAnaRegisterInterrupt( ulAuxBdNum, GSC_INTR_ANA_CHANNELSREADY,
					 isrChannelsReadyHandler )) != 0)
    printf("Error registering for channels ready interrupt, errno = %d\n", status );

  GscAnaSoftwareSync( ulAuxBdNum ); // Sync ALL Boards, Software sync

#ifdef __linux__
  msecs = 0;
  while (((status = msgrcv(callback_qid,(void *)&threadMessage, sizeof(int),
                           MSG_GSCANA_CHANNELSREADY, IPC_NOWAIT)) == -1)
         && (msecs < 6000))
  {
    usleep(50000);
    msecs += 50;
  }

  if (msecs < 6000)
  {
    // Wait for channels ready interrupt from second board - need to see
    // interrupt from both boards.
    while (((status = msgrcv(callback_qid,(void *)&threadMessage, sizeof(int),
                             MSG_GSCANA_CHANNELSREADY, IPC_NOWAIT)) == -1)
           && (msecs < 6000))
    {
      usleep(50000);
      msecs += 50;
    }
  }

  EventStatus = (status >= 0) ? 0 : -1;

  switch(EventStatus)
  {
  case 0:
  case 1:
#else
  EventStatus = WaitForMultipleObjects(2,theHandles,TRUE,6 * 1000); // Wait for notification
  switch(EventStatus)
  {
  case WAIT_OBJECT_0:
  case WAIT_OBJECT_0+1:
#endif
    PutCursor(CurX,19);
    cprintf("ALL Boards are Synchronized...                     \n");
    //fprintf(TestData,"ALL Boards are Synchronized...                     \n");
    Sleep(1000);
    break;
  default:
    PutCursor(CurX,16);
    cprintf("ALL Boards are NOT Synchronized...                  \n");
    //fprintf(TestData,"ALL Boards are NOT Synchronized...                  \n");
    GscLocalRegisterRead(ulBdNum, GSC_ANALOG_BCR, &indata);
    PutCursor(CurX,17);
    cprintf("Target Failed to Sync : BCR=%08lX...                 \n",indata);
    //fprintf(TestData,"Target Failed to Sync : BCR=%08lX...                 \n",indata);
    GscLocalRegisterRead(ulAuxBdNum, GSC_ANALOG_BCR, &indata);
    PutCursor(CurX,18);
    cprintf("Initiator Failed to Sync (AUX): BCR=%08lX...         \n",indata);
    //fprintf(TestData,"Initiator Failed to Sync (AUX): BCR=%08lX...         \n",indata);
    PutCursor(CurX,19);
    cprintf("                                                     \n",indata);
    Sleep(200);
    anykey();
    return;
    break;
  }

  // set clear buffer on sync bit for both boards
  GscAnaSetOption( ulAuxBdNum, GSC_ANA_CLEARBUFFER_ON_SYNC, GSC_ENABLED ); // Master
  GscAnaSetOption( ulBdNum, GSC_ANA_CLEARBUFFER_ON_SYNC, GSC_ENABLED ); // Slave

  PutCursor(CurX,19);
  cprintf("Autocal in Progress... Master                                                ");
  GscAnaAutoCal( ulAuxBdNum, 1 );

  PutCursor(CurX,19);
  cprintf("Autocal in Progress... Slave                                                 ");
  GscAnaAutoCal( ulBdNum, 1 );

#ifndef __linux__
  hBufferThresholdEvent = CreateEvent( NULL, TRUE, FALSE, (LPCWSTR)"BufferReadEvent" );
#endif

  if ((status = GscAnaRegisterInterrupt( ulBdNum, GSC_INTR_ANA_CHANNELSREADY,
					 NULL )) != 0)
    printf("Error unregistering channels ready interrupt, errno = %d\n", status );

  if ((status = GscAnaRegisterInterrupt( ulBdNum, GSC_INTR_ANA_THRESHOLDLOWTOHIGH,
					 isrBufferThresholdHandler )) != 0)
    printf("Error registering for buffer threshold interrupt, errno = %d\n", status );

  // Perform a software sync
  GscAnaSoftwareSync( ulAuxBdNum );

#ifdef __linux__
  msecs = 0;
  while (((status = msgrcv(callback_qid,(void *)&threadMessage, sizeof(int),
                           MSG_GSCANA_THRESHOLDLOWTOHIGH, IPC_NOWAIT)) == -1)
         && (msecs < 10000))
  {
    usleep(50000);
    msecs += 50;
  }
  EventStatus = (status >= 0) ? 0 : -1;
  switch(EventStatus)
  {
  case 0:
#else
  EventStatus = WaitForSingleObject(hChannelsReadyEventAux,
				    10 * 1000); // Wait for notification from interrupt callback
  switch(EventStatus)
  {
  case WAIT_OBJECT_0:
#endif
    PutCursor(CurX,19);
    cprintf("Buffers Cleared and ADC's are Synchronized...                         ",ulBdNum);
    //Sleep(1000);// had to comment this out to not miss firing of threshold low-->high interrupt.
    break;
  default:
    PutCursor(CurX,19);
    GscLocalRegisterRead(ulBdNum, GSC_ANALOG_BCR, &indata);
    GscLocalRegisterRead(ulAuxBdNum, GSC_ANALOG_BCR, &data_in);
    cprintf("Failed Sync : BCR%ld=%lX : BCR%ld=%lX               \n",ulBdNum,indata,ulAuxBdNum,data_in);
    Sleep(1000);
    anykey();
    return;
    break;
  }

#ifndef __linux__
  ResetEvent( hBufferThresholdEvent );
#endif

  if ((status = GscAnaRegisterInterrupt( ulAuxBdNum, GSC_INTR_ANA_CHANNELSREADY,
					 NULL )) != 0)
    cprintf("Error unregistering channels ready interrupt, errno = %d\n", status );

  if ((status = GscAnaRegisterInterrupt( ulAuxBdNum, GSC_INTR_ANA_THRESHOLDLOWTOHIGH,
					 isrBufferThresholdHandler )) != 0)
    cprintf("Error registering for buffer threshold interrupt, errno = %d\n", status );

  GscAnaClearInputBuffer( ulBdNum );
  GscAnaClearInputBuffer( ulAuxBdNum );
  PutCursor(CurX,19);
#ifdef __linux__
  msecs = 0;
  while (((status = msgrcv(callback_qid,(void *)&threadMessage, sizeof(int),
                           MSG_GSCANA_THRESHOLDLOWTOHIGH, IPC_NOWAIT)) == -1)
         && (msecs < 10000))
  {
    usleep(50000);
    msecs += 50;
  }
  EventStatus = (status >= 0) ? 0 : -1;
  switch(EventStatus)
  {
  case 0:
#else
  EventStatus = WaitForSingleObject(hBufferThresholdEvent,
				    10 * 1000); // Wait for the LO-HI buffer interrupt
  switch(EventStatus)
  {
  case WAIT_OBJECT_0:
#endif
    cprintf("Acquisition Started.. E to exit                      \n");
    cprintf("Clear Buffer       .. C to restart                   \n");
    cprintf("Verify below ~19h within the range of 7400h-8C00h above    ");
    break;
  default:
    cprintf("Interrupt timeout                           ");
    break;
  }

  PutCursor(CurX,19);
  ok = 1;

  do {
    PutCursor(CurX,CurY);

    mmchar = prompt_for_key("  Press anykey to Update now \n");

    switch(mmchar)
    {
    case 'E':
      ok=0;
      break;
    case 'C':
#ifndef __linux__
      ResetEvent( hBufferThresholdEvent );
#endif
      PutCursor(CurX,19);
      // Perform a software sync
      GscAnaSoftwareSync( ulAuxBdNum );
#ifdef __linux__
      msecs = 0;
      while (((status = msgrcv(callback_qid,(void *)&threadMessage, sizeof(int),
                               MSG_GSCANA_THRESHOLDLOWTOHIGH, IPC_NOWAIT)) == -1)
             && (msecs < 10000))
      {
        usleep(50000);
        msecs += 50;
      }
      EventStatus = (status >= 0) ? 0 : -1;
      switch(EventStatus)
      {
      case 0:
#else
      EventStatus = WaitForSingleObject(hBufferThresholdEvent,
		    		    10 * 1000); // Wait for notification from interrupt callback
      switch(EventStatus)
      {
      case WAIT_OBJECT_0:
#endif
        cprintf("Acquisition ReStarted.. E to exit                   \n");
        cprintf("Clear Buffer         .. C to restart                \n");
        cprintf("Verify below ~19h within the range of 7400h-8C00h above  ");
        break;
      default:
        cprintf("Interrupt timeout after sync            ");
        break;
      }

      break;
    default:
      PutCursor(0,CurY+1);
      cprintf("Slave            \n");
      for(i=0; i<32; i++){
        GscLocalRegisterRead(ulBdNum, GSC_ANALOG_IDB, &uData[i]);
        if(!(i%8))
          cprintf("\n");
        cprintf("%08lX ",uData[i]);
      }
      PutCursor(0,CurY+8);
      cprintf("Master            \n");
      for(i=0; i<32; i++){
        GscLocalRegisterRead(ulAuxBdNum, GSC_ANALOG_IDB, &Data[i]);
        if(!(i%8))
          cprintf("\n");
        cprintf("%08lX ",Data[i]);
      }
      PutCursor(0,CurY+21);
      cprintf("ABS Phase Difference            \n");
      for(i=0; i<32; i++){
        if(!(i%8))
          cprintf("\n");
        inphase[i] = abs(uData[i]-Data[i]);
        cprintf("%08lX ",abs(uData[i]-Data[i]));
      }
      i = 0;
    }

  }while(ok);
  kbflush();
}
