//------------------------------------------------------------------------------
// 24DSI12 Sample Code
//------------------------------------------------------------------------------
// Revision History:
//------------------------------------------------------------------------------
// Revision  Date        Name      Comments
// 1.0       02/07/03    Gary      Created
// 1.1       04/03/2008  M. Giesy  Ported to GscApi interface
//                                 Modified to run on both windows and linux.
// 1.1       05/01/2008  M. Giesy  Modified for DSI analog board.
//
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

#ifdef __linux__
#define _XOPEN_SOURCE 500
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include <ncurses.h>
#include <sys/select.h>
#include <sys/vfs.h>
#include <pthread.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/time.h>
#include <locale.h>
#else
#include <conio.h>
#include <stdio.h>
#include <wtypes.h>
#include <windows.h>
#include <math.h>
#include "Tools.h"
#include "time.h"
#endif

#include "GscApi.h"

#ifdef __linux__

#if !defined(ULONGLONG)
typedef unsigned long long         ULONGLONG;
#endif

#endif

void analog_inputs(void);
void SelfTest(void);
void CmdChainingDMATest(void);
void ShuttleDMATest(void);
void SglDMATest(void);
void BlockDMATest(void);
void ContinuousSaveData( int nChannels );
void MultiBoardExample(void);

#ifdef __linux__
void* InterruptAttachThreadDma(void *);
#else
DWORD WINAPI
InterruptAttachThreadDma(
    LPVOID pParam
    );
#endif


// Subs for the AI64SS
void  init_verify(void);
void  auto_cal_tst(void);
int   auto_cal(U32 BdNum);
void  get_avg_reading(int num_channels, int sample_size);
void  Scan_display();
void  PutCursor(U32,U32);
void  set_vRange(double range);

// ncurses implementation of screen functions
// linux system call implementation of keyboard functions
//
#ifdef __linux__

WINDOW *term;

void kbflush();

void PutCursor( U32 y, U32 x )
{
  move(x, y);
}

void CursorVisible(BOOL CVState)
{
  if (CVState == TRUE)
    curs_set(1);
  else
    curs_set(0);
}

void ClrScr(void)
{
  clear();
}

int cprintf( const char * format, ...)
{
  char sLine[120];
  va_list args;
  int chars_printed = 0;

  va_start( args, format);

  chars_printed = vsprintf( sLine, format, args );
  addstr( sLine );
  refresh();
  return chars_printed;
}

char anykey(void)
{
  char ch;

  //addstr("\n  Press AnyKey to Continue...."); // Prompt for AnyKey
  addstr(" Press AnyKey to Continue...."); // Prompt for AnyKey
  refresh();
  ch = getch();
  kbflush();                                    // Empty KB Buffer
  return(toupper( ch ));    // Return the toupper of the key
}

void ShowAPIError(U32 APIReturnCode)
{
  char errorMsg[100];

  clear();
  printf("\n  An Error has Occured in the API for the Device Driver.");
  printf("\n  Below is all the information that was available about ");
  printf("\n  the encountered Error.                             \n ");

  GscGetErrorString( APIReturnCode, errorMsg );

  printf( "\n Return Code %d = %s", (int)APIReturnCode, errorMsg );
  refresh();
  anykey();
}

int kbhit(void)
{
  struct timeval tv;
  fd_set read_fd;

  /* Do not wait at all, not even a microsecond */
  tv.tv_sec=0;
  tv.tv_usec=0;

  /* Must be done first to initialize read_fd */
  FD_ZERO(&read_fd);

  /* Makes select() ask if input is ready:
   * 0 is the file descriptor for stdin    */
  FD_SET(0,&read_fd);

  /* The first parameter is the number of the
   * largest file descriptor to check + 1. */
  if(select(1, &read_fd,
            NULL, /*No writes*/
            NULL, /*No exceptions*/
            &tv)
     == -1)
    return 0;  /* An error occured */

  /* read_fd now holds a bit map of files that are
   * readable. We test the entry for the standard
   * input (file 0). */
  if(FD_ISSET(0,&read_fd))
    /* Character pending on stdin */
    return 1;

  /* no characters were pending */
  return 0;
}

void kbflush()
{
  char ch;

  while (kbhit())
  {
    read(0, &ch, 1);
  }
}

char prompt_for_key(char *message)
{
  char sLine[120];

  sprintf(sLine, "%s", message); // Display the Message
  addstr(sLine);
  kbflush();                     // Empty KB Buffer
  refresh();
  do{}while( !kbhit() );         // Wait for a Key Press
  return(toupper( getch() ));    // Return the toupper of the key
}

void Busy_Signal(int speed)
{
  int BS_x, BS_y;
  U32 BSLoop;

  // Save Cursor Position
  getyx( term, BS_y, BS_x );

  for(BSLoop = speed;BSLoop>0x0000;BSLoop--)
  {
    move(2, 70);
    //attrset(COLOR_PAIR( COLOR_CYAN ));
    addstr("*\b");
    refresh();
    usleep(10000);

    move(2, 70);
    //attrset(COLOR_PAIR( COLOR_RED ));
    addstr("|");
    refresh();
    usleep(10000);

    move(2, 70);
    //attrset(COLOR_PAIR( COLOR_MAGENTA ));
    addstr("/");
    refresh();
    usleep(10000);

    move(2, 70);
    //attrset(COLOR_PAIR( COLOR_WHITE ));
    addstr("-");
    usleep(10000);

    move(2, 70);
    //attrset(COLOR_PAIR( COLOR_BLUE ));
    //attron( A_STANDOUT );
    addstr("\\");
    refresh();
    usleep(10000);

    move(2, 70);
    //attrset(COLOR_PAIR( COLOR_GREEN ));
    addstr("|");
    refresh();
    usleep(10000);

    move(2, 70);
    //attrset(COLOR_PAIR( COLOR_CYAN ));
    addstr("/");
    refresh();
    usleep(10000);

    move(2, 70);
    //attrset(COLOR_PAIR( COLOR_RED ));
    addstr("-");
    refresh();
    usleep(10000);

    move(2, 70);
    //attrset(COLOR_PAIR( COLOR_MAGENTA ));
    addstr("\\");
    refresh();
    usleep(10000);

  }

  move(2, 70);
  addstr(" ");
  //use_default_colors();
  //attrset(COLOR_PAIR(0));
  move(BS_x, BS_y);
  refresh();

}

void Sleep(int ms)
{
  usleep( ms * 1000 );
}

// pthreads variables
int callback_qid = 0;

// ipc structs and constants
#define WAIT_OBJECT_0                   0
#define MSG_GSCANA_AUTOCALCOMPLETE      1
#define MSG_GSCANA_THRESHOLDLOWTOHIGH   2

// Structure for passing thread messages
typedef struct _THREAD_MESSAGE
{
  long int type;
  int command;
} THREAD_MESSAGE, *PTHREAD_MESSAGE;

#endif

U32 i, j, *BuffPtr, *NewBuffPtr;
int ValueRead, indata, data_in;
int ulNumBds, ulBdNum, ulAuxBdNum, ulErr = 0;
U16 saved_x, saved_y, CurX=2, CurY=2;
char mmchar, kbchar, datebuf[16], timebuf[16], test_mode=0x00;
char cPlusMinus[3];

GSC_DEVICES_STRUCT      gscDevices[5];
BOARD_INFO              boardInfo[5];
#ifndef __linux__
HANDLE                  hBufferThresholdEvent;
HANDLE                  hAutoCalCompleteEvent;
HANDLE                  hMultiIniBoardBuffLowHigh;
HANDLE                  hMultiAuxBoardBuffLowHigh;
#endif
double elapsedDMATime = 0.0;


int uData[65535], Data[262144];
int dNumChan = 0;
U32 b1Ready,b2Ready,b1Done,b2Done;

U32 *Buff1;
U32 *Buff2;
U32 *wBuff;

U8 contRunning;
volatile U8 contThreadStatus;


//==============================================================================
//
//==============================================================================
int main(int argc, char * argv[])
{

  int status = 0;
  char sLine[120];

#ifdef __linux__
  // create message queue for interrupt callback communication
  callback_qid = msgget(IPC_PRIVATE, 0666 | IPC_CREAT| IPC_EXCL);
  if(callback_qid == -1)
  {
    printf("Problem creating message queue, err = %s.\n", strerror(errno));
    return 1;
  }

  // initialize ncurses
  term = initscr();
  nonl();
  cbreak();
  noecho();
  //  start_color();
  //init_pair(COLOR_BLACK, COLOR_BLACK, COLOR_BLACK);
  //init_pair(COLOR_GREEN, COLOR_GREEN, COLOR_BLACK);
  //init_pair(COLOR_RED, COLOR_RED, COLOR_BLACK);
  //init_pair(COLOR_CYAN, COLOR_CYAN, COLOR_BLACK);
  //init_pair(COLOR_WHITE, COLOR_WHITE, COLOR_BLACK);
  //init_pair(COLOR_MAGENTA, COLOR_MAGENTA, COLOR_BLACK);
  //init_pair(COLOR_BLUE, COLOR_BLUE, COLOR_BLACK);
  //init_pair(COLOR_YELLOW, COLOR_YELLOW, COLOR_BLACK);

#endif
  // Latin-1 plusminus character as UCN for linux xterm windows with utf-8 lang
  // and windows console
  strcpy(cPlusMinus, "\u00b1");

  CursorVisible(FALSE);

  if ((status = GscFindBoards( &ulNumBds, gscDevices )) != 0)
  {
    ShowAPIError(status);
#ifdef __linux__
    // Delete the message queue
    if (msgctl(callback_qid, IPC_RMID, (struct msqid_ds *)0) == -1)
    {
      sprintf(sLine, "Problem deleting message queue, error = %s, queue id = %d",
              strerror(errno), callback_qid);
      addstr(sLine);
    }
    endwin();
#endif

   exit(0);
  }
  if(ulNumBds < 1)
  {
#ifdef __linux__
    // Delete the message queue
    if (msgctl(callback_qid, IPC_RMID, (struct msqid_ds *)0) == -1)
    {
      sprintf(sLine, "Problem deleting message queue, error = %s, queue id = %d",
              strerror(errno), callback_qid);
      addstr(sLine);
    }
    endwin();
#endif
      printf("  ERROR: No Boards found in the system\n");
  }
  else
  {
    ulBdNum = 0;
    ClrScr();
#ifdef __linux__
    addstr("\n\n");
    addstr("  ====================================================\n");
    addstr("   Select Board Number to Test                          \n");
    addstr("  ====================================================\n");

    addstr("        Bus/   Vendor/                DLL     Driver     FPGA    \n");
    addstr(" Board  Slot   Device    Serial #   Version   Version   Version  \n");
    addstr(" -----  ----  ---------  ---------  --------  --------  -------- \n");
#else
    printf("\n\n");
    printf("  ====================================================\n");
    printf("   Select Board Number to Test                          \n");
    printf("  ====================================================\n");

    printf("        Bus/   Vendor/                DLL     Driver     FPGA    \n");
    printf(" Board  Slot   Device    Serial #   Version   Version   Version  \n");
    printf(" -----  ----  ---------  ---------  --------  --------  -------- \n");
#endif
    for(i=0; i < ulNumBds; i++)
    {
      // Open the board
      if ((status = GscOpen( i+1, GSC_API_VERSION)) != 0)
      {
        ShowAPIError(status);
        exit(0);
      }

      // Get the board info
      if ((status = GscGetInfo( i+1, &boardInfo[ i])) == 0)
      {
#ifdef __linux__
        sprintf(sLine, "   %d    %d/%d  %04X/%04X   %s ",
               (int)i+1,
               gscDevices[i].busNumber,
               gscDevices[i].slotNumber,
               gscDevices[i].vendorId,
               gscDevices[i].deviceId,
               gscDevices[i].serialNumber);
        addstr( sLine );

        // Display the versions
        sprintf(sLine, "       %s    %s  %s\n",
                boardInfo[i].apiVersion, boardInfo[i].driverVersion,
                boardInfo[i].fpgaVersion);
        addstr( sLine );
#else
        printf("   %d    %d/%d  %04X/%04X  %s ",
               i+1,
               gscDevices[i].busNumber,
               gscDevices[i].slotNumber,
               gscDevices[i].vendorId,
               gscDevices[i].deviceId,
               gscDevices[i].serialNumber);

        // Display the versions
        printf(" %s  %s  %s\n", boardInfo[i].apiVersion, boardInfo[i].driverVersion,
               boardInfo[i].fpgaVersion);
#endif
      }
      else
        printf("Problem getting board info, err = %d\n", status);

      // Close the board
      //GscClose( i+1);
    }

    printf("\n");
    do
    {
      kbchar = prompt_for_key(" Please Make selection: \n");
      switch(kbchar)
      {
      case '1':
        {
          ulBdNum = 1;
          break;
        }
      case '2':
        {
          ulBdNum = 2;
          break;
        }
      case '3':
        {
          ulBdNum = 3;
          break;
        }
      case '4':
        {
          ulBdNum = 4;
          break;
        }
      case '5':
        {
          ulBdNum = 5;
          break;
        }
      }
      if((ulBdNum == 0) || (ulBdNum > ulNumBds))
        printf("  Error - This is not a valid entry");
    }while((ulBdNum == 0) || (ulBdNum > ulNumBds));

    ulAuxBdNum = 0;
    if(ulNumBds >1){
      ulAuxBdNum = ulBdNum;
      ClrScr();
      cprintf("\n\n");
      cprintf("  ====================================================\n");
      cprintf("   Select Auxillary Board Number to Use               \n");
      cprintf("   For Examples Requiring Two Boards (0 = Disabled)   \n");
      cprintf("  ====================================================\n");
#ifdef __linux__
      addstr("        Bus/   Vendor/                DLL     Driver     FPGA    \n");
      addstr(" Board  Slot   Device    Serial #   Version   Version   Version  \n");
      addstr(" -----  ----  ---------  ---------  --------  --------  -------- \n");
#else
      printf("        Bus/   Vendor/                DLL     Driver     FPGA    \n");
      printf(" Board  Slot   Device    Serial #   Version   Version   Version  \n");
      printf(" -----  ----  ---------  ---------  --------  --------  -------- \n");
#endif
      for(i=0; i < ulNumBds; i++)
      {
        // Get the board info
        if ((status = GscGetInfo( i+1, &boardInfo[ i])) == 0)
        {
#ifdef __linux__
          sprintf(sLine, "   %d    %d/%d  %04X/%04X   %s ",
                  (int)i+1,
                  gscDevices[i].busNumber,
                  gscDevices[i].slotNumber,
                  gscDevices[i].vendorId,
                  gscDevices[i].deviceId,
                  gscDevices[i].serialNumber);
          addstr( sLine );

          // Display the versions
          sprintf(sLine, "       %s    %s  %s\n",
                  boardInfo[i].apiVersion, boardInfo[i].driverVersion,
                  boardInfo[i].fpgaVersion);
          addstr( sLine );
#else
          printf("   %d    %d/%d  %04X/%04X  %s ",
                 i+1,
                 gscDevices[i].busNumber,
                 gscDevices[i].slotNumber,
                 gscDevices[i].vendorId,
                 gscDevices[i].deviceId,
                 gscDevices[i].serialNumber);

          // Display the versions
          printf(" %s  %s  %s\n", boardInfo[i].apiVersion, boardInfo[i].driverVersion,
                 boardInfo[i].fpgaVersion);
#endif
        }
        else
          printf("Problem getting board info, err = %d\n", status);

      }

      printf("\n");
      do
      {
        kbchar = prompt_for_key(" Please Make selection: \n");
        switch(kbchar)
        {
        case '0':
          {
            ulAuxBdNum = 0;
            break;
          }
        case '1':
          {
            ulAuxBdNum = 1;
            break;
          }
        case '2':
          {
            ulAuxBdNum = 2;
            break;
          }
        case '3':
          {
            ulAuxBdNum = 3;
            break;
          }
        case '4':
          {
            ulAuxBdNum = 4;
            break;
          }
        case '5':
          {
            ulAuxBdNum = 5;
            break;
          }
        }
        if((ulAuxBdNum == ulBdNum) || (ulAuxBdNum > ulNumBds))
          printf("  Error - This is not a valid entry");
      } while((ulAuxBdNum == ulBdNum) || (ulBdNum > ulNumBds));

      if(ulAuxBdNum)
        if ((status = GscOpen( ulAuxBdNum, GSC_API_VERSION )) != 0)
        {
          ShowAPIError(status);
          do{}while( !kbhit() );         // Wait for a Key Press
          exit(0);
        }
    }

    cprintf(" Initializing board(s), please wait...\n");

    status = GscAnaInitializeBoard( ulBdNum );
    if (ulAuxBdNum != 0)
      status = GscAnaInitializeBoard( ulAuxBdNum );

    do
    {
      ClrScr();
#ifdef __linux__
      addstr("\n\n");
      addstr("  ====================================================\n");
      sprintf(sLine, "   Analog Board Sample Code - Board # %d                  \n",ulBdNum);
      addstr( sLine );
      if(ulAuxBdNum)
      {
        sprintf(sLine, "                     Aux  Board # %d                 \n",ulAuxBdNum);
        addstr( sLine );
      }
      addstr("  ====================================================\n");
      addstr("   1 - Board Initialization                           \n");
      addstr("   2 - Auto Calibration                               \n");
      addstr("   3 - Analog Inputs                                  \n");
      addstr("   4 - SelfTest Modes                                 \n");
      addstr("   5 - Sgl DMA Mode with Interrupts (& Data Packing if available)  \n");
      addstr("   6 - Continuous Data (Optionally to Disk)           \n");
      addstr("   7 - Continuous Data 8 Channels (SSA boards only, optionally to Disk)\n");
      if(ulAuxBdNum){
        addstr("   8 - Multiple Board Ext Sync Example               \n");
      }
      addstr("   X - EXIT (return to DOS)                           \n");
#else
      printf("\n\n");
      printf("  ====================================================\n");
      printf("   Analog Board Sample Code - Board # %d                  \n",ulBdNum);
      if(ulAuxBdNum)
        printf("                     Aux  Board # %d                 \n",ulAuxBdNum);
      printf("  ====================================================\n");
      printf("   1 - Board Initialization                           \n");
      printf("   2 - Auto Calibration                               \n");
      printf("   3 - Analog Inputs                                  \n");
      printf("   4 - SelfTest Modes                                 \n");
      printf("   5 - Sgl DMA Mode with Interrupts (& Data Packing if available)  \n");
      printf("   6 - Continuous Data (Optionally to Disk)           \n");
      printf("   7 - Continuous Data 8 Channels (SSA boards only, optionally to Disk)\n");
      if(ulAuxBdNum){
        printf("   8 - Multiple Board Ext Sync Example               \n");
      }
      printf("   X - EXIT (return to DOS)                           \n");
#endif
      mmchar = prompt_for_key("  Please Make selection: \n");

      switch(mmchar)
      {
      case '1':
        init_verify();
        break;
      case '2':
        auto_cal_tst();
        break;
      case '3':
        analog_inputs();
        break;
      case '4':
        SelfTest();
        break;
      case '5':
        SglDMATest();
        break;
      case '6':
        GscAnaGetOption( ulBdNum, GSC_ANA_MAXCHANNELS, &dNumChan);
        ContinuousSaveData( dNumChan ); // use all available channels
        break;
      case '7':
        ContinuousSaveData(8); // use only 8 channels, if
        break;
      case '8':
        MultiBoardExample();
        break;
      case 'X':
        ClrScr();
        break;
      default:
        printf("  Invalid Selection: Press AnyKey to continue...\n");
        getch();
        break;
      }
    }while(mmchar!='X');

    status = GscClose( ulBdNum );

#ifdef __linux__
    // Delete the message queue
    if (msgctl(callback_qid, IPC_RMID, (struct msqid_ds *)0) == -1)
    {
      sprintf(sLine, "Problem deleting message queue, error = %s, queue id = %d",
              strerror(errno), callback_qid);
      addstr(sLine);
    }
#endif

    CursorVisible(TRUE);

#ifdef __linux__
    endwin();
#endif

    for(i=0; i < ulNumBds; i++)
      GscClose( i+1 );
  }
  return 0;
} /* end main */

//------------------------------------------------------------------------------
// Initialization Verification
//------------------------------------------------------------------------------
void init_verify(void)
{
  int status = 0;

  ClrScr();
  CurX=CurY=2;

  PutCursor(CurX,CurY++);
  cprintf("Initializing board number %08X", ulBdNum);
  status = GscAnaInitializeBoard( ulBdNum );

  PutCursor(CurX,CurY++);
  GscLocalRegisterRead(ulBdNum, GSC_ANALOG_BCR, &ValueRead );
  cprintf("Board Control Register      = %08lX", ValueRead);

  PutCursor(CurX,CurY++);
  if ((status = GscLocalRegisterRead( ulBdNum, GSC_ANALOG_ICR, &ValueRead )) == 0)
    cprintf("Interrupt Control Register  = %08lX", ValueRead);
  else
    cprintf("Interrupt Control Register  = N/A");

  PutCursor(CurX,CurY++);
  GscLocalRegisterRead( ulBdNum, GSC_ANALOG_IDB, &ValueRead );
  cprintf("Input Data Buffer Register  = %08lX", ValueRead);

  PutCursor(CurX,CurY++);
  GscLocalRegisterRead( ulBdNum, GSC_ANALOG_IBC, &ValueRead );
  cprintf("Input Data Control Register = %08lX", ValueRead);

  PutCursor(CurX,CurY++);
  GscLocalRegisterRead( ulBdNum, GSC_ANALOG_RAG, &ValueRead );
  cprintf("Rate Control A Register     = %08lX", ValueRead);

  PutCursor(CurX,CurY++);
  GscLocalRegisterRead( ulBdNum, GSC_ANALOG_RBG, &ValueRead);
  cprintf("Rate Control B Register     = %08lX", ValueRead);

  PutCursor(CurX,CurY++);
  if ((status = GscLocalRegisterRead( ulBdNum, GSC_ANALOG_SSC, &ValueRead )) == 0)
    cprintf("Scan Control Register       = %08lX", ValueRead);
  else
    cprintf("Scan Control Register       = N/A");

  PutCursor(CurX,CurY++);
  GscLocalRegisterRead( ulBdNum, GSC_ANALOG_FWR, &ValueRead );
  status = GscGetNumberOfChannels( ulBdNum, &dNumChan );
  cprintf("FW_REV     * Undocumented * = %08lX : %02d Channels ",  ValueRead, dNumChan);

  PutCursor(CurX,CurY++);
  ValueRead = GscLocalRegisterRead( ulBdNum, GSC_ANALOG_AV, &ValueRead );
  cprintf("AUTOCAL    * Undocumented * = %08lX",  ValueRead);

  PutCursor(CurX,CurY++);
  PutCursor(CurX,CurY++);

  anykey();
}


//==============================================================================
// Callback for auto calibration interrupt
void CALLBACK isrAutoCalCompleteHandler( int board, int channel, int type)
{
#ifdef __linux__
  THREAD_MESSAGE  threadMessage;
  int status = 0;
  char sLine[80];
#endif

#ifdef __linux__
  memset( &threadMessage, 0, sizeof(threadMessage));
  threadMessage.type = MSG_GSCANA_AUTOCALCOMPLETE;
  status = msgsnd(callback_qid, (void *)&threadMessage, sizeof(int), 0);
  if (status)
  {
    sprintf(sLine, "msgsnd status = %s, queue id = %d",
            strerror(errno), callback_qid);
    addstr( sLine );
  }
#else
  SetEvent( hAutoCalCompleteEvent );
#endif
}

//------------------------------------------------------------------------------
// Auto Calibration Test
//------------------------------------------------------------------------------
void auto_cal_tst(void)
{
#ifdef __linux__
  THREAD_MESSAGE threadMessage;
  int msecs = 0;
#else
  int EventStatus;
#endif
  int status = 0;
  int acStatus = 0;

  ClrScr();
  CurX=CurY=2;
  PutCursor(CurX,CurY);
  cprintf("Auto Calibration Test:");

#ifndef __linux__
  hAutoCalCompleteEvent = CreateEvent( NULL, TRUE, FALSE, "AutoCalThread" );
#endif

  if ((status = GscAnaRegisterInterrupt( ulBdNum, GSC_INTR_ANA_AUTOCALCOMPLETED,
					 isrAutoCalCompleteHandler )) != 0)
    printf("Error registering for autocal complete interrupt, errno = %d\n", status );

  //Busy_Signal(5);
  // Initiate autocal without polling for completion
  status = GscAnaAutoCal( ulBdNum, 0 );

#ifdef __linux__
  msecs = 0;
  while (((status = msgrcv(callback_qid,(void *)&threadMessage, sizeof(THREAD_MESSAGE),
                           MSG_GSCANA_AUTOCALCOMPLETE, IPC_NOWAIT)) == -1)
         && (msecs < 10000))
  {
    usleep(50000);
    msecs += 50;
  }

  PutCursor(CurX,CurY++);
  if (status != -1)
    addstr("Interrupt was requested    ...");
  else
    addstr("Interrupt was NOT requested    ...");
#else
  EventStatus = WaitForSingleObject(hAutoCalCompleteEvent,
				    10 * 1000); // Wait for notification from interrupt callback
  PutCursor(CurX,CurY++);
  switch(EventStatus)
  {
  case WAIT_OBJECT_0:
    cprintf("Interrupt was requested    ...");
    break;
  default:
    cprintf("Interrupt was NOT requested...");
    break;
  }
  ResetEvent( hAutoCalCompleteEvent );
#endif

  status = GscAnaGetAutoCalStatus( ulBdNum, &acStatus );

  PutCursor(CurX,CurY++);
  if((acStatus & GSC_ANA_AUTOCAL_INIT) != 0)
    cprintf("Auto Cal BCR Bit did NOT clear...");
  else
    cprintf("Auto Cal BCR Bit Cleared   ...   ");

  PutCursor(CurX,CurY++);
  if(acStatus & GSC_ANA_AUTOCAL_PASS)
    cprintf("Auto Cal Status Says PASSED... %04lx",indata);
  else
    cprintf("Auto Cal Status Says FAILED... %04lx",indata);
  PutCursor(CurX,CurY++);
  cprintf(" ================================================================\n");
  cprintf("        Gain Off         Gain Off         Gain Off         Gain Off\n");

  if(dNumChan == 0)
    status = GscAnaGetOption( ulBdNum, GSC_ANA_NUMACTIVECHANNELS, &dNumChan );

  for(i=0; i< (U32)(2*dNumChan);)
  {
    GscLocalRegisterWrite( ulBdNum, GSC_ANALOG_AV, i);
    GscLocalRegisterRead( ulBdNum, GSC_ANALOG_AV, &indata );
    GscLocalRegisterWrite( ulBdNum, GSC_ANALOG_AV, i+1);
    GscLocalRegisterRead( ulBdNum, GSC_ANALOG_AV, &data_in );
    cprintf("   Ch%02d %04lX:%04lX", i/2,data_in,indata );
	  i+=2;
    if((i%8)==0)
      cprintf("\n");
  }

  if ((status = GscAnaRegisterInterrupt( ulBdNum, GSC_INTR_ANA_AUTOCALCOMPLETED,
					 NULL )) != 0)
    printf("Error unregistering autocal complete interrupt callback, errno = %d\n", status );

  anykey();
}


//------------------------------------------------------------------------------
// Analog Inputs
//------------------------------------------------------------------------------
void analog_inputs(void)
{
  char freq = 0;
  int status = 0;

  ClrScr();
  CurX=CurY=2;
  PutCursor(CurX,CurY++);
  cprintf("Analog Input Channels:");
  PutCursor(CurX,CurY++);
  //Busy_Signal(20);

  PutCursor(CurX,CurY++);
  PutCursor(CurX,CurY++);
  cprintf("Please select the desired Sample Rate to test ...");
  PutCursor(CurX,CurY++);
  cprintf("     1 -  10K Samples/Sec");
  PutCursor(CurX,CurY++);
  cprintf("     2 - 200K Samples/Sec");
  PutCursor(CurX,CurY++);
  cprintf("Selection: ");

#ifdef __linux__
  prompt_for_key("");
#else
  do
  {
    freq = toupper(getch());
    //Sleep(250);
  }while(freq != '1' && freq != '2');
#endif

  if (freq == '1')
    GscAnaSetOption( ulBdNum, GSC_ANA_SAMPLE_RATE, 10000 );
  else
    GscAnaSetOption( ulBdNum, GSC_ANA_SAMPLE_RATE, 200000 );

  ClrScr();
  CurX=CurY=2;
  PutCursor(CurX,CurY++);
  cprintf("Analog Input Channels:");

  if(dNumChan == 0)
    GscAnaGetOption( ulBdNum, GSC_ANA_NUMACTIVECHANNELS, &dNumChan );

  PutCursor(CurX,21);
  cprintf("Performing AutoCalibration                               ");

  set_vRange(2.5);
  GscAnaSetOption( ulBdNum, GSC_ANA_NUMACTIVECHANNELS, dNumChan );
  PutCursor(CurX,21);
  status = auto_cal(ulBdNum);

  if (status != 0)
    cprintf("Autocal FAILED BD#%ld  ...",ulBdNum);
  else
  {
    PutCursor(CurX,21);
    cprintf("                                            ");
  }
  PutCursor(CurX,22);
  cprintf("%s2.5V Range  %02d Single Ended Inputs   ", cPlusMinus, dNumChan);
  PutCursor(CurX,23);
  cprintf("Press anykey for the next Selection...                   ");
  Scan_display();


  PutCursor(CurX,21);
  cprintf("Performing AutoCalibration                               ");
  set_vRange(5.0);
  PutCursor(CurX,21);
  status = auto_cal(ulBdNum);
  if (status != 0)
    cprintf("Autocal FAILED BD#%ld  ...",ulBdNum);
  else
  {
    PutCursor(CurX,21);
    cprintf("                                            ");
  }
  PutCursor(1,22);
  cprintf("          ");
  PutCursor(CurX,22);
  cprintf("%s5V Range   %02d Single Ended Inputs   ", cPlusMinus, dNumChan);
  PutCursor(CurX,23);
  cprintf("Press anykey for the next Selection...                   ");
  Scan_display();

  PutCursor(CurX,21);
  cprintf("Performing AutoCalibration                               ");
  set_vRange(10.0);
  PutCursor(CurX,21);
  status = auto_cal(ulBdNum);
  if (status != 0)
    cprintf("Autocal FAILED BD#%ld  ...",ulBdNum);
  else
  {
    PutCursor(CurX,21);
    cprintf("                                            ");
  }
  PutCursor(1,22);
  cprintf("          ");
  PutCursor(CurX,22);
  cprintf("%s10V Range   %02d Single Ended Inputs   ", cPlusMinus, dNumChan);
  PutCursor(CurX,23);
  cprintf("Press anykey for the next Selection...                   ");
  Scan_display();

  if ((status = GscAnaSetOption( ulBdNum, GSC_ANA_INPUT_TYPE, GSC_ANA_PSEUDO_DIFF )) == 0)
  {
    // Pseudo Diff Mode
    PutCursor(CurX,21);
    cprintf("Performing AutoCalibration                               ");
    PutCursor(CurX,21);
    status = auto_cal(ulBdNum);
    if (status != 0)
      cprintf("Autocal FAILED BD#%ld  ...",ulBdNum);
    else
    {
      PutCursor(CurX,21);
      cprintf("                                            ");
    }
    PutCursor(1,22);
    cprintf("          ");
    PutCursor(CurX,22);
    cprintf("%s10V Range   %02d Pseudo Diff Inputs   ", cPlusMinus, dNumChan-1);
    PutCursor(CurX,23);
    cprintf("Press anykey for the next Selection...                   ");
    Scan_display();
  }

  if ((status = GscAnaSetOption( ulBdNum, GSC_ANA_INPUT_TYPE, GSC_ANA_DIFFERENTIAL )) == 0)
  {
    // Full Diff Mode
    PutCursor(CurX,21);
    cprintf("Performing AutoCalibration                               ");
    PutCursor(CurX,21);
    status = auto_cal(ulBdNum);
    if (status != 0)
      cprintf("Autocal FAILED BD#%ld  ...",ulBdNum);
    else
    {
      PutCursor(CurX,21);
      cprintf("                                            ");
    }
    PutCursor(1,22);
    cprintf("          ");
    PutCursor(CurX,22);
    cprintf("%s10V Range   %02d Full Diff Inputs     ", cPlusMinus, dNumChan/2);
    PutCursor(CurX,23);
    cprintf("Press anykey for the next Selection...                   ");
    Scan_display();
  }

  GscAnaSetOption( ulBdNum, GSC_ANA_INPUT_TYPE, GSC_ANA_SINGLE_ENDED ); // Std SE Mode
}


//------------------------------------------------------------------------------
// Selftest Verification
//------------------------------------------------------------------------------
void SelfTest(void)
{
  int status = 0;
  int nominalVrefScanValue = 0;

  ClrScr();
  CurX=CurY=2;
  PutCursor(CurX,CurY++);
  cprintf("SelfTest Modes:");

  Busy_Signal(5);

  if(dNumChan == 0)
    GscAnaGetOption( ulBdNum, GSC_ANA_NUMACTIVECHANNELS, &dNumChan );

  CurY=10;
  PutCursor(CurX,19);
  cprintf("Performing AutoCalibration   ");

  GscAnaSetOption( ulBdNum, GSC_ANA_INPUT_MODE, GSC_ANA_ZERO); // Zero Selftest
  set_vRange(2.5);
  GscAnaSetOption( ulBdNum, GSC_ANA_NUMACTIVECHANNELS, dNumChan );
  PutCursor(CurX,19);
  status = auto_cal(ulBdNum);
  if (status != 0)
    cprintf("Autocal FAILED BD#%ld  ...", ulBdNum);
  else
  {
    PutCursor(CurX,19);
    cprintf("                                            ");
  }
  PutCursor(CurX,20);
  cprintf("%s2.5V Range - Zero SelfTest - BiPolar   ", cPlusMinus);
  PutCursor(CurX,21);
  cprintf("Verify 0x8000 Nominal Value              ");
  Scan_display();

  nominalVrefScanValue = 0xFEB8; // nominal +vref scan value for DSI boards
  if ((status = GscAnaSetOption( ulBdNum, GSC_ANA_INPUT_RANGE, GSC_ANA_UNI_5_0 )) == 0)
  {
    // this is an SSA board, set the expected +vref nominal scan value
    nominalVrefScanValue = 0xFFDF;

    // UniPolar - this is a guess (5 or 10v?)
    PutCursor(CurX,19);
    cprintf("Performing AutoCalibration   ");
    PutCursor(CurX,19);
    status = auto_cal(ulBdNum);
    if (status != 0)
      cprintf("Autocal FAILED BD#%ld  ...",ulBdNum);
    else
    {
      PutCursor(CurX,19);
      cprintf("                                            ");
    }
    PutCursor(CurX,20);
    cprintf("0-5V Range - Zero SelfTest - UniPolar    ");
    PutCursor(CurX,21);
    cprintf("Verify 0x0000 Nominal Value              ");
    Scan_display();

    // +Vref Selftest
    status = GscAnaSetOption( ulBdNum, GSC_ANA_INPUT_MODE, GSC_ANA_VREF);
    PutCursor(CurX,19);
    cprintf("Performing AutoCalibrotion   ");
    status = auto_cal(ulBdNum);
    if (status != 0)
      cprintf("Autocal FAILED BD#%ld  ...",ulBdNum);
    else
    {
      PutCursor(CurX,19);
      cprintf("                                            ");
    }
    PutCursor(CurX,20);
    cprintf("0-5V Range - +Vref SelfTest - UniPolar    ");
    PutCursor(CurX,21);
    cprintf("Verify 0x7FDF Nominal Value              ");
    Scan_display();
  }
  else
    status = GscAnaSetOption( ulBdNum, GSC_ANA_INPUT_MODE, GSC_ANA_VREF); // +Vref Selftest

  PutCursor(CurX,19);
  cprintf("Performing AutoCalibration   ");
  //GscLocalRegisterWrite( ulBdNum, GSC_ANALOG_SSC, 4); // 16 Channels
  GscAnaSetOption( ulBdNum, GSC_ANA_NUMACTIVECHANNELS, 16 );
  set_vRange(5.0);
  PutCursor(CurX,19);
  status = auto_cal(ulBdNum);
  if (status != 0)
    cprintf("Autocal FAILED BD#%ld  ...",ulBdNum);
  else
  {
    PutCursor(CurX,19);
    cprintf("                                            ");
  }
  PutCursor(CurX,20);
  cprintf("%s5.0V Range - +Vref SelfTest - BiPolar      ", cPlusMinus);
  PutCursor(CurX,21);
  cprintf("Verify 0x%X Nominal Value              ", nominalVrefScanValue);
  Scan_display();

  PutCursor(CurX,19);
  cprintf("Performing AutoCalibration   ");
  set_vRange(10.0);
  PutCursor(CurX,19);
  status = auto_cal(ulBdNum);
  if (status != 0)
    cprintf("Autocal FAILED BD#%ld  ...",ulBdNum);
  else
  {
    PutCursor(CurX,19);
    cprintf("                                            ");
  }
  PutCursor(CurX,20);
  cprintf("%s10.0V Range - +Vref SelfTest - BiPolar     ", cPlusMinus);
  PutCursor(CurX,21);
  cprintf("Verify 0x%X Nominal Value              ", nominalVrefScanValue);
  Scan_display();


  PutCursor(CurX,19);
  cprintf("Performing AutoCalibration   ");
  GscAnaSetOption( ulBdNum, GSC_ANA_INPUT_MODE, GSC_ANA_ZERO); // Zero Selftest
  PutCursor(CurX,19);
  status = auto_cal(ulBdNum);
  if (status != 0)
    cprintf("Autocal FAILED BD#%ld  ...",ulBdNum);
  else
  {
    PutCursor(CurX,19);
    cprintf("                                            ");
  }
  PutCursor(CurX,20);
  cprintf("%s10.0V Range - Zero SelfTest - BiPolar     ", cPlusMinus);
  PutCursor(CurX,21);
  cprintf("Verify 0x8000 Nominal Value              ");
  Scan_display();

  GscAnaSetOption( ulBdNum, GSC_ANA_INPUT_MODE, GSC_ANA_NORMAL); // SE Input Mode
}



//------------------------------------------------------------------------------
// Auto Calibration Sub
//------------------------------------------------------------------------------
int auto_cal(U32 boardNum)
{
  U32 ulValue;
  ulValue = GscAnaAutoCal( boardNum, 1 );
  return ulValue;
}


//==============================================================================
void get_avg_reading(int num_channels, int sample_size)
{
  U32 chan_samp;

  chan_samp = (U32)(sample_size / num_channels);
  memset(uData,0,256); // Set memory
  for(j=0; j<(U32)sample_size; j+=(U32)num_channels)
  {
    for(i=0; i<(U32)num_channels; i++)
      uData[i] += (Data[j+i]&0xFFFF);

  }
  for(i=0; i<(U32)num_channels; i++)
    uData[i] /= chan_samp;
  return;
}


//==============================================================================
void Scan_display()		// Based on # of chans set in Scan & Sync Register
{
  static int prev_chans=64;  // Max Chans
  int chan_num, channels, sample_size, wait_cnt,  proc_mode=1;
  int chan_mask = 0;
  U32 yPos = 4, xPos = 7;
  int status = 0;
  int regval = 0;

  GscAnaSetOption( ulBdNum, GSC_ANA_DATA_WIDTH, GSC_ANA_DATAWIDTH_16); // Ensure default data width
  GscAnaSetOption( ulBdNum, GSC_ANA_DATA_PACKING, GSC_DISABLED); // Normal input data organization
  GscAnaGetOption( ulBdNum, GSC_ANA_NUMACTIVECHANNELS, &channels );
  if(channels==1) // Single Channel mode
  {
    if ((status = GscAnaGetOption( ulBdNum, GSC_ANA_SINGLEACTIVECHANNEL, &chan_mask )) != 0)
      chan_mask = 0;
  }
  else
    chan_mask = channels;

  sample_size = 16 * channels;

  for(i=0,j=0; i<64; i+=8,j+=2){
    PutCursor(xPos-7,yPos+j);
    if(i >= (U32)channels)
      cprintf("                                                          ");
    else
      cprintf("CH%02d                                                    ",(int)i);
  }

  memset(uData,0,16384); // Set memory
  memset(Data,0,65535); //Set memory

  if ((status = GscAnaGetOption( ulBdNum, GSC_ANA_INPUT_TYPE, &regval)) == 0) // Determine Processing mode
    if(regval == GSC_ANA_DIFFERENTIAL)
      proc_mode = 2;

  GscAnaSetOption( ulBdNum, GSC_ANA_RATEGEN, GSC_DISABLED );
  GscAnaSetInputBufferThreshold( ulBdNum, sample_size ); // Write threshold value = sample_size
  Sleep(100);

  GscAnaSetOption( ulBdNum, GSC_ANA_INPUTBUFFER, GSC_ENABLED );
  GscAnaSetOption( ulBdNum, GSC_ANA_RATEGEN, GSC_ENABLED );

  do
  {
    Sleep(250);

    GscAnaClearInputBuffer( ulBdNum ); // Clear buffer

    Sleep(1);
    wait_cnt = 0;
    do
    {
      GscLocalRegisterRead( ulBdNum, GSC_ANALOG_BS, &indata );
      wait_cnt++;
    } while(((int)indata < sample_size) && (wait_cnt < 10000));
    for(i=0; i<(U32)sample_size; i++)
      GscLocalRegisterRead( ulBdNum, GSC_ANALOG_IDB, &Data[i] );
    PutCursor(xPos,yPos-1);
    cprintf("UPDATE ");
    PutCursor(xPos,yPos);
    get_avg_reading(channels, sample_size);

    if(channels == 1){
      PutCursor(xPos-7,yPos);
      cprintf("CH%02d %04X ", uData[chan_mask]);
    }
    else{
      for(chan_num=0; chan_num < channels/proc_mode; chan_num++)
      {
        if(chan_num == 8)
          PutCursor(xPos,yPos+2);
        if(chan_num == 16)
          PutCursor(xPos,yPos+4);
        if(chan_num == 24)
          PutCursor(xPos,yPos+6);
        if(chan_num == 32)
          PutCursor(xPos,yPos+8);
        if(chan_num == 40)
          PutCursor(xPos,yPos+10);
        if(chan_num == 48)
          PutCursor(xPos,yPos+12);
        if(chan_num == 56)
          PutCursor(xPos,yPos+14);
        cprintf("%04X ", uData[chan_num]);
      }
      Sleep(250);
      PutCursor(xPos,yPos-1);
      cprintf("        ");
    }
#ifdef __linux__
    refresh();
#endif
  }while(!kbhit());

  kbflush();
  prev_chans = channels;
}


#ifndef __linux__
//==============================================================================
void PutCursor(U32 FixedX, U32 FixedY)
{
  PositionCursor((U16) FixedX,(U16) FixedY);
}
#endif

//==============================================================================
void set_vRange(double range)
{
  int opVal = 0;

  if(range == 10.0)
    opVal = GSC_ANA_BIP_10_0;
  else if(range == 5.0)
    opVal = GSC_ANA_BIP_5_0;
  else if (range == 2.5)
    opVal = GSC_ANA_BIP_2_5;
  GscAnaSetOption( ulBdNum, GSC_ANA_INPUT_RANGE, opVal );
}


//==============================================================================
void CmdChainingDMATest(){
}

//==============================================================================
// Callback for buffer threshold interrupt
void CALLBACK isrBufferThresholdHandler( int board, int channel, int type)
{
  //U32 ulBuffSize = 0;
  //U32 indata = 0;
#ifdef __linux__
  THREAD_MESSAGE  threadMessage;
#endif
  //QueryPerformanceCounter( &presentTime );
  //GscAnaRegisterInterrupt( board, GSC_INTR_ANA_THRESHOLDLOWTOHIGH, NULL); // Lo->Hi transition
  //GscLocalRegisterRead( ulBdNum, GSC_ANALOG_RAG, &indata); // Disable Clock
  //GscLocalRegisterWrite( ulBdNum, GSC_ANALOG_RAG, indata | 0x10000); // Disable Clock
  //GscAnaSetOption( board, GSC_ANA_CLOCK_SOURCE, GSC_ANA_CLOCK_MANUAL); // turn sampling off
  //GscLocalRegisterRead( board, GSC_ANALOG_BS, &ulBuffSize );
  //GscLocalRegisterRead( ulBdNum, GSC_ANALOG_IBC,&ulIbc ); // Threshold
  //printf("Got buffer threshold low==>high transition interrupt, buffer size = %d.\n", ulBuffSize);
  //printf("Got buffer threshold low==>high transition interrupt, ibc reg = 0x%x.\n", ulIbc);
  //PutCursor( 5, 10);
  //elapsedDMATime = ((double) (presentTime.QuadPart - startTime.QuadPart)) / (double) frequency.QuadPart;
  //printf("intr: scan time = %lf secs \n", elapsedDMATime);
  //Sleep(100);
#ifdef __linux__
  memset( &threadMessage, 0, sizeof(threadMessage));
  threadMessage.type = MSG_GSCANA_THRESHOLDLOWTOHIGH;
  //addstr("Got buffer threshold low==>high interrupt");
  msgsnd(callback_qid, (void *)&threadMessage, sizeof(int), 0);
#else
  SetEvent( hBufferThresholdEvent );
#endif
}

//==============================================================================
void SglDMATest()
{
  //HANDLE myHandle;
#ifdef __linux__
  THREAD_MESSAGE threadMessage;
  int msecs = 0;
#endif
  //DWORD EventStatus;
  int EventStatus;
  U32 mode = 0, ulMode;
  U32 yPos = 4, xPos = 7;
  U32 sampleSize;
  int iBuffSize = 0;
  U32 myData[200000];
  int iIcr = 0;
  //U32 ulNon55Count = 0;
  //int i = 0;
  U32 samples = 200;
  int status = 0;

  ClrScr();
  CurX=CurY=2;
  PutCursor(CurX,CurY++);
  cprintf("Sgl DMA Mode:");
  PutCursor(CurX,CurY++);
  PutCursor(CurX,CurY++);
  cprintf("Please select the desired Data Mode ...");
  PutCursor(CurX,CurY++);
  cprintf("     1 -  Normal");	// Packing only applies to the DMA transfer
  PutCursor(CurX,CurY++);
  cprintf("     2 -  Packed");
  PutCursor(CurX,CurY++);
  cprintf("Selection: ");
#ifdef __linux__
  prompt_for_key("");
#else
  do
  {
    mode = toupper(getch());
  } while( mode != '1' && mode != '2' );
#endif

  if (mode == '1')
    ulMode = GSC_DISABLED;
  else
    ulMode = GSC_ENABLED;


  ClrScr();
  CurX=CurY=2;
  PutCursor(CurX,CurY++);
  cprintf("Sgl DMA Mode:");

  Busy_Signal(10);

  GscAnaSetOption( ulBdNum, GSC_ANA_DATA_PACKING, ulMode);

  if(dNumChan == 0)
    status = GscAnaGetOption( ulBdNum, GSC_ANA_NUMACTIVECHANNELS, &dNumChan );

  status = GscAnaSetOption( ulBdNum, GSC_ANA_DMA_DEMANDMODE, GSC_ANA_DMA_AUTONOMOUS );

  CurY=10;
  PutCursor(CurX,21);
  cprintf("Performing AutoCalibration                               ");
  sampleSize = samples * dNumChan;
  GscAnaSetInputBufferThreshold( ulBdNum, sampleSize - 1); // Threshold
  PutCursor(CurX,21);
  //auto_cal(ulBdNum);
  PutCursor(CurX,19);
  cprintf("%s10V Range - SE                         ", cPlusMinus);
  PutCursor(CurX,20);

#ifndef __linux__
  hBufferThresholdEvent = CreateEvent( NULL, TRUE, FALSE, "BufferReadThread" );
#endif

  memset(myData, 55, 131072);
  status = GscAnaRegisterInterrupt( ulBdNum, GSC_INTR_ANA_THRESHOLDLOWTOHIGH,
                                    isrBufferThresholdHandler); // Lo->Hi transition

  GscAnaSetOption( ulBdNum, GSC_ANA_RATEGEN, GSC_DISABLED ); // Disable Clock

  do {

    ulErr = GscAnaScanInputs( ulBdNum, samples, 10000.0, 0, dNumChan - 1, (char *)myData );
#ifdef __linux__
    msecs = 0;
    while (((status = msgrcv(callback_qid,(void *)&threadMessage, sizeof(int),
                             MSG_GSCANA_THRESHOLDLOWTOHIGH, IPC_NOWAIT)) == -1)
           && (msecs < 2000))
    {
      usleep(50000);
      msecs += 50;
    }
    EventStatus = (status >= 0) ? 0 : -1;
    switch(EventStatus)
    {		// If packed data, we will
    case 0:	// have 2 channels per long word
#else
    EventStatus = WaitForSingleObject(hBufferThresholdEvent, 2 * 1000); // Wait for the interrupt

    switch(EventStatus)
    {						// If packed data, we will
    case WAIT_OBJECT_0:	// have 2 channels per long word
#endif
      PutCursor(xPos,yPos-1);
      if (ulErr) {
        cprintf(" ScanInput error: %d\n", ulErr);
        myData[0]=0x55AA;
      }
      else {
        //printf("myData[0] = %d\n", myData[0]);
        //printf("myData[1] = %d\n", myData[1]);
      }
      PutCursor(xPos,yPos);

      GscAnaGetInputBufferCount( ulBdNum, &iBuffSize );
      // Use this to determine if overrun (If it hits 0xFFFF we overrun)
      // Of course, the call takes time, so eliminate if critical
      cprintf("%04X : %08X", iBuffSize, myData[0]);
      //GscLocalRegisterWrite( ulBdNum, GSC_ANALOG_IBC, sampleSize - 1); // Clear threshold flag
      //Sleep(1000);
#ifndef __linux__
      ResetEvent( hBufferThresholdEvent );
#endif
      break;

    default:
      PutCursor(2,9);
      cprintf("\nError... Interrupt Timeout\n");
      GscAnaGetInputBufferCount( ulBdNum, &iBuffSize );
      GscLocalRegisterRead( ulBdNum, GSC_ANALOG_ICR, &iIcr );  // the FIFO
      cprintf("Buff Size, %08X : ICR, %08X", iBuffSize, iIcr); // interrupt. If it hits 0xFFFF we overrun
      anykey();
      return;
    }

#ifdef __linux__
    refresh();
#endif
  } while(!kbhit());

  kbflush();
}

//==============================================================================
void BlockDMATest(){
}

//==============================================================================
void ContinuousSaveData( int nChannels )
{
  //HANDLE myHandle;
#ifdef __linux__
  pthread_t       writerThread;
  pthread_attr_t  fsThreadAttribute;
  THREAD_MESSAGE  threadMessage;
  int             msecs = 0;
#else
  HANDLE hThread;
  HANDLE myThread;
  //DWORD EventStatus;
  DWORD pThreadId;
  ULONGLONG i64FreeBytesToCaller;
  ULONGLONG i64TotalBytes;
  ULONGLONG i64FreeBytes;
  U8 fResult;
#endif
  U32 EventStatus;
  double bytes_needed;
  U32 yPos = 4, xPos = 7;
  int sampleRate;
  U32 sampleSize;
  //U32 ulBuffSize = 0;
  //int iBuffSize = 0;
  //U32 ulIbc = 0;
  //int iIbc = 0;
  U32 offset;
  U32 memNeeded;
  U32 *myData;
  U32 scans_sec;
  int sec_to_run;
  U32 i,j,k;
  U8 enable_write = 0;
  U8 ok = 1;
  double z;
  int status = 0;
  int scanStatus = 0;
  int sampleByteCount = 0;
#ifdef __linux__
  int fResult = 0;
  struct statfs fsbuf;
  char *sdir = (char *) NULL;
#endif
  //clock_t tnow = 0;
  int cnt = 0;
  short int adreading = 0;
  char sInput[16];

  adreading = 0xffff;

  ClrScr();
  CurX=CurY=2;
  PutCursor(CurX,CurY++);
  cprintf("Continuous Data             ");
  PutCursor(CurX,CurY++);
  PutCursor(CurX,CurY++);
  kbflush();

  if ((status = GscAnaSetOption( ulBdNum, GSC_ANA_NUMACTIVECHANNELS, nChannels)) == 0)
    dNumChan = nChannels;
  else if(dNumChan == 0)
    status = GscAnaGetOption( ulBdNum, GSC_ANA_NUMACTIVECHANNELS, &dNumChan );

  do
  {
    cprintf("Please enter the sample rate (Hz)... ");
    //scanf("%ld",&sampleRate);
#ifdef __linux__
    echo();
    getstr(sInput);
    noecho();
    sscanf(sInput, "%d", &sampleRate);
#else
    scanf("%d",&sampleRate);
#endif
    kbflush();
    if((sampleRate < 5000) || (sampleRate > 200000)){
      cprintf("\n  Invalid Rate %ld (5K Min : 200000 Max)",sampleRate);
      anykey();
      kbflush();
      ClrScr();
      PutCursor(CurX,CurY);
    }
  } while ((sampleRate < 5000) || (sampleRate > 200000));

  PutCursor(CurX,CurY++);
  cprintf("Please enter the desired collection time (Sec)... ");
#ifdef __linux__
  echo();
  getstr(sInput);
  noecho();
  sscanf(sInput, "%d", &sec_to_run);
#else
  scanf("%d",&sec_to_run);
#endif
  kbflush();

  PutCursor(CurX,CurY++);
  kbchar = prompt_for_key("Write data to Disk ? Y or N");
  if(kbchar == 'Y')
    enable_write = 1;
  kbflush();

  memNeeded = (U32)((double)sampleRate * (double)dNumChan);
  sampleSize = memNeeded / 4;
  i = 4;
  while (sampleSize > 32768) sampleSize = memNeeded / i++;
  scans_sec = (U32)(memNeeded/sampleSize); // Switch at ~1 second rate
  // NOTE: scans_sec MUST be >= 1, and ideally >= 4
  bytes_needed = ((double)memNeeded*(double)sec_to_run*(double)4.0);
  if(enable_write) // NOTE: It will ALWAYS be slightly less than 1 second
  {
#ifdef __linux__
    sdir = getcwd(sdir, 4096);
    fResult = statfs(sdir, &fsbuf);
    if (sdir)
      free( sdir );
    if(fResult == -1)
#else
    fResult = GetDiskFreeSpaceEx ("c:",
                                  (PULARGE_INTEGER)&i64FreeBytesToCaller,
                                  (PULARGE_INTEGER)&i64TotalBytes,
                                  (PULARGE_INTEGER)&i64FreeBytes);
    if(!fResult)
#endif
    {
      cprintf("\n  Unable to determine available disk space\n  Cannot continue");
      anykey();
      kbflush();
      return;
    }
    else{
#ifdef __linux__
      if (fsbuf.f_bsize * fsbuf.f_bavail < bytes_needed)
      {
#else
      if(i64FreeBytesToCaller < (ULONGLONG)bytes_needed)
      {
#endif
        cprintf("\n  Not Enough Disk Space for %.1lf MBytes",
          ((double)memNeeded*(double)sec_to_run*4.0)/1000000.0);
        PutCursor(CurX,CurY);
        anykey();
        kbflush();
        ClrScr();
        PutCursor(CurX,CurY);
        return;
      }
    }
  }

  Buff1 = (U32 *)calloc(memNeeded + 262144,sizeof(U32));
  Buff2 = (U32 *)calloc(memNeeded + 262144,sizeof(U32));
  wBuff = (U32 *)calloc(memNeeded + 262144,sizeof(U32));
  ///if((Buff1 == (PU32)NULL) || (Buff2 == (PU32)NULL) || (wBuff == (PU32)NULL)){
  if((Buff1 == (U32 *)NULL) || (Buff2 == (U32 *)NULL) || (wBuff == (U32 *)NULL)){
    cprintf("\n  Not Enough Memory for %ld Bytes",(memNeeded*4));
    PutCursor(CurX,CurY);
    anykey();
    kbflush();
    ClrScr();
    PutCursor(CurX,CurY);
    return;
  }

  myData = Buff1;

  ClrScr();
  CurX=CurY=2;
  PutCursor(CurX,CurY++);
  cprintf("Continuous Data %s for %ld seconds:",((enable_write)?"to disk":"Monitor"),sec_to_run);

  Busy_Signal(5);

  CurY=10;
  PutCursor(CurX,19);
  cprintf("Performing AutoCalibration  ");

  // set input range
  status = GscAnaSetOption( ulBdNum, GSC_ANA_INPUT_RANGE, GSC_ANA_BIP_10_0 );

  status = GscAnaSetOption( ulBdNum, GSC_ANA_DMA_DEMANDMODE, GSC_ANA_DMA_AUTONOMOUS );

  // Setup the sample rate for the board.
  status = GscAnaSetOption( ulBdNum, GSC_ANA_SAMPLE_RATE, sampleRate );

  GscAnaSetInputBufferThreshold( ulBdNum, sampleSize - 1 ); // Threshold
  auto_cal(ulBdNum);

  status = GscAnaSetOption(ulBdNum, GSC_ANA_RATEGEN, GSC_DISABLED );

  status = GscAnaSetOption( ulBdNum, GSC_ANA_DATA_WIDTH, GSC_ANA_DATAWIDTH_16 );
  status = GscAnaSetOption( ulBdNum, GSC_ANA_DATA_PACKING, GSC_DISABLED );
  PutCursor(CurX,19);

#ifdef __linux__
  // TODO: set thread priority here.
#else
  myThread = GetCurrentThread();
  SetThreadPriority(myThread,THREAD_PRIORITY_TIME_CRITICAL);
#endif
  if(enable_write)
  {
    contThreadStatus = 0;
    contRunning = 1;
    b1Ready=0;
    b2Ready=0;

#ifdef __linux__
    // Initialize the thread attribute structure
    status = pthread_attr_init(&fsThreadAttribute);
    if (status) return;

    // Set the thread to detached
    status = pthread_attr_setdetachstate(&fsThreadAttribute, PTHREAD_CREATE_DETACHED);
    if (status) return ;

    // Start the thread
    status = pthread_create(&writerThread, &fsThreadAttribute, InterruptAttachThreadDma,
                            (void *)&bytes_needed);
    if (status) return;
#else
    hThread = CreateThread(
                           NULL,                          // Security
                           0,                             // Same stack size
                           InterruptAttachThreadDma,      // Thread start routine
                           (LPVOID)&bytes_needed,           //
                           0,                             // Creation flags
                           &pThreadId                     // Thread ID
                           );

    SetThreadPriority(hThread,THREAD_PRIORITY_LOWEST);
#endif

    Sleep(300);
    // Wait till thread starts
    while (contThreadStatus != 1)
    {
      Sleep(1);
#ifdef __linux__
      if(pthread_kill(writerThread, 0)) {
#else
      if(GetExitCodeThread(hThread,&pThreadId) != STILL_ACTIVE) {
#endif
        cprintf("\nThread Aborted - Cannot continue");
        anykey();
        kbflush();
        return;
      }
    }
  }
  else{
    contThreadStatus = 0;
    contRunning = 0;
  }

  status = GscAnaRegisterInterrupt( ulBdNum, GSC_INTR_ANA_THRESHOLDLOWTOHIGH,
                           isrBufferThresholdHandler); // Lo->Hi transition

#ifndef __linux__
  hBufferThresholdEvent = CreateEvent( NULL, TRUE, FALSE, "BufferThresholdLowToHigh" );
  ResetEvent( hBufferThresholdEvent );
#endif

  status = GscAnaClearInputBuffer( ulBdNum );

  offset = 0;
  i=0;
  j=0;
  k=0;
  z = 0.0;

  status = GscAnaSetOption( ulBdNum, GSC_ANA_RATEGEN, GSC_ENABLED );

  PutCursor(CurX,19);
  cprintf("Data %s Started                                 ",(enable_write)?"Collection":"Monitor");
  PutCursor(xPos,yPos);

  do
  {
#ifdef __linux__
    msecs = 0;
    while (((status = msgrcv(callback_qid,(void *)&threadMessage, sizeof(int),
                             MSG_GSCANA_THRESHOLDLOWTOHIGH, IPC_NOWAIT)) == -1)
           && (msecs < 5000))
    {
      usleep(50000);
      msecs += 50;
    }
    EventStatus = (status >= 0) ? 0 : -1;
#else
    EventStatus = WaitForSingleObject( hBufferThresholdEvent,
                                       5 * 1000 ); // wait for buffer theshold low-->high
#endif

    switch(EventStatus)
    {
    case WAIT_OBJECT_0:
      {

        status = GscAnaReceiveScan( ulBdNum, (char *)(myData+offset), &sampleByteCount, &scanStatus );

        if ((sampleByteCount >> 2) < sampleSize) continue; // spurious unexplained interrupt came our way, still
                                                           // below threshold, so wait for next interrupt

        if(k > (sampleSize-128)) {
          k=0;
          PutCursor(xPos,yPos);
          cprintf("                      ");
          PutCursor(xPos,yPos);
        }

        offset += (sampleByteCount >> 2);
        if (offset >= memNeeded)
          j = scans_sec; // already got our ~ 1sec of sample data, can't fit another sample in buffer
        else
          j++;
        if(j == scans_sec) { // We got our ~1 sec of data to be saved
          if(enable_write) {
            if(!b1Ready) {
              if(!b1Done) {
                cprintf("Not fast enough - exiting");
                contRunning = 0;
                while (contThreadStatus == 1)
                  Sleep(1);
                i = sec_to_run;
              }
              b1Ready = offset;
              b2Ready = 0;
              myData = Buff2;
            }
            else {
              if(!b2Done) {
                cprintf("Not fast enough - exiting");
                contRunning = 0;
                while (contThreadStatus == 1)
                  Sleep(1);
                i = sec_to_run;
              }
              b1Ready = 0;
              b2Ready = offset;
              myData = Buff1;
            }
          }
          else
            cprintf("*");

          j=0;
          offset = 0;
          i++;
        }
        z+=sampleByteCount;
        k+=64;
      }
#ifndef __linux__
        ResetEvent( hBufferThresholdEvent );
#endif
      //GscAnaSetOption( ulBdNum, GSC_ANA_RATEGEN, GSC_ENABLED );
      break;

    default:
      GscAnaGetInputBufferCount( ulBdNum, &cnt);
      cprintf("Error... Interrupt Timeout, input buffer count = 0x%x\n", cnt);
      ok=0;
      break;

    } // End Switch EventStatus


  } while(i < sec_to_run && ok);

#ifdef __linux__
  // TODO: restore original thread priority
#else
  SetThreadPriority(myThread,THREAD_PRIORITY_NORMAL);
#endif

  if (enable_write)
    Sleep(1000); // allow time for last buffer to be written to disk

  // unregister buffer threshold interrupt callback
  status = GscAnaRegisterInterrupt( ulBdNum, GSC_INTR_ANA_THRESHOLDLOWTOHIGH,
                           NULL);

  contRunning = 0;
  while (contThreadStatus == 1)
    Sleep(1);
  if(ok && !enable_write)
    cprintf("\n  No Errors\n  %.1lfMSamples Monitored", z/4000000.0);

  PutCursor(CurX,19);
  cprintf("Data %s Done                                 \n",(enable_write)?"Collection":"Monitor");

  free(Buff1);
  free(Buff2);
  free(wBuff);
  anykey();
  kbflush();
}


// Function to write data to disk when told

#ifdef __linux__

void *InterruptAttachThreadDma( void *pParam )

#else

DWORD WINAPI
InterruptAttachThreadDma(
    LPVOID pParam
    )

#endif
{
  FILE *myFile[1024];
  U32 wSize;
  U32 b1Written = 0;
  U32 b2Written = 0;
  U32 samplesWritten = 0;
  char fName[256];
  int filenum = 0;
  int k,l;
  int files_needed;
  double total_written;
  double bytes_needed;

  if (pParam)
    bytes_needed = *((double *)(pParam));
  else bytes_needed = 0;
  if(bytes_needed < 3000000000.0)
    files_needed = 1;
  else
    files_needed = ((int)(bytes_needed / 3000000000.0)+1);
  if(files_needed >= 1024)
    files_needed = 1024;
  for(k=0; k<files_needed; k++)
  {
  	sprintf(fName,"AI64SSData%04d.txt",k);
    myFile[k] = fopen(fName,"w+b"); // Doesn't append, always overwrites
    if(myFile[k] == NULL){
      cprintf("\n  Error opening file for save.. Cannot continue");
      l=k;
      for(l=0; l<k; l++)
        fclose(myFile[l]);
      contThreadStatus = 0;
      return 0;
    }
  } // end For()

  b1Done = 1;
  b2Done = 1;
  total_written = 0.0;

  // Signal that thread is ready
  contThreadStatus = 1;

  while(contRunning)
  {
    if(samplesWritten > 750000000) {
      total_written += (double)samplesWritten;
      if(filenum < 1023)
        filenum++;
      samplesWritten = 0;
    }
    if(b1Ready && !b1Written) {
      b1Done = 0;
      wSize = b1Ready;
      samplesWritten += b1Ready;
      memmove(wBuff,Buff1,wSize*4); // BYTES
      b1Done = 1;
      fwrite(wBuff,sizeof(U32),wSize,myFile[filenum]);
      cprintf("*");
      b1Written = 1;
      b2Written = 0;
    }
    if(b2Ready && !b2Written) {
      b2Done = 0;
      wSize = b2Ready;
      samplesWritten += b2Ready;
      memmove(wBuff,Buff2,wSize*4); // BYTES
      b2Done = 1;
      fwrite(wBuff,sizeof(U32),wSize,myFile[filenum]);
      cprintf("+");
      b2Written = 1;
      b1Written = 0;
    }
    Sleep(50);
  } // End while(contRunning)

  for(k=0; k<files_needed; k++)
    fclose(myFile[k]);
  if(filenum == 0)
    total_written = (double)samplesWritten;
  cprintf("\n  %.1lf Samples successfully written ..   \n",total_written);
  contThreadStatus = 0;
  return 0;
}


// Callback for initiator board buffer threshold interrupt
void CALLBACK isrInitiatorBuffThresholdHandler( int board, int channel, int type)
{
#ifdef __linux__
  THREAD_MESSAGE  threadMessage;
#endif
  //U32 ulBuffSize = 0;
  //U32 ulIbc = 0;
  //int i = 0;

  //QueryPerformanceCounter( &currentTime);

  GscAnaSetOption( board, GSC_ANA_CLOCK_SOURCE, GSC_ANA_CLOCK_MANUAL); // turn sampling off
  //GscLocalRegisterRead( board, GSC_ANALOG_BS, &ulBuffSize );
  //GscLocalRegisterRead( ulBdNum, GSC_ANALOG_IBC,&ulIbc ); // Threshold
  //printf("Got buffer threshold low==>high transition interrupt, buffer size = %d.\n", ulBuffSize);
  //printf("Got buffer threshold low==>high transition interrupt, ibc reg = 0x%x.\n", ulIbc);
  //elapsedDMATime = ((double) (currentTime.QuadPart - startTime.QuadPart)) / (double) frequency.QuadPart;
  //printf("Got buffer threshold low==>high transition interrupt, elapsed scan time = %lf secs \n",
  //  elapsedDMATime);
  //Sleep(100);
#ifdef __linux__
  memset( &threadMessage, 0, sizeof(threadMessage));
  threadMessage.type = MSG_GSCANA_THRESHOLDLOWTOHIGH;
  msgsnd(callback_qid, (void *)&threadMessage, sizeof(int), 0);
#else
  SetEvent( hMultiIniBoardBuffLowHigh );
#endif
}

// Callback for auxilliary board buffer threshold interrupt
void CALLBACK isrAuxBuffThresholdHandler( int board, int channel, int type)
{
#ifdef __linux__
  THREAD_MESSAGE  threadMessage;
#endif
  //U32 ulBuffSize = 0;
  //U32 ulIbc = 0;
  //int i = 0;

  //QueryPerformanceCounter( &currentTime);

  GscAnaSetOption( board, GSC_ANA_CLOCK_SOURCE, GSC_ANA_CLOCK_MANUAL); // turn sampling off
  //GscLocalRegisterRead( board, GSC_ANALOG_BS, &ulBuffSize );
  //GscLocalRegisterRead( ulBdNum, GSC_ANALOG_IBC,&ulIbc ); // Threshold
  //printf("Got buffer threshold low==>high transition interrupt, buffer size = %d.\n", ulBuffSize);
  //printf("Got buffer threshold low==>high transition interrupt, ibc reg = 0x%x.\n", ulIbc);
  //elapsedDMATime = ((double) (currentTime.QuadPart - startTime.QuadPart)) / (double) frequency.QuadPart;
  //printf("Got buffer threshold low==>high transition interrupt, elapsed scan time = %lf secs \n",
  //  elapsedDMATime);
  //Sleep(100);
#ifdef __linux__
  memset( &threadMessage, 0, sizeof(threadMessage));
  threadMessage.type = MSG_GSCANA_THRESHOLDLOWTOHIGH;
  msgsnd(callback_qid, (void *)&threadMessage, sizeof(int), 0);
#else
  SetEvent( hMultiAuxBoardBuffLowHigh );
#endif
}

//==============================================================================
//
// This example uses the external sync pin to clock the target board
// from the initiator board. Connect pin b40 from board1 to b40 of board2
//
//==============================================================================

void MultiBoardExample(){
  // modified to use GscApi library and SGL DMA
  // TODO when merge
  //    - use correct handles, create a callback function for each board and register them for the low -> high transition interrupt.
  //    - may need to modify GscAnaScanInputs to check whether board in externally sync'd (initiator or target)
  //    - the way scan inputs is written, we will need multiple threads when scanning more than one board. Do we want a "checkfordata" function
  //      to use with asynchronous receive call? This would work like the 66 lib call Open_DMA_Channel used in this function.

  //DWORD EventStatus;
  U32 EventStatus;
#ifdef __linux__
  pthread_t       writerThread;
  pthread_attr_t  fsThreadAttribute;
  THREAD_MESSAGE  threadMessage;
  struct statfs fsbuf;
  int fResult;
  char *sdir = (char *) NULL;
  int status = 0;
  int msecs = 0;
#else
  HANDLE theHandles[MAXIMUM_WAIT_OBJECTS];
  HANDLE hThread;
  HANDLE myThread;
  DWORD pThreadId;
  ULONGLONG i64FreeBytesToCaller;
  ULONGLONG i64TotalBytes;
  ULONGLONG i64FreeBytes;
  U8 fResult;
#endif
  //HANDLE fbHandle;
  //HANDLE sbHandle;
  double bytes_needed;
  U32 yPos = 4, xPos = 7;
  //U32 buff_size[2];
  int buff_size[2];
  int sampleRate;
  U32 sampleSizePri,sampleSizeAux;
  //  U32 ulBuffSizePri = 0;
  // U32 ulBuffSizeAux = 0;
  U32 offset;
  U32 memNeeded;
  U32 *myData;
  U32 scans_sec;
  int sec_to_run;
  U32 i,j,k;
  U8 enable_write = 0;
  U8 ok = 1;
  double z;
  int scanStatus = 0;
  int sampleByteCount = 0;
  GSC_ANALOG_CONFIG anacfg;

  ClrScr();
  CurX=CurY=2;
  PutCursor(CurX,CurY++);
  cprintf("Continuous Data             ");
  PutCursor(CurX,CurY++);
  PutCursor(CurX,CurY++);
  kbflush();
  if(dNumChan == 0){ // Assumes both the same
    //ValueRead = AI64SSA_Read_Local32(ulBdNum, &ulErr, FW_REV);
    GscLocalRegisterRead( ulBdNum, GSC_ANALOG_FWR, &ValueRead );
    dNumChan = (ValueRead&0x10000) ? 32:64;
  }
  sampleSizePri = 65536;
  sampleSizeAux = 65536;
BADRATEB:
  cprintf("Please enter the sample rate (Hz)... ");
  scanf("%d",&sampleRate);
  kbflush();
  if((sampleRate < 5000) || (sampleRate > 200000)){
      cprintf("\n  Invalid Rate %ld (5K Min : 200K Max)",sampleRate);
	  anykey();
	  kbflush();
      ClrScr();
	  PutCursor(CurX,CurY);
	  goto BADRATEB;
  }
  PutCursor(CurX,CurY++);

  cprintf("Please enter the desired collection time (Sec)... ");
  scanf("%d",&sec_to_run);
  kbflush();

  PutCursor(CurX,CurY++);

  kbchar = prompt_for_key("Write data to Disk ? Y or N");
  if(kbchar == 'Y')
	  enable_write = 1;
  kbflush();
  ok = 1;
  memNeeded = (U32)((double)sampleRate * (double)dNumChan * 2);// 2 Boards
  scans_sec = (U32)(memNeeded/(sampleSizePri + sampleSizeAux)); // Switch at ~1 second rate
  // NOTE: scans_sec MUST be >= 1, and ideally >= 4
  bytes_needed = ((double)memNeeded*(double)sec_to_run*(double)4.0);
  if(enable_write){					// NOTE: It will ALWAYS be slightly less than 1 second
#ifdef __linux__
    sdir = getcwd(sdir, 4096);
    fResult = statfs(sdir, &fsbuf);
    if (sdir)
      free( sdir );
    if(fResult == -1){
#else
    fResult = GetDiskFreeSpaceEx ("c:",
                                  (PULARGE_INTEGER)&i64FreeBytesToCaller,
                                  (PULARGE_INTEGER)&i64TotalBytes,
                                  (PULARGE_INTEGER)&i64FreeBytes);
    if(!fResult){
#endif

      cprintf("\n  Unable to determine available disk space\n  Cannot continue");
      anykey();
      kbflush();
      return;
    }
    else{
#ifdef __linux__
      if (fsbuf.f_bsize * fsbuf.f_bavail < bytes_needed) {
#else
      if(i64FreeBytesToCaller < (ULONGLONG)bytes_needed){
#endif
        cprintf("\n  Not Enough Disk Space for %.1lf MBytes",
                ((double)memNeeded*(double)sec_to_run*4.0)/1000000.0);
        PutCursor(CurX,CurY);
        anykey();
        kbflush();
        ClrScr();
        PutCursor(CurX,CurY);
        goto BADRATEB;
      }
    }
  }
  Buff1 = (U32 *)calloc(memNeeded,sizeof(U32));
  Buff2 = (U32 *)calloc(memNeeded,sizeof(U32));
  wBuff = (U32 *)calloc(memNeeded,sizeof(U32));
  if((Buff1 == (U32 *)NULL) || (Buff2 == (U32 *)NULL) || (wBuff == (U32 *)NULL)){
    cprintf("\n  Not Enough Memory for %ld Bytes",(memNeeded*4));
    PutCursor(CurX,CurY);
    anykey();
    kbflush();
    ClrScr();
    PutCursor(CurX,CurY);
    goto BADRATEB;
  }

  myData = Buff1;


  ClrScr();
  CurX=CurY=2;
  PutCursor(CurX,CurY++);
  cprintf("Continuous Data %s for %ld seconds:",((enable_write)?"to disk":"Monitor"),sec_to_run);

  //AI64SSA_Initialize(ulBdNum, &ulErr);
  GscOpen( ulBdNum, GSC_API_VERSION );
  GscAnaGetDefaults( ulBdNum, &anacfg );
  anacfg.masterClockFrequency = 50000000;
  GscAnaSetConfig( ulBdNum, anacfg );
  //AI64SSA_Initialize(ulAuxBdNum, &ulErr);
  GscOpen( ulAuxBdNum, GSC_API_VERSION );
  GscAnaGetDefaults( ulAuxBdNum, &anacfg );
  anacfg.masterClockFrequency = 50000000;
  GscAnaSetConfig( ulBdNum, anacfg );
  Busy_Signal(10);

  CurY=10;
  PutCursor(CurX,19);
  cprintf("Performing AutoCalibration                               ");
  //ValueRead = AI64SSA_Read_Local32(ulBdNum, &ulErr, BCR);
  GscLocalRegisterRead( ulBdNum, GSC_ANALOG_BCR, &ValueRead );
  //AI64SSA_Write_Local32(ulBdNum, &ulErr, BCR, ValueRead & 0x7FFFF); // Demand Mode
  //GscLocalRegisterWrite( ulBdNum, GSC_ANALOG_BCR, ValueRead & 0x7FFFF ); // Demand Mode
  //ValueRead = AI64SSA_Read_Local32(ulAuxBdNum, &ulErr, BCR);
  GscLocalRegisterRead( ulAuxBdNum, GSC_ANALOG_BCR, &ValueRead );
  //AI64SSA_Write_Local32(ulBdNum, &ulErr, BCR, ValueRead & 0x7FFFF); // Demand Mode
  //GscLocalRegisterWrite( ulAuxBdNum, GSC_ANALOG_BCR, ValueRead & 0x7FFFF ); // Demand Mode

  // Set the sample rate on the initiator board
  // This will be used to clk both boards when we are ready to go
  //AI64SSA_Write_Local32(ulBdNum, &ulErr, RATE_A, ((50000000/sampleRate)&0xFFFF)); // Sample Rate
  GscLocalRegisterWrite( ulBdNum, GSC_ANALOG_RAG, ((50000000/sampleRate)&0xFFFF) ); // Sample Rate
  // Set the sample rate on the target board
  // This is used to do autocal only, then will switch to external
  //AI64SSA_Write_Local32(ulAuxBdNum, &ulErr, RATE_A, ((50000000/sampleRate)&0xFFFF)); // Sample Rate
  GscLocalRegisterWrite( ulAuxBdNum, GSC_ANALOG_RAG, ((50000000/sampleRate)&0xFFFF) ); // Sample Rate

  //AI64SSA_Write_Local32(ulBdNum, &ulErr, SCAN_CNTRL,(dNumChan==64)?6:5);
  GscLocalRegisterWrite( ulBdNum, GSC_ANALOG_SSC,(dNumChan==64)?6:5 );
  //AI64SSA_Write_Local32(ulAuxBdNum, &ulErr, SCAN_CNTRL,(dNumChan==64)?6:5);
  GscLocalRegisterWrite( ulAuxBdNum, GSC_ANALOG_SSC, (dNumChan==64)?6:5 );

  //AI64SSA_Write_Local32(ulBdNum, &ulErr, IN_DATA_CNTRL,(sampleSizePri-1)); // Threshold
  GscLocalRegisterWrite( ulBdNum, GSC_ANALOG_IBC, (sampleSizePri-1) ); // Threshold
  //AI64SSA_Write_Local32(ulAuxBdNum, &ulErr, IN_DATA_CNTRL,(sampleSizeAux-1)); // Threshold
  GscLocalRegisterWrite( ulAuxBdNum, GSC_ANALOG_IBC, (sampleSizeAux-1) ); // Threshold
  auto_cal(ulBdNum);
  auto_cal(ulAuxBdNum);

  // Setup the target board for external sync
  //data_in = AI64SSA_Read_Local32(ulAuxBdNum, &ulErr, SCAN_CNTRL);
  GscLocalRegisterRead( ulAuxBdNum, GSC_ANALOG_SSC, &data_in );
  //AI64SSA_Write_Local32(ulAuxBdNum, &ulErr, SCAN_CNTRL,0x10 | data_in); // Target
  GscLocalRegisterWrite( ulAuxBdNum, GSC_ANALOG_SSC, 0x10 | data_in ); // Target

  //data_in = AI64SSA_Read_Local32(ulBdNum, &ulErr, BCR); // Initiator
  GscLocalRegisterRead( ulBdNum, GSC_ANALOG_BCR, &data_in ); // Initiator
  //AI64SSA_Write_Local32(ulBdNum, &ulErr, BCR,0x80 | data_in);
  GscLocalRegisterWrite( ulBdNum, GSC_ANALOG_BCR, 0x80 | data_in );
  //data_in = AI64SSA_Read_Local32(ulAuxBdNum, &ulErr, BCR); // Target
  GscLocalRegisterRead( ulAuxBdNum, GSC_ANALOG_BCR, &data_in ); // Target
  //AI64SSA_Write_Local32(ulAuxBdNum, &ulErr, BCR,0x80 | data_in);
  GscLocalRegisterWrite( ulAuxBdNum, GSC_ANALOG_BCR, 0x80 | data_in );
  // Ensure we have a clock to the target board before we proceed
  //data_in = AI64SSA_Read_Local32(ulAuxBdNum, &ulErr, IN_DATA_CNTRL); // Clear Buffer
  GscLocalRegisterRead( ulAuxBdNum, GSC_ANALOG_IBC, &data_in ); // Clear Buffer
  //AI64SSA_Write_Local32(ulAuxBdNum, &ulErr, IN_DATA_CNTRL, data_in|0x40000);
  GscLocalRegisterWrite( ulAuxBdNum, GSC_ANALOG_IBC, data_in|0x40000);
  Sleep(250);
  //data_in = AI64SSA_Read_Local32(ulAuxBdNum, &ulErr, BUFF_SIZE); // Read number of samples
  GscLocalRegisterRead( ulAuxBdNum, GSC_ANALOG_BS, &data_in ); // Read number of samples
  if(data_in < 100){ // Will be 0 if no clock
    PutCursor(CurX,19);
    cprintf("No Clock to target board - Aborting                        \n");
    anykey();
    return;
  }

  // Stop the clock till we get all dressed up and ready to go
  //data_in = AI64SSA_Read_Local32(ulBdNum, &ulErr, RATE_A);
  GscLocalRegisterRead( ulBdNum, GSC_ANALOG_RAG, &data_in );
  //AI64SSA_Write_Local32(ulBdNum, &ulErr, RATE_A, 0x10000 | data_in); // Sample Rate
  GscLocalRegisterWrite( ulBdNum, GSC_ANALOG_RAG, 0x10000 | data_in ); // Sample Rate
  PutCursor(CurX,19);
  cprintf("Data %s Started                                 ",(enable_write)?"Collection":"Monitor");

  //AI64SSA_Dma_DataMode(ulBdNum, 0);
  GscAnaSetOption( ulBdNum, GSC_ANA_DATA_WIDTH, GSC_ANA_DATAWIDTH_32 );
  //AI64SSA_Dma_DataMode(ulAuxBdNum, 0);
  GscAnaSetOption( ulAuxBdNum, GSC_ANA_DATA_WIDTH, GSC_ANA_DATAWIDTH_32 );
  PutCursor(CurX,19);
#ifdef __linux__
  // TODO: set thread priority here.
#else
  myThread = GetCurrentThread();
  SetThreadPriority(myThread,THREAD_PRIORITY_TIME_CRITICAL);
#endif
  if(enable_write){
    contThreadStatus = 0;
    contRunning = 1;
    b1Ready=0;
    b2Ready=0;

#ifdef __linux__
    callback_qid = msgget(IPC_PRIVATE, 0666 | IPC_CREAT| IPC_EXCL);
    if(callback_qid == -1)
    {
      return;
    }

    // Initialize the thread attribute structure
    status = pthread_attr_init(&fsThreadAttribute);
    if (status) return;

    // Set the thread to detached
    status = pthread_attr_setdetachstate(&fsThreadAttribute, PTHREAD_CREATE_DETACHED);
    if (status) return ;

    // Start the thread
    status = pthread_create(&writerThread, &fsThreadAttribute, InterruptAttachThreadDma, (void *)(0x0));
    if (status) return;
#else
    hThread = CreateThread(
                           NULL,                          // Security
                           0,                             // Same stack size
                           InterruptAttachThreadDma,      // Thread start routine
                           (LPVOID)&bytes_needed,           //
                           0,                             // Creation flags
                           &pThreadId                     // Thread ID
                           );

    SetThreadPriority(hThread,THREAD_PRIORITY_LOWEST);
#endif
    Sleep(300);
    // Wait till thread starts
    while (contThreadStatus != 1) {
      Sleep(1);
#ifdef __linux__
      if(pthread_kill(writerThread, 0)) {
#else
      if(GetExitCodeThread(hThread,&pThreadId) != STILL_ACTIVE) {
#endif
        cprintf("\nThread Aborted - Cannot continue");
        anykey();
        kbflush();
        return;
      }
    }
  }
  else{
    contThreadStatus = 0;
    contRunning = 0;
  }

  //data_in = AI64SSA_Read_Local32(ulBdNum, &ulErr, IN_DATA_CNTRL); // Clear Buffer
  GscLocalRegisterRead( ulBdNum, GSC_ANALOG_IBC, &data_in ); // Clear Buffer
  //AI64SSA_Write_Local32(ulBdNum, &ulErr, IN_DATA_CNTRL, data_in|0x40000);
  GscLocalRegisterWrite( ulBdNum, GSC_ANALOG_IBC, data_in|0x40000 );
  //data_in = AI64SSA_Read_Local32(ulAuxBdNum, &ulErr, IN_DATA_CNTRL); // Clear Buffer
  data_in = GscLocalRegisterRead( ulAuxBdNum, GSC_ANALOG_IBC, &data_in ); // Clear Buffer
  //AI64SSA_Write_Local32(ulAuxBdNum, &ulErr, IN_DATA_CNTRL, data_in|0x40000);
  GscLocalRegisterWrite( ulAuxBdNum, GSC_ANALOG_IBC, data_in|0x40000 );
  /*
    AI64SSA_OpenBlock_DMA_Channel(ulBdNum, 0, &ulErr);
    if(ulErr){
    cprintf("Error Opening DMA Channel 0\n");
    anykey();
    return;
    }
    AI64SSA_OpenBlock_DMA_Channel(ulAuxBdNum, 1, &ulErr);
    if(ulErr){
    cprintf("Error Opening DMA Channel 1\n");
    anykey();
    return;
    }
  */
  //AI64SSA_Attach_Interrupt(ulBdNum, &fbHandle, 0x10, &ulErr);
  GscAnaRegisterInterrupt( ulBdNum, GSC_INTR_ANA_THRESHOLDLOWTOHIGH, isrInitiatorBuffThresholdHandler); // Lo->Hi transition
  //AI64SSA_Attach_Interrupt(ulAuxBdNum, &sbHandle, 0x10, &ulErr);
  GscAnaRegisterInterrupt( ulBdNum, GSC_INTR_ANA_THRESHOLDLOWTOHIGH, isrAuxBuffThresholdHandler); // Lo->Hi transition

#ifndef __linux__
  hMultiIniBoardBuffLowHigh = CreateEvent( NULL, TRUE, FALSE, "MultiInitiatorBuffLowHigh" );
  hMultiAuxBoardBuffLowHigh = CreateEvent( NULL, TRUE, FALSE, "MultiAuxBuffLowHigh" );

  //theHandles[0] = fbHandle;
  theHandles[0] = hMultiIniBoardBuffLowHigh;
  //theHandles[1] = sbHandle;
  theHandles[1] = hMultiAuxBoardBuffLowHigh;
#endif

  offset = 0;
  i=0;
  j=0;
  k=0;
  z = 0.0;
  PutCursor(xPos,yPos);

  //indata = AI64SSA_Read_Local32(ulBdNum, &ulErr, RATE_A);         // Enable Clock
  //AI64SSA_Write_Local32(ulBdNum, &ulErr, RATE_A, indata & 0xFFFEFFFF);

  do{

#ifdef __linux__
    msecs = 0;
    while (((status = msgrcv(callback_qid,(void *)&threadMessage, sizeof(int),
                             MSG_GSCANA_THRESHOLDLOWTOHIGH, IPC_NOWAIT)) == -1)
           && (msecs < 5000))
    {
      usleep(50000);
      msecs += 50;
    }
    if (msecs < 5000)
    {
      // must have both buffer threshold transition interrupts - wait for the second one here.
      while (((status = msgrcv(callback_qid,(void *)&threadMessage, sizeof(int),
                               MSG_GSCANA_THRESHOLDLOWTOHIGH, IPC_NOWAIT)) == -1)
             && (msecs < 5000))
      {
        usleep(50000);
        msecs += 50;
      }
    }
    EventStatus = (status >= 0) ? 0 : -1;
    switch(EventStatus)
    {
    case 0:
    case 1:
      {
#else
    EventStatus = WaitForMultipleObjects(2,theHandles,TRUE, 0); // See if buffer threshold interrupts occurred.
    switch(EventStatus)
    {
    case WAIT_OBJECT_0:
    case WAIT_OBJECT_0+1:
      {
        //AI64SSA_ReAttach_Interrupt(ulBdNum, &fbHandle, &ulErr);
        //AI64SSA_ReAttach_Interrupt(ulAuxBdNum, &sbHandle, &ulErr);
        ResetEvent( hMultiIniBoardBuffLowHigh );
        ResetEvent( hMultiAuxBoardBuffLowHigh );
#endif
        //AI64SSA_BlockDMA_FROM_Buffer(ulBdNum, 0, sampleSizePri, myData+offset, &ulErr);
        //AI64SSA_BlockDMA_FROM_Buffer(ulAuxBdNum, 1, sampleSizeAux, myData+offset+sampleSizePri, &ulErr);
        GscAnaReceiveScan( ulBdNum, (char *)myData+offset, &sampleByteCount, &scanStatus );
        GscAnaReceiveScan( ulAuxBdNum, (char *)myData+offset+sampleSizePri, &sampleByteCount, &scanStatus );
        if(k > (sampleSizePri-128)){
          k=0;
          PutCursor(xPos,yPos);
          cprintf("                      ");
          PutCursor(xPos,yPos);
        }

        if(!(*(myData+offset+k) & 0x10000)){
          ok = 0;
          cprintf("\nError occurred @ %.1lfMSamples",(z*(double)sampleSizePri)/1000000.0);
          break;
        }
        offset += sampleSizePri;
        if(!(*(myData+offset+k) & 0x10000)){
          ok = 0;
          cprintf("\nError occurred @ %.1lfMSamples",(z*((double)sampleSizeAux+(double)sampleSizePri))/1000000.0);
          break;
        }
        offset += sampleSizeAux;
        j++;
        if(j == scans_sec){ // We got our ~1 sec of data to be saved
          if(enable_write){
            if(!b1Ready){
              if(!b1Done){
                cprintf("Not fast enough - exiting");
                contRunning = 0;
                while (contThreadStatus == 1)
                  Sleep(1);
                i = sec_to_run;
              } // end if(!b1Done)
              b1Ready = offset;
              b2Ready = 0;
              myData = Buff2;
            } // end if(!b1Ready)
            else{
              if(!b2Done){
                cprintf("Not fast enough - exiting");
                contRunning = 0;
                while (contThreadStatus == 1)
                  Sleep(1);
                i = sec_to_run;
              } // end if(!b2Done)
              b1Ready = 0;
              b2Ready = offset;
              myData = Buff1;
            } // end else
          } // end if(enable_write)
          else
            cprintf("*");

          j=0;
          offset = 0;
          i++;
        } // end if(j == scans_sec)
        z+=1.0;
        k+=64;

      } // end case
      break;
    default:
      cprintf("Error... Interrupt Timeout\n");
      //data_in = AI64SSA_Read_Local32(ulBdNum, &ulErr, BCR); // Initiator
      GscLocalRegisterRead( ulBdNum, GSC_ANALOG_BCR, &data_in ); // Initiator
      //indata = AI64SSA_Read_Local32(ulAuxBdNum, &ulErr, BCR); // Target
      GscLocalRegisterRead( ulAuxBdNum, GSC_ANALOG_BCR, &indata ); // Target
      //buff_size[0] = AI64SSA_Read_Local32(ulBdNum, &ulErr, BUFF_SIZE); // Initiator
      GscLocalRegisterRead( ulBdNum, GSC_ANALOG_BS, &buff_size[0] ); // Initiator
      //buff_size[1] = AI64SSA_Read_Local32(ulAuxBdNum, &ulErr, BUFF_SIZE); // Target
      GscLocalRegisterRead( ulAuxBdNum, GSC_ANALOG_BS, &buff_size[1] ); // Target
      cprintf("\nInitiator BCR = %ld, buffer size = %ld",data_in,buff_size[0]);
      cprintf("\nTarget    BCR = %ld, buffer size = %ld",indata,buff_size[1]);
      ok=0;
      break;

    } // End Switch EventStatus


  }while(i <= sec_to_run && ok);


  contRunning = 0;
  while (contThreadStatus == 1)
    Sleep(1);
  if(ok && !enable_write)
    cprintf("\n  No Errors\n  %.1lfMSamples Monitored",(z*(double)sampleSizePri+z*(double)sampleSizeAux)/1000000.0);
  PutCursor(CurX,19);
  cprintf("Data %s Done                                 \n",(enable_write)?"Collection":"Monitor");

  //data_in = AI64SSA_Read_Local32(ulAuxBdNum, &ulErr, BCR);
  data_in = GscLocalRegisterRead( ulAuxBdNum, GSC_ANALOG_BCR, &data_in );
  //AI64SSA_Write_Local32(ulAuxBdNum, &ulErr, BCR,data_in & 0xFFFFFF7F);
  GscLocalRegisterWrite( ulAuxBdNum, GSC_ANALOG_BCR, data_in & 0xFFFFFF7F );
  //AI64SSA_CloseBlock_DMA_Channel(ulBdNum, 0, &ulErr);
  //AI64SSA_CloseBlock_DMA_Channel(ulAuxBdNum, 1, &ulErr);
  //AI64SSA_Reset_Device(ulAuxBdNum, &ulErr);
  GscClose( ulAuxBdNum );
  Sleep(300);
  //AI64SSA_Reset_Device(ulBdNum, &ulErr);
  GscClose( ulBdNum );
#ifdef __linux__
  // TODO: restore original thread priority
#else
  SetThreadPriority(myThread,THREAD_PRIORITY_NORMAL);
#endif
  free(Buff1);
  free(Buff2);
  free(wBuff);
  anykey();
  kbflush();
}

