#include <stdio.h>
#if defined( WIN32 )
#include <conio.h>
#include <windows.h>
#elif defined( PLX_LINUX )
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#endif
#include "GscApi.h"

#ifdef WIN32
#define usleep(val) Sleep(val / 1000 + 200);
#endif

//--------------------------------------------------------
//	Global vars/function declarations:
//--------------------------------------------------------
char*	ucpTxBuffer;
char*	ucpRxBuffer;
int				status ;

//--------------------------------------------------------
//	DisplayErrorMessage();
//--------------------------------------------------------
void DisplayErrorMessage( int errorCode)
{
  // Buffer to hold the error message
  char errorMessage[80];

  // Call the API to interpret the message
  GscGetErrorString( errorCode, errorMessage);

  // Display the error
  printf("\n Error Code: %d, %s\n", errorCode, errorMessage);
}


//--------------------------------------------------------
//	main();
//--------------------------------------------------------
int main (void)
{
  int		board = 1;
  int		status;
  int		txChannel = 1, rxChannel = 2, j, bufferSize, errors;
  int		txid, rxid, bytesTransferred;

  GSC_SYNC_CONFIG syncConfig;

  //	Open SIO4B/BX/SYNC Card:
  status = GscOpen( board, GSC_API_VERSION);
  if(status) DisplayErrorMessage( status );

  bufferSize = 0x20;
  errors = 0;


  //	Allocate memory and init if successful:
  ucpTxBuffer = (char *) malloc(bufferSize);
  if(ucpTxBuffer == NULL)
  {
    printf("\n unable to allocate memory...exiting");
    exit(1);
  }
  else
  {
    for(j = 0; j < bufferSize; j++)
      *(ucpTxBuffer + j) = (char)j;
  }

  //	Allocate memory and init if successful:
  ucpRxBuffer = (char *) malloc(bufferSize);
  if(ucpRxBuffer == NULL)
  {
    printf("\n unable to allocate memory...exiting");
    exit(1);
  }
  else
    memset(ucpRxBuffer, 0x5a, bufferSize);


  printf("\n SIO4B/BX-SYNC Channel to Channel Loopback Test");
  printf("\n Transmit Chan: %d  Receive Chan: %d", txChannel, rxChannel);
  printf("\n NOTE: Test requires loopback connection via cable");
  printf("\n Press any key to run test...");
  getchar();


  //	reset/config all chans:
  printf("\n\n Resetting channels...");


  //	Channel resets (sets base parameters):
  status = GscSio4ChannelReset(board, txChannel);
  if(status) DisplayErrorMessage( status );

  //	Channel resets (sets base parameters):
  status = GscSio4ChannelReset(board, rxChannel);
  if(status) DisplayErrorMessage( status );

  printf("done");
  printf("\n --------------------------------------");
  printf("\n Configuring channels...");


  //	Get default values and set both channels:
  status = GscSio4SyncGetDefaults( &syncConfig );
  if(status) DisplayErrorMessage( status );


  //	Tweak default sync config:
  syncConfig.encoding =		GSC_ENCODING_NRZ;
  syncConfig.bitRate =		1000000;
  syncConfig.txClockSource = GSC_CLOCK_INTERNAL;
  syncConfig.rxClockSource = GSC_CLOCK_EXTERNAL;
  syncConfig.txStatus =		GSC_ENABLED;
  syncConfig.rxStatus =		GSC_ENABLED;
  syncConfig.interfaceMode =	GSC_PIN_DTE;
  syncConfig.txEnvPinMode =	GSC_PIN_AUTO;
  syncConfig.rxEnvPinMode =	GSC_PIN_AUTO;

  //	set txChannel sync config:
  status = GscSio4SyncSetConfig(board, txChannel, syncConfig);
  if(status) DisplayErrorMessage( status );

  //	set rxChannel to DCE mode:
  syncConfig.interfaceMode = GSC_PIN_DCE;

  //	set rxChannel sync config:
  status = GscSio4SyncSetConfig(board, rxChannel, syncConfig);
  if(status) DisplayErrorMessage( status );


  printf("done");
  printf("\n --------------------------------------");


  //	start data reception:
  status = GscSio4ChannelReceiveData(board, rxChannel, ucpRxBuffer, bufferSize, &rxid);
  if(status) DisplayErrorMessage( status );

  usleep(200000);

  //	write data to txChannel/non-blocking
  status = GscSio4ChannelTransmitData(board, txChannel, ucpTxBuffer, bufferSize, &txid);
  if(status) DisplayErrorMessage( status );
  else printf("\n Write of %x bytes to channel %d...", bufferSize, txChannel);

  //	little delay to allow tx data to transmit
  usleep(200000);
  printf("\n --------------------------------------");


  //	wait for data transfer to complete:
  status = GscSio4ChannelWaitForTransfer(board, rxChannel, 2000, rxid, &bytesTransferred);
  if(status) DisplayErrorMessage( status );
  else
  {
    if(bytesTransferred == -1)
      printf("\n Read %x bytes from channel %d...done", bufferSize, rxChannel);
    else
      printf("\n Read %x bytes from channel %d...done", bytesTransferred, rxChannel);

  }

  printf("\n --------------------------------------");


  //	Verify data after transfer -
  //	only display data if an error is detected
  errors = 0;
  if(memcmp(ucpTxBuffer, ucpRxBuffer, bufferSize) != 0)
  {
    //	display erroneous data:
    for(j = 0; j < bufferSize; j++)
    {
      if( *(ucpRxBuffer + j) != *(ucpTxBuffer + j) )
      {
        errors++;
        if(errors < 10)
          printf("\n   ucpRxBuffer[%d] = 0x%02X, ucpTxBuffer[%d] = 0x%02X  <== Error",
                 j, *(ucpRxBuffer + j), j, *(ucpTxBuffer + j) );
      }
    }	//	for(j)
  }	//	if(memcmp)


  if(errors)
    printf("\n\n Test Status ==> Failed");
  else
    printf("\n\n Test Status ==> Passed");


  //	Done - close SIO4B/BX/SYNC Card:
  status = GscClose(board);
  if(status) DisplayErrorMessage( status );

  printf("\n\n ");

  return 0;
}	//	main()



