//-------------------------------------------------------------------------------------------------
//
// File Name:
//
//     GscAnaDSI.h
//
// Description:
//
//     This file contains the interface specification for the GSC DSI analog board class.
//
// Programmer's notes:
//
//     06-06-2008:  Created file. Only  24dsi12 supported and tested. --mrg
//
//-------------------------------------------------------------------------------------------------

#include <string>
#include "GscApiLocal.h"
#include "GscApiPlxLegacy.h"
#include "GscAnalogBoard.h"

using namespace std;

class GscAna24DSI12 : public GscAnalogBoard
{
public:
  GscAna24DSI12();
  GscAna24DSI12(int bd);
  virtual ~GscAna24DSI12();

  virtual int GetConfigDefaults( PGSC_ANALOG_CONFIG config );
  virtual int GetConfig( PGSC_ANALOG_CONFIG config );
  virtual int SetConfig( GSC_ANALOG_CONFIG config );

  virtual int GetOption( enum GSC_OPTION_NAME optionName, int optionValue[] );
  virtual int SetOption( enum GSC_OPTION_NAME optionName, int optionValue );

  virtual int LocalRegisterRead( int reg, int *result );
  virtual int LocalRegisterWrite( int reg, int value );

  virtual int ScanInputs( int samples, double samplesPerSecond, int firstChannel, int lastChannel, char *buffer );
  virtual int AutoCal( bool bWaitForCompletion = true );
  virtual int ReceiveScan( char *buffer, int *count, int *transferStatus );

  virtual int RegisterInterrupt( int interrupt, GSC_CB_FUNCTION *function );
  virtual int ClearInterrupts();
  virtual int InvokeInterruptCallback();

  virtual int GetInputBufferSize( int *FifoByteCount );
  virtual int ClearInputBuffer();
  virtual int SetInputBufferThreshold( int threshold );
  virtual int SetSampleRate(int sampleHz);
  virtual int GetIDBRegOffset() { return REGOFFSET_IDB; };
  virtual int GetNumChannelsPresent();
  virtual int Initialize();
  virtual int GetAutoCalStatus( int *acStatus );
  virtual int SoftwareSync();

private:
  static const int MAX_SAMPLE_CLOCK	= 200000;
  static const int MIN_SAMPLE_CLOCK = 2000;

  // Board register offsets
  static const int REGOFFSET_BCR = 0x0000; // Board control register
  static const int REGOFFSET_RAC = 0x0004; // Rate-a control register
  static const int REGOFFSET_RBC = 0x0008; // Rate-b control register
  static const int REGOFFSET_RAR = 0x000c; // Rate assignment register
  static const int REGOFFSET_DIV = 0x0010; // Rate divisor register
  static const int REGOFFSET_IBC = 0x0020; // Input buffer control register
  static const int REGOFFSET_FWR = 0x0024; // Board configuration register
  static const int REGOFFSET_IBS = 0x0028; // Input buffer size register
  static const int REGOFFSET_ACV = 0x002c; // Autocal values.
  static const int REGOFFSET_IDB = 0x0030; // Input Data Buffer

  int     mBoardNo;
  bool    mValidHw;

  // config options corresponding to GSC_ANALOG_CONFIG struct members
  int     mInputTypeMode;
  int     mInputType;
  int     mInputRange;
  int     mDataFormat;
  int     mExternalSync;
  int     mDataWidth;
  int     mDataPacking;
  int     mScanMarkingMode;
  int     mScanMarkingValue;
  int     mSampleClockSource;
  int     mMasterClockHz;
  int     mSamplingHz;
  int     mNumChannelsPresent;

  // maybe can use these
  int     mSelfTestSettleTimeMs; // doc section 3.4.1
  int     mLegacyRateGen;

  INTERRUPT_INFO analogIntrInfo[4];
};
