//-------------------------------------------------------------------------------------------------
//
// File Name:
//
//     GscAnaSSA.h
//
// Description:
//
//     This file contains the interface specification for the GSC SSA analog board class.
//
// Programmer's notes:
//
//     06-06-2008:  Created file. Only 6616AI64 supported and tested. --mrg
//
//-------------------------------------------------------------------------------------------------

#include <string>
#include "GscApiLocal.h"
#include "GscApiPlxLegacy.h"
#include "GscAnalogBoard.h"

using namespace std;

class GscAna6616AI64SSA : public GscAnalogBoard
{
public:
  GscAna6616AI64SSA();
  GscAna6616AI64SSA(int bd);
  virtual ~GscAna6616AI64SSA();

  virtual int GetConfigDefaults( PGSC_ANALOG_CONFIG config );
  virtual int GetConfig( PGSC_ANALOG_CONFIG config );
  virtual int SetConfig( GSC_ANALOG_CONFIG config );

  virtual int GetOption( enum GSC_OPTION_NAME optionName, int optionValue[] );
  virtual int SetOption( enum GSC_OPTION_NAME optionName, int optionValue );

  virtual int LocalRegisterRead( int reg, int *result );
  virtual int LocalRegisterWrite( int reg, int value );

  virtual int ScanInputs( int samples, double samplesPerSecond, int firstChannel, int lastChannel, char *buffer );
  virtual int AutoCal( bool bWaitForCompletion = true );
  virtual int ReceiveScan( char *buffer, int *count, int *transferStatus );

  virtual int RegisterInterrupt( int interrupt, GSC_CB_FUNCTION *function );
  virtual int ClearInterrupts();
  virtual int InvokeInterruptCallback();

  virtual int GetInputBufferSize( int *FifoByteCount );
  virtual int ClearInputBuffer();
  virtual int SetInputBufferThreshold( int threshold );
  virtual int SetSampleRate(int sampleHz);
  virtual int GetIDBRegOffset() { return REGOFFSET_IDB; };
  virtual int GetNumChannelsPresent();
  virtual int Initialize();
  virtual int GetAutoCalStatus( int *acStatus );

private:
  static const int MAX_SAMPLE_CLOCK	= 200000;
  static const double MIN_SAMPLE_CLOCK;

  // Board register offsets
  static const int REGOFFSET_BCR = 0x00; // board control register
  static const int REGOFFSET_ICR = 0x04; // Interrupt Control
  static const int REGOFFSET_IDB = 0x08; // Input Data Buffer
  static const int REGOFFSET_IBC = 0x0c; // Input Buffer Control
  static const int REGOFFSET_RAG = 0x10; // Rate A Generator
  static const int REGOFFSET_RBG = 0x14; // Rate B Generator
  static const int REGOFFSET_IBS = 0x18; // Input Buffer Size
  static const int REGOFFSET_SSC = 0x20; // scan and sync control
  static const int REGOFFSET_ACA = 0x24; // active channel assignment
  static const int REGOFFSET_FWR = 0x28; // Firmware revision.
  static const int REGOFFSET_ACV = 0x2c; // Autocal values.
  static const int REGOFFSET_SMF = 0x38; // scan marker first value
  static const int REGOFFSET_SMS = 0x3c; // scan marker second value

  // int     mBoardType;
  //  string  mBoardSubId;
  int     mBoardNo;

  int     mInputTypeMode;
  int     mInputType;
  int     mInputRange;
  int     mDataFormat;
  int     mExternalSync;
  int     mDataWidth;
  int     mDataPacking;
  int     mScanMarkingMode;
  int     mScanMarkingValue;
  int     mSampleClockSource;
  int     mMasterClockHz;
  int     mSamplingHz;
  bool    mValidHw;
  int     mNumberOfActiveChannels;
  int     mNumChannelsPresent;

  INTERRUPT_INFO analogIntrInfo[4];

};

