#ifndef _ANALOG_BOARD_H
#define _ANALOG_BOARD_H

#include "GscApiLocal.h"
#include <string>

extern "C" GSC_CB_FUNCTION *AnalogInterruptRoutines[ MAXIMUM_NUMBER_OF_BOARDS][4];

using namespace std;

// TODO:
//    - resetFifo method. (probably)
//    - timeout argument for ScanInputs() method. (probably)
//    - where to disable clock. Probably at end of ScanInputs(). What about ReceiveScan()?

// Abstract base class for all analog board subclasses
class GscAnalogBoard
{
public:
  GscAnalogBoard();
  GscAnalogBoard(int BoardNo);
  virtual ~GscAnalogBoard();

  virtual int GetConfigDefaults( PGSC_ANALOG_CONFIG config );
  virtual int GetConfig( PGSC_ANALOG_CONFIG config );
  virtual int SetConfig( GSC_ANALOG_CONFIG config );

  virtual int GetOption( enum GSC_OPTION_NAME optionName, int optionValue[] );
  virtual int SetOption( enum GSC_OPTION_NAME optionName, int optionValue );

  virtual int LocalRegisterRead( int reg, int *result );
  virtual int LocalRegisterWrite( int reg, int value );

  virtual int ScanInputs( int samples, double samplesPerSecond, int firstChannel, int lastChannel, char *buffer );
  virtual int AutoCal( bool bWaitForCompletion = true ) = 0;
  virtual int ReceiveScan( char *buffer, int *count, int *transferStatus );

  virtual int RegisterInterrupt( int interrupt, GSC_CB_FUNCTION *function );
  virtual int ClearInterrupts();
  virtual int InvokeInterruptCallback();

  virtual int GetInputBufferSize( int *FifoByteCount ) = 0;
  virtual int ClearInputBuffer() = 0;
  virtual int SetSampleRate(int sampleHz);
  virtual int GetIDBRegOffset() = 0;
  virtual int GetNumChannelsPresent() = 0;
  virtual int Initialize() = 0;
  virtual int GetAutoCalStatus( int *acStatus ) = 0;
  virtual int SetInputBufferThreshold( int threshold ) = 0;
  virtual int SoftwareSync() { return GscApiInvalidBoardType; };

private:
  int     mBoardType;
  //  string  mBoardSubId;
  int     mBoardNo;

  int     mInputTypeMode;
  int     mInputType;
  int     mInputRange;
  int     mDataFormat;
  int     mExternalSync;
  int     mDataWidth;
  int     mDataPacking;
  int     mScanMarkingMode;
  int     mScanMarkingValue;
  int     mSampleClockSource;
  int     mMasterClockHz;
  bool    mValidHw;
};

#endif
