/////////////////////////////////////////////////////////////////////////////
//
//  GscApi.c
//
//  This is the main library file for the General Standards Corp API
// implementation.
//
#include <stdio.h>
#ifdef PLX_LINUX
#include <unistd.h>
#include <stdlib.h>
#include <pthread.h>
#include <sched.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/shm.h>
#include <dlfcn.h>
#include <sys/stat.h>
#endif

#define __MAIN__
#include "GscApiLocal.h"
#include "GscApiPlxLegacy.h"
#include "GscApiPlxdload.h"

extern int gscCreateBoardObj( int BoardNo );

// Forward declarations
int gscPlxDynamicLink();
int gscPlxStaticLink();

//
// Private (but global) variables
//

#ifdef __linux__
void *shmPool = NULL;

#else
#endif

#ifdef PLX_LINUX
void *handle = (void *) NULL;
int shmid;
#else
HINSTANCE hInstLib;
unsigned int bStaticLibInitialized = FALSE;
#endif

// Mutext names
#ifndef __linux__
LPCTSTR lpPlxApiCalls = "PlxApiCalls";
LPCTSTR lpGscApiCalls = "GscApiCalls";
LPCTSTR lpqmutex = "qmutex";
#endif

#ifdef PLX_LINUX

// Linux-specific Forward declarations
int gscShmInit(int firstLoad);


void __attribute__ ((constructor)) GscSio4InitializeApi()
{
  int i, ch;
  int status = 0;
  struct shmid_ds shmInfo;

  shmid = shmget( 1234, 1048576, IPC_CREAT | IPC_EXCL | S_IRUSR | S_IWUSR | S_IWOTH | S_IROTH );

  if (shmid != -1)
  {
    if ((shmPool = (int *)shmat( shmid, NULL, 0 )) == (void *)-1)
      printf("Error attaching shared memory\n");
    else
      status = gscShmInit(0);
  }
  else
  {
    shmid = shmget( 1234, sizeof(int), IPC_CREAT );
    if ((shmPool = (int *)shmat( shmid, NULL, 0 )) == (void *)-1)
      printf("Error attaching shared memory\n");
    else
    {
      shmctl( shmid, IPC_STAT, &shmInfo ); // query the number of attachments for the shm.

      if (shmInfo.shm_nattch == 1)
      {
        // cleaup orphaned shm segment and msg queues
        status = gscShmInit(1);
        for (i=0; i < MAXIMUM_NUMBER_OF_BOARDS; i++)
        {
          if (bdRefCount[i] > 0)
          {
            // Delete the dma thread queue
            if (msgctl(boardInformation[i].dmaQueueId, IPC_RMID, 0) == -1)
              puts( "Message queue cleanup failed!");

            // Delete the interrupt thread message queue
            if (msgctl(boardInformation[i].intrQueueId, IPC_RMID, 0) == -1)
              puts( "Message queue cleanup failed!");
          }
        }

        memset(shmPool, 0, 1048576);
        status = gscShmInit(0);
      }
      else
        status = gscShmInit(1);
    }
  }

  drvMajorVersion = 0;
  drvMinorVersion = 0;

  // Dynamically link Plx Api functions
  status = gscPlxDynamicLink();
  if (status != ApiSuccess)
  {
    if (status != ApiNoActiveDriver)
      puts( "Dynamic loading of Plx Api functions failed!");
    exit(1);
  }

  if (*soRefCount < 2)
  {
    // Initialize the "per board" info
    for (i=0; i<MAXIMUM_NUMBER_OF_BOARDS; i++)
    {
      // Clear the "handles"
      sio4BoardHandles6[i].hDevice = (PLX_DRIVER_HANDLE)0;
      sio4BoardHandles[i].hDevice = (PLX_DRIVER_HANDLE)0;
      sio4BoardHandlesLegacy[i] = (HANDLE)0;

      // Initialize all of the "per channel" info
      for (ch=0; ch<NUMBER_OF_CHANNELS; ch++)
      {
        // Set all channels frequency to 20MHz
        boardInformation[i].channelFrequency[ch] = 0;
        GscSio4ChannelSetClock( i+1, ch+1, 20000000);

        // Set all channels to uninitialized
        boardInformation[i].channelMode[ch] = GSC_MODE_UNINITIALIZED;

        // Set all channels to NRZ
        boardInformation[i].channelEncoding[ch] = GSC_ENCODING_NRZ;
      }
    }
  }
  else
  {
    // Initialize the "per board" info
    for (i=0; i<MAXIMUM_NUMBER_OF_BOARDS; i++)
    {
      // Clear the "handles"
      sio4BoardHandles6[i].hDevice = (PLX_DRIVER_HANDLE)0;
      sio4BoardHandles6[i].IsValidTag = PLX_TAG_INVALID;
      sio4BoardHandles[i].hDevice = (PLX_DRIVER_HANDLE)0;
      sio4BoardHandles[i].IsValidTag = PLX_TAG_INVALID;
      sio4BoardHandlesLegacy[i] = (HANDLE)0;
    }
  }

  // Get the board count
  status = GscFindBoards( &i, NULL);
}

void __attribute__ ((destructor)) GscSio4UnloadApi()
{
  struct shmid_ds shmInfo;

  (*soRefCount)--;

  if (*soRefCount == 0)
    shmctl( shmid, IPC_RMID, &shmInfo ); // mark shared memory segment for deletion on lib unload.

  //shmdt( shmPool );
  dlclose( handle );
}

#else

//-------------------------------------------------------------------- DllMain
//
// Main entry point for the DLL.
//
#ifdef _USERDLL
BOOL APIENTRY DllMain( HANDLE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
                                         )
{
  int i, ch;
  int status = 0;
  int nPlxReloadAttempts = 0;

  switch (ul_reason_for_call)
  {
  case DLL_PROCESS_ATTACH:

    // Increment dll ref count
    soRefCount++;

    InitCriticalSection(&PlxApiCalls, lpPlxApiCalls);
    InitCriticalSection(&GscApiCalls, lpGscApiCalls);
    InitCriticalSection(&qmutex, lpqmutex);

    drvMajorVersion = 0;
    drvMinorVersion = 0;

    // Dynamically link Plx Api functions
    status = gscPlxDynamicLink();
    if (status != ApiSuccess)
    {
      nPlxReloadAttempts = 0;
      while (status != ApiSuccess && nPlxReloadAttempts++ <= 3)
        status = gscPlxDynamicLink();
      if (status != ApiSuccess)
      {
        puts( "Dynamic loading of Plx Api functions failed!");
        return FALSE;
      }
    }

    // Initialize the critical section objects
    if (soRefCount < 2)
    {
      // Initialize the "per board" info
      for (i=0; i<MAXIMUM_NUMBER_OF_BOARDS; i++)
      {
        // Clear the "handles" array
        boardInformation[i].handle = NULL;
        sio4BoardHandles6[i].hDevice = (PLX_DRIVER_HANDLE)NULL;
        //sio4BoardHandles6[i].IsValidTag = PLX_TAG_INVALID;
        sio4BoardHandles[i].hDevice = (PLX_DRIVER_HANDLE)NULL;
        //sio4BoardHandles[i].IsValidTag = PLX_TAG_INVALID;
        sio4BoardHandlesLegacy[i] = (HANDLE)NULL;

        // Generate the abort thread strings
        sprintf(threadAbort[i], "AbortThread%d", i+1);

        // Generate the transfer complete event strings
        sprintf(transferCompleteEvent[i], "TransferComplete%d", i+1);
        sprintf(rxTransferCompleteEvent[i], "RxTransferComplete%d", i+1);

        // Initialize all of the "per channel" info
        for (ch=0; ch<NUMBER_OF_CHANNELS; ch++)
        {
          // Set all channels frequency to 20MHz
          boardInformation[i].channelFrequency[ch] = 0;
          GscSio4ChannelSetClock(i+1, ch+1, 20000000);

          // Set all channels to uninitialized
          boardInformation[i].channelMode[ch] = GSC_MODE_UNINITIALIZED;

          // Set all channels to NRZ
          boardInformation[i].channelEncoding[ch] = GSC_ENCODING_NRZ;
        }

        // Initialize board reference counts
        bdRefCount[i] = 0;
      }
    }
    else
    {
      // Initialize the "per board" info
      for (i=0; i<MAXIMUM_NUMBER_OF_BOARDS; i++)
      {
        // Clear the "handles"
        sio4BoardHandles6[i].hDevice = (PLX_DRIVER_HANDLE)0;
        sio4BoardHandles6[i].IsValidTag = PLX_TAG_INVALID;
        sio4BoardHandles[i].hDevice = (PLX_DRIVER_HANDLE)0;
        sio4BoardHandles[i].IsValidTag = PLX_TAG_INVALID;
        sio4BoardHandlesLegacy[i] = (HANDLE)0;
      }
    }


    // Set the timer resolution to 1ms
    timeBeginPeriod( 1);

    // Get the board count
    GscFindBoards( &i, NULL);
    break;

  case DLL_THREAD_ATTACH:
  case DLL_THREAD_DETACH:
    break;

  case DLL_PROCESS_DETACH:

    // Decrement dll ref count.
    soRefCount--;

    // Let the timer resolution go back to the default
    timeEndPeriod( 1);

    // Release the critical section objects
    if (soRefCount == 0)
    {
      DelCriticalSection(PlxApiCalls);
      DelCriticalSection(GscApiCalls);
      DelCriticalSection(qmutex);
    }

    FreeLibrary( hInstLib );
#ifdef DATA_LOGGING
    // Save the data log if it is not empty
    if (rxDataLogPointer != 0x00)
    {
      char fileBuffer[1000];
      DWORD count;
      int ch;

      // Open the log file
      HANDLE hFile = CreateFile("RxDataLog.txt", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

      // Save all of the data
      for (ch=0; ch<rxDataLogPointer; ch++)
      {
        int size = sprintf(fileBuffer, "%04X to %08X (%6.2f ms, %08X) (reason=%d, timer=%d)\n",
                           rxDataLog[ch].size,
                           (int)rxDataLog[ch].buffer,
                           (double)rxDataLog[ch].delta/100,
                           rxDataLog[ch].time,
                           rxDataLog[ch].reason,
                           rxDataLog[ch].timerDelay
                           );
        WriteFile(hFile, fileBuffer, size, &count, NULL);
      }

      // Close the log file
      CloseHandle(hFile);
    }

    if (txDataLogPointer != 0x00)
    {
      char fileBuffer[1000];
      DWORD count;
      int ch;

      // Open the log file
      HANDLE hFile = CreateFile("TxDataLog.txt", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

      // Save all of the data
      for (ch=0; ch<txDataLogPointer; ch++)
      {
        int size = sprintf(fileBuffer, "%04X to %08X (%6.2f ms, %08X) (reason=%d, timer=%d) fifoCount=%08X, tcbSent = %d\n",
                           txDataLog[ch].size,
                           (int)txDataLog[ch].buffer,
                           (double)txDataLog[ch].delta/100,
                           txDataLog[ch].time,
                           txDataLog[ch].reason,
                           txDataLog[ch].timerDelay,
                           txDataLog[ch].fifoCount,
                           txDataLog[ch].tcbSent
                           );
        WriteFile(hFile, fileBuffer, size, &count, NULL);
      }

      // Close the log file
      CloseHandle(hFile);
    }
#endif
    break;
  }

  return TRUE;
}
#endif

//-------------------------------------------------------------------- gscStaticLibInit
//
// Initialization routine used when developing apps with GscApi as a static library.
//
#ifndef _USERDLL
int gscStaticLibInit()
{
  int status = 0;
  int i, ch;

  // Setup PlxApi function pointers to reference statically linked PlxApi 6.3 functions
  status = gscPlxStaticLink();

  // Keep dll ref count at zero. It will be unused when linking statically with GscApi.lib
  soRefCount = 0;

  // Initialize the critical section objects
  InitCriticalSection(&PlxApiCalls, lpPlxApiCalls);
  InitCriticalSection(&GscApiCalls, lpGscApiCalls);
  InitCriticalSection(&qmutex, lpqmutex);

  drvMajorVersion = 6; // Only supporting our delivered PlxApi version when linking statically
  drvMinorVersion = 3;

  // Initialize the "per board" info
  for (i=0; i<MAXIMUM_NUMBER_OF_BOARDS; i++)
  {
    // Clear the "handles" array
    boardInformation[i].handle = NULL;
    sio4BoardHandles6[i].hDevice = (PLX_DRIVER_HANDLE)NULL;
    //sio4BoardHandles6[i].IsValidTag = PLX_TAG_INVALID;
    sio4BoardHandles[i].hDevice = (PLX_DRIVER_HANDLE)NULL;
    //sio4BoardHandles[i].IsValidTag = PLX_TAG_INVALID;
    sio4BoardHandlesLegacy[i] = (HANDLE)NULL;

    // Generate the abort thread strings
    sprintf(threadAbort[i], "AbortThread%d", i+1);

    // Generate the transfer complete event strings
    sprintf(transferCompleteEvent[i], "TransferComplete%d", i+1);
    sprintf(rxTransferCompleteEvent[i], "RxTransferComplete%d", i+1);

    // Initialize all of the "per channel" info
    for (ch=0; ch<NUMBER_OF_CHANNELS; ch++)
    {
      // Set all channels frequency to 20MHz
      boardInformation[i].channelFrequency[ch] = 0;
      GscSio4ChannelSetClock(i+1, ch+1, 20000000);

      // Set all channels to uninitialized
      boardInformation[i].channelMode[ch] = GSC_MODE_UNINITIALIZED;

      // Set all channels to NRZ
      boardInformation[i].channelEncoding[ch] = GSC_ENCODING_NRZ;
    }

    // Initialize board reference counts
    bdRefCount[i] = 0;
  }

  // Set the timer resolution to 1ms
  timeBeginPeriod( 1);

  // Get the board count
  //GscFindBoards( &i, NULL);

  bStaticLibInitialized = TRUE;

  return 0;
}
#endif
#endif


#ifdef __linux__
//--------------------------------------------------------------------------- gscShmInit
//
// Initialize shared memory for GscApi globals to be shared amongst all
// attached processes.
//
int gscShmInit(int firstLoad)
{
  int i = 0;
  int j = 0;
  int bdOffset = 0;

  /* initialize all globals to shared mem pool offsets
   */
  sio4BoardsLegacy3 = (DEVICE_LOCATION3 *)shmPool;
  sio4BoardsLegacy  = (DEVICE_LOCATION *) (shmPool + 1024);
  sio4BoardInfo     = (BOARD_INFO *) (shmPool + 3072);

  for(i=0; i < MAXIMUM_NUMBER_OF_BOARDS; i++)
  {
    //threadAbort[i]             = (char *) (shmPool + 5120 + (i * 20));
    //transferCompleteEvent[i]   = (char *) (shmPool + 6144 + (i * 20));
    //rxTransferCompleteEvent[i] = (char *) (shmPool + 7168 + (i * 20));
    txDmaQueueTo[i]            = (int *)  (shmPool + 8192 + (i * NUMBER_OF_CHANNELS * sizeof(int)));
    txDmaQueueFrom[i]          = (int *)  (shmPool + 9216 + (i * NUMBER_OF_CHANNELS * sizeof(int)));
    rxDmaQueueTo[i]            = (int *)  (shmPool + 10240 + (i * NUMBER_OF_CHANNELS * sizeof(int)));
    rxDmaQueueFrom[i]          = (int *)  (shmPool + 11264 + (i * NUMBER_OF_CHANNELS * sizeof(int)));
    rxPacketTo[i] = (int *) (shmPool + 11520 + (i * NUMBER_OF_CHANNELS * sizeof(int)));
    rxPacketFrom[i] = (int  *) (shmPool + 11776 + (i * NUMBER_OF_CHANNELS * sizeof(int)));

    for(j=0; j < NUMBER_OF_CHANNELS; j++)
    {
      //txStartEvent[i][j] = (char *) (shmPool + 13312 + (j * 20));
      //rxStartEvent[i][j] = (char *) (shmPool + 14336 + (j * 20));
      bdOffset = i * NUMBER_OF_CHANNELS * sizeof(int) * MAXIMUM_RX_PACKETS;
      txDmaQueue[i][j]   = (DMA_TRANSFER_QUEUE *) (shmPool + 65536 + bdOffset + (j * sizeof(DMA_TRANSFER_QUEUE)) *
                                                   MAXIMUM_DMA_QUEUE_ITEMS);
      rxDmaQueue[i][j]   = (DMA_TRANSFER_QUEUE *) (shmPool + 262144 + bdOffset + (j * sizeof(DMA_TRANSFER_QUEUE)) *
                                                   MAXIMUM_DMA_QUEUE_ITEMS);
      rxPacketLengths[i][j] = (int *) (shmPool + 491520 + bdOffset + (j * sizeof(int) * MAXIMUM_RX_PACKETS));
      rxPacketStatus[i][j] = (int *) (shmPool + 720869 + bdOffset + (j * sizeof(int) * MAXIMUM_RX_PACKETS));
    }
  }

  dmaRequestQueueId      = (int *) (shmPool + 12288);

  for(i=0; i < MAXIMUM_NUMBER_OF_BOARDS; i++)
    intrThread[i]        = (pthread_t *) (shmPool + 13312 + (i * sizeof(pthread_t)));

  for(i=0; i < MAXIMUM_NUMBER_OF_BOARDS; i++)
    rxIntrThread[i]      = (pthread_t *) (shmPool + 13824 + (i * sizeof(pthread_t)));

  qmutex                 = (pthread_mutex_t *) (shmPool + 15360);
  PlxApiCalls            = (pthread_mutex_t *) (shmPool + 17408);
  GscApiCalls            = (pthread_mutex_t *) (shmPool + 18432);
  threadMessageId        = (int *) (shmPool + 19456);
  boardInformation       = (BOARD_INFORMATION *) (shmPool + 20480);
  //sio4BoardHandlesLegacy = (HANDLE *) (shmPool + 38912);
  sio4BoardInfo          = (BOARD_INFO *) (shmPool + 40960);
  soRefCount             = (int *) (shmPool + 41984);

  if (firstLoad == 0)
  {
    // Initialize the critical section objects
    InitCriticalSection(PlxApiCalls);
    InitCriticalSection(qmutex);
    InitCriticalSection(GscApiCalls);
    *soRefCount = 1;
  }
  else
    (*soRefCount)++;

  bdRefCount  = (int *) (shmPool + 40964);
  for(i=0; i < MAXIMUM_NUMBER_OF_BOARDS; i++)
  {
    if (firstLoad == 0) // 1st load of shared lib
      bdRefCount[i] = 0;
  }

  return 0;
}
#endif

#ifndef _USERDLL
int gscPlxStaticLink()
{
  PlxPci_ApiVersion_Rtdl                = PlxPci_ApiVersion;
  PlxPci_DriverVersion_Rtdl             = PlxPci_DriverVersion;
  PlxPci_DriverVersion6_Rtdl            = NULL;
  PlxDriverVersion_Rtdl                 = NULL;
  PlxPci_DeviceFind_Rtdl                = PlxPci_DeviceFind;
  PlxPci_DeviceFind6_Rtdl               = NULL;
  PlxPciDeviceFind_Rtdl                 = NULL;
  PlxPciDeviceFind3_Rtdl                = NULL;
  PlxPci_DeviceOpen_Rtdl                = PlxPci_DeviceOpen;
  PlxPci_DeviceOpen6_Rtdl               = NULL;
  PlxPciDeviceOpen_Rtdl                 = NULL;
  PlxPciDeviceOpen3_Rtdl                = NULL;
  PlxPci_DeviceClose_Rtdl               = PlxPci_DeviceClose;
  PlxPci_DeviceClose6_Rtdl              = NULL;
  PlxPciDeviceClose_Rtdl                = NULL;
  PlxPci_PciBarSpaceRead_Rtdl           = PlxPci_PciBarSpaceRead;
  PlxPci_PciBarSpaceRead6_Rtdl          = NULL;
  PlxPci_PciBarSpaceWrite_Rtdl          = PlxPci_PciBarSpaceWrite;
  PlxPci_PciBarSpaceWrite6_Rtdl         = NULL;
  PlxBusIopRead_Rtdl                    = NULL;
  PlxBusIopWrite_Rtdl                   = NULL;
  PlxPci_DmaTransferUserBuffer_Rtdl     = PlxPci_DmaTransferUserBuffer;
  PlxPci_DmaTransferUserBuffer6_Rtdl    = NULL;
  PlxDmaSglTransfer_Rtdl                = NULL;
  PlxDmaSglTransfer3_Rtdl               = NULL;
  PlxPci_DmaChannelClose_Rtdl           = PlxPci_DmaChannelClose;
  PlxPci_DmaChannelClose6_Rtdl          = NULL;
  PlxDmaSglChannelClose_Rtdl            = NULL;
  PlxPci_DmaChannelOpen_Rtdl            = PlxPci_DmaChannelOpen;
  PlxPci_DmaChannelOpen6_Rtdl           = NULL;
  PlxDmaSglChannelOpen_Rtdl             = NULL;
  PlxDmaSglChannelOpen3_Rtdl            = NULL;
  PlxPci_DmaControl_Rtdl                = PlxPci_DmaControl;
  PlxPci_DmaControl6_Rtdl               = NULL;
  PlxDmaControl_Rtdl                    = NULL;
  PlxPci_InterruptEnable_Rtdl           = PlxPci_InterruptEnable;
  PlxPci_InterruptEnable6_Rtdl          = NULL;
  PlxPci_InterruptDisable_Rtdl          = PlxPci_InterruptDisable;
  PlxPci_InterruptDisable6_Rtdl         = NULL;
  PlxIntrDisable_Rtdl                   = NULL;
  PlxIntrDisable3_Rtdl                  = NULL;
  PlxIntrEnable_Rtdl                    = NULL;
  PlxIntrEnable3_Rtdl                   = NULL;
  PlxPci_NotificationRegisterFor_Rtdl   = PlxPci_NotificationRegisterFor;
  PlxPci_NotificationRegisterFor6_Rtdl  = NULL;
  PlxNotificationRegisterFor_Rtdl       = NULL;
  PlxPci_NotificationWait_Rtdl          = PlxPci_NotificationWait;
  PlxPci_NotificationWait6_Rtdl         = NULL;
  PlxNotificationWait_Rtdl              = NULL;
  PlxPci_NofificationCancel_Rtdl        = PlxPci_NotificationCancel;
  PlxPci_NofificationCancel6_Rtdl       = NULL;
  PlxIntrAttach_Rtdl                    = NULL;
  PlxIntrWait_Rtdl                      = NULL;
  PlxPci_PciRegisterReadFast_Rtdl       = PlxPci_PciRegisterReadFast;
  PlxPci_PciRegisterReadFast6_Rtdl      = NULL;
  PlxPciRegisterReadFast_Rtdl           = NULL;

  return 0;
}
#endif

//--------------------------------------------------------------------------- gscPlxDynamicLink
//
// dynamically load Plx functions that do not appear in all versions of the PlxApi lib
//
int gscPlxDynamicLink()
{
  int status = 0;
  U8 revision = 0;

  // Open the Plx shared lib for the appropriate platform
#ifdef PLX_LINUX
  handle = dlopen("libPlxApi.so", RTLD_LAZY);
  if (!handle)
  {
    fputs( dlerror(), stderr );
    fputs("\n",stderr);
    return ApiFailed;
  }
  PlxPci_DeviceFind_Rtdl = NULL;
  PlxPci_ApiVersion_Rtdl = (PlxPciApiVersion)dlsym( handle, "PlxPci_ApiVersion");
  if (PlxPci_ApiVersion_Rtdl)
  {
    status = PlxPci_ApiVersion_Rtdl( &drvMajorVersion, &drvMinorVersion, &revision );
    if (status != ApiSuccess)
    {
      fputs( "Function PlxPci_ApiVersion failed" , stderr);
      return status;
    }
  }
  else
  {
    fputs( "Function PlxPci_ApiVersion not found" , stderr);
    return ApiFailed;
  }

  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
  {
    PlxPci_DriverVersion_Rtdl = (PlxPciDrvVersion)dlsym( handle, "PlxPci_DriverVersion");
    if (!PlxPci_DriverVersion_Rtdl)
    {
      fputs( "Function PlxPci_DriverVersion not found" , stderr);
      return ApiFailed;
    }
    PlxPci_DeviceOpen_Rtdl = (PlxPciDevOpen)dlsym( handle, "PlxPci_DeviceOpen");
    if (!PlxPci_DeviceOpen_Rtdl)
    {
      fputs( "Function PlxPci_DeviceOpen not found" , stderr);
      return ApiFailed;
    }
  }
  else if (drvMajorVersion >= 5)
  {
    PlxPci_DriverVersion6_Rtdl = (PlxPciDrvVersion6)dlsym( handle, "PlxPci_DriverVersion");
    if (!PlxPci_DriverVersion6_Rtdl)
    {
      fputs( "Function PlxPci_DriverVersion not found" , stderr);
      return ApiFailed;
    }
    PlxPci_DeviceOpen6_Rtdl = (PlxPciDevOpen6)dlsym( handle, "PlxPci_DeviceOpen");
    if (!PlxPci_DeviceOpen6_Rtdl)
    {
      fputs( "Function PlxPci_DeviceOpen not found" , stderr);
      return ApiFailed;
    }
  }
  else
  {
    PlxDriverVersion_Rtdl = (PlxDrvVersionLegacy)dlsym( handle, "PlxDriverVersion");
    if (!PlxDriverVersion_Rtdl)
    {
      fputs( "Function PlxDriverVersion not found" , stderr);
      return ApiFailed;
    }
    PlxPciDeviceOpen_Rtdl = (PlxPciDevOpenLegacy)dlsym( handle, "PlxPciDeviceOpen");
    if (!PlxPciDeviceOpen_Rtdl)
    {
      fputs( "Function PlxPciDeviceOpen not found" , stderr);
      return ApiFailed;
    }
  }
#else

  // dynamically load the highest version of the plxapi dll installed on the system
  // and determine the version number
  if(hInstLib = LoadLibrary( "PlxApi650.dll" ))
  {
  // determine which function is available in the PLX API for retrieving the driver version
#ifdef PLX_LINUX
    PlxPci_DriverVersion_Rtdl = (PlxPciDrvVersion)dlsym( handle, "PlxPci_DriverVersion");
#else
    PlxPci_DriverVersion_Rtdl = (PlxPciDrvVersion)GetProcAddress( hInstLib, "PlxPci_DriverVersion");
#endif
    if (PlxPci_DriverVersion_Rtdl)
    {
      // must open a board in order to get driver version, so dynamically link the Plx open function
  #ifdef PLX_LINUX
      PlxPci_DeviceOpen_Rtdl = (PlxPciDevOpen)dlsym( handle, "PlxPci_DeviceOpen");
  #else
      PlxPci_DeviceOpen_Rtdl = (PlxPciDevOpen)GetProcAddress( hInstLib, "PlxPci_DeviceOpen");
  #endif
      if (!PlxPci_DeviceOpen_Rtdl)
      {
        fputs( "Function PlxPci_DeviceOpen not found" , stderr);
        return ApiFailed;
      }
      // open the device - try PLX_DEVICE_KEY
      sio4Boards[0].bus  = PCI_FIELD_IGNORE;
      sio4Boards[0].slot = PCI_FIELD_IGNORE;
      sio4Boards[0].function = PCI_FIELD_IGNORE;
      sio4Boards[0].VendorId = PCI_FIELD_IGNORE;
      sio4Boards[0].DeviceId = PCI_FIELD_IGNORE;
      sio4Boards[0].SubVendorId = PCI_FIELD_IGNORE;
      sio4Boards[0].SubDeviceId = PCI_FIELD_IGNORE;
      sio4Boards[0].Revision = PCI_FIELD_IGNORE;
      BeginCriticalSection( PlxApiCalls);
      status = PlxPci_DeviceOpen_Rtdl (&sio4Boards[0], &sio4BoardHandles[0]);
      EndCriticalSection( PlxApiCalls);
      if (status == ApiNoActiveDriver)
      {
        puts( "No active driver.");
        return status;
      }
      else
      {
        BeginCriticalSection( PlxApiCalls);
        status = PlxPci_DriverVersion_Rtdl( &sio4BoardHandles[0], &drvMajorVersion, &drvMinorVersion, &revision );
        EndCriticalSection( PlxApiCalls);
        if (status != ApiSuccess)
          return status;
      }
    }
  }
  else if(hInstLib = LoadLibrary( "PlxApi60.dll" ))
  {
  // determine which function is available in the PLX API for retrieving the driver version
#ifdef PLX_LINUX
    PlxPci_DriverVersion6_Rtdl = (PlxPciDrvVersion6)dlsym( handle, "PlxPci_DriverVersion");
#else
    PlxPci_DriverVersion6_Rtdl = (PlxPciDrvVersion6)GetProcAddress( hInstLib, "PlxPci_DriverVersion");
#endif
    if (PlxPci_DriverVersion6_Rtdl)
    {
      // must open a board in order to get driver version, so dynamically link the Plx open function
  #ifdef PLX_LINUX
      PlxPci_DeviceOpen6_Rtdl = (PlxPciDevOpen6)dlsym( handle, "PlxPci_DeviceOpen");
  #else
      PlxPci_DeviceOpen6_Rtdl = (PlxPciDevOpen6)GetProcAddress( hInstLib, "PlxPci_DeviceOpen");
  #endif
      if (!PlxPci_DeviceOpen6_Rtdl)
      {
        fputs( "Function PlxPci_DeviceOpen not found" , stderr);
        return ApiFailed;
      }
      // open the device - try PLX_DEVICE_KEY
      sio4Boards6[0].bus  = PCI_FIELD_IGNORE;
      sio4Boards6[0].slot = PCI_FIELD_IGNORE;
      sio4Boards6[0].function = PCI_FIELD_IGNORE;
      sio4Boards6[0].VendorId = PCI_FIELD_IGNORE;
      sio4Boards6[0].DeviceId = PCI_FIELD_IGNORE;
      sio4Boards6[0].SubVendorId = PCI_FIELD_IGNORE;
      sio4Boards6[0].SubDeviceId = PCI_FIELD_IGNORE;
      sio4Boards6[0].Revision = PCI_FIELD_IGNORE;
      BeginCriticalSection( PlxApiCalls);
      status = PlxPci_DeviceOpen6_Rtdl (&sio4Boards6[0], &sio4BoardHandles6[0]);
      EndCriticalSection( PlxApiCalls);
      if (status == ApiNoActiveDriver)
      {
        puts( "No active driver.");
        return status;
      }
      else
      {
        BeginCriticalSection( PlxApiCalls);
        status = PlxPci_DriverVersion6_Rtdl( &sio4BoardHandles6[0], &drvMajorVersion, &drvMinorVersion, &revision );
        EndCriticalSection( PlxApiCalls);
        if (status != ApiSuccess)
          return status;
      }
    }
  }
  else if(hInstLib = LoadLibrary( "PlxApi51.dll" ))
  {
  // determine which function is available in the PLX API for retrieving the driver version
#ifdef PLX_LINUX
    PlxPci_DriverVersion6_Rtdl = (PlxPciDrvVersion6)dlsym( handle, "PlxPci_DriverVersion");
#else
    PlxPci_DriverVersion6_Rtdl = (PlxPciDrvVersion6)GetProcAddress( hInstLib, "PlxPci_DriverVersion");
#endif
    if (PlxPci_DriverVersion6_Rtdl)
    {
      // must open a board in order to get driver version, so dynamically link the Plx open function
  #ifdef PLX_LINUX
      PlxPci_DeviceOpen6_Rtdl = (PlxPciDevOpen6)dlsym( handle, "PlxPci_DeviceOpen");
  #else
      PlxPci_DeviceOpen6_Rtdl = (PlxPciDevOpen6)GetProcAddress( hInstLib, "PlxPci_DeviceOpen");
  #endif
      if (!PlxPci_DeviceOpen6_Rtdl)
      {
        fputs( "Function PlxPci_DeviceOpen not found" , stderr);
        return ApiFailed;
      }
      // open the device - try PLX_DEVICE_KEY
      sio4Boards6[0].bus  = PCI_FIELD_IGNORE;
      sio4Boards6[0].slot = PCI_FIELD_IGNORE;
      sio4Boards6[0].function = PCI_FIELD_IGNORE;
      sio4Boards6[0].VendorId = PCI_FIELD_IGNORE;
      sio4Boards6[0].DeviceId = PCI_FIELD_IGNORE;
      sio4Boards6[0].SubVendorId = PCI_FIELD_IGNORE;
      sio4Boards6[0].SubDeviceId = PCI_FIELD_IGNORE;
      sio4Boards6[0].Revision = PCI_FIELD_IGNORE;
      BeginCriticalSection( PlxApiCalls);
      status = PlxPci_DeviceOpen6_Rtdl (&sio4Boards6[0], &sio4BoardHandles6[0]);
      EndCriticalSection( PlxApiCalls);
      if (status == ApiNoActiveDriver)
      {
        puts( "No active driver.");
        return status;
      }
      else
      {
        BeginCriticalSection( PlxApiCalls);
        status = PlxPci_DriverVersion6_Rtdl( &sio4BoardHandles6[0], &drvMajorVersion, &drvMinorVersion, &revision );
        EndCriticalSection( PlxApiCalls);
        if (status != ApiSuccess)
          return status;
      }
    }
  }
  else if (hInstLib = LoadLibrary( "PlxApi.dll" ))
  {
#ifdef PLX_LINUX
    PlxDriverVersion_Rtdl = (PlxDrvVersionLegacy)dlsym( handle, "PlxDriverVersion");
#else
    PlxDriverVersion_Rtdl = (PlxDrvVersionLegacy)GetProcAddress( hInstLib, "PlxDriverVersion");
#endif
    if (PlxDriverVersion_Rtdl)
    {
      // must open a board in order to get driver version, so dynamically link the Plx open function
#ifdef PLX_LINUX
      PlxPciDeviceOpen_Rtdl = (PlxPciDevOpenLegacy)dlsym( handle, "PlxPciDeviceOpen");
#else
      PlxPciDeviceOpen_Rtdl = (PlxPciDevOpenLegacy)GetProcAddress( hInstLib, "PlxPciDeviceOpen");
#endif
      if (!PlxPciDeviceOpen_Rtdl)
      {
        fputs( "Function PlxPciDeviceOpen not found" , stderr);
        exit(1);
      }
      sio4BoardsLegacy[0].BusNumber  = -1;
      sio4BoardsLegacy[0].SlotNumber = -1;
      sio4BoardsLegacy[0].VendorId   = -1;
      sio4BoardsLegacy[0].DeviceId   = -1;
      sio4BoardsLegacy[0].SerialNumber[0] = '\0';
      gscBoardCountLegacy = FIND_AMOUNT_MATCHED;
      BeginCriticalSection( PlxApiCalls);
      status = PlxPciDeviceOpen_Rtdl (&sio4BoardsLegacy[0], &sio4BoardHandlesLegacy[0]);
      EndCriticalSection( PlxApiCalls);
      if (status == ApiNoActiveDriver)
      {
        puts( "No active driver.");
        exit(1);
      }
      else if (status != ApiSuccess)
      {
        // The open may not have worked because PlxApi 3.4 is the installed. See if the PlxApi 3.4 version
        // of the open function works.
#ifdef PLX_LINUX
        PlxPciDeviceOpen3_Rtdl = (PlxPciDevOpenLegacy3)dlsym( handle, "PlxPciDeviceOpen");
#else
        PlxPciDeviceOpen3_Rtdl = (PlxPciDevOpenLegacy3)GetProcAddress( hInstLib, "PlxPciDeviceOpen");
#endif
        if (!PlxPciDeviceOpen3_Rtdl)
        {
          fputs( "Function PlxPciDeviceOpen not found" , stderr);
          exit(1);
        }
        sio4BoardsLegacy3[0].BusNumber  = -1;
        sio4BoardsLegacy3[0].SlotNumber = -1;
        sio4BoardsLegacy3[0].VendorId   = -1;
        sio4BoardsLegacy3[0].DeviceId   = -1;
        sio4BoardsLegacy3[0].SerialNumber[0] = '\0';
        gscBoardCountLegacy = FIND_AMOUNT_MATCHED;
        BeginCriticalSection( PlxApiCalls);
        status = PlxPciDeviceOpen3_Rtdl (&sio4BoardsLegacy3[0], &sio4BoardHandlesLegacy[0]);
        EndCriticalSection( PlxApiCalls);
        if (status != ApiSuccess) return status;
      }

      // Get the driver version
      BeginCriticalSection( PlxApiCalls);
      status = PlxDriverVersion_Rtdl(sio4BoardHandlesLegacy[0], &drvMajorVersion, &drvMinorVersion, &revision);
      EndCriticalSection( PlxApiCalls);
    }
    else
    {
      fputs( "Function PlxPci_DriverVersion not found" , stderr);
      exit(1);
    }
  }

  if (!hInstLib)
  {
    puts( "GscApi DLL Failed to Load!");
    return ApiFailed;
  }
#endif

    // Need the Plx close function entry point in order to close the board just opened
  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
  {
#ifdef PLX_LINUX
    PlxPci_DeviceClose_Rtdl = (PlxPciDevClose)dlsym( handle, "PlxPci_DeviceClose");
#else
    PlxPci_DeviceClose_Rtdl = (PlxPciDevClose)GetProcAddress( hInstLib, "PlxPci_DeviceClose");
#endif
    if (!PlxPci_DeviceClose_Rtdl)
    {
      fputs( "Function PlxPci_DeviceClose not found" , stderr);
      return ApiFailed;
    }
    // close the device
#ifndef PLX_LINUX
    BeginCriticalSection( PlxApiCalls);
    status = PlxPci_DeviceClose_Rtdl( &sio4BoardHandles[0]);
    EndCriticalSection( PlxApiCalls);
    sio4BoardHandles6[0].hDevice = sio4BoardHandles[0].hDevice = (PLX_DRIVER_HANDLE)0;
    if ( status != ApiSuccess)
      return status;
#endif
  }
  else if (drvMajorVersion >= 5)
  {
#ifdef PLX_LINUX
    PlxPci_DeviceClose6_Rtdl = (PlxPciDevClose6)dlsym( handle, "PlxPci_DeviceClose");
#else
    PlxPci_DeviceClose6_Rtdl = (PlxPciDevClose6)GetProcAddress( hInstLib, "PlxPci_DeviceClose");
#endif
    if (!PlxPci_DeviceClose6_Rtdl)
    {
      fputs( "Function PlxPci_DeviceClose not found" , stderr);
      return ApiFailed;
    }
    // close the device
#ifndef PLX_LINUX
    BeginCriticalSection( PlxApiCalls);
    status = PlxPci_DeviceClose6_Rtdl( &sio4BoardHandles6[0]);
    EndCriticalSection( PlxApiCalls);
    sio4BoardHandles6[0].hDevice = sio4BoardHandles[0].hDevice = (PLX_DRIVER_HANDLE)0;
    if ( status != ApiSuccess)
      return status;
#endif
  }
  else // Plx 4.x and 3.4
  {
    // Need the Plx close function entry point in order to close the board just opened
#ifdef PLX_LINUX
    PlxPciDeviceClose_Rtdl = (PlxPciDevCloseLegacy)dlsym( handle, "PlxPciDeviceClose");
#else
    PlxPciDeviceClose_Rtdl = (PlxPciDevCloseLegacy)GetProcAddress( hInstLib, "PlxPciDeviceClose");
#endif
    if (!PlxPciDeviceClose_Rtdl)
    {
      fputs( "Function PlxPciDeviceClose not found" , stderr);
      exit(1);
    }
    // close the device
#ifndef PLX_LINUX
    BeginCriticalSection( PlxApiCalls);
    status = PlxPciDeviceClose_Rtdl( sio4BoardHandlesLegacy[0]);
    EndCriticalSection( PlxApiCalls);
    sio4BoardHandlesLegacy[0] = (HANDLE)0;
    if (status != ApiSuccess) return status;
#endif
  }

  //
  // Dynamically load all PLX version 6.1 or greater functions used by the GscApi dll.
  //

  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
  {
#ifdef PLX_LINUX
    PlxPci_DeviceFind_Rtdl = (PlxPciDevFind)dlsym( handle, "PlxPci_DeviceFind");
#else
    PlxPci_DeviceFind_Rtdl = (PlxPciDevFind)GetProcAddress( hInstLib, "PlxPci_DeviceFind");
#endif
    if (!PlxPci_DeviceFind_Rtdl)
    {
      fputs( "Function PlxPci_DeviceFind not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_PhysicalMemoryAllocate_Rtdl = (PlxPciPhysicalMemoryAllocate)dlsym( handle, "PlxPci_PhysicalMemoryAllocate");
#else
    PlxPci_PhysicalMemoryAllocate_Rtdl = (PlxPciPhysicalMemoryAllocate)GetProcAddress( hInstLib, "PlxPci_PhysicalMemoryAllocate");
#endif
    if (!PlxPci_PhysicalMemoryAllocate_Rtdl)
    {
      fputs( "Function PlxPci_PhysicalMemoryAllocate not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_PhysicalMemoryMap_Rtdl = (PlxPciPhysicalMemoryMap)dlsym( handle, "PlxPci_PhysicalMemoryMap");
#else
    PlxPci_PhysicalMemoryMap_Rtdl = (PlxPciPhysicalMemoryMap)GetProcAddress( hInstLib, "PlxPci_PhysicalMemoryMap");
#endif
    if (!PlxPci_PhysicalMemoryMap_Rtdl)
    {
      fputs( "Function PlxPci_PhysicalMemoryMap not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_PhysicalMemoryUnmap_Rtdl = (PlxPciPhysicalMemoryMap)dlsym( handle, "PlxPci_PhysicalMemoryUnmap");
#else
    PlxPci_PhysicalMemoryUnmap_Rtdl = (PlxPciPhysicalMemoryMap)GetProcAddress( hInstLib, "PlxPci_PhysicalMemoryUnmap");
#endif
    if (!PlxPci_PhysicalMemoryUnmap_Rtdl)
    {
      fputs( "Function PlxPci_PhysicalMemoryUnmap not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_PciBarSpaceRead_Rtdl = (PlxPciBarSpaceIO)dlsym( handle, "PlxPci_PciBarSpaceRead");
#else
    PlxPci_PciBarSpaceRead_Rtdl = (PlxPciBarSpaceIO)GetProcAddress( hInstLib, "PlxPci_PciBarSpaceRead");
#endif
    if (!PlxPci_PciBarSpaceRead_Rtdl)
    {
      fputs( "Function PlxPci_PciBarSpaceRead not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_PciBarSpaceWrite_Rtdl = (PlxPciBarSpaceIO)dlsym( handle, "PlxPci_PciBarSpaceWrite");
#else
    PlxPci_PciBarSpaceWrite_Rtdl = (PlxPciBarSpaceIO)GetProcAddress( hInstLib, "PlxPci_PciBarSpaceWrite");
#endif
    if (!PlxPci_PciBarSpaceWrite_Rtdl)
    {
      fputs( "Function PlxPci_PciBarSpaceWrite not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_DmaTransferUserBuffer_Rtdl = (PlxPciDmaTransfer)dlsym( handle, "PlxPci_DmaTransferUserBuffer");
#else
    PlxPci_DmaTransferUserBuffer_Rtdl = (PlxPciDmaTransfer)GetProcAddress( hInstLib, "PlxPci_DmaTransferUserBuffer");
#endif
    if (!PlxPci_DmaTransferUserBuffer_Rtdl)
    {
      fputs( "Function PlxPci_DmaTransferUserBuffer not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_DmaTransferBlock_Rtdl = (PlxPciDmaTransfer)dlsym( handle, "PlxPci_DmaTransferBlock");
#else
    PlxPci_DmaTransferBlock_Rtdl = (PlxPciDmaTransfer)GetProcAddress( hInstLib, "PlxPci_DmaTransferBlock");
#endif
    if (!PlxPci_DmaTransferBlock_Rtdl)
    {
      fputs( "Function PlxPci_DmaTransferBlock not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_DmaChannelClose_Rtdl = (PlxPciDmaChannelClose)dlsym( handle, "PlxPci_DmaChannelClose");
#else
    PlxPci_DmaChannelClose_Rtdl = (PlxPciDmaChannelClose)GetProcAddress( hInstLib, "PlxPci_DmaChannelClose");
#endif
    if (!PlxPci_DmaChannelClose_Rtdl)
    {
      fputs( "Function PlxPci_DmaChannelClose not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_DmaChannelOpen_Rtdl = (PlxPciDmaChannelOpen)dlsym( handle, "PlxPci_DmaChannelOpen");
#else
    PlxPci_DmaChannelOpen_Rtdl = (PlxPciDmaChannelOpen)GetProcAddress( hInstLib, "PlxPci_DmaChannelOpen");
#endif
    if (!PlxPci_DmaChannelOpen_Rtdl)
    {
      fputs( "Function PlxPci_DmaChannelOpen not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_DmaControl_Rtdl = (PlxPciDmaControl)dlsym( handle, "PlxPci_DmaControl");
#else
    PlxPci_DmaControl_Rtdl = (PlxPciDmaControl)GetProcAddress( hInstLib, "PlxPci_DmaControl");
#endif
    if (!PlxPci_DmaControl_Rtdl)
    {
      fputs( "Function PlxPci_DmaControl not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_InterruptDisable_Rtdl = (PlxPciInterrupt)dlsym( handle, "PlxPci_InterruptDisable");
#else
    PlxPci_InterruptDisable_Rtdl = (PlxPciInterrupt)GetProcAddress( hInstLib, "PlxPci_InterruptDisable");
#endif
    if (!PlxPci_InterruptDisable_Rtdl)
    {
      fputs( "Function PlxPci_InterruptDisable not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_InterruptEnable_Rtdl = (PlxPciInterrupt)dlsym( handle, "PlxPci_InterruptEnable");
#else
    PlxPci_InterruptEnable_Rtdl = (PlxPciInterrupt)GetProcAddress( hInstLib, "PlxPci_InterruptEnable");
#endif
    if (!PlxPci_InterruptEnable_Rtdl)
    {
      fputs( "Function PlxPci_InterruptEnable not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_NotificationRegisterFor_Rtdl = (PlxPciNotifReg)dlsym( handle, "PlxPci_NotificationRegisterFor");
#else
    PlxPci_NotificationRegisterFor_Rtdl = (PlxPciNotifReg)GetProcAddress( hInstLib, "PlxPci_NotificationRegisterFor");
#endif
    if (!PlxPci_NotificationRegisterFor_Rtdl)
    {
      fputs( "Function PlxPci_NotificationRegisterFor not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_NotificationWait_Rtdl = (PlxPciNotifWait)dlsym( handle, "PlxPci_NotificationWait");
#else
    PlxPci_NotificationWait_Rtdl = (PlxPciNotifWait)GetProcAddress( hInstLib, "PlxPci_NotificationWait");
#endif
    if (!PlxPci_NotificationWait_Rtdl)
    {
      fputs( "Function PlxPci_NotificationWait not found", stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_NofificationCancel_Rtdl = (PlxPciNotifCancel)dlsym( handle, "PlxPci_NotificationCancel");
#else
    PlxPci_NofificationCancel_Rtdl = (PlxPciNotifCancel)GetProcAddress( hInstLib, "PlxPci_NotificationCancel");
#endif
    if (!PlxPci_NofificationCancel_Rtdl)
    {
      fputs( "Function PlxPci_NotificationCancel not found", stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_PciRegisterReadFast_Rtdl = (PlxPciRegReadFast)dlsym( handle, "PlxPci_PciRegisterReadFast");
#else
    PlxPci_PciRegisterReadFast_Rtdl = (PlxPciRegReadFast)GetProcAddress( hInstLib, "PlxPci_PciRegisterReadFast");
#endif
    if (!PlxPci_PciRegisterReadFast_Rtdl)
    {
      fputs( "Function PlxPci_PciRegisterReadFast not found", stderr);
      exit(1);
    }
  }

  //
  // Dynamically load all PLX version 5.1 - 6.0 functions used by the GscApi dll.
  //

  else if (drvMajorVersion >= 5)
  {
#ifdef PLX_LINUX
    PlxPci_DeviceFind6_Rtdl = (PlxPciDevFind6)dlsym( handle, "PlxPci_DeviceFind");
#else
    PlxPci_DeviceFind6_Rtdl = (PlxPciDevFind6)GetProcAddress( hInstLib, "PlxPci_DeviceFind");
#endif
    if (!PlxPci_DeviceFind6_Rtdl)
    {
      fputs( "Function PlxPci_DeviceFind not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_PciBarSpaceRead6_Rtdl = (PlxPciBarSpaceIO6)dlsym( handle, "PlxPci_PciBarSpaceRead");
#else
    PlxPci_PciBarSpaceRead6_Rtdl = (PlxPciBarSpaceIO6)GetProcAddress( hInstLib, "PlxPci_PciBarSpaceRead");
#endif
    if (!PlxPci_PciBarSpaceRead6_Rtdl)
    {
      fputs( "Function PlxPci_PciBarSpaceRead not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_PciBarSpaceWrite6_Rtdl = (PlxPciBarSpaceIO6)dlsym( handle, "PlxPci_PciBarSpaceWrite");
#else
    PlxPci_PciBarSpaceWrite6_Rtdl = (PlxPciBarSpaceIO6)GetProcAddress( hInstLib, "PlxPci_PciBarSpaceWrite");
#endif
    if (!PlxPci_PciBarSpaceWrite6_Rtdl)
    {
      fputs( "Function PlxPci_PciBarSpaceWrite not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_DmaTransferUserBuffer6_Rtdl = (PlxPciDmaTransfer6)dlsym( handle, "PlxPci_DmaTransferUserBuffer");
#else
    PlxPci_DmaTransferUserBuffer6_Rtdl = (PlxPciDmaTransfer6)GetProcAddress( hInstLib, "PlxPci_DmaTransferUserBuffer");
#endif
    if (!PlxPci_DmaTransferUserBuffer6_Rtdl)
    {
      fputs( "Function PlxPci_DmaTransferUserBuffer not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_DmaChannelClose6_Rtdl = (PlxPciDmaChannelClose6)dlsym( handle, "PlxPci_DmaChannelClose");
#else
    PlxPci_DmaChannelClose6_Rtdl = (PlxPciDmaChannelClose6)GetProcAddress( hInstLib, "PlxPci_DmaChannelClose");
#endif
    if (!PlxPci_DmaChannelClose6_Rtdl)
    {
      fputs( "Function PlxPci_DmaChannelClose not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_DmaChannelOpen6_Rtdl = (PlxPciDmaChannelOpen6)dlsym( handle, "PlxPci_DmaChannelOpen");
#else
    PlxPci_DmaChannelOpen6_Rtdl = (PlxPciDmaChannelOpen6)GetProcAddress( hInstLib, "PlxPci_DmaChannelOpen");
#endif
    if (!PlxPci_DmaChannelOpen6_Rtdl)
    {
      fputs( "Function PlxPci_DmaChannelOpen not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_DmaControl6_Rtdl = (PlxPciDmaControl6)dlsym( handle, "PlxPci_DmaControl");
#else
    PlxPci_DmaControl6_Rtdl = (PlxPciDmaControl6)GetProcAddress( hInstLib, "PlxPci_DmaControl");
#endif
    if (!PlxPci_DmaControl6_Rtdl)
    {
      fputs( "Function PlxPci_DmaControl not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_InterruptDisable6_Rtdl = (PlxPciInterrupt6)dlsym( handle, "PlxPci_InterruptDisable");
#else
    PlxPci_InterruptDisable6_Rtdl = (PlxPciInterrupt6)GetProcAddress( hInstLib, "PlxPci_InterruptDisable");
#endif
    if (!PlxPci_InterruptDisable6_Rtdl)
    {
      fputs( "Function PlxPci_InterruptDisable not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_InterruptEnable6_Rtdl = (PlxPciInterrupt6)dlsym( handle, "PlxPci_InterruptEnable");
#else
    PlxPci_InterruptEnable6_Rtdl = (PlxPciInterrupt6)GetProcAddress( hInstLib, "PlxPci_InterruptEnable");
#endif
    if (!PlxPci_InterruptEnable6_Rtdl)
    {
      fputs( "Function PlxPci_InterruptEnable not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_NotificationRegisterFor6_Rtdl = (PlxPciNotifReg6)dlsym( handle, "PlxPci_NotificationRegisterFor");
#else
    PlxPci_NotificationRegisterFor6_Rtdl = (PlxPciNotifReg6)GetProcAddress( hInstLib, "PlxPci_NotificationRegisterFor");
#endif
    if (!PlxPci_NotificationRegisterFor6_Rtdl)
    {
      fputs( "Function PlxPci_NotificationRegisterFor not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_NotificationWait6_Rtdl = (PlxPciNotifWait6)dlsym( handle, "PlxPci_NotificationWait");
#else
    PlxPci_NotificationWait6_Rtdl = (PlxPciNotifWait6)GetProcAddress( hInstLib, "PlxPci_NotificationWait");
#endif
    if (!PlxPci_NotificationWait6_Rtdl)
    {
      fputs( "Function PlxPci_NotificationWait not found", stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_NofificationCancel6_Rtdl = (PlxPciNotifCancel6)dlsym( handle, "PlxPci_NotificationCancel");
#else
    PlxPci_NofificationCancel6_Rtdl = (PlxPciNotifCancel6)GetProcAddress( hInstLib, "PlxPci_NotificationCancel");
#endif
    if (!PlxPci_NofificationCancel6_Rtdl)
    {
      fputs( "Function PlxPci_NotificationCancel not found", stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPci_PciRegisterReadFast6_Rtdl = (PlxPciRegReadFast6)dlsym( handle, "PlxPci_PciRegisterReadFast");
#else
    PlxPci_PciRegisterReadFast6_Rtdl = (PlxPciRegReadFast6)GetProcAddress( hInstLib, "PlxPci_PciRegisterReadFast");
#endif
    if (!PlxPci_PciRegisterReadFast6_Rtdl)
    {
      fputs( "Function PlxPci_PciRegisterReadFast not found", stderr);
      exit(1);
    }
  }
  else
  {

    //
    // Dynamically load all PLX legacy functions used by the GscApi dll.
    //

#ifdef PLX_LINUX
    PlxPciDeviceFind_Rtdl = (PlxPciDevFindLegacy)dlsym( handle, "PlxPciDeviceFind");
#else
    PlxPciDeviceFind_Rtdl = (PlxPciDevFindLegacy)GetProcAddress( hInstLib, "PlxPciDeviceFind");
#endif
    if (!PlxPciDeviceFind_Rtdl)
    {
      fputs( "Function PlxPciDeviceFind not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxPciDeviceFind3_Rtdl = (PlxPciDevFindLegacy3)dlsym( handle, "PlxPciDeviceFind");
#else
    PlxPciDeviceFind3_Rtdl = (PlxPciDevFindLegacy3)GetProcAddress( hInstLib, "PlxPciDeviceFind");
#endif
    if (!PlxPciDeviceFind3_Rtdl)
    {
      fputs( "Function PlxPciDeviceFind not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxBusIopRead_Rtdl = (PlxBusIopLegacy)dlsym( handle, "PlxBusIopRead");
#else
    PlxBusIopRead_Rtdl = (PlxBusIopLegacy)GetProcAddress( hInstLib, "PlxBusIopRead");
#endif
    if (!PlxBusIopRead_Rtdl)
    {
      fputs( "Function PlxBusIopRead not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxBusIopWrite_Rtdl = (PlxBusIopLegacy)dlsym( handle, "PlxBusIopWrite");
#else
    PlxBusIopWrite_Rtdl = (PlxBusIopLegacy)GetProcAddress( hInstLib, "PlxBusIopWrite");
#endif
    if (!PlxBusIopWrite_Rtdl)
    {
      fputs( "Function PlxBusIopWrite not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxDmaSglTransfer_Rtdl = (PlxDmaSglTransferLegacy)dlsym( handle, "PlxDmaSglTransfer");
#else
    PlxDmaSglTransfer_Rtdl = (PlxDmaSglTransferLegacy)GetProcAddress( hInstLib, "PlxDmaSglTransfer");
#endif
    if (!PlxDmaSglTransfer_Rtdl)
    {
      fputs( "Function PlxDmaSglTransfer not found" , stderr);
      exit(1);
    }

    // for legacy SDK major versions earlier than 4. Function name is the same, but one param type is different
#ifdef PLX_LINUX
    PlxDmaSglTransfer3_Rtdl = (PlxDmaSglTransferLegacy3)dlsym( handle, "PlxDmaSglTransfer");
#else
    PlxDmaSglTransfer3_Rtdl = (PlxDmaSglTransferLegacy3)GetProcAddress( hInstLib, "PlxDmaSglTransfer");
#endif
    if (!PlxDmaSglTransfer3_Rtdl)
    {
      fputs( "Function PlxDmaSglTransfer not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxDmaSglChannelClose_Rtdl = (PlxDmaSglChannelCloseLegacy)dlsym( handle, "PlxDmaSglChannelClose");
#else
    PlxDmaSglChannelClose_Rtdl = (PlxDmaSglChannelCloseLegacy)GetProcAddress( hInstLib, "PlxDmaSglChannelClose");
#endif
    if (!PlxDmaSglChannelClose_Rtdl)
    {
      fputs( "Function PlxDmaSglChannelClose not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxDmaSglChannelOpen_Rtdl = (PlxDmaSglChannelOpenLegacy)dlsym( handle, "PlxDmaSglChannelOpen");
#else
    PlxDmaSglChannelOpen_Rtdl = (PlxDmaSglChannelOpenLegacy)GetProcAddress( hInstLib, "PlxDmaSglChannelOpen");
#endif
    if (!PlxDmaSglChannelOpen_Rtdl)
    {
      fputs( "Function PlxDmaSglChannelOpen not found" , stderr);
      exit(1);
    }
    if (drvMajorVersion == 4)
    {
#ifdef PLX_LINUX
      PlxPciRegisterReadFast_Rtdl = (PlxPciRegReadFastLegacy)dlsym( handle, "PlxPciRegisterReadFast");
#else
      PlxPciRegisterReadFast_Rtdl = (PlxPciRegReadFastLegacy)GetProcAddress( hInstLib, "PlxPciRegisterReadFast");
#endif
      if (!PlxPciRegisterReadFast_Rtdl)
      {
        fputs( "Function PlxPciRegisterReadFast not found", stderr);
        exit(1);
      }
    }

    //
    // For legacy SDK major versions earlier than 4. Function name is the same, but one param type is different
    //

#ifdef PLX_LINUX
    PlxDmaSglChannelOpen3_Rtdl = (PlxDmaSglChannelOpenLegacy3)dlsym( handle, "PlxDmaSglChannelOpen");
#else
    PlxDmaSglChannelOpen3_Rtdl = (PlxDmaSglChannelOpenLegacy3)GetProcAddress( hInstLib, "PlxDmaSglChannelOpen");
#endif
    if (!PlxDmaSglChannelOpen3_Rtdl)
    {
      fputs( "Function PlxDmaSglChannelOpen not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxDmaControl_Rtdl = (PlxDmaControlLegacy)dlsym( handle, "PlxDmaControl");
#else
    PlxDmaControl_Rtdl = (PlxDmaControlLegacy)GetProcAddress( hInstLib, "PlxDmaControl");
#endif
    if (!PlxDmaControl_Rtdl)
    {
      fputs( "Function PlxDmaControl not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxIntrDisable_Rtdl = (PlxInterruptLegacy)dlsym( handle, "PlxIntrDisable");
#else
    PlxIntrDisable_Rtdl = (PlxInterruptLegacy)GetProcAddress( hInstLib, "PlxIntrDisable");
#endif
    if (!PlxIntrDisable_Rtdl)
    {
      fputs( "Function PlxIntrDisable not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxIntrDisable3_Rtdl = (PlxInterruptLegacy3)dlsym( handle, "PlxIntrDisable");
#else
    PlxIntrDisable3_Rtdl = (PlxInterruptLegacy3)GetProcAddress( hInstLib, "PlxIntrDisable");
#endif
    if (!PlxIntrDisable3_Rtdl)
    {
      fputs( "Function PlxIntrDisable not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxIntrEnable_Rtdl = (PlxInterruptLegacy)dlsym( handle, "PlxIntrEnable");
#else
    PlxIntrEnable_Rtdl = (PlxInterruptLegacy)GetProcAddress( hInstLib, "PlxIntrEnable");
#endif
    if (!PlxIntrEnable_Rtdl)
    {
      fputs( "Function PlxIntrEnable not found" , stderr);
      exit(1);
    }
#ifdef PLX_LINUX
    PlxIntrEnable3_Rtdl = (PlxInterruptLegacy3)dlsym( handle, "PlxIntrEnable");
#else
    PlxIntrEnable3_Rtdl = (PlxInterruptLegacy3)GetProcAddress( hInstLib, "PlxIntrEnable");
#endif
    if (!PlxIntrEnable3_Rtdl)
    {
      fputs( "Function PlxIntrEnable not found" , stderr);
      exit(1);
    }

    //
    // dynamically load interrupt-related PLX API functions that differ across versions older than 5.0
    //

    if (drvMajorVersion == 4 && drvMinorVersion == 4)
    {
      // load Plx 4.4 functions
#ifdef PLX_LINUX
      PlxNotificationRegisterFor_Rtdl = (PlxNotifRegLegacy)dlsym( handle, "PlxNotificationRegisterFor");
#else
      PlxNotificationRegisterFor_Rtdl = (PlxNotifRegLegacy)GetProcAddress( hInstLib, "PlxNotificationRegisterFor");
#endif
      if (!PlxNotificationRegisterFor_Rtdl)
      {
        fputs( "Function PlxNotificationRegisterFor not found" , stderr);
        exit(1);
      }
#ifdef PLX_LINUX
      PlxNotificationWait_Rtdl = (PlxNotifWaitLegacy)dlsym( handle, "PlxNotificationWait");
#else
      PlxNotificationWait_Rtdl = (PlxNotifWaitLegacy)GetProcAddress( hInstLib, "PlxNotificationWait");
#endif
      if (!PlxNotificationWait_Rtdl)
      {
        fputs( "Function PlxNotificationWait not found", stderr);
        exit(1);
      }
    }
    else
    {
      // load Plx 4.0, 3.4 functions
#ifdef PLX_LINUX
      PlxIntrAttach_Rtdl = (PlxIntAttLegacy)dlsym( handle, "PlxIntrAttach");
#else
      PlxIntrAttach_Rtdl = (PlxIntAttLegacy)GetProcAddress( hInstLib, "PlxIntrAttach");
#endif
      if (!PlxIntrAttach_Rtdl)
      {
        fputs( "Function PlxIntrAttach not found", stderr);
        exit(1);
      }
      if (drvMajorVersion == 4)
      {
#ifdef PLX_LINUX
        PlxIntrWait_Rtdl = (PlxIntWaitLegacy)dlsym( handle, "PlxIntrWait");
#else
        PlxIntrWait_Rtdl = (PlxIntWaitLegacy)GetProcAddress( hInstLib, "PlxIntrWait");
#endif
        if (!PlxIntrWait_Rtdl)
        {
          fputs( "Function PlxIntrWait not found", stderr);
          exit(1);
        }
      }
    }
  }

  return ApiSuccess;
}

//---------------------------------------------------------- GscFindBoards
//
//  This routine finds and reports all of the GSC SIO4  and analog
// boards in the system. It returns both a count of boards found and
// a structure with the board info in it (assuming the structure
// pointer is not NULL).
//
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscFindBoards( int *boardCount, GSC_DEVICES_STRUCT results[])
{
  int status=0x00, i, j, number;
  int ix = 0;
  int idevdup = 0;
  U16 DeviceTypes[MAX_NUM_OF_DRIVERS] = {0x8000, 0x6000, 0x9050, 0x9030,
                                         0x9080, 0x9054, 0x9056, 0x8311};

#ifdef WIN32
#ifndef _USERDLL
  if (bStaticLibInitialized == FALSE)
    status = gscStaticLibInit();
#endif
#endif

  if ((results != NULL) && sio4BoardHandles[0].hDevice != (PLX_DRIVER_HANDLE)0)
  {
    if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    {
      for(i=0; (sio4BoardHandles[i].hDevice != (PLX_DRIVER_HANDLE)0) && i < 10; i++)
      {
        results[i].busNumber = sio4Boards[i].bus;
        results[i].slotNumber = sio4Boards[i].slot;
        results[i].vendorId = sio4Boards[i].VendorId;
        results[i].deviceId = sio4Boards[i].DeviceId;
        strcpy( results[i].serialNumber, "N/A");
      }
    }
    else
    {
      for(i=0; (sio4BoardHandles6[i].hDevice != (PLX_DRIVER_HANDLE)0) && i < 10; i++)
      {
        results[i].busNumber = sio4Boards6[i].bus;
        results[i].slotNumber = sio4Boards6[i].slot;
        results[i].vendorId = sio4Boards6[i].VendorId;
        results[i].deviceId = sio4Boards6[i].DeviceId;
        strcpy( results[i].serialNumber, "N/A");
      }
    }
    gscBoardCount = *boardCount = i;
    return 0;
  }

  // Assume no boards will be found
  *boardCount = 0;

  // Update the board info for all boards
  for (i=0; i < MAX_NUM_OF_DRIVERS; i++)
  {
    for (j=0,idevdup=0; j < MAXIMUM_NUMBER_OF_BOARDS; j++)
    {
      // SetUp Search Criteria (find all)
      if (drvMajorVersion == 6 && drvMinorVersion >= 1)
      {
        sio4Boards[*boardCount + j - idevdup].bus  = PCI_FIELD_IGNORE;
        sio4Boards[*boardCount + j - idevdup].slot = PCI_FIELD_IGNORE;
        sio4Boards[*boardCount + j - idevdup].function = PCI_FIELD_IGNORE;
        sio4Boards[*boardCount + j - idevdup].VendorId = PCI_FIELD_IGNORE;
        sio4Boards[*boardCount + j - idevdup].DeviceId = DeviceTypes[i];
        sio4Boards[*boardCount + j - idevdup].SubVendorId = PCI_FIELD_IGNORE;
        sio4Boards[*boardCount + j - idevdup].SubDeviceId = PCI_FIELD_IGNORE;
        sio4Boards[*boardCount + j - idevdup].Revision = PCI_FIELD_IGNORE;
      }
      else if (drvMajorVersion >= 5)
      {
        sio4Boards6[*boardCount + j - idevdup].bus  = PCI_FIELD_IGNORE;
        sio4Boards6[*boardCount + j - idevdup].slot = PCI_FIELD_IGNORE;
        sio4Boards6[*boardCount + j - idevdup].function = PCI_FIELD_IGNORE;
        sio4Boards6[*boardCount + j - idevdup].VendorId = PCI_FIELD_IGNORE;
        sio4Boards6[*boardCount + j - idevdup].DeviceId = DeviceTypes[i];
        sio4Boards6[*boardCount + j - idevdup].SubVendorId = PCI_FIELD_IGNORE;
        sio4Boards6[*boardCount + j - idevdup].SubDeviceId = PCI_FIELD_IGNORE;
        sio4Boards6[*boardCount + j - idevdup].Revision = PCI_FIELD_IGNORE;
      }
      else
      {
        sio4BoardsLegacy[*boardCount + j].BusNumber   = -1;
        sio4BoardsLegacy[*boardCount + j].SlotNumber  = -1;
        sio4BoardsLegacy[*boardCount + j].VendorId    = -1;
        sio4BoardsLegacy[*boardCount + j].DeviceId    = DeviceTypes[i];
        sio4BoardsLegacy3[*boardCount + j].SerialNumber[0] = '\0';
        sio4BoardsLegacy3[*boardCount + j].BusNumber  = -1;
        sio4BoardsLegacy3[*boardCount + j].SlotNumber = -1;
        sio4BoardsLegacy3[*boardCount + j].VendorId    = -1;
        sio4BoardsLegacy3[*boardCount + j].DeviceId    = DeviceTypes[i];
        sio4BoardsLegacy[*boardCount + j].SerialNumber[0] = '\0';
      }

      // Look for the current board
      number = j;

      // Perform the search
      BeginCriticalSection( PlxApiCalls);
      if (drvMajorVersion == 6 && drvMinorVersion >= 1)
        status = PlxPci_DeviceFind_Rtdl(&sio4Boards[*boardCount + j - idevdup], number);
      else if (drvMajorVersion >= 5)
        status = PlxPci_DeviceFind6_Rtdl(&sio4Boards6[*boardCount + j - idevdup], number);
      else if (drvMajorVersion == 4)
        status = PlxPciDeviceFind_Rtdl(&sio4BoardsLegacy[*boardCount + j], (unsigned int *)&number);
      else
        status = PlxPciDeviceFind3_Rtdl(&sio4BoardsLegacy3[*boardCount + j], (unsigned int *)&number);

      EndCriticalSection( PlxApiCalls);
      if (status == ApiSuccess)
      {
        if (results != NULL)
        {
          if (drvMajorVersion == 6 && drvMinorVersion >= 1)
          {
            // Do not add device if already in the list - this check is needed on linux as
            // the driver IOCTL command returns duplicate PLX_DEVICE_KEY structures for each
            // board it knows about. The Plx 5.1 driver does not behave this way on Windows,
            // so as of Plx 5.1 this check is not necesary on that platform.
            // Note: This check is done in the Plx SDK 5.1 samples and was borrowed from there.
            for (ix=0; ix < *boardCount +j; ix++)
              if ((results[ix].busNumber == sio4Boards[*boardCount + j - idevdup].bus) &&
                  (results[ix].slotNumber == sio4Boards[*boardCount + j - idevdup].slot) &&
                  (results[ix].deviceId == sio4Boards[*boardCount + j - idevdup].DeviceId))
                break;
            if (ix == *boardCount + j)
            {
              results[*boardCount + j - idevdup].busNumber = sio4Boards[*boardCount + j - idevdup].bus;
              results[*boardCount + j - idevdup].slotNumber = sio4Boards[*boardCount + j - idevdup].slot;
              results[*boardCount + j - idevdup].vendorId = sio4Boards[*boardCount + j - idevdup].VendorId;
              results[*boardCount + j - idevdup].deviceId = sio4Boards[*boardCount + j - idevdup].DeviceId;
              strcpy( results[*boardCount + j - idevdup].serialNumber, "N/A");
            }
            else
            {
              idevdup++;
            }
          }
          else if (drvMajorVersion >= 5)
          {
            // Do not add device if already in the list - this check is needed on linux as
            // the driver IOCTL command returns duplicate PLX_DEVICE_KEY structures for each
            // board it knows about. The Plx 5.1 driver does not behave this way on Windows,
            // so as of Plx 5.1 this check is not necesary on that platform.
            // Note: This check is done in the Plx SDK 5.1 samples and was borrowed from there.
            for (ix=0; ix < *boardCount +j; ix++)
              if ((results[ix].busNumber == sio4Boards6[*boardCount + j - idevdup].bus) &&
                  (results[ix].slotNumber == sio4Boards6[*boardCount + j - idevdup].slot) &&
                  (results[ix].deviceId == sio4Boards6[*boardCount + j - idevdup].DeviceId))
                break;
            if (ix == *boardCount + j)
            {
              results[*boardCount + j - idevdup].busNumber = sio4Boards6[*boardCount + j - idevdup].bus;
              results[*boardCount + j - idevdup].slotNumber = sio4Boards6[*boardCount + j - idevdup].slot;
              results[*boardCount + j - idevdup].vendorId = sio4Boards6[*boardCount + j - idevdup].VendorId;
              results[*boardCount + j - idevdup].deviceId = sio4Boards6[*boardCount + j - idevdup].DeviceId;
              strcpy( results[*boardCount + j - idevdup].serialNumber, "N/A");
            }
            else
              idevdup++;
          }
          else if (drvMajorVersion == 4)
          {
            results[*boardCount + j].busNumber = sio4BoardsLegacy[*boardCount + j].BusNumber;
            results[*boardCount + j].slotNumber = sio4BoardsLegacy[*boardCount + j].SlotNumber;
            results[*boardCount + j].vendorId = sio4BoardsLegacy[*boardCount + j].VendorId;
            results[*boardCount + j].deviceId = sio4BoardsLegacy[*boardCount + j].DeviceId;
            strcpy( results[*boardCount + j].serialNumber,
                    (char *)sio4BoardsLegacy[*boardCount + j].SerialNumber);
          }
          else
          {
            results[*boardCount + j].busNumber = sio4BoardsLegacy3[*boardCount + j].BusNumber;
            results[*boardCount + j].slotNumber = sio4BoardsLegacy3[*boardCount + j].SlotNumber;
            results[*boardCount + j].vendorId = sio4BoardsLegacy3[*boardCount + j].VendorId;
            results[*boardCount + j].deviceId = sio4BoardsLegacy3[*boardCount + j].DeviceId;
            strcpy( results[*boardCount + j].serialNumber,
                    (char *)sio4BoardsLegacy3[*boardCount + j].SerialNumber);
          }
        }
      }
      else
      {
        *boardCount += j - idevdup;
        break;
      }
    }
  }

  // Set the board count maintained by the library
  gscBoardCount = *boardCount;

  return 0x00;
}


//---------------------------------------------------------------- GscOpen
//
//  This routine is used to reset the entire SIO or analog board.
// For the SIO boards, the Zilog chips, FIFOs, and FPGAs are all set
// to their power on state. This routine also gets a handle for use by
// the PLX driver if one has not already been assigned.
//
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscOpen( int boardNumber, int headerVersion)
{
  int status = 0x00, index;
  int bn = boardNumber-1;

#ifdef PLX_LINUX
  pthread_t  rxDmaThread, txDmaThread;
  pthread_attr_t  dmaThreadAttribute;
// struct sched_param dmaSchedParam;
  key_t msg_key;
  int channel;
  long int id;
#else
  HANDLE hThread, hThread1;
  int dmaThreadParam = (bdRefCount[bn] << 4) + boardNumber;
#endif

#ifdef WIN32
#ifndef _USERDLL
  if (bStaticLibInitialized == FALSE)
  {
    status = gscStaticLibInit();
    GscFindBoards(&status, NULL);
  }
#endif
#endif

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // Check the header version against the DLL version
  if (headerVersion != GSC_API_VERSION) return GscApiDllHeaderMismatch;

  BeginCriticalSection( GscApiCalls );

  // If there is no handle for this board, get one
  if ((drvMajorVersion == 6 && drvMinorVersion >=1 && sio4BoardHandles[bn].hDevice == (PLX_DRIVER_HANDLE)0) ||
      ((drvMajorVersion == 5 || (drvMajorVersion == 6 && drvMinorVersion == 0)) && sio4BoardHandles6[bn].hDevice == (PLX_DRIVER_HANDLE)0) ||
      (drvMajorVersion < 5 && boardInformation[ bn].handle == (HANDLE)0))

  {
    // Open the device
    BeginCriticalSection( PlxApiCalls);
    if (drvMajorVersion == 6 && drvMinorVersion >= 1)
      status = PlxPci_DeviceOpen_Rtdl (&sio4Boards[bn], &sio4BoardHandles[bn]);
    else if (drvMajorVersion >= 5)
      status = PlxPci_DeviceOpen6_Rtdl (&sio4Boards6[bn], &sio4BoardHandles6[bn]);
    else if (drvMajorVersion == 4)
      status = PlxPciDeviceOpen_Rtdl (&sio4BoardsLegacy[bn], &boardInformation[bn].handle);
    else
      status = PlxPciDeviceOpen3_Rtdl (&sio4BoardsLegacy3[bn], &boardInformation[bn].handle);
    EndCriticalSection( PlxApiCalls);
    if (status != ApiSuccess)
    {
      EndCriticalSection( GscApiCalls );
      return status;
    }

  }
  else
    return 0;

  // Determine the board type information
  status = determineBoardType( boardNumber);
  if (status)
  {
    EndCriticalSection( GscApiCalls );
    return status;
  }

  // Reset the board
  if (bdRefCount[bn] == 0)
  {
    resetBoard( boardNumber);

    // If there is a zilog chip, initialize it
    if (boardInformation[ bn].features & GSC_FEATURE_HAS_ZILOG_CHIP)
    {
      // Write to each of the BCR registers
      status = GscSio4ChannelRegisterWrite( boardNumber, 1, 0x0000, 0x0000);
      if (status)
      {
        EndCriticalSection( GscApiCalls );
        return status;
      }

      status = GscSio4ChannelRegisterWrite( boardNumber, 3, 0x0000, 0x0000);
      if (status)
      {
        EndCriticalSection( GscApiCalls );
        return status;
      }
    }
  }

#ifdef PLX_LINUX
  if (bdRefCount[bn] == 0)
  {
    // Get the DMA thread message queue ID
    msg_key = 1234;

    boardInformation[bn].dmaQueueId = msgget(IPC_PRIVATE, 0666 | IPC_CREAT| IPC_EXCL);
    if(boardInformation[bn].dmaQueueId == -1)
      return ApiFailed;

    // Create a queue for the interrupt thread
    boardInformation[bn].intrQueueId = msgget(IPC_PRIVATE, 0666 | IPC_CREAT | IPC_EXCL);
  }

  // Initialize the thread attribute structure
  //dmaSchedParam.sched_priority = 30;
  status = pthread_attr_init(&dmaThreadAttribute);
  //status = pthread_attr_setinheritsched( &dmaThreadAttribute, PTHREAD_EXPLICIT_SCHED );
  //status = sched_setscheduler( 0, SCHED_RR, &dmaSchedParam );
  //status = pthread_attr_setschedpolicy( &dmaThreadAttribute, SCHED_RR );
  //status = pthread_attr_setschedparam( &dmaThreadAttribute, &dmaSchedParam );
  if (status) return ApiFailed;

  // Set the thread to detached
  status = pthread_attr_setdetachstate(&dmaThreadAttribute, PTHREAD_CREATE_DETACHED);
  if (status) return ApiFailed;

  // Start all of the Dma threads
  if (boardInformation[ bn].boardType != GSC_BOARD_TYPE_ANALOG)
  {
    for (channel=1; channel <=NUMBER_OF_CHANNELS; channel++)
    {
      // Start the Tx thread
      if (channel == 1)
      {
        // Determine the channel ID
        id = ((getpid() & 0x03ff) << 20) + ((bdRefCount[bn] + 1) << 12) + (boardNumber<<4) + GSC_THREAD_TX_DMA_TYPE;

        // Start the Tx thread
        status = pthread_create(&txDmaThread, &dmaThreadAttribute, gscSio4TxDmaHandler, (void *)&id);
        if (status) return ApiFailed;
      }

      usleep(50000); // pthreads causes sporadic problems without this delay
      // Start the Rx thread
      if (channel == 1)
      {
        id = ((getpid() & 0x03ff) << 20) + ((bdRefCount[bn] + 1) << 12) + (boardNumber << 4) + GSC_THREAD_RX_DMA_TYPE;
        status = pthread_create(&rxDmaThread, &dmaThreadAttribute, gscSio4RxDmaHandler, (void *)&id);
        if (status) return ApiFailed;
      }
      usleep(50000); // pthreads causes sporadic problems without this delay
    }
  }
  else
  {
    // Start the Rx thread
    id = ((bdRefCount[bn] + 1) << 12) + (boardNumber << 4) + 1 + 0x100;
    status = pthread_create(&rxDmaThread, &dmaThreadAttribute, gscSio4RxDmaHandler, (void *)&id);
    if (status) return ApiFailed;
  }

  // Destroy the attribute object
  (void)pthread_attr_destroy(&dmaThreadAttribute);

#else
  // Generate the abort thread string
  sprintf(threadAbort[bn], "AbortThread%d%d", boardNumber, GetCurrentProcessId() );

  // Start the Rx DMA handler and increase its priority
  dmaThreadParam = ((bdRefCount[bn] + 1) << 4) + boardNumber;
  hThread = CreateThread(0,0,gscSio4RxDmaHandler, (LPVOID)&dmaThreadParam, 0, NULL);
  if (hThread != NULL) SetThreadPriority( hThread, THREAD_PRIORITY_HIGHEST);
  Sleep(250);

  // Start the Tx DMA handler and increase its priority
  if (boardInformation[ bn].boardType != GSC_BOARD_TYPE_ANALOG)
  {
    hThread1 = CreateThread(0,0,gscSio4TxDmaHandler, (LPVOID)&dmaThreadParam, 0, NULL);
    if (hThread1 != NULL) SetThreadPriority( hThread1, THREAD_PRIORITY_HIGHEST);
  }

  // Generate the transfer complete event strings
  sprintf(transferCompleteEvent[bn], "TrComp%d%d", bn, GetCurrentProcessId() );
  sprintf(rxTransferCompleteEvent[bn], "RxTrComp%d%d", bn, GetCurrentProcessId() );

  Sleep(50);
#endif

  // Open both of the DMA channels for this board
  if (bdRefCount[bn] == 0)
  {
    if (drvMajorVersion >= 5)
      status = gscSio4OpenDma( boardNumber, 0);
    else
      status = gscSio4OpenDma( boardNumber, PrimaryPciChannel0);
    if (status)
    {
      EndCriticalSection( GscApiCalls );
      return status;
    }

    if (drvMajorVersion >= 5)
      status = gscSio4OpenDma( boardNumber, 1);
    else
      status = gscSio4OpenDma( boardNumber, PrimaryPciChannel1);
    if (status)
    {
      EndCriticalSection( GscApiCalls );
      return status;
    }
  }

  //
  // Only do this on SIO4/8 boards
  //
  if (boardInformation[ bn].boardType != GSC_BOARD_TYPE_ANALOG)
  {
    // Initialize the interrupt handler
    status = gscSio4InitializeInterrupts( boardNumber);

    if (bdRefCount[bn] == 0)
    {
    // Disable all interrupts
    status = fpgaWrite(boardNumber, INTERRUPT_CONTROL_REG, 0x0000);
    if (status)
    {
      EndCriticalSection( GscApiCalls );
      return status;
    }

    // Clear all pending interrupts
    status = fpgaWrite(boardNumber, INTERRUPT_STATUS_REG, 0xffff);
    if (status)
    {
      EndCriticalSection( GscApiCalls );
      return status;
    }

    // Initialize the channel modes. Also initialize the rxComplete channel list for the current process
    for (index=0; index<NUMBER_OF_CHANNELS; index++)
    {
      boardInformation[ boardNumber-1].channelMode[ index] = GSC_MODE_UNINITIALIZED;
      rxCompleteChannels[boardNumber - 1][index] = FALSE;
    }
    }
  }
  else
  {
    // create AnalogBoard-derived object
    gscCreateBoardObj( boardNumber );

    // Initialize interrupt handler for analog boards.
    status = gscAnaInitializeInterrupts( boardNumber );

    // This sleep alleviates a rare, harmful race condition effect that occurs in the PlxApi
    // library. So far it has been encountered only on linux Debian 2.6.25 amd64.
    Sleep(50);

    // Disable all interrupts - ??
    //status = fpgaWrite( boardNumber, GSC_ANALOG_ICR, 0x0 );
    //if (status) return status;

    // Clear all pending interrupts
    status = gscAnaClearInterrupts( boardNumber );
  }

  //      // Determine the clock chip configuration
  //      status = fpgaRead( boardNumber, FEATURES_REG, &index);
  //      if (status) return status;
  //      if ((index & 0x000f) == 0x04) boardInformation[ boardNumber-1].dualClockChip = TRUE;
  //      else boardInformation[ boardNumber-1].dualClockChip = FALSE;

  // Return the status
  bdRefCount[bn]++;
  processBoardRef[boardNumber - 1] = bdRefCount[bn];

  EndCriticalSection( GscApiCalls );

  if (status != ApiSuccess) return status;
  return 0;
}

//---------------------------------------------------------------- GscClose
//
int GscClose( int boardNumber)
{
  int status = 0x00;
#ifdef PLX_LINUX
  THREAD_MESSAGE message;
  int i;
#else
  HANDLE hEvent;
#endif

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  BeginCriticalSection( GscApiCalls );

  // If there is no handle for this board, it is already closed so just return
  if ((drvMajorVersion == 6 && drvMinorVersion >=1 &&sio4BoardHandles[boardNumber - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      ((drvMajorVersion == 5 || (drvMajorVersion == 6 && drvMinorVersion == 0)) && sio4BoardHandles6[boardNumber - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      (drvMajorVersion < 5 && boardInformation[boardNumber - 1].handle == (HANDLE)0))
  {
    EndCriticalSection( GscApiCalls );
    return 0;
  }
#ifdef PLX_LINUX

  // Load the thread abort command
  message.command = GSC_THREAD_ABORT_CMD;

  // Signal all of the DMA threads to shut down
  for (i=1; i<=4; i++)
  {
    // Send message to the Tx threads
    message.type = ((getpid() & 0x03ff) << 20) + (processBoardRef[boardNumber - 1] << 12) + GSC_THREAD_TX_DMA_TYPE + (boardNumber << 4) + i;
    if(msgsnd(boardInformation[boardNumber - 1].dmaQueueId,(void*)&message,sizeof(int),0)==-1)
      return ApiFailed;

    // Send message to the Rx thread
    if (i == 1)
    {
      message.type = ((getpid() & 0x03ff) << 20) + (processBoardRef[boardNumber - 1] << 12) + GSC_THREAD_RX_DMA_TYPE + (boardNumber << 4);
      if(msgsnd(boardInformation[boardNumber - 1].dmaQueueId,(void*)&message,sizeof(int),0)==-1)
        return ApiFailed;
    }
  }

  // Give the threads time to exit
  usleep(100000);

  // decrement board reference count
  if (bdRefCount[boardNumber - 1] > 0)
    (bdRefCount[boardNumber - 1])--;

  if (bdRefCount[boardNumber - 1] == 0)
  {
    // Signal the Completion Interrupt thread to shut down
    message.type = GSC_THREAD_COMPLETION_TYPE;
    if (msgsnd(boardInformation[boardNumber - 1].dmaQueueId, (void*)&message,sizeof(int),0)==-1)
      return ApiFailed;

    // Signal the Rx Completion Interrupt thread to shut down
    message.type = GSC_THREAD_RX_COMPLETION_TYPE;
    if (msgsnd(boardInformation[boardNumber - 1].dmaQueueId,(void*)&message,sizeof(int),0)==-1)
      return ApiFailed;

    // Signal the FPGA Interrupt thread to shut down
    message.type = GSC_THREAD_FPGA_INTR_TYPE;
    if (msgsnd(boardInformation[boardNumber - 1].intrQueueId,(void*)&message,sizeof(int),0)==-1)
    {
      EndCriticalSection( GscApiCalls );
      return ApiFailed;
    }

    // Delete the dma thread queue
    if (msgctl(boardInformation[boardNumber - 1].dmaQueueId, IPC_RMID, 0) == -1)
      return ApiFailed;

    // Delete the interrupt thread message queue
    if (msgctl(boardInformation[boardNumber - 1].intrQueueId, IPC_RMID, 0) == -1)
      return ApiFailed;
  }

#else

  // decrement board reference count
  if (bdRefCount[boardNumber - 1] > 0)
  {
    (bdRefCount[boardNumber - 1])--;
  }

  // Signal the threads to shut down
  hEvent = CreateEvent( NULL, TRUE, FALSE, threadAbort[ boardNumber-1]);
  SetEvent ( hEvent );
  CloseHandle ( hEvent );

#endif

  // Close both of the DMA channels for this board
  if (bdRefCount[boardNumber - 1] == 0)
  {
    if (drvMajorVersion >= 5)
      status = gscSio4CloseDma( boardNumber, 0);
    else
      status = gscSio4CloseDma( boardNumber, PrimaryPciChannel0);
    //if (status) return status;

    if (drvMajorVersion >= 5)
      status = gscSio4CloseDma( boardNumber, 1);
    else
      status = gscSio4CloseDma( boardNumber, PrimaryPciChannel1);
    //if (status) return status;
    //
  }

  // Close the device
  BeginCriticalSection( PlxApiCalls);
  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    status = PlxPci_DeviceClose_Rtdl( &sio4BoardHandles[boardNumber - 1]);
  else if (drvMajorVersion >= 5)
    status = PlxPci_DeviceClose6_Rtdl( &sio4BoardHandles6[ boardNumber-1]);
  else
    status = PlxPciDeviceClose_Rtdl( boardInformation[ boardNumber-1].handle);

  EndCriticalSection( PlxApiCalls);
  //if ( status != ApiSuccess) return status;

  // Clear the handle
  boardInformation[ boardNumber-1].handle = (HANDLE)0;
  sio4BoardHandles[ boardNumber-1].hDevice = (PLX_DRIVER_HANDLE)0;
  sio4BoardHandles6[ boardNumber-1].hDevice = (PLX_DRIVER_HANDLE)0;
  sio4BoardHandlesLegacy[ boardNumber-1] = (HANDLE)0;

  EndCriticalSection( GscApiCalls );

  if ( status != ApiSuccess) return status;
  return 0;
}


//--------------------------------------------------- GscLocalRegisterRead
//
//  This routine is used to read a local register (FPGA)
// on the GSC SIO and analog boards.
//
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscLocalRegisterRead( int boardNumber, int reg, int *result)
{
  int status = 0x00;

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // If there is no handle, get one
  if ((drvMajorVersion == 6 && drvMinorVersion >=1 &&sio4BoardHandles[boardNumber - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      ((drvMajorVersion == 5 || (drvMajorVersion == 6 && drvMinorVersion == 0)) && sio4BoardHandles6[boardNumber - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      (drvMajorVersion < 5 && boardInformation[boardNumber - 1].handle == (HANDLE)0))
  {
    status = GscOpen( boardNumber, GSC_API_VERSION);
    if (status) return status;
  }

  // Perform the read
  if (boardInformation[boardNumber - 1].boardType == GSC_BOARD_TYPE_ANALOG)
    return GscAnaLocalRegisterRead( boardNumber, reg, result);
  else
    return fpgaRead( boardNumber, reg, result);
}

//-------------------------------------------------- GscLocalRegisterWrite
//
//  This routine is used to write to a local register (FPGA)
// on the GSC SIO and analog boards.
//
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscLocalRegisterWrite(int boardNumber, int reg, int value)
{
  int status;

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // If there is no handle, get one
  if ((drvMajorVersion == 6 && drvMinorVersion >=1 &&sio4BoardHandles[boardNumber - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      ((drvMajorVersion == 5 || (drvMajorVersion == 6 && drvMinorVersion == 0)) && sio4BoardHandles6[boardNumber - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      (drvMajorVersion < 5 && boardInformation[boardNumber - 1].handle == (HANDLE)0))
  {
    status = GscOpen( boardNumber, GSC_API_VERSION);
    if (status) return status;
  }

  // Perform the write
  if (boardInformation[boardNumber - 1].boardType == GSC_BOARD_TYPE_ANALOG)
    return GscAnaLocalRegisterWrite( boardNumber, reg, value);
  else
    return fpgaWrite( boardNumber, reg, value);
}


//--------------------------------------------------- GscLocalRegisterRead16
//
//  This routine is used to read a local register (FPGA)
// on the GSC SIO and analog boards.
//
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscLocalRegisterRead16( int boardNumber, int reg, unsigned short *result)
{
  int status = 0x00;

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // If there is no handle, get one
  if ((drvMajorVersion == 6 && drvMinorVersion >=1 && sio4BoardHandles[boardNumber - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      ((drvMajorVersion == 5 || (drvMajorVersion == 6 && drvMinorVersion == 0)) && sio4BoardHandles6[boardNumber - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      (drvMajorVersion < 5 && boardInformation[boardNumber - 1].handle == (HANDLE)0))
  {
    status = GscOpen( boardNumber, GSC_API_VERSION);
    if (status) return status;
  }

  // Perform the read
  return fpgaRead16( boardNumber, reg, result);
}

//-------------------------------------------------- GscLocalRegisterWrite16
//
//  This routine is used to write to a local register (FPGA)
// on the GSC SIO and analog boards.
//
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscLocalRegisterWrite16(int boardNumber, int reg, unsigned short value)
{
  int status;

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // If there is no handle, get one
  if ((drvMajorVersion == 6 && drvMinorVersion >=1 &&sio4BoardHandles[boardNumber - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      ((drvMajorVersion == 5 || (drvMajorVersion == 6 && drvMinorVersion == 0)) && sio4BoardHandles6[boardNumber - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      (drvMajorVersion < 5 && boardInformation[boardNumber - 1].handle == (HANDLE)0))
  {
    status = GscOpen( boardNumber, GSC_API_VERSION);
    if (status) return status;
  }

  // Perform the write
  return fpgaWrite16( boardNumber, reg, value);
}


//------------------------------------------------- GscSio4ChannelRegisterRead
//
//  This routine is used to read a USC register
// on the SIO4 board.
//
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscSio4ChannelRegisterRead(int boardNumber, int channel, int reg, int *value)
{
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // If there is no handle, get one
  if ((drvMajorVersion == 6 && drvMinorVersion >=1 &&sio4BoardHandles[boardNumber - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      ((drvMajorVersion == 5 || (drvMajorVersion == 6 && drvMinorVersion == 0)) && sio4BoardHandles6[boardNumber- 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      (drvMajorVersion < 5 && boardInformation[boardNumber - 1].handle == (HANDLE)0))
  {
    status = GscOpen( boardNumber, GSC_API_VERSION);
    if (status) return status;
  }

  // Read the register
  status = zilogRead( boardNumber, channel, reg, value);
  return status;
}

//------------------------------------------------ GscSio4ChannelRegisterWrite
//
//  This routine is used to write to a USC register
// on the SIO4 board.
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscSio4ChannelRegisterWrite(int boardNumber, int channel, int reg, int value)
{
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // If there is no handle, get one
  if ((drvMajorVersion == 6 && drvMinorVersion >=1 &&sio4BoardHandles[boardNumber - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      ((drvMajorVersion == 5 || (drvMajorVersion == 6 && drvMinorVersion == 0)) && sio4BoardHandles6[boardNumber - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      (drvMajorVersion < 5 && boardInformation[boardNumber - 1].handle == (HANDLE)0))
  {
    status = GscOpen( boardNumber, GSC_API_VERSION);
    if (status) return status;
  }

  // Write the register
  status = zilogWrite( boardNumber, channel, reg, value);
  return status;
}

//--------------------------------------------------------- GscGetVersions
//
//  This routine gets the 32 bit value of the firmware revision
// register and places into the variable passed into the routine.
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscGetVersions( int boardNumber, int *libVersion, int *driverVersion, int *fpgaVersion)
{
  int status = 0x00;
  U8 major=0, minor=0, revision=0;

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // Get the library version (if requested)
  if (libVersion != NULL) *libVersion = GSC_API_VERSION;

  // Get the driver version (if requested)
  if (driverVersion != NULL)
  {
    // Make sure the board is open
    if ((drvMajorVersion == 6 && drvMinorVersion >=1 &&sio4BoardHandles[boardNumber - 1].hDevice != (PLX_DRIVER_HANDLE)0) ||
        ((drvMajorVersion == 5 || (drvMajorVersion == 6 && drvMinorVersion == 0)) && sio4BoardHandles6[boardNumber - 1].hDevice != (PLX_DRIVER_HANDLE)0) ||
        (drvMajorVersion < 5 && boardInformation[boardNumber - 1].handle != (HANDLE)0))
    {
      // Read the info from the driver
      BeginCriticalSection( PlxApiCalls);
      if (drvMajorVersion == 6 && drvMinorVersion >= 1)
        status = PlxPci_DriverVersion_Rtdl( &sio4BoardHandles[boardNumber-1], &major, &minor, &revision );
      else if (drvMajorVersion >= 5)
        status = PlxPci_DriverVersion6_Rtdl( &sio4BoardHandles6[boardNumber-1], &major, &minor, &revision );
      else
        status = PlxDriverVersion_Rtdl(boardInformation[boardNumber-1].handle, &major, &minor, &revision);
      EndCriticalSection( PlxApiCalls);
      if (status != ApiSuccess) return status;

      // Return it
      *driverVersion = (major<<16) + (minor<<8) + revision;
    }
  }


  // Read the version register (if requested)
  if (fpgaVersion != NULL) status = fpgaRead( boardNumber, FW_REVISION_REG, fpgaVersion);

  // Return the status of the operation
  return status;
}


//------------------------------------------------------------- GscGetInfo
//
//  This routine returns the general information about the
// selected board. The data is returned in a board info
// structure.
//
int GscGetInfo( int boardNumber, PBOARD_INFO info)
{
  int status = 0x00;
  U8 major=0, minor=0, revision=0;
  //      int value;
  int bn = boardNumber - 1;

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // Get the library version
  sprintf(info->apiVersion, "%d.%02d.%02d", (GSC_API_VERSION>>16)&0xff, (GSC_API_VERSION>>8)&0xff, GSC_API_VERSION&0xff);

  // Make sure the board is open
  if ((drvMajorVersion == 6 && drvMinorVersion >=1 &&sio4BoardHandles[bn].hDevice != (PLX_DRIVER_HANDLE)0) ||
      ((drvMajorVersion == 5 || (drvMajorVersion == 6 && drvMinorVersion == 0)) && sio4BoardHandles6[bn].hDevice != (PLX_DRIVER_HANDLE)0) ||
      (drvMajorVersion < 5 && boardInformation[ bn].handle != (HANDLE)0))
  {
    // Read the driver info from the driver
    BeginCriticalSection( PlxApiCalls);
    if (drvMajorVersion == 6 && drvMinorVersion >= 1)
      status = PlxPci_DriverVersion_Rtdl( &sio4BoardHandles[bn], &major, &minor, &revision );
    else if (drvMajorVersion >= 5)
      status = PlxPci_DriverVersion6_Rtdl( &sio4BoardHandles6[bn], &major, &minor, &revision );
    else
      status = PlxDriverVersion_Rtdl(boardInformation[bn].handle, &major, &minor, &revision);
    EndCriticalSection( PlxApiCalls);
    if (status != ApiSuccess) return status;

    // Return it
    sprintf(info->driverVersion, "%d.%02d.%02d", major, minor, revision);
  }

  //      // Read the version register
  //      status = fpgaRead( boardNumber, FW_REVISION_REG, &value);
  //      if (status) return status;
  //
  // Return the FPGA version
  //      sprintf(info->fpgaVersion, "%08X", value);
  strcpy(info->fpgaVersion, boardInformation[ bn].fpgaVersionText);

  //      // Check for invalid board information
  //      if (boardInformation[bn].formFactor[0] == '?') strcpy(info->boardType, "Unknown");
  //
  //      // Otherwise, generate a description
  //      else
  //      {
  //              // Get the form factor
  //              strcpy(info->boardType, boardInformation[bn].formFactor);
  //
  //              // Append the sub type
  //              strcat(info->boardType, boardInformation[bn].subId);
  //
  //              // Append the board version
  //              strcat(info->boardType, boardInformation[bn].version);
  //      }

  // Return the board description
  strcpy(info->boardType, boardInformation[ bn].boardDescription);

  // Return the status of the operation
  if (status != ApiSuccess)
    return status;
  else
    return 0;
}


//-------------------------------------------------------- GscAllocPhysicalMemory
//
//  This allocates a buffer of physical (logical, dma-able) memory.
//
int GscAllocPhysicalMemory( int boardNumber, PLX_PHYSICAL_MEM *pciMem )
{
  int status = 0;

  status = PlxPci_PhysicalMemoryAllocate_Rtdl(&sio4BoardHandles[ boardNumber-1], pciMem, FALSE);

  // Return if an error occurred
  if(status != ApiSuccess) return status;
  return 0;
}


//-------------------------------------------------------- GscMapPhysicalMemory
//
//  This allocates a buffer of physical (logical, dma-able) memory.
//
int GscMapPhysicalMemory( int boardNumber, PLX_PHYSICAL_MEM *pciMem )
{
  int status = 0;

  status = PlxPci_PhysicalMemoryMap_Rtdl(&sio4BoardHandles[ boardNumber-1], pciMem);

  // Return if an error occurred
  if(status != ApiSuccess) return status;
  return 0;
}


//-------------------------------------------------------- GscUnmapPhysicalMemory
//
//  This allocates a buffer of physical (logical, dma-able) memory.
//
int GscUnmapPhysicalMemory( int boardNumber, PLX_PHYSICAL_MEM *pciMem )
{
  int status = 0;

  status = PlxPci_PhysicalMemoryUnmap_Rtdl(&sio4BoardHandles[ boardNumber-1], pciMem );

  // Return if an error occurred
  if(status != ApiSuccess) return status;
  return 0;
}


//-------------------------------------------------------- GscSio4ChannelReset
//
//  This routine resets a single channel on the SIO4 board.
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscSio4ChannelReset(int boardNumber, int channel)
{
  int offset, status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  BeginCriticalSection( GscApiCalls );

  // Reset the board info structure copy of the mode info and packet framing state
  boardInformation[ boardNumber-1].channelMode[channel-1] = GSC_MODE_UNINITIALIZED;
        boardInformation[ boardNumber-1].channelPacketFraming[ channel-1] = GSC_DISABLED;

  // Only write to the USC on non-sync boards
  if ( boardInformation[ boardNumber-1].boardType == GSC_BOARD_TYPE_NORMAL)
  {
    // Reset the channel by Setting the reset bit in the CCAR register
    status = zilogWrite(boardNumber, channel, USC_CCAR, 0x0400);
    if (status) return status;
    Sleep(10);

    // Finish the reset by Clearing the reset bit in the CCAR register
    status = zilogWrite(boardNumber, channel, USC_CCAR, 0x0000);
    if (status) return status;

    // Set the RxREQ and TxReq pins to outputs
    // Set RxC, TxC as inputs and TxD as an output
    status = zilogWrite(boardNumber, channel, USC_IOCR, 0x0500);
    if (status) return status;

    // Setup RxAck and TxAck pins and disable BRG0/1
    status = zilogWrite(boardNumber, channel, USC_HCR, 0x0044);
    if (status) return status;
  }

  // Set the Tx almost values to their default
  status = GscSio4ChannelSetTxAlmost( boardNumber, channel, 0x00100010);
  if (status) return status;

  // Set the Rx almost values to their default
  status = GscSio4ChannelSetRxAlmost( boardNumber, channel, 0x00100010);
  if (status) return status;

  // Clear the sync byte for this channel
  offset = (channel - 1) * 4;
  status = fpgaWrite(boardNumber, SYNC_CHARACTER_BASE_REG + offset, 0x0000);
  if (status) return status;

  EndCriticalSection( GscApiCalls );

  return status;
}

//-------------------------------------------------- GscSio4ChannelResetTxFifo
//
//  This routine resets the Tx fifo for a single channel.
//
int GscSio4ChannelResetTxFifo(int boardNumber, int channel)
{
  int offset;
  int tempValue;
  int status;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Calculate the offset to the proper register
  offset = ((channel-1)<<4);

  // Read the control/status register
  status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + offset, &tempValue);
  if (status) return status;

  // Keep only the transceiver enable info for SIO4B, and bits 30-24 for -SYNC boards
  tempValue &= 0x7f00003c;

  // Reset the FIFO to force the new value to program
  status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG + offset, tempValue | 0x0001);
  Sleep(5);

  // Return the status
  return status;
}

//-------------------------------------------------- GscSio4ChannelResetRxFifo
//
//  This routine resets the Rx fifo for a single channel.
//
int GscSio4ChannelResetRxFifo(int boardNumber, int channel)
{
//      int offset;
//      int tempValue;
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Reset the FIFO and return the result
  return resetRxFifo( boardNumber, channel);

//      // Calculate the offset to the proper register
//      offset = ((channel-1)<<4);
//
//      // Read the control/status register
//      status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + offset, &tempValue);
//      if (status) return status;
//
//      // Keep only the transceiver enable info
//      tempValue &= 0x003c;
//
//      // Reset the FIFO to force the new value to program
//      status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG + offset, tempValue | 0x0002);
//      Sleep(5);
//
//      // Return the status
//      return status;
}


//------------------------------------------------- GscSioGetOption
//
// This routine is used to individually retrieve protocol options for all channels.
//
GSCAPI_API int GscSio4GetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue[] )
{
  int status      = 0x00;
  int offset      = 0;
  int mode        = 0;
  int bitRate     = 0;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Get the mode for calling mode-specific setoption() function if
  // the desired option is not among the common options.
  status = GscSio4ChannelGetMode( boardNumber, channel, &mode, &bitRate );
  if (status)
    mode = GSC_MODE_UNINITIALIZED;


  // Handle options common to all supported protocols on
  // NORMAL board.
  switch ( optionName )
  {
  case GSC_SIO_ENCODING: // Return the current encoding for a channel
    {
      int *encoding = &optionValue[0];

      // Return the encoding
      *encoding = boardInformation[ boardNumber-1].channelEncoding[ channel-1];
    }

    break;

  case GSC_SIO_PROTOCOL:
    {
      int *protocol    = &optionValue[0];
      int *termination = &optionValue[1];

      // Get the values
      *protocol = boardInformation[ boardNumber-1].channelProtocol[ channel-1];
      *termination = boardInformation[ boardNumber-1].channelTermination[ channel-1];
    }

    break;

  case GSC_SIO_PARITY: // Return the current parity setting for a channel
    {
      int *parity = &optionValue[0];

      // Retrieve the current parity setting
      *parity = boardInformation[ boardNumber-1].channelParity[ channel-1];
    }

    break;

  case GSC_SIO_DTEDCE: // Return the current setting of the DTE or DCE mode.
    {
      int *mode = &optionValue[0];

      // Return the info
      *mode = boardInformation[ boardNumber-1].channelDteDce[ channel-1];
    }

    break;

  case GSC_SIO_TXALMOST:
    {
      int *almostValue = &optionValue[0];

      // Calculate the offset to the proper register
      offset = ((channel-1)<<4);

      // Get the current almost value
      status = fpgaRead(boardNumber, TX_ALMOST_BASE_REG + offset, almostValue);
    }

    break;

  case GSC_SIO_RXALMOST:
    {
      int *almostValue = &optionValue[0];

      // Calculate the offset to the proper register
      offset = ((channel-1)<<4);

      // Upper 16 bits = Almost Full, lower 16 bits = Almost empty
      status = fpgaRead(boardNumber, RX_ALMOST_BASE_REG + offset, almostValue);
    }

    break;

  case GSC_SIO_LOOPBACK:
    {
      int *loopMode = &optionValue[0];

      // Return the loopback mode
      *loopMode = boardInformation[ boardNumber-1].channelLoopback[ channel-1];
    }

    break;

  case GSC_SIO_CLOCKSOURCE:
    {
      int *txSource = &optionValue[0];
      int *rxSource = &optionValue[1];

      status = gscsio4ChannelGetClockSource( boardNumber, channel, txSource, rxSource);
    }
    break;

  default:
    // sync board-specific option, so call the sync getoption() function
    if ((boardInformation[boardNumber-1].boardType == GSC_BOARD_TYPE_SYNC) ||
        (gscSio4ChannelIsBxrSync(boardNumber, channel)))
      return gscSio4SyncGetOption( boardNumber, channel, optionName, optionValue );
    // bisync16 board-specific option, so call the bisync16 getoption() function
    else if ( boardInformation[boardNumber-1].boardType == GSC_BOARD_TYPE_BISYNC16 )
      return gscSio4BiSync16GetOption( boardNumber, channel, optionName, optionValue );

    // invoke mode-specific setoption() for NORMAL board
    switch( mode )
    {
    case GSC_MODE_ASYNC:
      return gscSio4AsyncGetOption( boardNumber, channel, optionName, optionValue );
    case GSC_MODE_HDLC:
      return gscSio4HdlcGetOption( boardNumber, channel, optionName, optionValue );
    case GSC_MODE_BISYNC:
      return gscSio4BiSyncGetOption( boardNumber, channel, optionName, optionValue );
    }
    return GscApiInvalidBoardType;
  }

  return status;
}

//------------------------------------------------- GscSio4SetOption
//
// This routine is used to individually set protocol options for all channels.
//
GSCAPI_API int GscSio4SetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue)
{
  int status    = 0x00;
  int tempValue = 0;
  int mode      = 0;
  int bitRate   = 0;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Get the mode for calling mode-specific setoption() function if
  // the desired option is not among the common options.
  status = GscSio4ChannelGetMode( boardNumber, channel, &mode, &bitRate );
  if (status)
    mode = GSC_MODE_UNINITIALIZED;

  // Handle options common to all supported protocols on
  // NORMAL board.

  switch ( optionName )
  {
  case GSC_SIO_PROTOCOL:
    {
      int protocol    = (optionValue & 0xffff0000) >> 16;
      int termination = optionValue & 0x0000ffff;

      // Clear the protocol and termination fields
      tempValue &= ~0x4f000000;

      // Handle the proper protocol
      switch( protocol)
      {
      case GSC_PROTOCOL_RS422_RS485:  tempValue |= 0x00000000; break;
      case GSC_PROTOCOL_RS423:                tempValue |= 0x01000000; break;
      case GSC_PROTOCOL_RS232:                tempValue |= 0x02000000; break;
      case GSC_PROTOCOL_RS530_1:              tempValue |= 0x04000000; break;
      case GSC_PROTOCOL_RS530_2:              tempValue |= 0x05000000; break;
      case GSC_PROTOCOL_V35_1:                tempValue |= 0x06000000; break;
      case GSC_PROTOCOL_V35_2:                tempValue |= 0x07000000; break;
      case GSC_PROTOCOL_RS422_RS423_1:tempValue |= 0x08000000; break;
      case GSC_PROTOCOL_RS422_RS423_2:tempValue |= 0x09000000; break;
      default:                                                return GscApiInvalidParameter;
      }

      // Handle the termination
      switch( termination)
      {
      case GSC_TERMINATION_ENABLED:   tempValue |= 0x00000000; break;
      case GSC_TERMINATION_DISABLED:  tempValue |= 0x40000000; break;
      default:                                                return GscApiInvalidParameter;
      }

      // Update the board info
      boardInformation[ boardNumber-1].channelProtocol[ channel-1] = protocol;
      boardInformation[ boardNumber-1].channelTermination[ channel-1] = termination;

      // Make sure the New (non-legacy) mode is selected
      tempValue |= 0x80000000;

      // Write the new Pin Source Register value
      status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, tempValue);
    }

    break;

  case GSC_SIO_PARITY:
    {
      int parity = optionValue;

      // Make the changes
      status = quickSetParity( boardNumber, channel, parity);
    }

    break;

  case GSC_SIO_DTEDCE:
    {
      int mode = optionValue;

      // Make the changes
      status = quickSetDteDce( boardNumber, channel, mode);
    }

    break;

  case GSC_SIO_TXALMOST:
    {
      int offset;
      int almostValue = optionValue;

      // Calculate the offset to the proper register
      offset = ((channel-1)<<4);

      // Upper 16 bits = Almost Full, lower 16 bits = Almost empty
      status = fpgaWrite(boardNumber, TX_ALMOST_BASE_REG + offset, almostValue);
      if (status) return status;

      // Reset the Fifo
      status = GscSio4ChannelResetTxFifo( boardNumber, channel);
    }

    break;

  case GSC_SIO_RXALMOST:
    {
      int offset;
      int almostValue = optionValue;

      // Calculate the offset to the proper register
      offset = ((channel-1)<<4);

      // Upper 16 bits = Almost Full, lower 16 bits = Almost empty
      status = fpgaWrite(boardNumber, RX_ALMOST_BASE_REG + offset, almostValue);
      if (status) return status;

      // Reset the FIFO
      status = GscSio4ChannelResetRxFifo( boardNumber, channel);
    }

    break;

  case GSC_SIO_LOOPBACK:
    {
      int loopMode = optionValue;

      // Make the change
      status = quickSetLoopBack( boardNumber, channel, loopMode);
    }

    break;

  case GSC_SIO_CLOCKSOURCE:
    {
      int txSource = (optionValue & 0xffff0000) >> 16;
      int rxSource = optionValue & 0x0000ffff;

      status = gscSio4ChannelSetClockSource( boardNumber, channel, txSource, rxSource);
    }
    break;

  default:
    // sync board-specific option, so call the sync setoption() function
    if ((boardInformation[boardNumber-1].boardType == GSC_BOARD_TYPE_SYNC) ||
        (gscSio4ChannelIsBxrSync(boardNumber, channel)))
      return gscSio4SyncSetOption( boardNumber, channel, optionName, optionValue );
    // bisync16 board-specific option, so call the bisync16 setoption() function
    else if ( boardInformation[ boardNumber-1].boardType == GSC_BOARD_TYPE_BISYNC16 )
      return gscSio4BiSync16SetOption( boardNumber, channel, optionName, optionValue );


    // invoke mode-specific setoption() for standard board.
    switch( mode )
    {
    case GSC_MODE_ASYNC:
      return gscSio4AsyncSetOption( boardNumber, channel, optionName, optionValue );
    case GSC_MODE_HDLC:
      return gscSio4HdlcSetOption( boardNumber, channel, optionName, optionValue );
    case GSC_MODE_BISYNC:
      return gscSio4BiSyncSetOption( boardNumber, channel, optionName, optionValue );
    }
    return GscApiInvalidBoardType;
  }

  return status;
}

int GscSio4ChannelGetTxAlmost(int boardNumber, int channel, int *almostValue)
{
  int status = 0x00;
  int offset;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Calculate the offset to the proper register
  offset = ((channel-1)<<4);

  // Get the current almost value
  status = fpgaRead(boardNumber, TX_ALMOST_BASE_REG + offset, almostValue);
  return status;
}

//-------------------------------------------------- GscSio4ChannelSetTxAlmost
//
//  This routine sets the values of the Tx Fifo Almost
// Full and Almost Empty flags for a single channel.
// Once the values are written, the FIFO is reset to
// force the actual programming.
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscSio4ChannelSetTxAlmost(int boardNumber, int channel, int almostValue)
{
  int offset;
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Calculate the offset to the proper register
  offset = ((channel-1)<<4);

  // Upper 16 bits = Almost Full, lower 16 bits = Almost empty
  status = fpgaWrite(boardNumber, TX_ALMOST_BASE_REG + offset, almostValue);
  if (status) return status;

  // Reset the Fifo
  status = GscSio4ChannelResetTxFifo( boardNumber, channel);

  // Return the status
  return status;
}

int GscSio4ChannelGetRxAlmost(int boardNumber, int channel, int *almostValue)
{
  int status = 0x00;
  int offset;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Calculate the offset to the proper register
  offset = ((channel-1)<<4);

  // Upper 16 bits = Almost Full, lower 16 bits = Almost empty
  status = fpgaRead(boardNumber, RX_ALMOST_BASE_REG + offset, almostValue);
  return status;
}


//-------------------------------------------------- GscSio4ChannelSetRxAlmost
//
//  This routine sets the values of the Rx Fifo Almost
// Full and Almost Empty flags for a single channel.
// Once the values are written, the FIFO is reset to
// force the actual programming.
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscSio4ChannelSetRxAlmost(int boardNumber, int channel, int almostValue)
{
  int offset;
  int status;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Calculate the offset to the proper register
  offset = ((channel-1)<<4);

  // Upper 16 bits = Almost Full, lower 16 bits = Almost empty
  status = fpgaWrite(boardNumber, RX_ALMOST_BASE_REG + offset, almostValue);
  if (status) return status;

  // Reset the FIFO
  status = GscSio4ChannelResetRxFifo( boardNumber, channel);

  // Return the status
  return status;
}

//------------------------------------------------------ GscSio4ChannelGetMode
//
//  This routine returns the current mode and bitrate for
// a channel.
//
int GscSio4ChannelGetMode(int boardNumber, int channel, int *mode, int *bitRate)
{
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Return the info
  *mode = boardInformation[ boardNumber-1].channelMode[channel-1];
  *bitRate = boardInformation[ boardNumber-1].channelBitRate[channel-1];

  return status;
}

//------------------------------------------------------ GscSio4ChannelSetMode
//
//  This routine sets the mode and bitrate for a channel.
// It also sets several defaults.
//
int GscSio4ChannelSetMode(int boardNumber, int channel, int mode, int bitRate)
{
  int status = 0x00;
  int value = 0;
  int channelMode;
  GSC_HDLC_CONFIG hdlcConfig;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Handle the non-standard board types
  if ((boardInformation[ boardNumber-1].boardType == GSC_BOARD_TYPE_SYNC) ||
      ((!strcmp(boardInformation[ boardNumber-1].subId, "-SIO4BXR")) && (mode = GSC_MODE_SYNC)))
  {
    // If the board is a BXR, change the channel to SYNC mode
    if (boardInformation[boardNumber-1].boardType == GSC_BOARD_TYPE_NORMAL)
    {
      int chMode = 0;

      status = fpgaRead(boardNumber, FW_TYPE_REG, &chMode);
      if (status) return status;

      chMode = chMode & ~(0xff << ((channel - 1) * 8)); // clear channel type
      chMode |= (0x04 << ((channel - 1) * 8)); // set channel type to sync

      status = fpgaWrite(boardNumber, FW_TYPE_REG, chMode);
      if (status) return status;
    }
    // The -Sync board
    return syncBoardSetMode( boardNumber, channel, mode, bitRate);
  }

  // If the board is a BXR, change the channel to standard mode
  if (boardInformation[boardNumber-1].subId == "-SIO4BXR")
  {
    int chMode = 0;

    status = fpgaRead(boardNumber, FW_TYPE_REG, &chMode);
    if (status) return status;

    chMode = chMode & ~(0xff << ((channel - 1) * 8)); // clear channel type
    chMode |= (0x01 << ((channel - 1) * 8)); // set channel type to standard

    status = fpgaWrite(boardNumber, FW_TYPE_REG, chMode);
    if (status) return status;
  }

  // Range check the mode value
  if ((mode <= GSC_MODE_UNINITIALIZED) || (mode >= GSC_MODE_LAST_MODE)) return GscApiInvalidParameter;

  // Save the new info
  boardInformation[ boardNumber-1].channelMode[channel-1] = mode;
  boardInformation[ boardNumber-1].channelBitRate[channel-1] = bitRate;

  switch (mode)
  {
    // Asynchronous Mode
  case GSC_MODE_ASYNC:
    // Set to Async Mode, 16x clock, one stop bit
    status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, 0x0000);
    if (status) return status;

    // Make sure packet framing is disabled.
    boardInformation[ boardNumber-1].channelPacketFraming[ channel-1] = GSC_DISABLED;

          // Get the current Channel Control Register and make sure packet framing is off
          status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &value);
          if (status) return status;

          // Clear the TCB type bits
          value &= ~0xc000;

                // Write the new Channel Control Register value
                status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CCR, value);
                if (status) return status;

    // Save the transfer rate
    boardInformation[ boardNumber-1].bytesPerMs[ channel-1] = (double)bitRate/1000/10;

    // Enable the Tx and Rx data lines in their default locations (DTE mode)
    status = gscSio4ChannelSetDteDce( boardNumber, channel, GSC_PIN_DTE);
    if (status) return status;

    // Set the TxC and RxC cable pins to GPIO since clocks are not used in Async mode
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA,  GSC_PIN_AUTO);
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA,  GSC_PIN_AUTO);
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, GSC_PIN_GPIO);
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, GSC_PIN_GPIO);

    // Set the other pins to their default configurations
    GscSio4ChannelSetPinMode(boardNumber,channel,GSC_PIN_RTS, GSC_PIN_GPIO);
    GscSio4ChannelSetPinMode(boardNumber,channel,GSC_PIN_CTS, GSC_PIN_GPIO);

    // Set the encoding to NRZ as a default (will also set the baud rate)
    status = gscSio4ChannelSetEncoding( boardNumber, channel, GSC_ENCODING_NRZ);
    if (status) return status;

    // Enable the Receiver (no auto enables)
    status = GscSio4ChannelReceiverControl( boardNumber, channel, GSC_ENABLED);
    if (status) return status;

    // Enable the Transmitter (no auto enables)
    status = GscSio4ChannelTransmitterControl( boardNumber, channel, GSC_ENABLED);
    if (status) return status;
    break;

  case GSC_MODE_HDLC:
    // Get the defaults
    status = GscSio4HdlcGetDefaults( &hdlcConfig);
    if (status) return status;

    // Setup the requested Bit Rate
    hdlcConfig.bitRate = bitRate;

    // Program Channel
    status = GscSio4HdlcSetConfig( boardNumber, channel, hdlcConfig);
    if (status) return status;
    break;

  case GSC_MODE_BISYNC:
    // Save the transfer rate
    boardInformation[ boardNumber-1].bytesPerMs[ channel-1] = (double)bitRate/1000/8;

    // Setup the channel with the defaults
    status = gscSio4BiSyncInit( boardNumber, channel);
    if (status) return status;

    // Enable the Tx and Rx data lines in their default locations (DTE mode)
    status = gscSio4ChannelSetDteDce( boardNumber, channel, GSC_PIN_DTE);
    if (status) return status;

    // Set the cable pins to their proper modes
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA,  GSC_PIN_AUTO);
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA,  GSC_PIN_AUTO);
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, GSC_PIN_AUTO);
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, GSC_PIN_AUTO);

    // Set the other pins to their default configurations
    GscSio4ChannelSetPinMode(boardNumber,channel,GSC_PIN_RTS, GSC_PIN_GPIO);
    GscSio4ChannelSetPinMode(boardNumber,channel,GSC_PIN_CTS, GSC_PIN_GPIO);

    // Set the encoding to NRZ as a default (will also set the baud rate)
    status = gscSio4ChannelSetEncoding( boardNumber, channel, GSC_ENCODING_NRZ);
    if (status) return status;

    // Enable the Receiver (no auto enables)
    status = GscSio4ChannelReceiverControl( boardNumber, channel, GSC_ENABLED);
    if (status) return status;

    // Enable the Transmitter (no auto enables)
    status = GscSio4ChannelTransmitterControl( boardNumber, channel, GSC_ENABLED);
    if (status) return status;
    break;

  case GSC_MODE_BISYNC16:
    // Save the transfer rate
    boardInformation[ boardNumber-1].bytesPerMs[ channel-1] = (double)bitRate/1000/8;

    // Setup the channel with the defaults
    status = gscSio4BiSync16Init( boardNumber, channel);
    if (status) return status;

    // Enable the Tx and Rx data lines in their default locations (DTE mode)
    status = gscSio4ChannelSetDteDce( boardNumber, channel, GSC_PIN_DTE);
    if (status) return status;

    // Set the cable pins to their proper modes
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA,  GSC_PIN_AUTO);
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA,  GSC_PIN_AUTO);
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, GSC_PIN_AUTO);
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, GSC_PIN_AUTO);

    // Set the other pins to their default configurations
    GscSio4ChannelSetPinMode(boardNumber,channel,GSC_PIN_RTS, GSC_PIN_GPIO);
    GscSio4ChannelSetPinMode(boardNumber,channel,GSC_PIN_CTS, GSC_PIN_GPIO);

    // Set the encoding to NRZ as a default (will also set the baud rate)
    status = gscSio4ChannelSetEncoding( boardNumber, channel, GSC_ENCODING_NRZ);
    if (status) return status;

    // Enable the Receiver (no auto enables)
    status = GscSio4ChannelReceiverControl( boardNumber, channel, GSC_ENABLED);
    if (status) return status;

    // Enable the Transmitter (no auto enables)
    status = GscSio4ChannelTransmitterControl( boardNumber, channel, GSC_ENABLED);
    if (status) return status;
    break;

  default:
    // Determine the proper channel mode value
    switch (mode)
    {
    case GSC_MODE_EXT_SYNC:         channelMode = 0x0001;   break;
    case GSC_MODE_ISO:                      channelMode = 0x0202;   break;
      //              case GSC_MODE_ASYNC_CV:         channelMode = 0x0303;   break;
    case GSC_MODE_MONOSYNC:         channelMode = 0x0404;   break;
    case GSC_MODE_TRANS_BISYNC:     channelMode = 0x0707;   break;
      //              case GSC_MODE_NBIF:                     channelMode = 0x0808;   break;
    case GSC_MODE_802_3:            channelMode = 0x0909;   break;
    default: return GscApiInvalidParameter;
    }

    // Set to the proper Mode
    status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, channelMode);
    if (status) return status;

    // Save the transfer rate
    boardInformation[ boardNumber-1].bytesPerMs[ channel-1] = (double)bitRate/1000/8;

    // Enable the Tx and Rx data lines in their default locations (DTE mode)
    status = gscSio4ChannelSetDteDce( boardNumber, channel, GSC_PIN_DTE);
    if (status) return status;

    // Set the cable pins to their proper modes
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA,  GSC_PIN_AUTO);
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA,  GSC_PIN_AUTO);
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, GSC_PIN_AUTO);
    GscSio4ChannelSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, GSC_PIN_AUTO);

    // Set the other pins to their default configurations
    GscSio4ChannelSetPinMode(boardNumber,channel,GSC_PIN_RTS, GSC_PIN_GPIO);
    GscSio4ChannelSetPinMode(boardNumber,channel,GSC_PIN_CTS, GSC_PIN_GPIO);

    // Set the encoding to NRZ as a default (will also set the baud rate)
    status = gscSio4ChannelSetEncoding( boardNumber, channel, GSC_ENCODING_NRZ);
    if (status) return status;

    // Enable the Receiver (no auto enables)
    status = GscSio4ChannelReceiverControl( boardNumber, channel, GSC_ENABLED);
    if (status) return status;

    // Enable the Transmitter (no auto enables)
    status = GscSio4ChannelTransmitterControl( boardNumber, channel, GSC_ENABLED);
    if (status) return status;
    break;
  }

  // Return true
  return status;
}

//---------------------------------------------------- quickSetParity
//
int quickSetParity(int boardNumber, int channel, int parity)
{
  int status = 0x00;
  int newValue, oldValue;

  // Handle the proper parity
  switch( parity)
  {
  case GSC_PARITY_NONE:   newValue = 0x0000; break;
  case GSC_PARITY_EVEN:   newValue = 0x0020; break;
  case GSC_PARITY_ODD:    newValue = 0x0060; break;
  case GSC_PARITY_MARK:   newValue = 0x00e0; break;
  case GSC_PARITY_SPACE:  newValue = 0x00a0; break;
  default: return GscApiInvalidParameter;
  }

  // Save the new parity setting
  boardInformation[ boardNumber-1].channelParity[ channel-1] = parity;

  // Get current Transmit Mode Register Value
  status = zilogRead( boardNumber, channel, USC_TMR, &oldValue);
  if (status) return status;

  // Update the value and write it back out
  newValue = (oldValue & ~0x00e0) | newValue;
  status = zilogWrite( boardNumber, channel, USC_TMR, newValue);
  if (status) return status;

  // Get current Receiver Mode Register Value
  status = zilogRead( boardNumber, channel, USC_RMR, &oldValue);
  if (status) return status;

  // Update the value and write it back out
  newValue = (oldValue & ~0x00e0) | newValue;
  status = zilogWrite( boardNumber, channel, USC_RMR, newValue);
  return status;
}
//---------------------------------------------------- quickSetDteDce
//
int quickSetDteDce(int boardNumber, int channel, int mode)
{
  int status = 0x00;
  int tempValue;

  // Get the current Pin Source Register value
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, &tempValue);
  if (status) return status;

  // Handle the proper encoding
  switch( mode)
  {
  case GSC_PIN_DTE:       tempValue &= ~0x10000000;       break;
  case GSC_PIN_DCE:       tempValue |=  0x10000000;       break;
  default:                        return GscApiInvalidParameter;
  }

  // Update the board info
  boardInformation[ boardNumber-1].channelDteDce[ channel-1] = mode;

  // Make sure the New (non-legacy) mode is selected
  tempValue |= 0x80000000;

  // Write the new Pin Source Register value
  status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, tempValue);
  return status;
}


//-------------------------------------------------- GscSio4ChannelSetProtocol
//
int quickSetProtocol(int boardNumber, int channel, int protocol, int termination)
{
  int status = 0x00;
  int tempValue;

  // Get the current Pin Source Register value
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, &tempValue);
  if (status) return status;

  // Clear the protocol and termination fields
  tempValue &= ~0x4f000000;

  // Handle the proper protocol
  switch( protocol)
  {
  case GSC_PROTOCOL_RS422_RS485:  tempValue |= 0x00000000; break;
  case GSC_PROTOCOL_RS423:                tempValue |= 0x01000000; break;
  case GSC_PROTOCOL_RS232:                tempValue |= 0x02000000; break;
  case GSC_PROTOCOL_RS530_1:              tempValue |= 0x04000000; break;
  case GSC_PROTOCOL_RS530_2:              tempValue |= 0x05000000; break;
  case GSC_PROTOCOL_V35_1:                tempValue |= 0x06000000; break;
  case GSC_PROTOCOL_V35_2:                tempValue |= 0x07000000; break;
  case GSC_PROTOCOL_RS422_RS423_1:tempValue |= 0x08000000; break;
  case GSC_PROTOCOL_RS422_RS423_2:tempValue |= 0x09000000; break;
  default:                                                return GscApiInvalidParameter;
  }

  // Handle the termination
  switch( termination)
  {
  case GSC_TERMINATION_ENABLED:   tempValue |= 0x00000000; break;
  case GSC_TERMINATION_DISABLED:  tempValue |= 0x40000000; break;
  default:                                                return GscApiInvalidParameter;
  }

  // Update the board info
  boardInformation[ boardNumber-1].channelProtocol[ channel-1] = protocol;
  boardInformation[ boardNumber-1].channelTermination[ channel-1] = termination;

  // Make sure the New (non-legacy) mode is selected
  tempValue |= 0x80000000;

  // Write the new Pin Source Register value
  status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, tempValue);
  return status;
}

//----------------------------------------------- gscSio4ChannelGetMsbLsbOrder
//
int gscSio4ChannelGetMsbLsbOrder(int boardNumber, int channel, int *txOrder, int *rxOrder)
{
  int status = 0x00;
  int oldValue;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Only valid for -Sync boards
  if ((boardInformation[ boardNumber-1].boardType != GSC_BOARD_TYPE_SYNC) &&
      (!gscSio4ChannelIsBxrSync(boardNumber, channel)))
    return GscApiInvalidCombination;

  // Get the current Channel Control/Status Register
  status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+((channel-1)<<4), &oldValue);
  if (status) return status;

  // Get the Tx Order
  if (oldValue & 0x10000000) *txOrder = GSC_LSB_FIRST;
  else *txOrder = GSC_MSB_FIRST;

  // Get the Rx Order
  if (oldValue & 0x08000000) *rxOrder = GSC_LSB_FIRST;
  else *rxOrder = GSC_MSB_FIRST;

  return status;
}


//----------------------------------------------- gscSio4ChannelSetMsbLsbOrder
//
int gscSio4ChannelSetMsbLsbOrder(int boardNumber, int channel, int txOrder, int rxOrder)
{
  int status = 0x00;
  int oldValue;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Only valid for -Sync boards
        if ((boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_SYNC) &&
      (!gscSio4ChannelIsBxrSync(boardNumber, channel)))
    return GscApiInvalidCombination;

  // Get the current Channel Control/Status Register
  status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+((channel-1)<<4), &oldValue);
  if (status) return status;

  // Process the Tx Order
  switch(txOrder)
  {
  case GSC_MSB_FIRST: oldValue &= ~0x10000000; break;
  case GSC_LSB_FIRST: oldValue |=  0x10000000; break;
  default:                        return GscApiInvalidParameter;
  }

  // Process the Rx Source
  switch(rxOrder)
  {
  case GSC_MSB_FIRST: oldValue &= ~0x08000000; break;
  case GSC_LSB_FIRST: oldValue |=  0x08000000; break;
  default:                        return GscApiInvalidParameter;
  }

  // Write it back out
  status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG+((channel-1)<<4), oldValue);

  return status;
}

int gscSio4ChannelSetEncoding(int boardNumber, int channel, int encoding)
{
  int status = 0x00;

  // Convert board and channel into indexes
  int bn = boardNumber - 1;
  int ch = channel - 1;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Handle the Zilog based boards
  if (boardInformation[ bn].boardType == GSC_BOARD_TYPE_NORMAL)
  {
    status = zilogSetEncoding( boardNumber,
      channel,
      encoding,
      boardInformation[ bn].channelMode[ ch],
      boardInformation[ bn].channelBitRate[ ch]);
    return status;
  }

  // Handle the other boards
  return fpgaSetEncoding( boardNumber,
    channel,
    encoding,
    boardInformation[ bn].channelBitRate[ ch]);
}

//-------------------------------------------------- gscSio4ChannelGetProtocol
//
int gscSio4ChannelGetProtocol(int boardNumber, int channel, int *protocol, int *termination)
{
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Get the values
  *protocol = boardInformation[ boardNumber-1].channelProtocol[ channel-1];
  *termination = boardInformation[ boardNumber-1].channelTermination[ channel-1];

  return status;
}

//---------------------------------------------------- gscSio4ChannelGetParity
//
//  This routine returns the current parity setting
// for a channel.
//
int gscSio4ChannelGetParity(int boardNumber, int channel, int *parity)
{
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Save the new parity setting
  *parity = boardInformation[ boardNumber-1].channelParity[ channel-1];

  return status;
}

//-------------------------------------------------- gscSio4ChannelSetStopBits
//
int gscSio4ChannelSetStopBits(int boardNumber, int channel, int stopBits)
{
  int status = 0x00;
  int newValue, newValue2, oldValue;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Handle the proper stop bits setting
  switch( stopBits)
  {
  case GSC_STOP_BITS_0:   newValue = 0x8000; newValue2 = 0x0000; break;
  case GSC_STOP_BITS_1:   newValue = 0x0000; newValue2 = 0x0000; break;
  case GSC_STOP_BITS_1_5: newValue = 0xc000; newValue2 = 0x0700; break;
  case GSC_STOP_BITS_2:   newValue = 0x4000; newValue2 = 0x0000; break;
  default: return GscApiInvalidParameter;
  }

  // Save the Stop Bit value
  boardInformation[ boardNumber-1].channelStopBits[ channel-1] = stopBits;

  // Get current Channel Mode Register Value
  status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &oldValue);
  if (status) return status;

  // Update the value and write it back out
  newValue = (oldValue & ~0xc000) | newValue;
  status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, newValue);
  if (status) return status;

  // Get current Channel Control Register Value
  status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &oldValue);
  if (status) return status;

  // Update the value and write it back out
  newValue = (oldValue & ~0x0f00) | newValue2;
  status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CCR, newValue);
  if (status) return status;

  return status;
}

//---------------------------------------------------- gscSio4ChannelSetDteDce
//
//  This routine sets the current setting of the DTE
// or DCE mode.
//
int gscSio4ChannelSetDteDce(int boardNumber, int channel, int mode)
{
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Make the changes
  status = quickSetDteDce( boardNumber, channel, mode);

  return status;
}

int gscSio4ChannelSetClockSource( int boardNumber, int channel, int txSource, int rxSource)
{
  int status=0x00;
  int pinSource;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Handle the -Sync board separately
  if ((boardInformation[boardNumber-1].boardType == GSC_BOARD_TYPE_SYNC) ||
      (gscSio4ChannelIsBxrSync(boardNumber, channel)))
  {
    // Read the current pin source register
    status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG + (channel-1)*4, &pinSource);
    if (status) return status;

	// GSC_CLOCK_EXT_RX_AUX is not supported by SYNC boards.

	// Process the Tx Source
    switch(txSource)
    {
    case GSC_CLOCK_INTERNAL: pinSource &= ~0x0002; break;
    case GSC_CLOCK_EXTERNAL: pinSource |= 0x0002; break;
    default:                                 return GscApiInvalidParameter;
    }

    // Ignore the Rx Source

    // Write the new pin source register
    status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG + (channel-1)*4, pinSource);
  }

  // Otherwise, handle the standard board
  else
  {
    status = quickNormalSetClockSource( boardNumber, channel, txSource, rxSource);
  }

  return status;
}


//---------------------------------------------------- gscSio4ChannelSetFifoTimetagByte
//
//  This routine sets the current setting of the 24-bit FIFO timetag feature.
//
int gscSio4ChannelSetFifoTimetagByte(int boardNumber, int channel, int fifoTimetagSetting)
{
  int status = 0x00;
  int regValue;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Make the changes
  status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), &regValue);
  if (status) return status;

  if (fifoTimetagSetting == GSC_ENABLED)
    regValue |= 0x04;
  else
    regValue &= ~0x04;

  status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), regValue);

  return status;
}


//---------------------------------------------------- gscSio4ChannelSetFifoParityErrorByte
//
//  This routine sets the current setting of the 8-bit FIFO parity error feature.
//
int gscSio4ChannelSetFifoParityErrorByte(int boardNumber, int channel, int fifoParitySetting)
{
  int status = 0x00;
  int regValue;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Make the changes
  status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), &regValue);
  if (status) return status;

  if (fifoParitySetting == GSC_ENABLED)
    regValue |= 0x08;
  else
    regValue &= ~0x08;

  status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), regValue);

  return status;
}


//------------------------------------------------- GscSio4ChannelTransmitData
//
//  This routine adds a transmit request to the DMA
// handler's queue. It may return before the transfer
// has taken place.
//
int GscSio4ChannelTransmitData(int boardNumber, int channel, char *buffer, int count, int *id)
{
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Check buffer alignment
  if ((int)buffer & 0x03)
    return GscApiMisalignedBuffer;

  // Perform the Actual Transmit function
  return gscAddDmaRequest(boardNumber, channel, buffer, count, GSC_DATA_TRANSMIT, id);
}


//------------------------------------------------- GscSio4ChannelTransmitData
//
//  This routine adds a transmit request to the DMA
// handler's queue. It may return before the transfer
// has taken place.
//
int GscSio4ChannelTransmitPlxPhysData(int boardNumber, int channel, PLX_PHYSICAL_MEM *buffer, int count, int *id)
{
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Check buffer alignment
  if ((int)buffer & 0x03)
    return GscApiMisalignedBuffer;

  // Perform the Actual Transmit function
  return gscAddPlxPhysDmaRequest(boardNumber, channel, buffer, count, GSC_DATA_TRANSMIT, id);
}


//------------------------------------------ GscSio4ChannelTransmitDataAndWait
//
//  This routine adds a transmit request to the DMA
// handler's queue and then waits for the transfer to
// complete.
//
int GscSio4ChannelTransmitDataAndWait(int boardNumber, int channel, char *buffer, int count, int timeout, int *bytesTransferred)
{
  int status=0x00, txId;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Check buffer alignment
  if ((int)buffer & 0x03)
    return GscApiMisalignedBuffer;

  // Perform the Transmit function
  status = gscAddDmaRequest(boardNumber, channel, buffer, count, GSC_DATA_TRANSMIT, &txId);
  if (status) return status;

  // Wait for it to complete
  status = GscSio4ChannelWaitForTransfer(boardNumber, channel, timeout, txId, bytesTransferred);
  return status;
}


//------------------------------------------ GscSio4ChannelTransmitPlxPhysDataAndWait
//
//  This routine adds a transmit request using physical memory to the DMA
// handler's queue and then waits for the transfer to
// complete.
//
int GscSio4ChannelTransmitPlxPhysDataAndWait(int boardNumber, int channel, PLX_PHYSICAL_MEM *buffer, int count, int timeout, int *bytesTransferred)
{
  int status=0x00, txId;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Check buffer alignment
  if ((int)buffer & 0x03)
    return GscApiMisalignedBuffer;

  // Perform the Transmit function
  status = gscAddPlxPhysDmaRequest(boardNumber, channel, buffer, count, GSC_DATA_TRANSMIT, &txId);
  if (status) return status;

  // Wait for it to complete
  return GscSio4ChannelWaitForTransfer(boardNumber, channel, timeout, txId, bytesTransferred);
}


int GscSio4ChannelTransmitSinglePacket(int boardNumber, int channel, char *buffer, int count)
{
  int status = 0;
  int value = 0;
  int nAttempts = 0;
  int oldCMR = 0, oldCCSR = 0;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Check buffer alignment
  if ((int)buffer & 0x03)
    return GscApiMisalignedBuffer;

  // Wait for packet enable
  nAttempts = 0;
  do {
    status = fpgaRead(boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), &value);
    Sleep(1);
    nAttempts++;
  } while ((value & 0x2000000) && (nAttempts < 500));

  /*
  if (nAttempts == 1000)
    return GscApiOperationTimeout;
  */

  // Set packet size in sync character register to trigger Zilog transmit
  status = fpgaWrite( boardNumber, SYNC_CHARACTER_BASE_REG + ((channel - 1) * 4), count + 4);

  // Enable single packet
  status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), (value | 0x2000000));

  // Transmit a packet
  BeginCriticalSection( qmutex);

  // Read the CMR and CCSR
  status = GscSio4ChannelRegisterRead(boardNumber, channel, USC_CMR, &oldCMR);
  status = GscSio4ChannelRegisterRead(boardNumber, channel, USC_CCSR, &oldCCSR);

  // Write the TCB to the fifo
  fpgaWrite( boardNumber, DATA_FIFO_BASE_REG + ((channel - 1) * 0x10), oldCCSR & 0x1c );
  fpgaWrite( boardNumber, DATA_FIFO_BASE_REG + ((channel - 1) * 0x10), (oldCMR>>8) & 0xf0 );
  fpgaWrite( boardNumber, DATA_FIFO_BASE_REG + ((channel - 1) * 0x10), count & 0xff );
  fpgaWrite( boardNumber, DATA_FIFO_BASE_REG + ((channel - 1) * 0x10), (count>>8) & 0xff );

  if (drvMajorVersion >= 5)
    status = gscPerformDma( boardNumber, channel, buffer, count, GSC_DATA_TRANSMIT, 1);
  else
    status = gscPerformDma( boardNumber, channel, buffer, count, GSC_DATA_TRANSMIT, PrimaryPciChannel1);

  EndCriticalSection(qmutex);

  return status;
}


//------------------------------------------------- GscSio4ChannelCheckForData
//
//  This routine checks to see if a packet has been received.
// If one has been, A DMA transfer is setup to return the
// data. If not, the routine returns immediately.
//
int GscSio4ChannelReceivePacket(int boardNumber, int channel, char *buffer, int *count, int *transferStatus)
{
  int maxCount = *count;
  int status = 0x00;
  int value = 0;
  int rsbhi = 0;
  int rsblo = 0;
  int fifoCounts;
  int tempCount, isOdd;
  int usingRSB = 0;
  int usingRSB32 = 0;


  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Check buffer alignment
  if ((int)buffer & 0x03)
    return GscApiMisalignedBuffer;

  // Check the RSB enabled/disabled status
  status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &value);
  if (status) return status;
  if (value & 0x40) // using 16 bit RSB
    usingRSB = 1;
  if (value & 0x80) {// using 16 bit RSB
    usingRSB = 1;
    usingRSB32 = 1;
  }
  // Assume no data is available
  *count = 0;
  if (transferStatus != NULL) *transferStatus = 0;

  BeginCriticalSection( qmutex);

  // Just return if no new packets have been received
  if (rxPacketTo[ boardNumber-1][channel-1] == rxPacketFrom[ boardNumber-1][channel-1])
  {
    EndCriticalSection(qmutex);
    return status;
  }

  // Get the length of the data
  *count = rxPacketLengths[ boardNumber-1][channel-1][rxPacketFrom[ boardNumber-1][channel-1]];

  // Get the status of the transfer
  if (transferStatus != NULL)
    *transferStatus = rxPacketStatus[ boardNumber-1][channel-1][rxPacketFrom[ boardNumber-1][channel-1]];

  // Adjust the pointer
  if (++rxPacketFrom[ boardNumber-1][channel-1] >= MAXIMUM_RX_PACKETS) rxPacketFrom[ boardNumber-1][channel-1] = 0;

  // Range check the count value
  if (*count > maxCount)
  {
    EndCriticalSection(qmutex);
    return GscApiDataPacketTooLong;
  }

  // Make sure the data is still in the Rx FIFO
  //      status = GscSio4ChannelFifoCounts( boardNumber, channel, &fifoCounts);
  status = getRxFifoCount( boardNumber, channel, &fifoCounts);
  if (status)
  {
    EndCriticalSection(qmutex);
    return status;
  }
  //      fifoCounts = (fifoCounts >> 16) & 0xffff;
  if (fifoCounts < *count)
  {
    EndCriticalSection(qmutex);
    return GscApiFifoOverrun;
  }

  /* Check whether timetag will be inserted into Rx FIFO for each byte
     and adjust count accordingly */
  status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+((channel-1)<<4), &value);
  if (status)
  {
    EndCriticalSection(qmutex);
    return status;
  }
  tempCount = *count;
  if (value & 0x04) tempCount += *count * 3;
  if (value & 0x08) tempCount += *count;

  // If the count is odd, make it even
  isOdd = 0x00;
  if ( tempCount & 0x01)
  {
    isOdd = 0xff;
    tempCount &= ~0x01;
  }

  // Get the data using the second DMA channel
  if (drvMajorVersion >= 5)
    status = gscPerformDma( boardNumber, channel, buffer, tempCount, GSC_DATA_RECEIVE, 1);
  else
    status = gscPerformDma( boardNumber, channel, buffer, tempCount, GSC_DATA_RECEIVE, PrimaryPciChannel1);

  // If the count was odd
  if (isOdd)
  {
    // Read the last byte
    fpgaRead( boardNumber, (channel << 4) | 0x08, &isOdd);
    *(buffer + tempCount) = isOdd;
  }

  // read RSB if present, and return masked errors in transferStatus parameter
  if (usingRSB)
  {
    fpgaRead( boardNumber, (channel << 4) | 0x08, &rsblo);
    fpgaRead( boardNumber, (channel << 4) | 0x08, &rsbhi);
    if (transferStatus)
      *transferStatus = ((rsbhi << 8) + rsblo) & 0x012e;
  }
    // read RSB32 (packet size) from data stream and discard
  if (usingRSB32)
  {
    fpgaRead( boardNumber, (channel << 4) | 0x08, &rsblo);
    fpgaRead( boardNumber, (channel << 4) | 0x08, &rsbhi);
  }
  EndCriticalSection( qmutex);

  return status;
}
int GscSio4ChannelCheckForData(int boardNumber, int channel, char *buffer, int *count)
{
  return GscSio4ChannelReceivePacket( boardNumber, channel, buffer, count, NULL);
}


//-------------------------------------------------- GscSio4ChannelReceiveData
//
//  This routine adds a receive request to the DMA
// handler's queue. It may return before the transfer
// has taken place.
//
int GscSio4ChannelReceiveData(int boardNumber, int channel, char *buffer, int count, int *id)
{
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Check buffer alignment
  if ((int)buffer & 0x03)
    return GscApiMisalignedBuffer;

  // Perform the Transmit function
  return gscAddDmaRequest(boardNumber, channel, buffer, count, GSC_DATA_RECEIVE, id);
}



//-------------------------------------------------- GscSio4ChannelReceiveData
//
//  This routine adds a receive request to the DMA
// handler's queue. It may return before the transfer
// has taken place.
//
int GscSio4ChannelReceivePlxPhysData(int boardNumber, int channel, PLX_PHYSICAL_MEM *buffer, int count, int *id)
{
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Check buffer alignment
  if ((int)buffer & 0x03)
    return GscApiMisalignedBuffer;

  // Perform the Transmit function
  return gscAddPlxPhysDmaRequest(boardNumber, channel, buffer, count, GSC_DATA_RECEIVE, id);
}



//------------------------------------------- GscSio4ChannelReceiveDataAndWait
//
//  This routine adds a receive request to the DMA
// handler's queue and then waits for the transfer to
// complete.
//
int GscSio4ChannelReceiveDataAndWait(int boardNumber, int channel, char *buffer, int count, int timeout, int *bytesTransferred)
{
  int status=0x00, rxId;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Check buffer alignment
  if ((int)buffer & 0x03)
    return GscApiMisalignedBuffer;

  // Start the Receive function
  status = gscAddDmaRequest(boardNumber, channel, buffer, count, GSC_DATA_RECEIVE, &rxId);
  if (status) return status;

  // Wait for it to complete
  return GscSio4ChannelWaitForTransfer(boardNumber, channel, timeout, rxId, bytesTransferred);
}



//------------------------------------------- GscSio4ChannelPlxPhysReceiveDataAndWait
//
//  This routine adds a receive request using physical memory to the DMA
// handler's queue and then waits for the transfer to
// complete.
//
int GscSio4ChannelReceivePlxPhysDataAndWait(int boardNumber, int channel, PLX_PHYSICAL_MEM *buffer, int count, int timeout, int *bytesTransferred)
{
  int status=0x00, rxId;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Check buffer alignment
  if ((int)buffer & 0x03)
    return GscApiMisalignedBuffer;

  // Start the Receive function
  status = gscAddPlxPhysDmaRequest(boardNumber, channel, buffer, count, GSC_DATA_RECEIVE, &rxId);
  if (status) return status;

  // Wait for it to complete
  return GscSio4ChannelWaitForTransfer(boardNumber, channel, timeout, rxId, bytesTransferred);
}

//--------------------------------------------------- GscSio4ChannelFifoCounts
//
//  This routine returns the number of bytes currently
// in the Tx and Rx FIFOs for the specified channel.
// The upper 16 bits are the count for the Rx FIFO and
// the lower 16 bits are the count for the Tx FIFO.
//
int GscSio4ChannelFifoCounts( int boardNumber, int channel, int *counts)
{
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Read the counts
  status = fpgaRead( boardNumber, FIFO_COUNT_BASE_REG+(channel-1)*4, counts);

  return status;
}


// ---------------------------------------------------- GscGetNumberOfChannels
//
//  This routine routines the number of active channels
// for the specified board. Analog boards always return
// 1 channel.
int GscGetNumberOfChannels( int boardNumber, int *channels)
{
  int status = 0x00;
  int bn = boardNumber - 1;

  // Make sure the board number is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // If this is an analog board, set the number of channels to 1
  if ( boardInformation[ bn].boardType == GSC_BOARD_TYPE_ANALOG)
  {
    status = GscAnaGetOption( boardNumber, GSC_ANA_NUMACTIVECHANNELS, channels );
  }

  // Otherwise, look up the number
  else
  {
    *channels = boardInformation[ bn].numberOfChannels;
  }

  return status;
}

//----------------------------------------------------------- GscGetRxFifoSize
//
//  This routine returns the size of the Rx fifo on the
// specified board and channel. Analog cards ignore the
// channel value.
//
int GscGetRxFifoSize( int boardNumber, int channel, int *size)
{
  int status = 0x00;
  int bn = boardNumber - 1;
  int ch = channel - 1;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // If this is an analog board, set channel to 1 (ch to 0)
  if ( boardInformation[ bn].boardType == GSC_BOARD_TYPE_ANALOG) ch = 0;

  // Get the size
  *size = boardInformation[ bn].rxFifoSizes[ ch];

  return status;
}

//----------------------------------------------------------- GscGetTxFifoSize
//
//  This routine returns the size of the Tx fifo on the
// specified board and channel. Analog cards ignore the
// channel value.
//
int GscGetTxFifoSize( int boardNumber, int channel, int *size)
{
  int status = 0x00;
  int bn = boardNumber - 1;
  int ch = channel - 1;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // If this is an analog board, set channel to 1 (ch to 0)
  if ( boardInformation[ bn].boardType == GSC_BOARD_TYPE_ANALOG) ch = 0;

  // Get the size
  *size = boardInformation[ bn].txFifoSizes[ ch];

  return status;
}

//---------------------------------------------------- GscSio4ChannelFifoSizes
//
//  This routine returns the sizes of the Tx and Rx FIFOs
// for the specified channel. The upper 16 bits are the
// size of the Rx FIFO and the lower 16 bits are the size
// of the Tx FIFO.
//
int GscSio4ChannelFifoSizes( int boardNumber, int channel, int *sizes)
{
  int status = 0x00;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Read the sizes
  //      status = fpgaRead( boardNumber, FIFO_SIZE_BASE_REG+(channel-1)*4, sizes);
  *sizes = (boardInformation[ boardNumber-1].rxFifoSizes[ channel-1] << 16) | (boardInformation[ boardNumber-1].txFifoSizes[ channel-1]);

  return status;
}


int GscSio4ChannelTransmitterControl( int boardNumber, int channel, int txMode)
{
  int status = 0x00, oldData;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // If this is a -Sync card
  if ((boardInformation[ boardNumber-1].boardType == GSC_BOARD_TYPE_SYNC) ||
      (gscSio4ChannelIsBxrSync(boardNumber, channel)))
  {
    // Read the current Control/Status register
    status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &oldData);
    if (status) return status;

    // Process the desired Tx mode
    switch (txMode)
    {
    case GSC_ENABLED:       oldData |=  0x02000000; break;
    case GSC_DISABLED:      oldData &= ~0x02000000; break;
    default:                        return GscApiInvalidParameter;
    }

    // Write the new value back out
    status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), oldData);
    if (status) return status;
  }

  // Otherwise, its a Zilog Based card
  else status = zilogTransmitterControl( boardNumber, channel, txMode);

  return status;
}

int GscSio4ChannelReceiverControl( int boardNumber, int channel, int rxMode)
{
  int status = 0x00, oldData;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // If this is not a Zilog Based board
  if (boardInformation[ boardNumber-1].boardType != GSC_BOARD_TYPE_NORMAL)
  {
    // Read the current Control/Status register
    status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &oldData);
    if (status) return status;

    // Process the desired Rx mode
    switch (rxMode)
    {
    case GSC_ENABLED:       oldData |=  0x01000000; break;
    case GSC_DISABLED:      oldData &= ~0x01000000; break;
    default:                        return GscApiInvalidParameter;
    }

    // Write the new value back out
    status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), oldData);
    if (status) return status;
  }

  // Otherwise, its a Zilog Based card
  else status = zilogReceiverControl( boardNumber, channel, rxMode);

  return status;
}

int quickSetLoopBack( int boardNumber, int channel, int loopMode)
{
  int status = 0x00;
  int tempValue;

  // Get the current Pin Source Register value
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, &tempValue);
  if (status) return status;

  // Process the desired mode
  switch (loopMode)
  {
  case GSC_LOOP_NONE:             tempValue &= ~0x20000000;       break;
  case GSC_LOOP_EXTERNAL: tempValue |=  0x20000000;       break;
  default:                                return GscApiInvalidParameter;
  }

  // Save the loopback mode
  boardInformation[ boardNumber-1].channelLoopback[ channel-1] = loopMode;

  // Make sure the New (non-legacy) mode is selected
  tempValue |= 0x80000000;

  // Write the new Pin Source Register value
  status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, tempValue);
  return status;
}

//-------------------------------------------------------- GscSio4GetLastError
//
//  This routine returns a string that contains detailed
// information about the last error reported.
//
int GscSio4GetLastError( int boardNumber, int channel, int errorCode, char *errorString, char *detailString)
{
  int status = 0x00;

  // Get the error string if it is requested
  if (errorString != NULL)
  {
    status = GscGetErrorString( errorCode, errorString);
    if (status) return status;
  }

  // Get the detail string if it is requested
  if (detailString != NULL) strcpy( detailString, boardInformation[ boardNumber-1].lastError[ channel-1]);

  return status;
}

//------------------------------------------------------ GscGetErrorString
//
//  This routine takes an error code and returns the
// corresponding error string. The string is guaranteed
// to be less than 80 bytes long.
//
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscGetErrorString( int errorCode, char *errorString)
{
  int PlxErrorFound = TRUE;

  if (drvMajorVersion < 5)
  {
    // Switch on the error code
    switch (errorCode)
    {
      // Plx Driver errors
    case ApiSuccessLegacy:             strcpy(errorString, "Operation Successful");  break;
    case ApiFailedLegacy:              strcpy(errorString, "Operation Failed");  break;
    case ApiAccessDeniedLegacy:        strcpy(errorString, "ApiAccessDenied");  break;
    case ApiDmaChannelTypeErrorLegacy: strcpy(errorString, "ApiDmaChannelTypeError");  break;
    case ApiDmaNotPausedLegacy:        strcpy(errorString, "ApiDmaNotPaused");  break;
    case ApiDmaManReadyLegacy:         strcpy(errorString, "ApiDmaManReady");  break;
    case ApiDmaManNotReadyLegacy:      strcpy(errorString, "ApiDmaManNotReady");  break;
    case ApiDmaManCorruptedLegacy:     strcpy(errorString, "ApiDmaManCorrupted");  break;
    case ApiDmaInvalidElementIndexLegacy: strcpy(errorString, "ApiDmaInvalidElementIndex");  break;
    case ApiDmaNoMoreElementsLegacy:   strcpy(errorString, "ApiDmaNoMoreElements");  break;
    case ApiDmaSglInvalidLegacy:       strcpy(errorString, "ApiDmaSglInvalid");  break;
    case ApiDmaSglQueueFullLegacy:     strcpy(errorString, "ApiDmaSglQueueFull");  break;
    case ApiMuNotReadyLegacy:          strcpy(errorString, "ApiMuNotReady");  break;
    case ApiInvalidRegisterLegacy:     strcpy(errorString, "ApiInvalidRegister");  break;
    case ApiDoorbellClearFailedLegacy: strcpy(errorString, "ApiDoorbellClearFailed");  break;
    case ApiInvalidUserPinLegacy:      strcpy(errorString, "ApiInvalidUserPin");  break;
    case ApiInvalidUserStateLegacy:    strcpy(errorString, "ApiInvalidUserState");  break;
    case ApiEepromNotPresentLegacy:    strcpy(errorString, "ApiEepromNotPresent");  break;
    case ApiEepromTypeNotSupportedLegacy: strcpy(errorString, "ApiEepromTypeNotSupported");  break;
    case ApiEepromBlankLegacy:         strcpy(errorString, "ApiEepromBlank");  break;
    case ApiObjectAlreadyAllocatedLegacy: strcpy(errorString, "ApiObjectAlreadyAllocated");       break;
    case ApiAlreadyInitializedLegacy:  strcpy(errorString, "ApiAlreadyInitialized"); break;
    case ApiNotInitializedLegacy:      strcpy(errorString, "ApiNotInitialized");  break;
    case ApiBadConfigRegEndianModeLegacy: strcpy(errorString, "ApiBadConfigRegEndianMode");       break;
    case ApiFlybyNotSupportedLegacy:   strcpy(errorString, "ApiFlybyNotSupported");  break;
    case ApiNotSupportThisChannelLegacy:  strcpy(errorString, "ApiNotSupportThisChannel");        break;
    case ApiNoActionLegacy:            strcpy(errorString, "ApiNoAction");  break;
    case ApiVpdNotEnabledLegacy:       strcpy(errorString, "ApiVpdNotEnabled");  break;
    case ApiNoMoreCapLegacy:           strcpy(errorString, "ApiNoMoreCap");  break;
    case ApiBadPinDirectionLegacy:     strcpy(errorString, "ApiBadPinDirection");  break;
    case ApiPciTimeoutLegacy:          strcpy(errorString, "ApiPciTimeout");  break;
    case ApiDmaChannelClosedLegacy:    strcpy(errorString, "ApiDmaChannelClosed");  break;
    case ApiDmaChannelErrorLegacy:     strcpy(errorString, "ApiDmaChannelError");  break;
    case ApiBufferNotReadyLegacy:      strcpy(errorString, "ApiBufferNotReady");  break;
    case ApiDoNothingLegacy:           strcpy(errorString, "ApiDoNothing");  break;
    case ApiDmaSglBuildFailedLegacy:   strcpy(errorString, "ApiDmaSglBuildFailed");  break;
    case ApiPMNotSupportedLegacy:      strcpy(errorString, "ApiPMNotSupported");  break;
    case ApiWaitTimeoutLegacy:         strcpy(errorString, "ApiWaitTimeout");  break;
    case ApiWaitCanceledLegacy:        strcpy(errorString, "ApiWaitCanceled");  break;
    default:
      PlxErrorFound = FALSE;
    }
  }
  else
  {
    switch (errorCode)
    {
    case ApiSuccess:                    strcpy(errorString, "Operation Successful");  break;
    case ApiFailed:                     strcpy(errorString, "Operation Failed");  break;
    case ApiDmaChannelUnavailable:      strcpy(errorString, "ApiDmaChannelUnavailable");  break;
    case ApiDmaChannelInvalid:          strcpy(errorString, "ApiDmaChannelInvalid");  break;
    case ApiDmaInProgress:              strcpy(errorString, "ApiDmaInProgress");  break;
    case ApiDmaDone:                    strcpy(errorString, "ApiDmaDone");  break;
    case ApiDmaPaused:                  strcpy(errorString, "ApiDmaPaused");  break;
    case ApiDmaCommandInvalid:            strcpy(errorString, "ApiDmaCommandInvalid");          break;
    case ApiDmaInvalidChannelPriority:  strcpy(errorString, "ApiDmaInvalidChannelPriority");  break;
    case ApiNullParam:                                strcpy(errorString, "ApiNullParam");              break;
    case ApiInvalidBusIndex:                strcpy(errorString, "ApiInvalidBusIndex");                  break;
    case ApiUnsupportedFunction:          strcpy(errorString, "ApiUnsupportedFunction");  break;
    case ApiInvalidPciSpace:                strcpy(errorString, "ApiInvalidPciSpace");                  break;
    case ApiInvalidIopSpace:                strcpy(errorString, "ApiInvalidIopSpace");                  break;
    case ApiInvalidSize:                strcpy(errorString, "ApiInvalidSize");  break;
    case ApiInvalidAddress:             strcpy(errorString, "ApiInvalidAddress");  break;
    case ApiInvalidAccessType:            strcpy(errorString, "ApiInvalidAccessType");          break;
    case ApiInvalidIndex:               strcpy(errorString, "ApiInvalidIndex");  break;
    case ApiMuFifoEmpty:                strcpy(errorString, "ApiMuFifoEmpty");  break;
    case ApiMuFifoFull:                 strcpy(errorString, "ApiMuFifoFull");  break;
    case ApiConfigAccessFailed:           strcpy(errorString, "ApiConfigAccessFailed");         break;
    case ApiInvalidDeviceInfo:            strcpy(errorString, "ApiInvalidDeviceInfo");          break;
    case ApiNoActiveDriver:             strcpy(errorString, "ApiNoActiveDriver");  break;
    case ApiInsufficientResources:      strcpy(errorString, "ApiInsufficientResources");        break;
    case ApiInvalidPowerState:            strcpy(errorString, "ApiInvalidPowerState");          break;
    case ApiPowerDown:                  strcpy(errorString, "ApiPowerDown");  break;
    case ApiHSNotSupported:             strcpy(errorString, "ApiHSNotSupported");  break;
    case ApiVPDNotSupported:                strcpy(errorString, "ApiVPDNotSupported");                  break;
    case ApiInvalidOffset:              strcpy(errorString, "ApiInvalidOffset");  break;
    case ApiInvalidHandle:              strcpy(errorString, "ApiInvalidHandle");  break;
    case ApiInvalidData:                strcpy(errorString, "ApiInvalidData");  break;
    case ApiInvalidDriverVersion:         strcpy(errorString, "ApiInvalidDriverVersion");       break;
    default:
      PlxErrorFound = FALSE;
    }
  }

  if (PlxErrorFound == FALSE)
  {
    // GSC API errors
    switch (errorCode)
    {
    case GscApiInvalidBoardNumber:        strcpy(errorString, "Invalid Board Number");          break;
    case GscApiInvalidChannel:              strcpy(errorString, "Invalid Channel Number");              break;
    case GscApiInvalidDmaNumWords:        strcpy(errorString, "Invalid DMA Transfer Size");     break;
    case GscApiDmaBufferTooSmall:           strcpy(errorString, "DMA Buffer Too Small");                break;
    case GscApiInvalidDmaBufferAddr:  strcpy(errorString, "Invalid DMA Buffer Address");        break;
    case GscApiDmaChannelBusy:              strcpy(errorString, "DMA Channel Busy");                    break;
    case GscApiInvalidFWRev:                  strcpy(errorString, "Invalid Firmware Version");  break;
    case GscApiInvalidParameter:            strncpy(errorString, "Invalid Parameter", 80);                      break;
    case GscApiOperationTimeout:            strcpy(errorString, "Operation Timed Out");                 break;
    case GscApiInvalidCombination:        strcpy(errorString, "Invalid Parameter Combination"); break;
    case GscApiDllHeaderMismatch:           strcpy(errorString, "Header/DLL Version MisMatch"); break;
    case GscApiDataPacketTooLong:           strcpy(errorString, "Data Packet Too Long");                break;
    case GscApiFifoOverrun:                 strcpy(errorString, "FIFO Overrun"); break;
    case GscApiInvalidBoardType:            strncpy(errorString, "Feature not supported on this board", 80);    break;
    case GscApiCTCNoBufferedFrames:         strcpy(errorString, "No major frames buffered"); break;
    default:
      strcpy(errorString, "Invalid Error Code");
      return GscApiInvalidParameter;
    }
  }

  // Return a 0x00 since everything is ok
  return 0x00;
}


//int GscSio4ChannelGetTxAlmostEmpty(int boardNumber, int channel, int *almostValue)
//int GscSio4ChannelGetTxAlmostFull(int boardNumber, int channel, int *almostValue)
//int GscSio4ChannelGetRxAlmostEmpty(int boardNumber, int channel, int *almostValue)
//int GscSio4ChannelGetRxAlmostFull(int boardNumber, int channel, int *almostValue)
