//////////////////////////////////////////////////////////////////////////////
//
// Deprecated.c
//
//  This file contains all of the GscApi functions currently flagged as
// deprecated.
//
#include "GscApiLocal.h"

//
// Deprecated system Level Routines
//

//---------------------------------------------------------- GscSio4FindBoards
//
//  This routine finds and reports all of the GSC SIO4 boards
// in the system. It returns both a count of boards found and
// a structure with the board info in it (assuming the structure
// pointer is not NULL).
//
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscSio4FindBoards( int *boardCount, GSC_DEVICES_STRUCT results[])
{
  return GscFindBoards( boardCount, results );
}

//------------------------------------------------------ GscSio4GetErrorString
//
//  This routine takes an error code and returns the
// corresponding error string. The string is guaranteed
// to be less than 80 bytes long.
//
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscSio4GetErrorString( int errorCode, char *errorString)
{
  return GscGetErrorString( errorCode, errorString );
}

//
// Deprecated board level API functions
//

//---------------------------------------------------------------- GscSio4Open
//
//  This routine is used to reset the entire SIO4 board. The
// Zilog chips, FIFOs, and FPGAs are all set to their power on
// state. This routine also gets a handle for use by the PLX
// driver if one has not already been assigned.
//
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscSio4Open( int boardNumber, int headerVersion)
{
  return GscOpen( boardNumber, headerVersion );
}

//---------------------------------------------------------------- GscSio4Close
//
int GscSio4Close( int boardNumber)
{
  return GscClose( boardNumber );
}

//------------------------------------------------------------- GscSio4GetInfo
//
//  This routine returns the general information about the
// selected board. The data is returned in a board info
// structure.
//
int GscSio4GetInfo( int boardNumber, PBOARD_INFO info)
{
  return GscGetInfo( boardNumber, info );
}

//--------------------------------------------------------- GscSio4GetVersions
//
//  This routine gets the 32 bit value of the firmware revision
// register and places into the variable passed into the routine.
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscSio4GetVersions( int boardNumber, int *libVersion, int *driverVersion, int *fpgaVersion)
{
  return GscGetVersions( boardNumber, libVersion, driverVersion, fpgaVersion );
}

//--------------------------------------------------- GscSio4LocalRegisterRead
//
//  This routine is used to read a local register (FPGA)
// on the SIO4 board.
//
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscSio4LocalRegisterRead( int boardNumber, int reg, int *result)
{
  return GscLocalRegisterRead( boardNumber, reg, result );
}

//-------------------------------------------------- GscSio4LocalRegisterWrite
//
//  This routine is used to write to a local register (FPGA)
// on the SIO4 board.
//
//  This routine returns a 0x00 if successful or an error code
// if it fails.
//
int GscSio4LocalRegisterWrite(int boardNumber, int reg, int value)
{
  return GscLocalRegisterWrite( boardNumber, reg, value );
}

//
// BISYNC mode-specific deprecated API functions
//

//--------------------------------------------- GscSio4ChannelGetBiSyncPattern
//--------------------------------------------- GscSio4ChannelSetBiSyncPattern
//
//  These routines are used to get/set the sync pattern for BiSync mode.
//
int GscSio4ChannelGetBiSyncPattern( int boardNumber, int channel, int *TxSync, int *RxSync)
{
  // Call the internal library function and return its status
  return gscSio4ChannelGetBiSyncPattern( boardNumber, channel, TxSync, RxSync );
}

int GscSio4ChannelSetBiSyncPattern( int boardNumber, int channel, int TxSync, int RxSync)
{
  // Call the internal library function and return its status
  return gscSio4ChannelSetBiSyncPattern( boardNumber, channel, TxSync, RxSync );
}


//------------------------------------------ GscSio4ChannelGetBiSyncTxUnderrun
//------------------------------------------ GscSio4ChannelSetBiSyncTxUnderrun
//
//  These routines are used to get/set the Tx
// Underrun condition mode.
//
int GscSio4ChannelGetBiSyncTxUnderrun( int boardNumber, int channel, int *txUnderrun)
{

    // Call the internal library function and return its status
    return gscSio4ChannelGetBiSyncTxUnderrun( boardNumber, channel, txUnderrun );
}

int GscSio4ChannelSetBiSyncTxUnderrun( int boardNumber, int channel, int txUnderrun)
{
    int status = 0x00;
    int value;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    // Read the current Channel Mode Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
    if (status) return status;

    // Clear the bits
    value &= ~0xc000;

    // Determine the Tx Underrun Condition
    switch (txUnderrun)
    {
        case GSC_SYN1:          value |= 0x0000;	break;
        case GSC_SYN0_SYN1:     value |= 0x4000;	break;
        case GSC_CRC_SYN1:      value |= 0x8000;	break;
        case GSC_CRC_SYN0_SYN1: value |= 0xc000;    break;
    }

    // Write the new Channel Mode Register value
    status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, value);
    if (status) return status;

    return status;
}


//------------------------------------------ GscSio4ChannelGetBiSyncTxPreamble
//------------------------------------------ GscSio4ChannelSetBiSyncTxPreamble
//
//  These routines are used to get/set the Tx
// Preamble value.
//
int GscSio4ChannelGetBiSyncTxPreamble( int boardNumber, int channel, int *txPreamble)
{
    // Call the internal library function and return its status
    return gscSio4ChannelGetBiSyncTxPreamble( boardNumber, channel, txPreamble );
}

int GscSio4ChannelSetBiSyncTxPreamble( int boardNumber, int channel, int txPreamble)
{
    int status = 0x00;
    int value;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    // Read the current Channel Mode Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
    if (status) return status;

    // Write the new Channel Mode Register value
    status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, value);
    if (status) return status;

    return status;
}


//----------------------------------------- GscSio4ChannelGetBiSyncTxShortSync
//----------------------------------------- GscSio4ChannelSetBiSyncTxShortSync
//
//  These routines are used to get/set the Tx
// Sync Length (short or 8 bit).
//
int GscSio4ChannelGetBiSyncTxShortSync( int boardNumber, int channel, int *txShortSync)
{
    // Call the internal library function and return its status
    return gscSio4ChannelGetBiSyncTxShortSync( boardNumber, channel, txShortSync );
}

int GscSio4ChannelSetBiSyncTxShortSync( int boardNumber, int channel, int txShortSync)
{
    int status = 0x00;
    int value;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    // Read the current Channel Mode Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
    if (status) return status;

    // Write the new Channel Mode Register value
    status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, value);
    if (status) return status;

    return status;
}


//----------------------------------------- GscSio4ChannelGetBiSyncRxSyncStrip
//----------------------------------------- GscSio4ChannelSetBiSyncRxSyncStrip
//
//  These routines are used to get/set the Rx
// Sync Strip mode.
//
int GscSio4ChannelGetBiSyncRxSyncStrip( int boardNumber, int channel, int *rxSyncStrip)
{
    // Call the internal library function and return its status
    return gscSio4ChannelGetBiSyncRxSyncStrip( boardNumber, channel, rxSyncStrip );
}

int GscSio4ChannelSetBiSyncRxSyncStrip( int boardNumber, int channel, int rxSyncStrip)
{
    int status = 0x00;
    int value;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    // Read the current Channel Mode Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
    if (status) return status;

    // Write the new Channel Mode Register value
    status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, value);
    if (status) return status;

    return status;
}


//----------------------------------------- GscSio4ChannelGetBiSyncRxShortSync
//----------------------------------------- GscSio4ChannelSetBiSyncRxShortSync
//
//  These routines are used to get/set the Rx
// Sync Length (short or 8 bit).
//
int GscSio4ChannelGetBiSyncRxShortSync( int boardNumber, int channel, int *rxShortSync)
{
    // Call the internal library function and return its status
    return gscSio4ChannelGetBiSyncRxShortSync( boardNumber, channel, rxShortSync );
}

int GscSio4ChannelSetBiSyncRxShortSync( int boardNumber, int channel, int rxShortSync)
{
    int status = 0x00;
    int value;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    // Read the current Channel Mode Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
    if (status) return status;

    // Write the new Channel Mode Register value
    status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, value);
    if (status) return status;

    return status;
}


//------------------------------------ GscSio4ChannelGetBiSyncTxPreambleLength
//------------------------------------ GscSio4ChannelSetBiSyncTxPreambleLength
//
//  These routines are used to get/set the Tx
// Preamble Length.
//
int GscSio4ChannelGetBiSyncTxPreambleLength( int boardNumber, int channel, int *txPreambleLength)
{
    // Call the internal library function and return its status
    return gscSio4ChannelGetBiSyncTxPreambleLength( boardNumber, channel, txPreambleLength );
}

int GscSio4ChannelSetBiSyncTxPreambleLength( int boardNumber, int channel, int txPreambleLength)
{
    int status = 0x00;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    return status;
}


//----------------------------------- GscSio4ChannelGetBiSyncTxPreamblePattern
//----------------------------------- GscSio4ChannelSetBiSyncTxPreamblePattern
//
//  These routines are used to get/set the Tx
// Preamble pattern.
//
int GscSio4ChannelGetBiSyncTxPreamblePattern( int boardNumber, int channel, int *txPreamblePattern)
{
    // Call the internal library function and return its status
    return gscSio4ChannelGetBiSyncTxPreamblePattern( boardNumber, channel, txPreamblePattern );
}

int GscSio4ChannelSetBiSyncTxPreamblePattern( int boardNumber, int channel, int txPreamblePattern)
{
    int status = 0x00;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    return status;
}

//
// BISYNC16 mode-specific deprecated API functions
//

int GscSio4BiSync16GetSyncWord( int boardNumber, int channel, int *syncWord, int *idleWord)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, SYNC_CHARACTER_BASE_REG + (channel-1)*4, &value);
	if (status) return status;

	// Return values in the proper variables
	*syncWord = value & 0x0000ffff;
	*idleWord = (value >> 16) & 0x0000ffff;

	return status;
}

int GscSio4BiSync16SetSyncWord( int boardNumber, int channel, int syncWord, int idleWord)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Write new value
	value = ((idleWord<<16) & 0xffff0000) | (syncWord & 0x0000ffff);
	status = fpgaWrite( boardNumber, SYNC_CHARACTER_BASE_REG + (channel-1)*4, value);
	if (status) return status;

	return status;
}


int	GscSio4BiSync16GetOrdering( int boardNumber, int channel, int *byteOrder, int *bitOrder)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
	if (status) return status;

	// Return values in the proper variables
	if (value & 0x04000000) *byteOrder = GSC_MSB_FIRST;
	else *byteOrder = GSC_LSB_FIRST;

	if (value & 0x08000000) *bitOrder = GSC_LSB_FIRST;
	else *bitOrder = GSC_MSB_FIRST;

	return status;
}

int	GscSio4BiSync16SetOrdering( int boardNumber, int channel, int byteOrder, int bitOrder)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
	if (status) return status;

	// Clear bits of interest
	value &= ~0x0c000000;

	// Update value
	if (byteOrder == GSC_MSB_FIRST) value |= 0x04000000;
	if (bitOrder == GSC_LSB_FIRST) value |= 0x08000000;
	status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, value);
	if (status) return status;

	return status;
}


int GscSio4BiSync16GetMaxRxCount( int boardNumber, int channel, int *maxRxCount)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, MAX_RX_COUNT_BASE_REG + (channel-1)*4, &value);
	if (status) return status;

	// Return values in the proper variables
	*maxRxCount = value & 0xffff;

	return status;
}

int GscSio4BiSync16SetMaxRxCount( int boardNumber, int channel, int maxRxCount)
{
	int status = 0x00;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Set value
	status = fpgaWrite( boardNumber, MAX_RX_COUNT_BASE_REG + (channel-1)*4, maxRxCount);
	if (status) return status;

	return status;
}


int GscSio4BiSync16GetReceiver( int boardNumber, int channel, int *mode)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
	if (status) return status;

	// Return values in the proper variables
	if (value & 0x01000000) *mode = GSC_ENABLED;
	else *mode = GSC_DISABLED;

	return status;
}

int GscSio4BiSync16SetReceiver( int boardNumber, int channel, int mode)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
	if (status) return status;

	// Update values
	if (mode == GSC_ENABLED) value |= 0x01000000;
	else value &= ~0x01000000;

	// Write them back out
	status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, value);

	return status;
}


int GscSio4BiSync16GetTransmitter( int boardNumber, int channel, int *mode)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
	if (status) return status;

	// Return values in the proper variables
	if (value & 0x02000000) *mode = GSC_ENABLED;
	else *mode = GSC_DISABLED;

	return status;
}

int GscSio4BiSync16SetTransmitter( int boardNumber, int channel, int mode)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
	if (status) return status;

	// Update values
	if (mode == GSC_ENABLED) value |= 0x02000000;
	else value &= ~0x02000000;

	// Write them back out
	status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, value);

	return status;
}


//
// HDLC mode-specific deprecated API functions
//

//----------------------------------------------- GscSio4ChannelSetHdlcCrcMode
//
int GscSio4HdlcSetCrcMode(int boardNumber, int channel, int crcMode, int initialValue)
{
	int status = 0x00;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Make the changes
	status = zilogSetCrcMode( boardNumber, channel, crcMode, initialValue);
	return status;
}

//
// SYNC mode-specific (-SYNC board) deprecated API functions
//


//----------------------------------------------- GscSio4ChannelGetMsbLsbOrder
//
int GscSio4ChannelGetMsbLsbOrder(int boardNumber, int channel, int *txOrder, int *rxOrder)
{
  // call the internal function and return its status
  return gscSio4ChannelGetMsbLsbOrder( boardNumber, channel, txOrder, rxOrder );
}


//----------------------------------------------- GscSio4ChannelSetMsbLsbOrder
//
int GscSio4ChannelSetMsbLsbOrder(int boardNumber, int channel, int txOrder, int rxOrder)
{
  // Call the internal fn, return its status.
  return gscSio4ChannelSetMsbLsbOrder( boardNumber, channel, txOrder, rxOrder);
}

//
// Channel level deprecated API functions
//

//-------------------------------------------------- GscSio4ChannelGetDataSize
//
//  This routine returns the current data size for
// a channel.
//
int GscSio4ChannelGetDataSize(int boardNumber, int channel, int *dataSize)
{
	int status = 0x00;
	int oldValue;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Handle the proper board type
	switch (boardInformation[ boardNumber-1].boardType)
	{
	// -BISYNC16 Boards
	case GSC_BOARD_TYPE_BISYNC16:
		*dataSize = 16;
		break;

	// -SYNC Boards
	case GSC_BOARD_TYPE_SYNC:

		// Get the current Tx Count Register
		status = fpgaRead( boardNumber, TX_COUNT_BASE_REG+4*(channel-1), &oldValue);
		if (status) return status;

		// Adjust it to the new value
		*dataSize = oldValue & 0x0000ffff;
		break;

	// Zilog USC based Boards
	default:
		status = zilogGetDataLength( boardNumber, channel, dataSize, dataSize);
		if (status) return status;
		break;
	}

	return status;
}


//-------------------------------------------------- GscSio4ChannelSetDataSize
//
int GscSio4ChannelSetDataSize(int boardNumber, int channel, int dataSize)
{
	int status = 0x00;
	int oldValue;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Handle the proper board type
	switch (boardInformation[ boardNumber-1].boardType)
	{
	// -BISYNC16 Boards
	case GSC_BOARD_TYPE_BISYNC16:
		if (dataSize != 16) return GscApiInvalidParameter;
		break;

	// -SYNC Boards
	case GSC_BOARD_TYPE_SYNC:
		// Range check the data size
		if((dataSize < 1) || (dataSize > 0xffff)) return GscApiInvalidParameter;

		// Get the current Tx Count Register
		status = fpgaRead( boardNumber, TX_COUNT_BASE_REG+4*(channel-1), &oldValue);
		if (status) return status;

		// Adjust it to the new value
		oldValue &= 0xffff0000;
		oldValue |= dataSize;

		// Write it back out
		status = fpgaWrite( boardNumber, TX_COUNT_BASE_REG+4*(channel-1), oldValue);
		if (status) return status;
		break;

	// Zilog USC based Boards
	default:
		status = zilogSetDataLength( boardNumber, channel, dataSize, dataSize);
		if (status) return status;
		break;
	}

	return status;
}


//--------------------------------------------------- GscSio4ChannelGetGapSize
//
//  This routine returns the current gap size for
// a channel.
//
int GscSio4ChannelGetGapSize(int boardNumber, int channel, int *gapSize)
{
	int status = 0x00;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Return the data size
	*gapSize = boardInformation[ boardNumber-1].channelGapSize[ channel-1];

	return status;
}


//--------------------------------------------------- GscSio4ChannelSetGapSize
//
int GscSio4ChannelSetGapSize(int boardNumber, int channel, int gapSize)
{
	int status = 0x00;
	int oldValue;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Only valid for -Sync boards
	if (boardInformation[ boardNumber-1].boardType != GSC_BOARD_TYPE_SYNC) return GscApiInvalidCombination;

	// Range check the data size
	if((gapSize < 0) || (gapSize > 0xffff)) return GscApiInvalidParameter;

	// Save the new data size
	boardInformation[ boardNumber-1].channelGapSize[ channel-1] = gapSize;

	// Get the current Tx Count Register
	status = fpgaRead( boardNumber, TX_COUNT_BASE_REG+4*(channel-1), &oldValue);
	if (status) return status;

	// Adjust it to the new value
	oldValue &= 0x0000ffff;
	oldValue |= (gapSize << 16);

	// Write it back out
	status = fpgaWrite( boardNumber, TX_COUNT_BASE_REG+4*(channel-1), oldValue);
	if (status) return status;

	return status;
}

//-------------------------------------------------- GscSio4ChannelGetEncoding
//
//  This routine returns the current Encoding for a
// channel.
//
int GscSio4ChannelGetEncoding(int boardNumber, int channel, int *encoding)
{
	int status = 0x00;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Return the encoding
	*encoding = boardInformation[ boardNumber-1].channelEncoding[ channel-1];

	return status;
}

int GscSio4ChannelSetEncoding(int boardNumber, int channel, int encoding)
{
  // call the internal library function and return its status
  return gscSio4ChannelSetEncoding( boardNumber, channel, encoding );
}

//---------------------------------------------------- GscSio4ChannelGetParity
//
//  This routine returns the current parity setting
// for a channel.
//
int GscSio4ChannelGetParity(int boardNumber, int channel, int *parity)
{
  // Call the internal library function and return its status
  return gscSio4ChannelGetParity( boardNumber, channel, parity );
}

int GscSio4ChannelSetParity(int boardNumber, int channel, int parity)
{
	int status = 0x00;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Make the changes
	status = quickSetParity( boardNumber, channel, parity);
	return status;
}


//-------------------------------------------------- GscSio4ChannelGetStopBits
//
//  This routine returns the current Stop Bit setting.
//
int GscSio4ChannelGetStopBits(int boardNumber, int channel, int *stopBits)
{
	int status = 0x00;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Return the Stop Bit value
	*stopBits = boardInformation[ boardNumber-1].channelStopBits[ channel-1];

	return status;
}

//-------------------------------------------------- GscSio4ChannelSetStopBits
//
int GscSio4ChannelSetStopBits(int boardNumber, int channel, int stopBits)
{
  // Call the internal library function and return its status
  return gscSio4ChannelSetStopBits( boardNumber, channel, stopBits );
}

//---------------------------------------------------- GscSio4ChannelGetDteDce
//
//  This routine returns the current setting of the DTE
// or DCE mode.
//
int GscSio4ChannelGetDteDce(int boardNumber, int channel, int *mode)
{
	int status = 0x00;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Return the info
	*mode = boardInformation[ boardNumber-1].channelDteDce[ channel-1];

	return status;
}

int GscSio4ChannelSetDteDce(int boardNumber, int channel, int mode)
{
  // Call the internal library function and return its status
  return gscSio4ChannelSetDteDce( boardNumber, channel, mode );
}


//-------------------------------------------------- GscSio4ChannelGetProtocol
//
int GscSio4ChannelGetProtocol(int boardNumber, int channel, int *protocol, int *termination)
{

  // call the internal library function and return its status
  return gscSio4ChannelGetProtocol( boardNumber, channel, protocol, termination );
}

int GscSio4ChannelSetProtocol(int boardNumber, int channel, int protocol, int termination)
{
	int status = 0x00;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Make the changes
	status = quickSetProtocol( boardNumber, channel, protocol, termination);
	return status;
}

int GscSio4ChannelGetLoopBack( int boardNumber, int channel, int *loopMode)
{
	int status = 0x00;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Return the loopback mode
	*loopMode = boardInformation[ boardNumber-1].channelLoopback[ channel-1];

	return status;
}

int GscSio4ChannelSetLoopBack( int boardNumber, int channel, int loopMode)
{
	int status = 0x00;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Make the change
	status = quickSetLoopBack( boardNumber, channel, loopMode);
	return status;
}

int GscSio4ChannelSetClockSource( int boardNumber, int channel, int txSource, int rxSource)
{
  // Call the internal library function and return its status
  return gscSio4ChannelSetClockSource( boardNumber, channel, txSource, rxSource );
}

