//////////////////////////////////////////////////////////////////////////////
//
// GscApiDma.c
//
//  This module contains all of the DMA handling routines.
//
#include <stdio.h>
#ifdef PLX_LINUX
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/time.h>
#include <errno.h>
#else
#define _WIN32_WINNT 0x0500
#include <windows.h>
#include <sddl.h>
//#include <stdio.h>
#endif
#include "GscApiLocal.h"
#include "GscApiPlxLegacy.h"
#include "GscApiPlxdload.h"

// constants
#define WIN32_CHECKDMA_DELAY_MULTIPLIER -10000
#define LINUX_CHECKDMA_DELAY_MULTIPLIER -1000

//
// External variables
//
extern int		dmaHandlerAbort[ MAXIMUM_NUMBER_OF_BOARDS];
extern DATA_LOG	rxDataLog[ MAXIMUM_DATALOG_ENTRIES];
extern DATA_LOG	txDataLog[ MAXIMUM_DATALOG_ENTRIES];
extern int		rxDataLogPointer;
extern int		txDataLogPointer;
//extern int	rxPacketLengths[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS][MAXIMUM_RX_PACKETS];
//extern int	rxPacketTo[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];
//extern int  rxPacketFrom[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];

#ifndef PLX_LINUX

SECURITY_ATTRIBUTES sa;

//HANDLE WINAPI CreateWaitableTimerA(LPSECURITY_ATTRIBUTES lpTimerAttributes, BOOL bManualReset,
//                                   LPCSTR lpTimerName);
//BOOL WINAPI SetWaitableTimer(HANDLE hTimer, const LARGE_INTEGER *lpDueTime, LONG lPeriod,
//                             LPVOID pfnCompletionRoutine, LPVOID lpArgToCompletionRoutine, BOOL fResume);
//BOOL WINAPI CancelWaitableTimer(HANDLE hTimer);
#endif

//------------------------------------------------------- checkSingleRxChannel
//
//  This routine is called by the Rx DMA Handler. It checks
// a single channel to see if a transfer is needed and performs
// the transfer if possible.
//
int checkSingleRxChannel( int bn, int ch, int reason, int fifoSize, double maximumDelay, int* loopDelay)
{
  int to, *from;
  PDMA_TRANSFER_QUEUE transferQueue;
  int fifoCount, transferCount=0, savedTransferCount, lastByte;
  char *buffer = NULL;
  int status = 0;

#ifdef DATA_LOGGING
  LARGE_INTEGER frequency, currentTime;
#endif

  BeginCriticalSection( qmutex);

  // Get the to, from, and queue pointers
  to = rxDmaQueueTo[ bn][ ch];
  from = &rxDmaQueueFrom[ bn][ ch];
  transferQueue = (PDMA_TRANSFER_QUEUE)rxDmaQueue[ bn][ ch];

  // Assume we are going to do nothing
  *loopDelay = 0x00;

  // See if there is anything to do
  if (to != *from)
  {

    // Get the available space in the fifo
    //		GscSio4ChannelFifoCounts(bn+1, ch+1, &fifoCount);
    //		fifoCount = (fifoCount >> 16) & 0xffff;
    status = getRxFifoCount( bn+1, ch+1, &fifoCount);

    // The transfer count is the smaller of the available space and the requested amount
    transferCount = (fifoCount>=transferQueue[ *from].size)?transferQueue[ *from].size:fifoCount;

    // Make the transfer count a multiple of 4 (save the original)
    savedTransferCount = transferCount;

    if ((transferQueue[ *from].size - transferCount) != 0)
      transferCount = transferCount - (transferCount % 4);
    else
      // make transfer count event
      transferCount &= ~0x01;

    // Get the buffer address and update it for next time
    if (transferQueue[*from].IsBufferLogicalAddr != TRUE)
    {
      buffer = transferQueue[ *from].buffer;
      transferQueue[ *from].buffer += transferCount;
    }

    // Do the transfer
    if (transferCount != 0)
    {
      if (transferQueue[*from].IsBufferLogicalAddr == TRUE)
        status = gscPerformPlxPhysDma( bn+1, ch+1, &transferQueue[ *from].physBuffer, transferCount, GSC_DATA_RECEIVE, 0);
      else
        status = gscSio4PerformDma( bn+1, ch+1, buffer, transferCount, GSC_DATA_RECEIVE);
      //printf("checksingle rx dma: status =0x%x, transferCount = %d, savedTransferCount = %d\n", status, transferCount, savedTransferCount);
    }

    if (transferQueue[*from].IsBufferLogicalAddr == TRUE)
    {
      transferQueue[ *from].physBuffer.PhysicalAddr += transferCount;
      transferQueue[ *from].physBuffer.CpuPhysical += transferCount;
      transferQueue[ *from].physBuffer.UserAddr += transferCount;
      transferQueue[ *from].physBuffer.Size -= transferCount;
    }
    // Adjust the remaining size
    transferQueue[ *from].size -= transferCount;

    // If there is only a single byte left or we are flushing
    if ((savedTransferCount > transferCount) &&
      ((transferQueue[ *from].size == 1) || (transferQueue[ *from].forceFlush)))
    {
      do
      {
        // Read the single byte from the fifo and put it into the buffer
        fpgaRead( bn+1, (((ch+1) << 4) | 0x08), &lastByte);
        if (transferQueue[*from].IsBufferLogicalAddr == TRUE)
        {
          *((char *)transferQueue[ *from].physBuffer.UserAddr) = (char)lastByte;
          transferQueue[ *from].physBuffer.UserAddr++;
        }
        else
          *transferQueue[ *from].buffer++ = (char)lastByte;

        // Account for the read
        //transferQueue[ *from].size--;
      } while (--transferQueue[ *from].size > 0);
    }

    // Clear the force flag
    transferQueue[ *from].forceFlush = FALSE;

    // If there is nothing left to transfer
    if (transferQueue[ *from].size == 0x00)
    {
      // Increment to pointer
      *from = *from + 1;
      if (*from == MAXIMUM_DMA_QUEUE_ITEMS) *from = 0;

      // Flag that we just finished/removed a transfer
      *loopDelay = 1;

      // Perform the callback if one was specified, only if not in Packet Framing Mode
      /*
      if ((boardInformation[bn].boardType != GSC_BOARD_TYPE_ANALOG) &&
          (boardInformation[bn].channelPacketFraming[ch] != GSC_ENABLED))
        setCompletionPending( bn+1, ch+1, 1);
      */
    }

    // Otherwise, flag to start the timer
    else
    {
      // If less than 1/4 fifo left to do
      if (transferQueue[ *from].size < fifoSize/4)
      {
        // Determine the time to interrupt
#ifndef PLX_LINUX
        *loopDelay = (int)(transferQueue[ *from].size *
                           WIN32_CHECKDMA_DELAY_MULTIPLIER/boardInformation[ bn].bytesPerMs[ ch]);
#else
        *loopDelay = (int)(transferQueue[ *from].size *
                           LINUX_CHECKDMA_DELAY_MULTIPLIER/boardInformation[ bn].bytesPerMs[ ch]);
#endif
      }
      else
      {
        // Determine the time to interrupt
        *loopDelay = (int)(maximumDelay/boardInformation[ bn].bytesPerMs[ ch]);
      }
    }
  }

  // Log the transfer if enabled
#ifdef DATA_LOGGING
  // Get performance counter frequency and convert it to counts/ms
  QueryPerformanceFrequency( &frequency);
  frequency.QuadPart /= 100000;

  // Get the current time and log it
  QueryPerformanceCounter( &currentTime);
  rxDataLog[rxDataLogPointer].time = (int)currentTime.QuadPart;

  // Calculate the delta from last transfer
  if (rxDataLogPointer != 0)
    rxDataLog[rxDataLogPointer].delta = (int)((rxDataLog[rxDataLogPointer].time -
                                               rxDataLog[rxDataLogPointer-1].time)/frequency.QuadPart);
  else
    rxDataLog[rxDataLogPointer].delta = 0;

  // Save the other info
  rxDataLog[rxDataLogPointer].size=transferCount;
  rxDataLog[rxDataLogPointer].buffer = buffer;
  rxDataLog[rxDataLogPointer].reason = reason;
  rxDataLog[rxDataLogPointer].timerDelay = *loopDelay;

  // Bump the pointer
  if (++rxDataLogPointer >= MAXIMUM_DATALOG_ENTRIES) rxDataLogPointer=MAXIMUM_DATALOG_ENTRIES-1 ;
#endif

  EndCriticalSection( qmutex);
  return 0x00;
}


//-------------------------------------------------------- gscSio4RxDmaHandler
//
//  This routine processes the Rx DMA request queue
//
#ifdef PLX_LINUX
void *gscSio4RxDmaHandler(void * pParam)
{
  // Get the message ID
  long int messageId = *(long int *)pParam;

  // Get the board number
  int bn = ((messageId >> 4) & 0x0f) - 1;
  int bdRefCount = ((messageId >> 12) & 0x0f);
  int ch = 0;

  int savedRxFifoSize[NUMBER_OF_CHANNELS];
  int loopDelayInt;
  double maximumDelay;

  // Thread Message Buffer
  THREAD_MESSAGE threadMessage;

  // Initialize the transfer queue pointers and FIFO sizes
  for (ch=0; ch<NUMBER_OF_CHANNELS; ch++)
  {
    if (bdRefCount == 1)
    {
      // Rx Queue pointers start at 0
      rxDmaQueueTo[ bn][ ch] = 0;
      rxDmaQueueFrom[ bn][ ch] = 0;
    }
    GscSio4ChannelFifoSizes(bn+1, ch+1, &savedRxFifoSize[ch]);
    savedRxFifoSize[ch] = (savedRxFifoSize[ch] >> 16) & 0xffff;
  }

  if (bdRefCount == 1)
  {
    // Initialize the queue entry ID variable
    dmaRequestQueueId[ bn] = 0x00;

    // Initialize the data log pointer
    rxDataLogPointer = 0;
  }

  // Loop until requested to abort
  while (TRUE)
  {
    // Wait for something to happen
    if (msgrcv(boardInformation[bn].dmaQueueId,(void *)&threadMessage, sizeof(int), messageId, 0) == -1)
    {
      pthread_exit((void *)THREAD_STATUS(GSC_THREAD_ERROR_EXIT,0,0,GSC_THREAD_RX_DMA));
    }

    // Handle the proper command
    if ((threadMessage.command & 0x0f) == GSC_THREAD_ABORT_CMD)
    {
      pthread_exit((void*)THREAD_STATUS(GSC_THREAD_NORMAL_EXIT,0,0,GSC_THREAD_RX_DMA));
    }

    ch = ((threadMessage.command & 0xf0) >> 4) - 1;

    // Print the received command
    //printf("RxDma: Command Received = %08X, for channel %d\n", threadMessage.command,
    //           ch);

    // Clear the loop delay
    loopDelayInt = 0;

    // Determine maximum delay time (in 1us intervals)
    maximumDelay = (savedRxFifoSize[ch]/ 4) * LINUX_CHECKDMA_DELAY_MULTIPLIER;

    // See if there is anything to do
    checkSingleRxChannel( bn, ch, threadMessage.command, savedRxFifoSize[ch],
      (int)maximumDelay, &loopDelayInt);

    // Loop until transfer is complete
    if (loopDelayInt <= 0)
    {
      // See if we need to start the timers again (negative numbers are used for timers)
      if (loopDelayInt < 0)
      {
        // Wait the desired time
        usleep( -loopDelayInt);
        if (msgsnd(boardInformation[bn].dmaQueueId, (void *)&threadMessage, sizeof(int), 0) == -1)
        {
          return (void *)NULL;
        }
      }
    }
  }
}
#else
DWORD WINAPI gscSio4RxDmaHandler(LPVOID pParam)
{
  int bn, ch;		// Board number and channel number (0 based)
  int savedRxFifoSize[ NUMBER_OF_CHANNELS];
  HANDLE	hEvent[9];
  LARGE_INTEGER loopDelay;
  DWORD eventStatus;
  int loopDelayInt;
  double maximumDelay[ NUMBER_OF_CHANNELS];
  int numberOfEvents = 0x00;
  int bdRefCount;
  BOOL sts;
  TCHAR * szSD = TEXT("D:(A;NP;0x781f0003;;;WD)");
  PULONG nSize = 0;
  int i = 0;

  // Get the proper board number
  bn = ((*(int *)pParam) & 0x0f) - 1;

  // Get the board ref count
  bdRefCount = (*(int *)pParam) >> 4;

  // Initialize the queue entry ID variable
  if (bdRefCount == 1)
  {
    dmaRequestQueueId[ bn] = 0x00;
  }

  // Initialize the transfer queue pointers and FIFO sizes
  for (ch=0; ch<NUMBER_OF_CHANNELS; ch++)
  {
    if (bdRefCount == 1)
    {
      // Rx Queue pointers start at 0
      rxDmaQueueTo[ bn][ ch] = 0;
      rxDmaQueueFrom[ bn][ ch] = 0;
    }

    // Get FIFO sizes
    //		GscSio4ChannelFifoSizes(bn+1, ch+1, &savedRxFifoSize[ch]);
    //		savedRxFifoSize[ch] = (savedRxFifoSize[ch] >> 16) & 0xffff;
    savedRxFifoSize[ch] = boardInformation[ bn].rxFifoSizes[ ch];

    // Determine maximum delay time (in 100ns intervals)
    maximumDelay[ ch] = ((savedRxFifoSize[ch]/4) * WIN32_CHECKDMA_DELAY_MULTIPLIER);

    // Generate the Start Rx Event names for this channel
    sprintf( rxStartEvent[ bn][ ch], "RxStart_%d_%d_%d", bn+1, GetCurrentProcessId(), ch+1);
  }

  // Initialize the data log pointer
  rxDataLogPointer = 0;

  // Setup security descriptor for events
  // Convert the string to the security descriptor binary and return
  sts = ConvertStringSecurityDescriptorToSecurityDescriptor(
        szSD,                       // The ACE strings
        SDDL_REVISION_1,            // Standard revision level
        &(sa.lpSecurityDescriptor), // Pointer to the converted security descriptor
        nSize);                     // The size in byte the converted security descriptor
  sa.nLength = sizeof( SECURITY_ATTRIBUTES );
  sa.bInheritHandle = FALSE;

  // Get the handle to the abort object
  hEvent[numberOfEvents] = CreateEvent( &sa, TRUE, FALSE, threadAbort[ bn]);
  ResetEvent( hEvent[numberOfEvents++] );
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("RxDmaHandler: rx abort event creation failed... 0x%x\n", ew);
  }

  // Create a timer object for each channel
  hEvent[numberOfEvents++] = CreateWaitableTimerA(&sa,FALSE,NULL); // "DmaRxTimer_1_1");
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("RxDmaHandler: rx timer 1_1 creation failed... 0x%x\n", ew);
  }
  hEvent[numberOfEvents++] = CreateWaitableTimerA(&sa,FALSE,NULL); // "DmaRxTimer_1_2");
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("RxDmaHandler: rx timer 1_2 creation failed... 0x%x\n", ew);
  }
  hEvent[numberOfEvents++] = CreateWaitableTimerA(&sa,FALSE,NULL); // "DmaRxTimer_1_3");
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("RxDmaHandler: rx timer 1_3 creation failed... 0x%x\n", ew);
  }
  hEvent[numberOfEvents++] = CreateWaitableTimerA(&sa,FALSE,NULL); // "DmaRxTimer_1_4");
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("RxDmaHandler: rx timer 1_4 creation failed... 0x%x\n", ew);
  }

  // Create the rx start event objects
  hEvent[numberOfEvents++] = CreateEvent(&sa, TRUE, FALSE, rxStartEvent[bn][0]);
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("RxDmaHandler: rx start event 1 creation failed... 0x%x\n", ew);
  }
  hEvent[numberOfEvents++] = CreateEvent(&sa, TRUE, FALSE, rxStartEvent[bn][1]);
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("RxDmaHandler: rx start event 2 creation failed... 0x%x\n", ew);
  }
  hEvent[numberOfEvents++] = CreateEvent(&sa, TRUE, FALSE, rxStartEvent[bn][2]);
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("RxDmaHandler: rx start event 3 creation failed... 0x%x\n", ew);
  }
  hEvent[numberOfEvents++] = CreateEvent(&sa, TRUE, FALSE, rxStartEvent[bn][3]);
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("RxDmaHandler: rx start event 4 creation failed... 0x%x\n", ew);
  }

  // Loop until requested to abort
  while (TRUE)
  {
    // Wait for something to happen
    eventStatus = WaitForMultipleObjects( numberOfEvents, hEvent, FALSE, INFINITE);
    if (eventStatus == WAIT_FAILED)
    {
      DWORD ew = GetLastError();
      printf("RxDmaHandler: wait failed... 0x%x\n", ew);
      //Sleep(50);
    }

    // Handle the proper event
    switch (eventStatus)
    {
      // Abort the thread
    case WAIT_OBJECT_0:
      for(i=0; i < numberOfEvents; i++)
        CloseHandle( hEvent[i] );
      return THREAD_STATUS(GSC_THREAD_NORMAL_EXIT,0,0,GSC_THREAD_RX_DMA);

      // Timer expirations
    case WAIT_OBJECT_0+1:
    case WAIT_OBJECT_0+2:
    case WAIT_OBJECT_0+3:
    case WAIT_OBJECT_0+4:

      // Get the channel # (0-3)
      ch = eventStatus-WAIT_OBJECT_0-1;
      break;

      // Start Rx Event
    case WAIT_OBJECT_0+5:
    case WAIT_OBJECT_0+6:
    case WAIT_OBJECT_0+7:
    case WAIT_OBJECT_0+8:

      // Reset the event
      ResetEvent( hEvent[ eventStatus-WAIT_OBJECT_0]);
      //printf("rxdmahandler: got start event\n");

      // Get the channel # (0-3)
      ch = eventStatus-WAIT_OBJECT_0-5;
      break;
    }

    // See if the event really has anything to do
    checkSingleRxChannel( bn, ch, eventStatus, savedRxFifoSize[ch], maximumDelay[ch], &loopDelayInt);

    // See if we need to start the timers again (negative numbers are used for timers)
    if (loopDelayInt < 0)
    {
      // Determine the time to interrupt
      loopDelay.QuadPart = loopDelayInt;

      // Start the timer
      SetWaitableTimer( hEvent[ch+1], &loopDelay, 0, NULL, NULL, 0);
    }

    // Otherwise, see if we just finished a transfer and should check the fifo for another one
    else if (loopDelayInt > 0)
    {
      SetEvent( hEvent[ ch+5]);
    }
  }
}
#endif

//------------------------------------------------------- checkSingleTxChannel
//
//  This routine is called by the Tx DMA Handler. It checks
// a single channel to see if a transfer is needed and performs
// the transfer if possible.
//
int checkSingleTxChannel( int bn, int ch, int reason, int fifoSize, double maximumDelay, int* loopDelay)
{
  int to, *from, skip = FALSE;
  PDMA_TRANSFER_QUEUE transferQueue;
  int fifoCount, transferCount=0;
  char *buffer = NULL;
  int status = 0;

#ifdef DATA_LOGGING
  int tcbSent = 0;
  LARGE_INTEGER frequency, currentTime;
#endif

  BeginCriticalSection( qmutex);

  // Get the to, from, and queue pointers
  to = txDmaQueueTo[ bn][ ch];
  from = &txDmaQueueFrom[ bn][ ch];
  transferQueue = (PDMA_TRANSFER_QUEUE)txDmaQueue[ bn][ ch];

  // Assume we are going to do nothing
  *loopDelay = 0x00;

  // See if there is anything to do
  if (to != *from)
  {
    // If the TCB Needed flag is set, make sure that 4 bytes are available in the FIFO
    if (transferQueue[ *from].tcbNeeded == TRUE)
    {
      // Get the available space in the fifo and
      GscSio4ChannelFifoCounts(bn+1, ch+1, &fifoCount);
      fifoCount &= 0xffff;
      fifoCount = fifoSize - fifoCount;

      // If 4 bytes are available
      if (fifoCount >= 4)
      {
        // Write the 4 TCB bytes
        fpgaWrite( bn+1, DATA_FIFO_BASE_REG + (ch * 0x10), transferQueue[ *from].tcbBytes[0]);
        fpgaWrite( bn+1, DATA_FIFO_BASE_REG + (ch * 0x10), transferQueue[ *from].tcbBytes[1]);
        fpgaWrite( bn+1, DATA_FIFO_BASE_REG + (ch * 0x10), transferQueue[ *from].tcbBytes[2]);
        fpgaWrite( bn+1, DATA_FIFO_BASE_REG + (ch * 0x10), transferQueue[ *from].tcbBytes[3]);
        transferQueue[ *from].tcbNeeded = FALSE;
      }
      else
      {
        skip = TRUE;
      }
    }

    // If we are not skipping this transfer (waiting for TCB room)
    if (skip == FALSE)
    {
      // Get the available space in the fifo
      GscSio4ChannelFifoCounts(bn+1, ch+1, &fifoCount);
      fifoCount &= 0xffff;

      // Convert from "bytes in the fifo" to "bytes available in the fifo"
      fifoCount = fifoSize - fifoCount;

      // The transfer count is the smaller of the available space and the requested amount
      transferCount = (fifoCount>=transferQueue[ *from].size)?transferQueue[ *from].size:fifoCount;

      if ((transferQueue[ *from].size - transferCount) != 0)
        transferCount = transferCount - (transferCount % 4);


      // Get the buffer address and update it for next time
      if (transferQueue[*from].IsBufferLogicalAddr != TRUE)
      {
        buffer = transferQueue[ *from].buffer;
        transferQueue[ *from].buffer += transferCount;
      }

      // Do the transfer
      if (transferCount != 0)
      {
        if (transferQueue[*from].IsBufferLogicalAddr == TRUE)
          status = gscPerformPlxPhysDma( bn+1, ch+1, &transferQueue[ *from].physBuffer, transferCount, GSC_DATA_TRANSMIT, 0);
        else
          status = gscSio4PerformDma( bn+1, ch+1, buffer, transferCount, GSC_DATA_TRANSMIT);
        //printf("tx dma: transfer count = %d,  status =0x%x\n", transferCount, status);
      }

      if (transferQueue[*from].IsBufferLogicalAddr == TRUE)
      {
        transferQueue[ *from].physBuffer.PhysicalAddr += transferCount;
        transferQueue[ *from].physBuffer.CpuPhysical += transferCount;
        transferQueue[ *from].physBuffer.UserAddr += transferCount;
        transferQueue[ *from].physBuffer.Size -= transferCount;
      }
      // Adjust the remaining size
      transferQueue[ *from].size -= transferCount;
    }

    // If there is nothing left to transfer
    if (transferQueue[ *from].size == 0x00)
    {
      // Increment to pointer
      *from = *from + 1;
      if (*from == MAXIMUM_DMA_QUEUE_ITEMS) *from = 0;

      // Flag that we just finished/removed a transfer
      *loopDelay = 1;

      // Perform the callback if one was specified
      //setCompletionPending( bn+1, ch+1, 0);
    }

    // Otherwise, flag to start the timer
    else
    {
      // Determine the time to interrupt
      *loopDelay = (int)(maximumDelay/boardInformation[ bn].bytesPerMs[ ch]);
    }
  }

  // Log the transfer if enabled
#ifdef DATA_LOGGING
  // Get performance counter frequency and convert it to counts/ms
  QueryPerformanceFrequency( &frequency);
  frequency.QuadPart /= 100000;

  // Get the current time and log it
  QueryPerformanceCounter( &currentTime);
  txDataLog[txDataLogPointer].time = (int)currentTime.QuadPart;

  // Calculate the delta from last transfer
  if (txDataLogPointer != 0)
    txDataLog[txDataLogPointer].delta = (int)((txDataLog[txDataLogPointer].time -
                                               txDataLog[txDataLogPointer-1].time)/frequency.QuadPart);
  else
    txDataLog[txDataLogPointer].delta = 0;

  // Save the other info
  txDataLog[txDataLogPointer].size=transferCount;
  txDataLog[txDataLogPointer].buffer = buffer;
  txDataLog[txDataLogPointer].reason = reason;
  txDataLog[txDataLogPointer].timerDelay = *loopDelay;
  txDataLog[txDataLogPointer].fifoCount = fifoCount;
  txDataLog[txDataLogPointer].tcbSent = tcbSent;

  // Bump the pointer
  if (++txDataLogPointer >= MAXIMUM_DATALOG_ENTRIES) txDataLogPointer=MAXIMUM_DATALOG_ENTRIES-1 ;
#endif

  EndCriticalSection( qmutex);
  return 0x00;
}


//-------------------------------------------------------- gscSio4TxDmaHandler
//
//  This routine processes the Tx DMA request queue
//
#ifdef PLX_LINUX
void *gscSio4TxDmaHandler(void * pParam)
{
  // Get the message ID
  long int messageId = *(long int*)pParam;

  // Get the board and channel number
  int bn = ((messageId >> 4) & 0x0f) - 1;
  int bdRefCount = ((messageId >> 12) & 0x0f);
  int ch = 0;

  int savedTxFifoSize;
  int loopDelayInt;
  double maximumDelay;

  // Thread Message Buffer
  THREAD_MESSAGE threadMessage;

  // Tx Queue pointers start at 0
  if (bdRefCount == 1)
  {
    txDmaQueueTo[ bn][ ch] = 0;
    txDmaQueueFrom[ bn][ ch] = 0;
  }

  // Get FIFO sizes
  GscSio4ChannelFifoSizes(bn+1, ch+1, &savedTxFifoSize);
  savedTxFifoSize &= 0xffff;

  // Initialize the data log pointer
  txDataLogPointer = 0;

  // Loop until requested to abort
  while (TRUE)
  {
    // Wait for something to happen
    if (msgrcv(boardInformation[bn].dmaQueueId,(void *)&threadMessage, sizeof(int), messageId, 0) == -1)
    {
      pthread_exit((void *)THREAD_STATUS(GSC_THREAD_ERROR_EXIT,0,0,GSC_THREAD_TX_DMA));
    }

    // Check for an Abort command
    if (threadMessage.command == GSC_THREAD_ABORT_CMD)
    {
      pthread_exit((void*)THREAD_STATUS(GSC_THREAD_NORMAL_EXIT,0,0,GSC_THREAD_TX_DMA));
    }

    ch = ((threadMessage.command & 0xf0) >> 4) - 1;

    // Clear the loop delay
    loopDelayInt = 0;

    // Determine maximum delay time (in us intervals)
    maximumDelay = (savedTxFifoSize/2) * LINUX_CHECKDMA_DELAY_MULTIPLIER;

    // Loop until transfer is complete
    while (loopDelayInt <= 0)
    {
      // See if the event really has anything to do
      checkSingleTxChannel( bn, ch, threadMessage.command, savedTxFifoSize, (int)maximumDelay, &loopDelayInt);

      // See if we need to start the timers again (negative numbers are used for timers)
      if (loopDelayInt < 0)
      {
        // Wait the desired time
        usleep(-loopDelayInt);
      }
    }
  }
}

#else

DWORD WINAPI gscSio4TxDmaHandler(LPVOID pParam)
{
  int bn, ch;		// Board number and channel number (0 based)
  int savedTxFifoSize[ NUMBER_OF_CHANNELS];
  HANDLE	hEvent[9];
  LARGE_INTEGER loopDelay;
  DWORD eventStatus;
  int loopDelayInt;
  double maximumDelay[ NUMBER_OF_CHANNELS];
  int numberOfEvents = 0x00;
  int bdRefCount;
  SECURITY_ATTRIBUTES sa;
  BOOL sts;
  TCHAR * szSD = TEXT("D:(A;NP;0x781f0003;;;WD)");
  PULONG nSize = 0;
  int i = 0;


  // Get the proper board number
  bn = ((*(int*)pParam) & 0x0f) - 1;

  // Get the board ref count
  bdRefCount = (*(int *)pParam) >> 4;

  // Initialize the transfer queue pointers and FIFO sizes
  for (ch=0; ch<NUMBER_OF_CHANNELS; ch++)
  {
    if (bdRefCount == 1)
    {
      // Tx Queue pointers start at 0
      txDmaQueueTo[ bn][ ch] = 0;
      txDmaQueueFrom[ bn][ ch] = 0;
    }

    // Get FIFO sizes
    //		GscSio4ChannelFifoSizes(bn+1, ch+1, &savedTxFifoSize[ch]);
    //		savedTxFifoSize[ch] &= 0xffff;
    savedTxFifoSize[ ch] = boardInformation[ bn].txFifoSizes[ ch];

    // Determine maximum delay time (in 100ns intervals)
    maximumDelay[ ch] = (savedTxFifoSize[ch]/2) * WIN32_CHECKDMA_DELAY_MULTIPLIER;

    // Generate the Start Tx Event name for this channel
    sprintf( txStartEvent[ bn][ ch], "TxStart_%d_%d_%d", bn+1, GetCurrentProcessId(), ch+1);
  }

  // Initialize the data log pointer
  txDataLogPointer = 0;

  // Setup security descriptor for events
  // Convert the string to the security descriptor binary and return
  sts = ConvertStringSecurityDescriptorToSecurityDescriptor(
        szSD,                       // The ACE strings
        SDDL_REVISION_1,            // Standard revision level
        &(sa.lpSecurityDescriptor), // Pointer to the converted security descriptor
        nSize);                     // The size in byte the converted security descriptor
  sa.nLength = sizeof( SECURITY_ATTRIBUTES );
  sa.bInheritHandle = FALSE;

  // Get the handle to the abort object
  hEvent[numberOfEvents] = CreateEvent( &sa, TRUE, FALSE, threadAbort[ bn]);
  if (hEvent[numberOfEvents] == NULL)
  {
    DWORD ew = GetLastError();
    printf("TxDmaHandler: tx abort event creation failed... 0x%x\n", ew);
  }
  ResetEvent( hEvent[numberOfEvents++] );

  // Create a timer object for each channel
  hEvent[numberOfEvents++] = CreateWaitableTimerA(NULL,FALSE,NULL); // "DmaTxTimer_1_1");
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("TxDmaHandler: tx timer 1_1 creation failed... 0x%x\n", ew);
  }
  hEvent[numberOfEvents++] = CreateWaitableTimerA(&sa,FALSE,NULL); // "DmaTxTimer_1_2");
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("TxDmaHandler: tx timer 1_2 creation failed... 0x%x\n", ew);
  }
  hEvent[numberOfEvents++] = CreateWaitableTimerA(&sa,FALSE,NULL); // "DmaTxTimer_1_3");
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("TxDmaHandler: tx timer 1_3 creation failed... 0x%x\n", ew);
  }
  hEvent[numberOfEvents++] = CreateWaitableTimerA(&sa,FALSE,NULL); // "DmaTxTimer_1_4");
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("TxDmaHandler: tx timer 1_4 creation failed... 0x%x\n", ew);
  }

  // Create the tx start event objects
  hEvent[numberOfEvents++] = CreateEvent(&sa, TRUE, FALSE, txStartEvent[bn][0]);
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("TxDmaHandler: tx start event 1 creation failed... 0x%x\n", ew);
  }
  hEvent[numberOfEvents++] = CreateEvent(&sa, TRUE, FALSE, txStartEvent[bn][1]);
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("TxDmaHandler: tx start event 2 creation failed... 0x%x\n", ew);
  }
  hEvent[numberOfEvents++] = CreateEvent(&sa, TRUE, FALSE, txStartEvent[bn][2]);
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("TxDmaHandler: tx start event 3 creation failed... 0x%x\n", ew);
  }
  hEvent[numberOfEvents++] = CreateEvent(&sa, TRUE, FALSE, txStartEvent[bn][3]);
  if (hEvent[numberOfEvents - 1] == NULL)
  {
    DWORD ew = GetLastError();
    printf("TxDmaHandler: tx start event 4 creation failed... 0x%x\n", ew);
  }

  // Loop until requested to abort
  while (TRUE)
  {
    // Wait for something to happen
    eventStatus = WaitForMultipleObjects( numberOfEvents, hEvent, FALSE, INFINITE);
    if (eventStatus == WAIT_FAILED)
    {
      DWORD ew = GetLastError();
      //for (i=0; i < numberOfEvents; i++)
      //  if (hEvent[i] == NULL) printf("hEvent[%d] is null\n", i);
      printf("TxDmaHandler: wait failed... 0x%x\n", ew);
      //Sleep(50);
    }

    // Handle the proper event
    switch (eventStatus)
    {
      // Abort the thread
    case WAIT_OBJECT_0:
      for(i=0; i < numberOfEvents; i++)
        CloseHandle( hEvent[i] );
      return THREAD_STATUS(GSC_THREAD_NORMAL_EXIT,0,0,GSC_THREAD_TX_DMA);

      // Timer expirations
    case WAIT_OBJECT_0+1:
    case WAIT_OBJECT_0+2:
    case WAIT_OBJECT_0+3:
    case WAIT_OBJECT_0+4:

      // Get the channel # (0-3)
      ch = eventStatus-WAIT_OBJECT_0-1;
      break;

      // Start Tx Event
    case WAIT_OBJECT_0+5:
    case WAIT_OBJECT_0+6:
    case WAIT_OBJECT_0+7:
    case WAIT_OBJECT_0+8:

      // Reset the event
      ResetEvent( hEvent[ eventStatus-WAIT_OBJECT_0]);

      // Get the channel # (0-3)
      ch = eventStatus-WAIT_OBJECT_0-5;
      break;
    }

    // See if the event really has anything to do
    checkSingleTxChannel( bn, ch, eventStatus, savedTxFifoSize[ch], maximumDelay[ch], &loopDelayInt);

    // See if we need to start the timers again (negative numbers are used for timers)
    if (loopDelayInt < 0)
    {
      // Determine the time to interrupt
      loopDelay.QuadPart = loopDelayInt;

      // Start the timer
      SetWaitableTimer( hEvent[ch+1], &loopDelay, 0, NULL, NULL, 0);
    }

    // Otherwise, see if we just finished a transfer and should check the fifo for another one
    else if (loopDelayInt > 0)
    {
      SetEvent( hEvent[ ch+5]);
    }
  }
}

#endif


//------------------------------------------------ GscSio4ChannelFlushTransfer
//
//  This routine will cause any data that is in the FIFO
// to be transfered to memory (via DMA).
//
int GscSio4ChannelFlushTransfer( int boardNumber, int channel, int id)
{
  int status = 0x00;
  PDMA_TRANSFER_QUEUE	transferQueue;
  int to, from;
  int iFlushChecks = 0;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  BeginCriticalSection( qmutex);

  // If the Transfer ID is even, check the Tx queue
  if ((id & 0x01) == 0)
  {
    to = txDmaQueueTo[ boardNumber-1][ channel-1];
    from = txDmaQueueFrom[ boardNumber-1][ channel-1];
    while (to != from)
    {
      // Point to the queue item
      transferQueue = &txDmaQueue[ boardNumber-1][ channel-1][from++];
      if (transferQueue->id == id)
      {
        transferQueue->forceFlush = TRUE;
        EndCriticalSection( qmutex);
        while((transferQueue->forceFlush != FALSE) && (iFlushChecks++ < 10))
          Sleep(10);
        return status;
      }
    }
  }

  // Otherwise, check the Rx queue
  else
  {
    to = rxDmaQueueTo[ boardNumber-1][ channel-1];
    from = rxDmaQueueFrom[ boardNumber-1][ channel-1];
    while (to != from)
    {
      // Point to the queue item
      transferQueue = &rxDmaQueue[ boardNumber-1][ channel-1][from++];
      if (transferQueue->id == id)
      {
        transferQueue->forceFlush = TRUE;
        EndCriticalSection( qmutex);
        while((transferQueue->forceFlush != FALSE) && (iFlushChecks++ < 10))
          Sleep(10);
        return status;
      }
      if (from == MAXIMUM_DMA_QUEUE_ITEMS) from = 0;
    }
  }
  EndCriticalSection( qmutex);
  return status;
}

//----------------------------------------------- GscSio4ChannelRemoveTransfer
//
//  This routine will remove either a single transfer,
// or all of the entries, from the DMA transfer queue.
//
int GscSio4ChannelRemoveTransfer( int boardNumber, int channel, int id, int *bytesTransferred)
{
  int status = 0x00;
  int bn = boardNumber - 1;
  int ch = channel - 1;
  PDMA_TRANSFER_QUEUE	transferQueue;
  int to, from;

  // Initialize bytesTransferred
  if (bytesTransferred) *bytesTransferred = -1;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  BeginCriticalSection( qmutex);

  // If the ID is -1, flush everything
  if (id == -1)
  {
    // Reset the queue pointers
    txDmaQueueTo[ bn][ ch] = 0;
    txDmaQueueFrom[ bn][ ch] = 0;
    rxDmaQueueTo[ bn][ ch] = 0;
    rxDmaQueueFrom[ bn][ ch] = 0;
  }

  // Otherwise, just remove the single entry
  else
  {
    // If the Transfer ID is even, check the Tx queue
    if ((id & 0x01) == 0)
    {
      to = txDmaQueueTo[ boardNumber-1][ channel-1];
      from = txDmaQueueFrom[ boardNumber-1][ channel-1];
      while (to != from)
      {
        // Point to the queue item
        transferQueue = &txDmaQueue[ boardNumber-1][ channel-1][from++];
        if (transferQueue->id == id)
        {
          if (bytesTransferred) *bytesTransferred = transferQueue->originalSize - transferQueue->size;
          transferQueue->size = 0;
          EndCriticalSection( qmutex);
          return status;
        }
        if (from == MAXIMUM_DMA_QUEUE_ITEMS) from = 0;
      }
    }

    // Otherwise, check the Rx queue
    else
    {
      to = rxDmaQueueTo[ boardNumber-1][ channel-1];
      from = rxDmaQueueFrom[ boardNumber-1][ channel-1];
      while (to != from)
      {
        // Point to the queue item
        transferQueue = &rxDmaQueue[ boardNumber-1][ channel-1][from++];
        if (transferQueue->id == id)
        {
          if (bytesTransferred) *bytesTransferred = transferQueue->originalSize - transferQueue->size;
          transferQueue->size = 0;
          EndCriticalSection( qmutex);
          return status;
        }
        if (from == MAXIMUM_DMA_QUEUE_ITEMS) from = 0;
      }
    }
  }
  EndCriticalSection( qmutex);
  return status;
}


//------------------------------------------------ GscSio4ChannelQueryTransfer
//
//  This routine is used to determine if a DMA operation
// is still pending or is complete. It returns the number
// of bytes remaining to be transfered in "stat". (0 if
// operation is completed.)
//
int GscSio4ChannelQueryTransfer(int boardNumber, int channel, int *stat, int id)
{
  int status = 0x00;
  PDMA_TRANSFER_QUEUE	transferQueue;
  int to, from;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  BeginCriticalSection( qmutex);

  // If the Transfer ID is even, check the Tx Queue
  if ((id & 0x01) == 0)
  {
    to = txDmaQueueTo[ boardNumber-1][ channel-1];
    from = txDmaQueueFrom[ boardNumber-1][ channel-1];
    while (to != from)
    {
      // Point to the queue item
      transferQueue = &txDmaQueue[ boardNumber-1][ channel-1][from];

      // If the ids match, return the count
      if (transferQueue->id == id)
      {
        *stat = transferQueue->size;
        EndCriticalSection( qmutex);
        return status;
      }

      // Point to the next item
      if (++from == MAXIMUM_DMA_QUEUE_ITEMS) from = 0;
    }
  }

  // Otherwise, check the Rx Queue
  else
  {
    to = rxDmaQueueTo[ boardNumber-1][ channel-1];
    from = rxDmaQueueFrom[ boardNumber-1][ channel-1];
    while (to != from)
    {
      // Point to the queue item
      transferQueue = &rxDmaQueue[ boardNumber-1][ channel-1][from];

      // If the ids match, return the count
      if (transferQueue->id == id)
      {
        *stat = transferQueue->size;
        EndCriticalSection( qmutex);
        return status;
      }

      // Point to the next item
      if (++from == MAXIMUM_DMA_QUEUE_ITEMS) from = 0;
    }
  }

  // Could not find match, must be done
  *stat = 0x0000;
  EndCriticalSection( qmutex);
  return status;
}


//---------------------------------------------- GscSio4ChannelWaitForTransfer
//
//  This routine waits until a transfer completes or
// the timeout expires.
//
//  The routine returns 0x00 if the transfer completes
// or a non-zero error otherwise.
//
#ifdef WIN32
int GscSio4ChannelWaitForTransfer_A(int boardNumber, int channel, int timeout, int id, int *bytesTransferred)
{
  LARGE_INTEGER frequency, currentTime, endTime;

  int status = 0x00, count, noTimeout = FALSE;

  // Assume no bytes will be transferred
  if (bytesTransferred) *bytesTransferred = -1;

  // Get performance counter frequency and convert it to counts/ms
  QueryPerformanceFrequency( &frequency);
  frequency.QuadPart /= 1000;

  // If the timeout is 0, don't do a timeout
  if (timeout == 0x0000)
  {
    noTimeout = TRUE;
  }
  else if (timeout < 50) timeout = 50;


  // Otherwise, determine the time to terminate the loop
  {
    // Get the current time
    QueryPerformanceCounter( &currentTime);

    // Calculate the end time relative to the current time
    endTime.QuadPart = currentTime.QuadPart + (timeout * frequency.QuadPart);
  }

  // Loop until transfer complete or timeout
  do
  {
    // Get the number of bytes left to transfer
    status = GscSio4ChannelQueryTransfer( boardNumber, channel, &count, id);
    if (status) return status;

    // Return if count is 0x00
    if (count == 0x0000) return status;

    // Allow transfer to continue
    Sleep(0);

    // Get the current time
    QueryPerformanceCounter( &currentTime);

  } while ( noTimeout || (currentTime.QuadPart < endTime.QuadPart));

  // Timeout, flush transfer from queue and return count and error
  GscSio4ChannelFlushTransfer( boardNumber, channel, id);
  GscSio4ChannelRemoveTransfer( boardNumber, channel, id, bytesTransferred);
  return GscApiOperationTimeout;
}
#endif

//---------------------------------------------- GscSio4ChannelWaitForTransfer_B
//
//  This routine waits until a transfer completes or
// the timeout expires.
//
//  The routine returns 0x00 if the transfer completes
// or a non-zero error otherwise.
//
#ifdef WIN32
int GscSio4ChannelWaitForTransfer_B(int boardNumber, int channel, int timeout, int id, int *bytesTransferred)
{
  char event[30];
  HANDLE hEvent;
  DWORD eventStatus;

  // Assume no bytes will be transferred
  if (bytesTransferred) *bytesTransferred = -1;

  // If the timeout is 0, don't do a timeout
  if (timeout == 0x0000) timeout = INFINITE;

  // Otherwise, make sure it is at least 50
  else if (timeout < 50) timeout = 50;

  // Create the transfer complete object (Will be set if it doesn't already exist
  sprintf(event, "GSC_Completed_%d_%d",boardNumber, id);
  hEvent = CreateEvent( NULL, TRUE, TRUE, event);

  // Wait for something to happen
  eventStatus = WaitForSingleObject(hEvent, (DWORD)timeout);

  // Reset the event
  ResetEvent( hEvent);

  // If event occured, return good status
  if (eventStatus == WAIT_OBJECT_0) return 0x00;

  // Timeout, flush transfer from queue and return count and error
  GscSio4ChannelFlushTransfer( boardNumber, channel, id);
  GscSio4ChannelRemoveTransfer( boardNumber, channel, id, bytesTransferred);
  return GscApiOperationTimeout;
}
#endif

//---------------------------------------------- GscSio4ChannelWaitForTransfer
//
//  This routine waits until a transfer completes or
// the timeout expires.
//
//  The routine returns 0x00 if the transfer completes
// or a non-zero error otherwise.
//
int GscSio4ChannelWaitForTransfer(int boardNumber, int channel, int timeout, int id, int *bytesTransferred)
{
	int				count;
	int				noTimeout	= FALSE;
	struct timeval	starttime	= { .tv_sec = 0, .tv_usec = 0 };
	int				status		= 0x00;
	struct timeval	stoptime	= { .tv_sec = 0, .tv_usec = 0 };
	long			usec		= 0;

	// Assume no bytes will be transferred
	if (bytesTransferred)
		*bytesTransferred	= -1;

	// If the timeout is 0, don't do a timeout
	if (timeout == 0x0000)
	{
		noTimeout	= TRUE;
	}
	else	// Otherwise, determine the time to terminate the loop
	{
		// Get the current time
		gettimeofday( &starttime, NULL );

		// Calculate the end time relative to the current time
		stoptime.tv_sec	= starttime.tv_sec + (timeout / 1000);
		usec			= starttime.tv_usec + (timeout - ((stoptime.tv_sec - starttime.tv_sec) * 1000)) * 1000;

		if (usec > 1000000L)
		{
			stoptime.tv_sec++;
			usec	-= 1000000L;
		}

		stoptime.tv_usec	= usec;
	}

	// Loop until transfer complete or timeout
	do
	{
		// Get the number of bytes left to transfer
		status	= GscSio4ChannelQueryTransfer( boardNumber, channel, &count, id);

		// Return on error or if count is 0 (transfer done)
		if ((status != 0x00) || (count == 0x0000))
		{
			return status;
		}

		// Pause to allow transfer to continue
		usleep( 500);

		// Get the current time
		gettimeofday( &starttime, NULL );
	}
	while (	(noTimeout) ||
			(starttime.tv_sec < stoptime.tv_sec) ||
			(starttime.tv_sec == stoptime.tv_sec && starttime.tv_usec < stoptime.tv_usec));

	// Timeout, flush transfer from queue and return count and error
	GscSio4ChannelFlushTransfer( boardNumber, channel, id);
	GscSio4ChannelRemoveTransfer( boardNumber, channel, id, bytesTransferred);
	return GscApiOperationTimeout;
}


//----------------------------------------------------------- gscAddDmaRequest
//
//  This routine adds a DMA request to the queue for later
// processing. A unique Transfer ID is created for each
// DMA request. Tx transfers are given even IDs and Rx
// transfers are given odd IDs.
//
int gscAddDmaRequest(int boardNumber, int channel, char *buffer, int count, int direction, int *id)
{
	int status = 0x00, newId;
	PDMA_TRANSFER_QUEUE	transferQueue;
	int *to, from, tempTo;
	int value = 0;
	int tempCount = 0;
	int oldCMR, oldCCSR;
	long int messageId;
	THREAD_MESSAGE  threadMessage;

	//printf("Add dma request: direction = %d\n", direction);
	BeginCriticalSection( qmutex);

	// Make sure the direction is valid
	switch (direction)
	{
		case GSC_DATA_TRANSMIT:

			to				= &txDmaQueueTo[ boardNumber-1][ channel-1];
			from			= txDmaQueueFrom[ boardNumber-1][ channel-1];
			transferQueue	= &txDmaQueue[ boardNumber-1][ channel-1][*to];
			newId			= dmaRequestQueueId[ boardNumber-1];
			messageId		= GSC_THREAD_TX_DMA_TYPE;
			break;

		case GSC_DATA_RECEIVE:

			to				= &rxDmaQueueTo[ boardNumber-1][ channel-1];
			from			= rxDmaQueueFrom[ boardNumber-1][ channel-1];
			transferQueue	= &rxDmaQueue[ boardNumber-1][ channel-1][*to];
			newId			= dmaRequestQueueId[ boardNumber-1] + 1;
			messageId		= GSC_THREAD_RX_DMA_TYPE;
			/* Check whether timetag will be inserted into Rx FIFO for each byte
			   and adjust count accordingly */
			status			= fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+((channel-1)<<4), &value);
			//if (status) return status;
			tempCount		= count;
			if (value & 0x04)
				tempCount	+= count * 3;

			if (value & 0x08)
				tempCount	+= count;

			count			= tempCount;
			break;

		default:

			EndCriticalSection( qmutex);
			return GscApiInvalidParameter;
	}

	// Account for using a new Transfer ID value
	dmaRequestQueueId[ boardNumber-1] += 2;

	// Setup the TCB for transfers if packet framing is enabled
	if ((direction == GSC_DATA_TRANSMIT) &&
		(boardInformation[ boardNumber-1].channelPacketFraming[ channel-1] == GSC_ENABLED))
	{
		// Read the CMR and CCSR
		status	= GscSio4ChannelRegisterRead(boardNumber, channel, USC_CMR, &oldCMR);

	    if (status)
		{
			EndCriticalSection( qmutex);
			return status;
		}

		status	= GscSio4ChannelRegisterRead(boardNumber, channel, USC_CCSR, &oldCCSR);

		if (status)
		{
			EndCriticalSection( qmutex);
			return status;
		}

		transferQueue->tcbNeeded	= TRUE;
		transferQueue->tcbBytes[0]	= oldCCSR & 0x1c;
		transferQueue->tcbBytes[1]	= (oldCMR>>8) & 0xf0;
		transferQueue->tcbBytes[2]	= count & 0xff;
		transferQueue->tcbBytes[3]	= (count>>8) & 0xff;
	}
	else	// Otherwise, just clear the flag
	{
		transferQueue->tcbNeeded	= FALSE;
	}

	// Put info into the proper transfer queue
	transferQueue->size					= count;
	transferQueue->originalSize			= count;
	transferQueue->buffer				= buffer;
	transferQueue->id					= newId;
	transferQueue->function				= NULL;
	transferQueue->forceFlush			= FALSE;
	transferQueue->IsBufferLogicalAddr	= FALSE;

	// Adjust the fifo pointer
	//  *to = *to + 1;
	//  if (*to == MAXIMUM_DMA_QUEUE_ITEMS) *to = 0;
	//  if (*to == from) return GscApiDmaChannelBusy;
	tempTo = *to + 1;

	if (tempTo == MAXIMUM_DMA_QUEUE_ITEMS)
		tempTo = 0;

	if (tempTo == from)
	{
		EndCriticalSection( qmutex);
		return GscApiDmaChannelBusy;
	}

	*to = tempTo;

  // Return the queue id
  *id = newId;

  // Get the message ID
  messageId += ((getpid() & 0x03ff) << 20) + (processBoardRef[boardNumber - 1] << 12) + (boardNumber << 4);
  threadMessage.command = channel << 4;

  // Flag the proper dma handler
  threadMessage.type = messageId;
  threadMessage.command += GSC_THREAD_START_CMD;
  if (msgsnd(boardInformation[boardNumber - 1].dmaQueueId, (void *)&threadMessage, sizeof(int), 0) == -1)
  {
    EndCriticalSection( qmutex);
    return ApiFailed;
  }

  EndCriticalSection( qmutex);

  return status;
}




//----------------------------------------------------------- gscAddDmaRequest
//
//  This routine adds a DMA request to the queue for later
// processing. A unique Transfer ID is created for each
// DMA request. Tx transfers are given even IDs and Rx
// transfers are given odd IDs.
//
int gscAddPlxPhysDmaRequest(int boardNumber, int channel, PLX_PHYSICAL_MEM *buffer, int count, int direction, int *id)
{
  int status = 0x00, newId;
  PDMA_TRANSFER_QUEUE	transferQueue;
  int *to, from, tempTo;
  int value = 0;
  int tempCount = 0;
#ifdef WIN32
  HANDLE hEvent;
  PULONG nSize = 0;
#endif
  int oldCMR, oldCCSR;
#ifdef PLX_LINUX
  long int messageId;
  THREAD_MESSAGE  threadMessage;
#endif

  //printf("Add dma request: direction = %d\n", direction);
  BeginCriticalSection( qmutex);

  // Make sure the direction is valid
  switch (direction)
  {
  case GSC_DATA_TRANSMIT:
    to = &txDmaQueueTo[ boardNumber-1][ channel-1];
    from = txDmaQueueFrom[ boardNumber-1][ channel-1];
    transferQueue = &txDmaQueue[ boardNumber-1][ channel-1][*to];
    newId = dmaRequestQueueId[ boardNumber-1];
#ifdef PLX_LINUX
    messageId = GSC_THREAD_TX_DMA_TYPE;
#endif
    break;

  case GSC_DATA_RECEIVE:
    to = &rxDmaQueueTo[ boardNumber-1][ channel-1];
    from = rxDmaQueueFrom[ boardNumber-1][ channel-1];
    transferQueue = &rxDmaQueue[ boardNumber-1][ channel-1][*to];
    newId = dmaRequestQueueId[ boardNumber-1] + 1;
#ifdef PLX_LINUX
    messageId = GSC_THREAD_RX_DMA_TYPE;
#endif
    /* Check whether timetag will be inserted into Rx FIFO for each byte
       and adjust count accordingly */
    status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+((channel-1)<<4), &value);
    if (status) return status;
    tempCount = count;
    if (value & 0x04) tempCount += count * 3;
    if (value & 0x08) tempCount += count;
    count = tempCount;
    break;

  default:
    EndCriticalSection( qmutex);
    return GscApiInvalidParameter;
  }

  // Account for using a new Transfer ID value
  dmaRequestQueueId[ boardNumber-1] += 2;

  // Setup the TCB for transfers if packet framing is enabled
  if ((direction == GSC_DATA_TRANSMIT) &&
      (boardInformation[ boardNumber-1].channelPacketFraming[ channel-1] == GSC_ENABLED))
  {
    // Read the CMR and CCSR
    status = GscSio4ChannelRegisterRead(boardNumber, channel, USC_CMR, &oldCMR);
    if (status)
    {
      EndCriticalSection( qmutex);
      return status;
    }
    status = GscSio4ChannelRegisterRead(boardNumber, channel, USC_CCSR, &oldCCSR);
    if (status)
    {
      EndCriticalSection( qmutex);
      return status;
    }
    transferQueue->tcbNeeded = TRUE;
    transferQueue->tcbBytes[0] = oldCCSR & 0x1c;
    transferQueue->tcbBytes[1] = (oldCMR>>8) & 0xf0;
    transferQueue->tcbBytes[2] = count & 0xff;
    transferQueue->tcbBytes[3] = (count>>8) & 0xff;
  }

  // Otherwise, just clear the flag
  else
  {
    transferQueue->tcbNeeded = FALSE;
  }

  // Put info into the proper transfer queue
  transferQueue->size = count;
  transferQueue->originalSize = count;
  transferQueue->physBuffer.CpuPhysical = buffer->CpuPhysical;
  transferQueue->physBuffer.PhysicalAddr = buffer->PhysicalAddr;
  transferQueue->physBuffer.Size = buffer->Size;
  transferQueue->physBuffer.UserAddr = buffer->UserAddr;
  transferQueue->id = newId;
  transferQueue->function = NULL;
  transferQueue->forceFlush = FALSE;
  transferQueue->IsBufferLogicalAddr = TRUE;

  // Adjust the fifo pointer
  //  *to = *to + 1;
  //  if (*to == MAXIMUM_DMA_QUEUE_ITEMS) *to = 0;
  //  if (*to == from) return GscApiDmaChannelBusy;
  tempTo = *to + 1;
  if (tempTo == MAXIMUM_DMA_QUEUE_ITEMS) tempTo = 0;
  if (tempTo == from)
  {
    EndCriticalSection( qmutex);
    return GscApiDmaChannelBusy;
  }
  *to = tempTo;

  // Return the queue id
  *id = newId;

#ifdef PLX_LINUX
  // Get the message ID
  messageId += ((getpid() & 0x03ff) << 20) + (processBoardRef[boardNumber - 1] << 12) + (boardNumber << 4);
  threadMessage.command = channel << 4;

  // Flag the proper dma handler
  threadMessage.type = messageId;
  threadMessage.command += GSC_THREAD_START_CMD;
  if (msgsnd(boardInformation[boardNumber - 1].dmaQueueId, (void *)&threadMessage, sizeof(int), 0) == -1)
  {
    EndCriticalSection( qmutex);
    return ApiFailed;
  }
#else
  // Flag the proper dma handler
  if (direction == GSC_DATA_TRANSMIT)
  {
    hEvent = CreateEvent( &sa, TRUE, FALSE, txStartEvent[boardNumber-1][channel-1]);
  }
  else
  {
    hEvent = CreateEvent( &sa, TRUE, FALSE, rxStartEvent[boardNumber-1][channel-1]);
  }
  SetEvent ( hEvent );
  CloseHandle ( hEvent );
#endif

  EndCriticalSection( qmutex);

  return status;
}


int gscSio4PerformDma(int boardNumber, int channel, char *buffer, int count, int direction)
{
  if (drvMajorVersion >= 5)
    return gscPerformDma( boardNumber, channel, buffer, count, direction, 0);
  else
    return gscPerformDma( boardNumber, channel, buffer, count, direction, PrimaryPciChannel0);
}


int gscPerformDma(int boardNumber, int channel, char *buffer, int count, int direction, int dmaChannel)
{
  int status = 0x00;
  int bn = boardNumber - 1;
  DMA_TRANSFER_ELEMENT  DmaData;
  DMA_TRANSFER_ELEMENT3 DmaData3;
  PLX_DMA_PARAMS6       DmaParams6;
  PLX_DMA_PARAMS        DmaParams;
  int dataRegister;

  //printf("gscPerformDma: count = %d, direction = %d\n", count, direction);

  // Determine the correct data register
  if (boardInformation[ bn].boardType == GSC_BOARD_TYPE_ANALOG)
  {
    // For analog boards, get the buffer register from the data structure
    dataRegister = gscAnaGetIDBRegOffset( boardNumber );
  }
  else
  {
    // For the SIO boards, derive the buffer register from the channel number
    dataRegister = (channel << 4) | 0x08;
  }

#if 0
  //
  // ANALOG TEMP
  // Do PIO mode instead of DMA for the Analog Boards
  //
  if (boardInformation[ bn].boardType == GSC_BOARD_TYPE_ANALOG)
  {
    // Do the requested number of reads
    while (count > 0)
    {
      status = fpgaRead( boardNumber, dataRegister, (int *)buffer);
      if (status) return status;
      buffer = buffer + 4;
      count--;
    }
  }
  else
#endif
  {
    // Clear DMA channel parameters
    //  memset( &DmaData, 0x00, sizeof(DMA_CHANNEL_DESC));
    memset( &DmaData, 0x00, sizeof(DMA_TRANSFER_ELEMENT));

    // Setup the DMA parameters. PlxLinux uses PCI_CODE, windows IOP_CODE
    //#ifdef PCI_CODE

    if (drvMajorVersion >= 6 && drvMinorVersion >= 1)
    {
      memset(&DmaParams, 0, sizeof(PLX_DMA_PARAMS));
      DmaParams.UserVa            = (PLX_UINT_PTR)buffer;
      DmaParams.LocalAddr         = (U32)dataRegister;	 // Register offset of the input data buffer
      DmaParams.ByteCount         = (U32)count;
      DmaParams.Direction         = (direction==GSC_DATA_TRANSMIT)?PLX_DMA_PCI_TO_LOC:PLX_DMA_LOC_TO_PCI; // 1=Local=>PCI, 0=PCI=>Local
      //DmaParams6.TerminalCountIntr = 0;
    }
    else if (drvMajorVersion >= 5)
    {
      memset(&DmaParams6, 0, sizeof(PLX_DMA_PARAMS6));
      DmaParams6.u.UserVa          = (PLX_UINT_PTR)buffer;
      DmaParams6.LocalAddr         = (U32)dataRegister;	 // Register offset of the input data buffer
      DmaParams6.ByteCount         = (U32)count;
      DmaParams6.LocalToPciDma     = (unsigned int)((direction==GSC_DATA_TRANSMIT)?0:1); // 1=Local=>PCI, 0=PCI=>Local
      DmaParams6.TerminalCountIntr = 0;
    }
    else if (drvMajorVersion == 4)
    {
      DmaData.u.UserVa          = (PLX_UINT_PTR)buffer;
      DmaData.LocalAddr         = (U32)dataRegister;	 // Register offset of the input data buffer
      DmaData.ByteCount         = (U32)count;
      DmaData.LocalToPciDma     = (unsigned int)((direction==GSC_DATA_TRANSMIT)?0:1); // 1=Local=>PCI, 0=PCI=>Local
      DmaData.TerminalCountIntr = 0;
    }
    else
    {
#ifndef __LP64__
      DmaData3.Pci9080Dma.UserAddr          = (U32)buffer;
#endif
      DmaData3.Pci9080Dma.IopAddr           = (U32)dataRegister; // Register offset of the input data buffer
      DmaData3.Pci9080Dma.TransferCount     = count;
      DmaData3.Pci9080Dma.IopToPciDma       = ((direction==GSC_DATA_TRANSMIT)?0:1); // 1=Local=>PCI, 0=PCI=>Local
      DmaData3.Pci9080Dma.TerminalCountIntr = 0;
    }
    //#endif

    // Start the Transfer and wait for its completion
    BeginCriticalSection( PlxApiCalls);
    if (drvMajorVersion == 6 && drvMinorVersion >= 1)
      status = PlxPci_DmaTransferUserBuffer_Rtdl( &sio4BoardHandles[boardNumber-1], dmaChannel,
                                                  &DmaParams, PLX_TIMEOUT_INFINITE);
    else if (drvMajorVersion >= 5)
      status = PlxPci_DmaTransferUserBuffer6_Rtdl( &sio4BoardHandles6[boardNumber-1], dmaChannel,
                                                  &DmaParams6, PLX_TIMEOUT_INFINITE);
    else if (drvMajorVersion == 4)
      status = PlxDmaSglTransfer_Rtdl( boardInformation[boardNumber-1].handle, dmaChannel, &DmaData, FALSE);
    else
      status = PlxDmaSglTransfer3_Rtdl( boardInformation[boardNumber-1].handle, dmaChannel, &DmaData3, FALSE);
    EndCriticalSection( PlxApiCalls);
    if(status != ApiSuccess) return status;
  }
  return 0x00;
}


int gscPerformPlxPhysDma(int boardNumber, int channel, PLX_PHYSICAL_MEM *buffer, int count, int direction, int dmaChannel)
{
  int status = 0x00;
  int bn = boardNumber - 1;
  DMA_TRANSFER_ELEMENT  DmaData;
  DMA_TRANSFER_ELEMENT3 DmaData3;
  PLX_DMA_PARAMS6       DmaParams6;
  PLX_DMA_PARAMS        DmaParams;
  int dataRegister;

  //printf("gscPerformDma: count = %d, direction = %d\n", count, direction);

  // Determine the correct data register
  if (boardInformation[ bn].boardType == GSC_BOARD_TYPE_ANALOG)
  {
    // For analog boards, get the buffer register from the data structure
    dataRegister = gscAnaGetIDBRegOffset( boardNumber );
  }
  else
  {
    // For the SIO boards, derive the buffer register from the channel number
    dataRegister = (channel << 4) | 0x08;
  }

#if 0
  //
  // ANALOG TEMP
  // Do PIO mode instead of DMA for the Analog Boards
  //
  if (boardInformation[ bn].boardType == GSC_BOARD_TYPE_ANALOG)
  {
    // Do the requested number of reads
    while (count > 0)
    {
      status = fpgaRead( boardNumber, dataRegister, (int *)buffer);
      if (status) return status;
      buffer = buffer + 4;
      count--;
    }
  }
  else
#endif
  {
    // Clear DMA channel parameters
    //  memset( &DmaData, 0x00, sizeof(DMA_CHANNEL_DESC));
    memset( &DmaData, 0x00, sizeof(DMA_TRANSFER_ELEMENT));

    // Setup the DMA parameters. PlxLinux uses PCI_CODE, windows IOP_CODE
    //#ifdef PCI_CODE

    if (drvMajorVersion >= 6 && drvMinorVersion >= 1)
    {
      memset(&DmaParams, 0, sizeof(PLX_DMA_PARAMS));
      DmaParams.PciAddr           = buffer->PhysicalAddr;
      DmaParams.LocalAddr         = (U32)dataRegister;	 // Register offset of the input data buffer
      DmaParams.ByteCount         = (U32)count;
      DmaParams.Direction         = (direction==GSC_DATA_TRANSMIT)?PLX_DMA_PCI_TO_LOC:PLX_DMA_LOC_TO_PCI; // 1=Local=>PCI, 0=PCI=>Local
      //DmaParams6.TerminalCountIntr = 0;
    }
    else if (drvMajorVersion >= 5)
    {
      memset(&DmaParams6, 0, sizeof(PLX_DMA_PARAMS6));
      DmaParams6.u.UserVa          = (PLX_UINT_PTR)buffer;
      DmaParams6.LocalAddr         = (U32)dataRegister;	 // Register offset of the input data buffer
      DmaParams6.ByteCount         = (U32)count;
      DmaParams6.LocalToPciDma     = (unsigned int)((direction==GSC_DATA_TRANSMIT)?0:1); // 1=Local=>PCI, 0=PCI=>Local
      DmaParams6.TerminalCountIntr = 0;
    }
    else if (drvMajorVersion == 4)
    {
      DmaData.u.UserVa          = (PLX_UINT_PTR)buffer;
      DmaData.LocalAddr         = (U32)dataRegister;	 // Register offset of the input data buffer
      DmaData.ByteCount         = (U32)count;
      DmaData.LocalToPciDma     = (unsigned int)((direction==GSC_DATA_TRANSMIT)?0:1); // 1=Local=>PCI, 0=PCI=>Local
      DmaData.TerminalCountIntr = 0;
    }
    else
    {
#ifndef __LP64__
      DmaData3.Pci9080Dma.UserAddr          = (U32)buffer;
#endif
      DmaData3.Pci9080Dma.IopAddr           = (U32)dataRegister; // Register offset of the input data buffer
      DmaData3.Pci9080Dma.TransferCount     = count;
      DmaData3.Pci9080Dma.IopToPciDma       = ((direction==GSC_DATA_TRANSMIT)?0:1); // 1=Local=>PCI, 0=PCI=>Local
      DmaData3.Pci9080Dma.TerminalCountIntr = 0;
    }
    //#endif

    // Start the Transfer and wait for its completion
    BeginCriticalSection( PlxApiCalls);
    if (drvMajorVersion == 6 && drvMinorVersion >= 1)
      status = PlxPci_DmaTransferBlock_Rtdl( &sio4BoardHandles[boardNumber-1], dmaChannel,
                                             &DmaParams, PLX_TIMEOUT_INFINITE);
    else if (drvMajorVersion >= 5)
      status = PlxPci_DmaTransferUserBuffer6_Rtdl( &sio4BoardHandles6[boardNumber-1], dmaChannel,
                                                  &DmaParams6, PLX_TIMEOUT_INFINITE);
    else if (drvMajorVersion == 4)
      status = PlxDmaSglTransfer_Rtdl( boardInformation[boardNumber-1].handle, dmaChannel, &DmaData, FALSE);
    else
      status = PlxDmaSglTransfer3_Rtdl( boardInformation[boardNumber-1].handle, dmaChannel, &DmaData3, FALSE);
    EndCriticalSection( PlxApiCalls);
    if(status != ApiSuccess) return status;
  }
  return 0x00;
}


int gscSio4OpenDma( int boardNumber, int dmaChannel)
{
  int	status = 0x00;
  DMA_CHANNEL_DESC      DmaDesc;
  DMA_CHANNEL_DESC3     DmaDesc3;
  PLX_DMA_PROP6         DmaProps6;
  PLX_DMA_PROP          DmaProps;
  int dmaWidth;

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // Clear DMA channel parameters
  memset( &DmaDesc, 0, sizeof(DMA_CHANNEL_DESC));

  // Determine the correct dma bus width
  if (boardInformation[ boardNumber-1].boardType == GSC_BOARD_TYPE_ANALOG)
  {
    // For analog boards, is always 32
    dmaWidth = 2;
  }
  else
  {
    // For the SIO boards, is always 8
    dmaWidth = 0;
  }

  // Initialize the DMA Channel parameters
  if (drvMajorVersion >= 6 && drvMinorVersion >= 1)
  {
    memset(&DmaProps, 0, sizeof(PLX_DMA_PROP));
    DmaProps.ReadyInput = 1;
    DmaProps.Burst = 1;
    DmaProps.ConstAddrLocal = 1;
    DmaProps.LocalBusWidth = dmaWidth;
    DmaProps.DualAddressMode = 1;
  }
  else if (drvMajorVersion >= 5)
  {
    memset(&DmaProps6, 0, sizeof(PLX_DMA_PROP6));
    DmaProps6.ReadyInput = 1;
    DmaProps6.Burst = 1;
    DmaProps6.LocalAddrConst = 1;
    DmaProps6.LocalBusWidth = dmaWidth;
  }
  else if (drvMajorVersion == 4)
  {
    DmaDesc.EnableReadyInput         = 1;
    DmaDesc.EnableWriteInvalidMode   = 0;
    DmaDesc.EnableDmaEOTPin          = 0;
    DmaDesc.DmaStopTransferMode      = 0;
    DmaDesc.HoldIopAddrConst         = 1;
    DmaDesc.DemandMode               = 0;
    DmaDesc.EnableTransferCountClear = 0;
    DmaDesc.DmaChannelPriority       = ((dmaChannel==PrimaryPciChannel0) ? Channel0Highest:Channel1Highest);
    DmaDesc.WaitStates               = 0;
    DmaDesc.IopBusWidth              = dmaWidth;        // 0 -> 8 bit, 2/3 = 32-bit
    DmaDesc.TholdForIopWrites        = 0;
    DmaDesc.TholdForIopReads         = 0;
    DmaDesc.TholdForPciWrites        = 0;
    DmaDesc.TholdForPciReads         = 0;
    DmaDesc.EnableIopBurst           = 1;
    DmaDesc.EnableBTERMInput         = 0;
    DmaDesc.EnableDualAddressCycles  = 0;
  }
  else
  {
    DmaDesc3.EnableReadyInput         = 1;
    DmaDesc3.EnableWriteInvalidMode   = 0;
    DmaDesc3.EnableDmaEOTPin          = 0;
    DmaDesc3.DmaStopTransferMode      = 0;
    DmaDesc3.HoldIopAddrConst         = 1;
    DmaDesc3.HoldIopSourceAddrConst   = 0;
    DmaDesc3.HoldIopDestAddrConst     = 0;
    DmaDesc3.DemandMode               = 0;
    DmaDesc3.EnableTransferCountClear = 0;
    DmaDesc3.DmaChannelPriority       = ((dmaChannel==PrimaryPciChannel0) ? Channel0Highest:Channel1Highest);
    DmaDesc3.WaitStates               = 0;
    DmaDesc3.IopBusWidth              = dmaWidth;        // 0 -> 8 bit, 2/3 = 32-bit
    DmaDesc3.TholdForIopWrites        = 0;
    DmaDesc3.TholdForIopReads         = 0;
    DmaDesc3.TholdForPciWrites        = 0;
    DmaDesc3.TholdForPciReads         = 0;
    DmaDesc3.EnableIopBurst           = 1;
    DmaDesc3.EnableBTERMInput         = 0;
    DmaDesc3.EnableDualAddressCycles  = 0;
  }
  // Open the DMA channel
  BeginCriticalSection(PlxApiCalls);
  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    status = PlxPci_DmaChannelOpen_Rtdl(&sio4BoardHandles[boardNumber-1], dmaChannel, &DmaProps);
  else if (drvMajorVersion >= 5)
    status = PlxPci_DmaChannelOpen6_Rtdl(&sio4BoardHandles6[boardNumber-1], dmaChannel, &DmaProps6);
  else if (drvMajorVersion == 4)
    status = PlxDmaSglChannelOpen_Rtdl( boardInformation[boardNumber-1].handle, dmaChannel, &DmaDesc);
  else
    status = PlxDmaSglChannelOpen3_Rtdl( boardInformation[boardNumber-1].handle, dmaChannel, &DmaDesc3);
  EndCriticalSection(PlxApiCalls);

  if (status != ApiSuccess) return status;

  return 0;
}

int gscSio4CloseDma( int boardNumber, int dmaChannel)
{
  int status;

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  BeginCriticalSection(PlxApiCalls);
  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
  {
    PlxPci_DmaControl_Rtdl( &sio4BoardHandles[ boardNumber-1], dmaChannel, DmaAbort);
    status = PlxPci_DmaChannelClose_Rtdl(&sio4BoardHandles[boardNumber-1], dmaChannel);
  }
  else if (drvMajorVersion >= 5)
  {
    PlxPci_DmaControl6_Rtdl( &sio4BoardHandles6[ boardNumber-1], dmaChannel, DmaAbort);
    status = PlxPci_DmaChannelClose6_Rtdl(&sio4BoardHandles6[boardNumber-1], dmaChannel);
  }
  else
  {
    PlxDmaControl_Rtdl( boardInformation[boardNumber-1].handle, dmaChannel, DmaAbort);
    status = PlxDmaSglChannelClose_Rtdl( boardInformation[boardNumber-1].handle, dmaChannel);
  }
  EndCriticalSection(PlxApiCalls);

  if (status != ApiSuccess) return status;

  return 0;
}


#ifdef PLX_LINUX
int GscStopDmaThreads( int boardNumber )
{
  THREAD_MESSAGE message;
  int i = 0;
  int status = 0;

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  BeginCriticalSection( GscApiCalls );

  // Load the thread abort command
  message.command = GSC_THREAD_ABORT_CMD;

  // Signal all of the DMA threads to shut down
  for (i=1; i<=1; i++)
  {
    // Send message to the Tx threads
    message.type = ((getpid() & 0x03ff) << 20) + (processBoardRef[boardNumber - 1] << 12) + GSC_THREAD_TX_DMA_TYPE + (boardNumber << 4);
    if(msgsnd(boardInformation[boardNumber - 1].dmaQueueId,(void*)&message,sizeof(int),0)==-1)
      return ApiFailed;

    // Send message to the Rx thread
    message.type = ((getpid() & 0x03ff) << 20) + (processBoardRef[boardNumber - 1] << 12) + GSC_THREAD_RX_DMA_TYPE + (boardNumber << 4);
    if(msgsnd(boardInformation[boardNumber - 1].dmaQueueId,(void*)&message,sizeof(int),0)==-1)
      return ApiFailed;
  }

  // Give the threads time to exit
  usleep(100000);

  EndCriticalSection( GscApiCalls );

  return 0;
}
#endif
