//////////////////////////////////////////////////////////////////////////////
//
//  GscApiPlxLegacy.h
//
//  This header file supplements the PLX SDK v.5 PlxLegacy.h header file with data structures that
// differ from the PLX SDK v. 4.4 definitions, or were not present in v.4.4. These data structures
// are necessary for calling Plx API functions in older versions of the PLX SDK. (prior to v. 4.4)
//

//
// Typedefs for Plx Api functions that changed with major release 5, along with typedefs
// for the corresponding legacy functions found in earlier versions of the PLX Api.
//
typedef PLX_STATUS RETURN_CODE;

typedef PLX_STATUS (*PlxPciPhysicalMemoryAllocate)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice,
    PLX_PHYSICAL_MEM    *pMemoryInfo,
    BOOLEAN             bSmallerOk
    );

typedef PLX_STATUS (*PlxPciPhysicalMemoryMap)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice,
    PLX_PHYSICAL_MEM    *pMemoryInfo
    );

typedef PLX_STATUS (*PlxPciApiVersion)( // PLX 6.1
    U8                  *pVersionMajor,
    U8                  *pVersionMinor,
    U8                  *pVersionRevision
    );

typedef PLX_STATUS (*PlxPciDrvVersion)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice,
    U8                  *pVersionMajor,
    U8                  *pVersionMinor,
    U8                  *pVersionRevision
    );

typedef RETURN_CODE (*PlxPciDrvVersion6)( // PLX 5.1 - 6.0
    PLX_DEVICE_OBJECT6  *pDevice,
    U8                  *pVersionMajor,
    U8                  *pVersionMinor,
    U8                  *pVersionRevision
    );

typedef RETURN_CODE (*PlxDrvVersionLegacy)( // Legacy PLX
    HANDLE   pDevice,
    U8       *pVersionMajor,
    U8       *pVersionMinor,
    U8       *pVersionRevision
    );

typedef PLX_STATUS (*PlxPciDevFind)( // PLX 6.1
    PLX_DEVICE_KEY   *pKey,
    U8               DeviceNumber
    );

typedef RETURN_CODE (*PlxPciDevFind6)( // PLX 5.1 - 6.0
    PLX_DEVICE_KEY6   *pKey,
    U8                DeviceNumber
    );

typedef RETURN_CODE (*PlxPciDevFindLegacy)( // Legacy PLX
    DEVICE_LOCATION   *pKey,
    U32               *pRequestLimit
    );

typedef RETURN_CODE (*PlxPciDevFindLegacy3)( // Legacy PLX SDk 3.4
    DEVICE_LOCATION3  *pKey,
    U32               *pRequestLimit
    );

typedef PLX_STATUS (*PlxPciDevOpen)( // PLX 6.1
    PLX_DEVICE_KEY      *pKey,
    PLX_DEVICE_OBJECT   *pDevice
    );

typedef RETURN_CODE (*PlxPciDevOpen6)( // PLX 5.1 - 6.0
    PLX_DEVICE_KEY6      *pKey,
    PLX_DEVICE_OBJECT6   *pDevice
    );

typedef RETURN_CODE (*PlxPciDevOpenLegacy)( // Legacy PLX
    DEVICE_LOCATION     *pKey,
    HANDLE              *pDevice
    );

typedef RETURN_CODE (*PlxPciDevOpenLegacy3)( // Legacy PLX SDk 3.4
    DEVICE_LOCATION3    *pKey,
    HANDLE              *pDevice
    );

typedef PLX_STATUS (*PlxPciDevClose)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice
    );

typedef RETURN_CODE (*PlxPciDevClose6)( // PLX 5.1 - 6.0
    PLX_DEVICE_OBJECT6   *pDevice
    );

typedef RETURN_CODE (*PlxPciDevCloseLegacy)( // Legacy PLX
    HANDLE   hDevice
    );

typedef PLX_STATUS (*PlxPciBarSpaceIO)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice,
    U8                  BarIndex,
    U32                 offset,
    VOID                *pBuffer,
    U32                 ByteCount,
    PLX_ACCESS_TYPE     AccessType,
    BOOLEAN             bOffsetAsLocalAddr
    );

typedef RETURN_CODE (*PlxPciBarSpaceIO6)( // PLX 5.1 - 6.0
    PLX_DEVICE_OBJECT6  *pDevice,
    U8                  BarIndex,
    U32                 offset,
    VOID                *pBuffer,
    U32                 ByteCount,
    PLX_ACCESS_TYPE     AccessType,
    BOOLEAN             bOffsetAsLocalAddr
    );

typedef RETURN_CODE (*PlxBusIopLegacy) ( // Legacy PLX
  HANDLE      hDevice,
  IOP_SPACE   IopSpace,
  U32         address,
  BOOLEAN     bRemap,
  VOID        *pBuffer,
  U32         ByteCount,
  ACCESS_TYPE AccessType
  );

typedef PLX_STATUS (*PlxPciDmaTransfer)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice,
    U8                  channel,
    PLX_DMA_PARAMS      *pDmaParams,
    U64                 Timeout_ms
    );

typedef RETURN_CODE (*PlxPciDmaTransfer6)( // PLX 5.1 - 6.0
    PLX_DEVICE_OBJECT6  *pDevice,
    U8                  channel,
    PLX_DMA_PARAMS6     *pDmaParams,
    U64                 Timeout_ms
    );

typedef RETURN_CODE (*PlxDmaSglTransferLegacy)( // Legacy PLX
    HANDLE                hDevice,
    DMA_CHANNEL           channel,
    DMA_TRANSFER_ELEMENT  *dmaData,
    BOOLEAN               returnImmediate
    );

typedef RETURN_CODE (*PlxDmaSglTransferLegacy3)( // Legacy PLX SDk 3.4
    HANDLE                hDevice,
    DMA_CHANNEL           channel,
    DMA_TRANSFER_ELEMENT3 *dmaData,
    BOOLEAN               returnImmediate
    );

typedef PLX_STATUS (*PlxPciDmaChannelClose)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice,
    U8                  channel
    );

typedef RETURN_CODE (*PlxPciDmaChannelClose6)( // PLX 5.1 - 6.0
    PLX_DEVICE_OBJECT6  *pDevice,
    U8                  channel
    );

typedef RETURN_CODE (*PlxDmaSglChannelCloseLegacy)( // Legacy PLX
    HANDLE        hDevice,
    DMA_CHANNEL   channel
    );

typedef PLX_STATUS (*PlxPciDmaChannelOpen)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice,
    U8                  channel,
    PLX_DMA_PROP        *pDmaProp
    );

typedef RETURN_CODE (*PlxPciDmaChannelOpen6)( // PLX 5.1 - 6.0
    PLX_DEVICE_OBJECT6  *pDevice,
    U8                  channel,
    PLX_DMA_PROP6       *pDmaProp
    );

typedef RETURN_CODE (*PlxDmaSglChannelOpenLegacy)( // Legacy PLX
    HANDLE              hDevice,
    DMA_CHANNEL         channel,
    DMA_CHANNEL_DESC    *pDesc
    );

typedef RETURN_CODE (*PlxDmaSglChannelOpenLegacy3)( // Legacy PLX SDk 3.4
    HANDLE              hDevice,
    DMA_CHANNEL         channel,
    DMA_CHANNEL_DESC3   *pDesc
    );

typedef PLX_STATUS (*PlxPciDmaControl)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice,
    U8                  channel,
    PLX_DMA_COMMAND     command
    );

typedef RETURN_CODE (*PlxPciDmaControl6)( // PLX 5.1 - 6.0
    PLX_DEVICE_OBJECT6  *pDevice,
    U8                  channel,
    PLX_DMA_COMMAND     command
    );

typedef RETURN_CODE (*PlxDmaControlLegacy) ( // Legacy PLX
  HANDLE      hDevice,
  DMA_CHANNEL channel,
  DMA_COMMAND command
  );

typedef PLX_STATUS (*PlxPciInterrupt)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice,
    PLX_INTERRUPT       *pPlxIntr
    );

typedef RETURN_CODE (*PlxPciInterrupt6)( // PLX 5.1 - 6.0
    PLX_DEVICE_OBJECT6  *pDevice,
    PLX_INTERRUPT6      *pPlxIntr
    );

typedef RETURN_CODE (*PlxInterruptLegacy)( // Legacy PLX
    HANDLE    hDevice,
    PLX_INTR  *pPlxIntr
    );

typedef RETURN_CODE (*PlxInterruptLegacy3)( // Legacy PLX SDK 3.4
    HANDLE    hDevice,
    PLX_INTR3 *pPlxIntr
    );

typedef PLX_STATUS (*PlxPciNotifReg)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice,
    PLX_INTERRUPT       *pPlxIntr,
    PLX_NOTIFY_OBJECT   *pEvent
    );

typedef RETURN_CODE (*PlxPciNotifReg6)( // PLX 5.1 - 6.0
    PLX_DEVICE_OBJECT6  *pDevice,
    PLX_INTERRUPT6      *pPlxIntr,
    PLX_NOTIFY_OBJECT   *pEvent
    );

typedef RETURN_CODE (*PlxNotifRegLegacy)( // Legacy PLX
    HANDLE             hDevice,
    PLX_INTR          *pPlxIntr,
    PLX_NOTIFY_OBJECT *pEvent
    );

typedef PLX_STATUS (*PlxPciNotifWait)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice,
    PLX_NOTIFY_OBJECT   *pEvent,
    U64                 Timeout_ms
    );

typedef RETURN_CODE (*PlxPciNotifWait6)( // PLX 5.1 - 6.0
    PLX_DEVICE_OBJECT6  *pDevice,
    PLX_NOTIFY_OBJECT   *pEvent,
    U32                 Timeout_ms
    );

typedef RETURN_CODE (*PlxNotifWaitLegacy)( // Legacy PLX
    HANDLE             hDevice,
    PLX_NOTIFY_OBJECT *pEvent,
    U32                Timeout_ms
    );

typedef PLX_STATUS (*PlxPciNotifCancel)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice,
    PLX_NOTIFY_OBJECT   *pEvent
    );

typedef RETURN_CODE (*PlxPciNotifCancel6)( // PLX 5.1 - 6.0
    PLX_DEVICE_OBJECT6  *pDevice,
    PLX_NOTIFY_OBJECT   *pEvent
    );

typedef RETURN_CODE (*PlxIntAttLegacy)( // Legacy PLX SDK 3.4
    HANDLE    hDevice,
    PLX_INTR3 intrTypes,
    HANDLE   *pEventHdl
    );

typedef RETURN_CODE (*PlxIntWaitLegacy)( // Legacy PLX
    HANDLE hDevice,
    HANDLE hEvent,
    U32    Timeout_ms
    );

typedef U32 (*PlxPciRegReadFast)( // PLX 6.1
    PLX_DEVICE_OBJECT   *pDevice,
    U16                 offset,
    PLX_STATUS          *pReturnCode
    );

typedef RETURN_CODE (*PlxPciRegReadFast6)( // PLX 5.1 - 6.0
    PLX_DEVICE_OBJECT6  *pDevice,
    U16                 offset,
    RETURN_CODE         *pReturnCode
    );

typedef RETURN_CODE (*PlxPciRegReadFastLegacy)( // Legacy PLX
    HANDLE              hDevice,
    U16                 offset,
    RETURN_CODE         *pReturnCode
    );


//
// Declare/Extern global variables
//
#ifdef __MAIN__
 #define EXTERN
#else
 #define EXTERN extern
#endif

// PlxApi function pointers for dynamic loading from Plx Api dll.
EXTERN  PlxPciPhysicalMemoryAllocate  PlxPci_PhysicalMemoryAllocate_Rtdl;
EXTERN  PlxPciPhysicalMemoryMap       PlxPci_PhysicalMemoryMap_Rtdl;
EXTERN  PlxPciPhysicalMemoryMap       PlxPci_PhysicalMemoryUnmap_Rtdl;
EXTERN  PlxPciApiVersion              PlxPci_ApiVersion_Rtdl;
EXTERN  PlxPciDrvVersion              PlxPci_DriverVersion_Rtdl;
EXTERN  PlxPciDrvVersion6             PlxPci_DriverVersion6_Rtdl;
EXTERN  PlxDrvVersionLegacy           PlxDriverVersion_Rtdl;
EXTERN  PlxPciDevFind                 PlxPci_DeviceFind_Rtdl;
EXTERN  PlxPciDevFind6                PlxPci_DeviceFind6_Rtdl;
EXTERN  PlxPciDevFindLegacy           PlxPciDeviceFind_Rtdl;
EXTERN  PlxPciDevFindLegacy3          PlxPciDeviceFind3_Rtdl;
EXTERN  PlxPciDevOpen                 PlxPci_DeviceOpen_Rtdl;
EXTERN  PlxPciDevOpen6                PlxPci_DeviceOpen6_Rtdl;
EXTERN  PlxPciDevOpenLegacy           PlxPciDeviceOpen_Rtdl;
EXTERN  PlxPciDevOpenLegacy3          PlxPciDeviceOpen3_Rtdl;
EXTERN  PlxPciDevClose                PlxPci_DeviceClose_Rtdl;
EXTERN  PlxPciDevClose6               PlxPci_DeviceClose6_Rtdl;
EXTERN  PlxPciDevCloseLegacy          PlxPciDeviceClose_Rtdl;
EXTERN  PlxPciBarSpaceIO              PlxPci_PciBarSpaceRead_Rtdl;
EXTERN  PlxPciBarSpaceIO6             PlxPci_PciBarSpaceRead6_Rtdl;
EXTERN  PlxPciBarSpaceIO              PlxPci_PciBarSpaceWrite_Rtdl;
EXTERN  PlxPciBarSpaceIO6             PlxPci_PciBarSpaceWrite6_Rtdl;
EXTERN  PlxBusIopLegacy               PlxBusIopRead_Rtdl;
EXTERN  PlxBusIopLegacy               PlxBusIopWrite_Rtdl;
EXTERN  PlxPciDmaTransfer             PlxPci_DmaTransferUserBuffer_Rtdl;
EXTERN  PlxPciDmaTransfer             PlxPci_DmaTransferBlock_Rtdl;
EXTERN  PlxPciDmaTransfer6            PlxPci_DmaTransferUserBuffer6_Rtdl;
EXTERN  PlxDmaSglTransferLegacy       PlxDmaSglTransfer_Rtdl;
EXTERN  PlxDmaSglTransferLegacy3      PlxDmaSglTransfer3_Rtdl;
EXTERN  PlxPciDmaChannelClose         PlxPci_DmaChannelClose_Rtdl;
EXTERN  PlxPciDmaChannelClose6        PlxPci_DmaChannelClose6_Rtdl;
EXTERN  PlxDmaSglChannelCloseLegacy   PlxDmaSglChannelClose_Rtdl;
EXTERN  PlxPciDmaChannelOpen          PlxPci_DmaChannelOpen_Rtdl;
EXTERN  PlxPciDmaChannelOpen6         PlxPci_DmaChannelOpen6_Rtdl;
EXTERN  PlxDmaSglChannelOpenLegacy    PlxDmaSglChannelOpen_Rtdl;
EXTERN  PlxDmaSglChannelOpenLegacy3   PlxDmaSglChannelOpen3_Rtdl;
EXTERN  PlxPciDmaControl              PlxPci_DmaControl_Rtdl;
EXTERN  PlxPciDmaControl6             PlxPci_DmaControl6_Rtdl;
EXTERN  PlxDmaControlLegacy           PlxDmaControl_Rtdl;
EXTERN  PlxPciInterrupt               PlxPci_InterruptEnable_Rtdl;
EXTERN  PlxPciInterrupt6              PlxPci_InterruptEnable6_Rtdl;
EXTERN  PlxPciInterrupt               PlxPci_InterruptDisable_Rtdl;
EXTERN  PlxPciInterrupt6              PlxPci_InterruptDisable6_Rtdl;
EXTERN  PlxInterruptLegacy            PlxIntrDisable_Rtdl;
EXTERN  PlxInterruptLegacy3           PlxIntrDisable3_Rtdl;
EXTERN  PlxInterruptLegacy            PlxIntrEnable_Rtdl;
EXTERN  PlxInterruptLegacy3           PlxIntrEnable3_Rtdl;
EXTERN  PlxPciNotifReg                PlxPci_NotificationRegisterFor_Rtdl;
EXTERN  PlxPciNotifReg6               PlxPci_NotificationRegisterFor6_Rtdl;
EXTERN  PlxNotifRegLegacy             PlxNotificationRegisterFor_Rtdl;
EXTERN  PlxPciNotifWait               PlxPci_NotificationWait_Rtdl;
EXTERN  PlxPciNotifWait6              PlxPci_NotificationWait6_Rtdl;
EXTERN  PlxNotifWaitLegacy            PlxNotificationWait_Rtdl;
EXTERN  PlxPciNotifCancel             PlxPci_NofificationCancel_Rtdl;
EXTERN  PlxPciNotifCancel6            PlxPci_NofificationCancel6_Rtdl;
EXTERN  PlxIntAttLegacy               PlxIntrAttach_Rtdl;
EXTERN  PlxIntWaitLegacy              PlxIntrWait_Rtdl;
EXTERN  PlxPciRegReadFast             PlxPci_PciRegisterReadFast_Rtdl;
EXTERN  PlxPciRegReadFast6            PlxPci_PciRegisterReadFast6_Rtdl;
EXTERN  PlxPciRegReadFastLegacy       PlxPciRegisterReadFast_Rtdl;

